@@ -9,6 +9,6 @@
 #undef PACKAGE_SOURCE_DIR
 #undef ENABLE_BONOBO
 #undef USING_OAF
-#undef NEW_RENDER
 #undef ENABLE_AUTOTRACE
-
+#undef ENABLE_GNOME_PRINT
+#undef USE_MMX
