@@ -1,6 +1,22 @@
-#define SP_ARC_CONTEXT_C
+#define __SP_ARC_CONTEXT_C__
+
+/*
+ * Ellipse drawing context
+ *
+ * Authors:
+ *   Mitsuru Oka
+ *   Lauris Kaplinski <lauris@kaplinski.com>
+ *
+ * Copyright (C) 2000 Lauris Kaplinski
+ * Copyright (C) 2000-2001 Ximian, Inc.
+ * Copyright (C) 2002 Mitsuru Oka
+ * Copyright (C) 2002 Lauris Kaplinski
+ *
+ * Released under GNU GPL, read the file 'COPYING' for more information
+ */
 
 #include <math.h>
+#include "helper/sp-canvas.h"
 #include "sodipodi.h"
 #include "sp-ellipse.h"
 #include "document.h"
@@ -136,12 +152,15 @@ sp_arc_context_root_handler (SPEventContext * event_context, GdkEvent * event)
 	switch (event->type) {
 	case GDK_BUTTON_PRESS:
 		if (event->button.button == 1) {
+			NRPointF fp;
 			dragging = TRUE;
 			/* Position center */
-			sp_desktop_w2d_xy_point (event_context->desktop, &ac->center, event->button.x, event->button.y);
+			sp_desktop_w2d_xy_point (event_context->desktop, &fp, event->button.x, event->button.y);
+			ac->center.x = fp.x;
+			ac->center.y = fp.y;
 			/* Snap center to nearest magnetic point */
 			sp_desktop_free_snap (event_context->desktop, &ac->center);
-			gnome_canvas_item_grab (GNOME_CANVAS_ITEM (desktop->acetate),
+			sp_canvas_item_grab (SP_CANVAS_ITEM (desktop->acetate),
 						GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK,
 						NULL, event->button.time);
 			ret = TRUE;
@@ -149,7 +168,7 @@ sp_arc_context_root_handler (SPEventContext * event_context, GdkEvent * event)
 		break;
 	case GDK_MOTION_NOTIFY:
 		if (dragging && event->motion.state && GDK_BUTTON1_MASK) {
-			ArtPoint p;
+			NRPointF p;
 			sp_desktop_w2d_xy_point (event_context->desktop, &p, event->motion.x, event->motion.y);
 			sp_arc_drag (ac, p.x, p.y, event->motion.state);
 			ret = TRUE;
@@ -161,7 +180,7 @@ sp_arc_context_root_handler (SPEventContext * event_context, GdkEvent * event)
 			sp_arc_finish (ac);
 			ret = TRUE;
 		}
-		gnome_canvas_item_ungrab (GNOME_CANVAS_ITEM (desktop->acetate), event->button.time);
+		sp_canvas_item_ungrab (SP_CANVAS_ITEM (desktop->acetate), event->button.time);
 		break;
 	default:
 		break;
@@ -183,6 +202,7 @@ sp_arc_drag (SPArcContext * ac, double x, double y, guint state)
 	gdouble x0, y0, x1, y1;
 	GString * xs, * ys;
 	gchar status[80];
+	NRPointF fp;
 
 	desktop = SP_EVENT_CONTEXT (ac)->desktop;
 
@@ -267,8 +287,12 @@ sp_arc_drag (SPArcContext * ac, double x, double y, guint state)
 		sp_desktop_free_snap (desktop, &p1);
 	}
 
-	sp_desktop_d2doc_xy_point (desktop, &p0, p0.x, p0.y);
-	sp_desktop_d2doc_xy_point (desktop, &p1, p1.x, p1.y);
+	sp_desktop_dt2root_xy_point (desktop, &fp, p0.x, p0.y);
+	p0.x = fp.x;
+	p0.y = fp.y;
+	sp_desktop_dt2root_xy_point (desktop, &fp, p1.x, p1.y);
+	p1.x = fp.x;
+	p1.y = fp.y;
 
 	x0 = MIN (p0.x, p1.x);
 	y0 = MIN (p0.y, p1.y);
