@@ -4,14 +4,10 @@
 #include <config.h>
 #include <gnome.h>
 #include <bonobo.h>
+#include "../forward.h"
+#include "../desktop.h"
 #include "embeddable-document.h"
 
-#ifndef SP_DESKTOP_DEFINED
-#define SP_DESKTOP_DEFINED
-typedef struct _SPDesktop * SPDesktop;
-typedef struct _SPDesktopClass SPDesktopClass;
-#endif
-
 #define SP_EMBEDDABLE_DESKTOP_TYPE	(sp_embeddable_desktop_get_type ())
 #define SP_EMBEDDABLE_DESKTOP(o)  	(GTK_CHECK_CAST ((o), SP_EMBEDDABLE_DESKTOP_TYPE, SPEmbeddableDesktop))
 #define SP_EMBEDDABLE_DESKTOP_CLASS(k)    (GTK_CHECK_CLASS_CAST((k), SP_EMBEDDABLE_DESKTOP_TYPE, SPEmbeddableDesktopClass))
@@ -29,7 +25,7 @@ typedef struct _SPEmbeddableDesktopClass SPEmbeddableDesktopClass;
 struct _SPEmbeddableDesktop {
 	BonoboView view;
 	SPEmbeddableDocument * document;
-	SPDesktop * desktop;
+	SPDesktopWidget * desktop;
 };
 
 struct _SPEmbeddableDesktopClass {
