@@ -1,4 +1,22 @@
-#define SP_DESKTOP_EVENTS_C
+#define __SP_DESKTOP_EVENTS_C__
+
+/*
+ * Event handlers for SPDesktop
+ *
+ * Author:
+ *   Lauris Kaplinski <lauris@kaplinski.com>
+ *
+ * Copyright (C) 1999-2002 Lauris Kaplinski
+ *
+ * Released under GNU GPL, read the file 'COPYING' for more information
+ */
+
+#include <gtk/gtksignal.h>
+#include <gtk/gtkspinbutton.h>
+#include <gtk/gtkhbox.h>
+#include <gtk/gtklabel.h>
+#include <libgnomeui/gnome-stock.h>
+#include <libgnomeui/gnome-dialog.h>
 
 #include "helper/sp-guide.h"
 #include "helper/unit-menu.h"
@@ -13,7 +31,6 @@
 #include "sp-metrics.h"
 #include "sp-item.h"
 #include "desktop-events.h"
-#include "gnome.h"
 
 static void sp_dt_simple_guide_dialog (SPGuide * guide, SPDesktop * desktop);
 
@@ -22,12 +39,12 @@ static void sp_dt_simple_guide_dialog (SPGuide * guide, SPDesktop * desktop);
 
 
 void
-sp_desktop_root_handler (GnomeCanvasItem * item, GdkEvent * event, SPDesktop * desktop)
+sp_desktop_root_handler (SPCanvasItem * item, GdkEvent * event, SPDesktop * desktop)
 {
 	if (event->type == GDK_ENTER_NOTIFY) {
 		/* fixme: should it go here? 
 		   moved it to sp_desktop_set_focus (Frank)
-	  gnome_canvas_item_grab_focus ((GnomeCanvasItem *) desktop->main); 
+	  sp_canvas_item_grab_focus ((SPCanvasItem *) desktop->main); 
 		*/
 	}
 	sp_event_context_root_handler (desktop->event_context, event);
@@ -40,7 +57,7 @@ sp_desktop_root_handler (GnomeCanvasItem * item, GdkEvent * event, SPDesktop * d
  */
 
 void
-sp_desktop_item_handler (GnomeCanvasItem * item, GdkEvent * event, gpointer data)
+sp_desktop_item_handler (SPCanvasItem * item, GdkEvent * event, gpointer data)
 {
 	gpointer ddata;
 	SPDesktop * desktop;
@@ -66,8 +83,7 @@ static gint
 sp_dt_ruler_event (GtkWidget * widget, GdkEvent * event, gpointer data, gboolean horiz)
 {
 	static gboolean dragging = FALSE;
-	static GnomeCanvasItem * guide = NULL;
-	static ArtPoint p;
+	static SPCanvasItem * guide = NULL;
 	SPDesktopWidget *dtw;
 	SPDesktop *desktop;
 
@@ -78,7 +94,7 @@ sp_dt_ruler_event (GtkWidget * widget, GdkEvent * event, gpointer data, gboolean
 	case GDK_BUTTON_PRESS:
 		if (event->button.button == 1) {
 			dragging = TRUE;
-			guide = gnome_canvas_item_new (desktop->guides, SP_TYPE_GUIDELINE,
+			guide = sp_canvas_item_new (desktop->guides, SP_TYPE_GUIDELINE,
 						       "orientation",
 						       horiz ? SP_GUIDELINE_ORIENTATION_HORIZONTAL : SP_GUIDELINE_ORIENTATION_VERTICAL,
 						       "color", desktop->namedview->guidehicolor,
@@ -91,19 +107,15 @@ sp_dt_ruler_event (GtkWidget * widget, GdkEvent * event, gpointer data, gboolean
 		break;
 	case GDK_MOTION_NOTIFY:
 		if (dragging) {
-		    /* we have to substract (x|y)thickness to the position 
-		     * since there is a frame between ruler and canvas */
-			gnome_canvas_window_to_world (dtw->canvas,
-						      event->motion.x - 
-						      (horiz ? 0//-widget->style->klass->xthickness
-						             : widget->allocation.width + 
-						               widget->style->klass->xthickness),
-						      event->motion.y - 
-						      (horiz ? widget->allocation.height +
-						               widget->style->klass->ythickness 
-						             : 0),//-widget->style->klass->ythickness),
-						      &p.x, &p.y);
-			sp_desktop_w2d_xy_point (desktop, &p, p.x, p.y);
+			NRPointF p;
+			double px, py;
+			/* we have to substract (x|y) thickness from the position */
+			/* since there is a frame between ruler and canvas */
+			sp_canvas_window_to_world (dtw->canvas,
+						      event->motion.x - (horiz ? 0 : widget->allocation.width + widget->style->klass->xthickness),
+						      event->motion.y - (horiz ? widget->allocation.height + widget->style->klass->ythickness : 0),
+						      &px, &py);
+			sp_desktop_w2d_xy_point (desktop, &p, px, py);
 			sp_guideline_moveto ((SPGuideLine *) guide, p.x, p.y);
 			switch (SP_GUIDELINE(guide)->orientation){
 			case SP_GUIDELINE_ORIENTATION_HORIZONTAL:
@@ -113,10 +125,20 @@ sp_dt_ruler_event (GtkWidget * widget, GdkEvent * event, gpointer data, gboolean
 				sp_desktop_set_coordinate_status (desktop, p.x, p.y, SP_COORDINATES_UNDERLINE_X);
 				break;
 			}
+			sp_view_set_position (SP_VIEW (desktop), p.x, p.y);
 		}
 		break;
 	case GDK_BUTTON_RELEASE:
 		if (dragging && event->button.button == 1) {
+			NRPointF p;
+			double px, py;
+			/* we have to substract (x|y) thickness from the position */
+			/* since there is a frame between ruler and canvas */
+			sp_canvas_window_to_world (dtw->canvas,
+						      event->motion.x - (horiz ? 0 : widget->allocation.width + widget->style->klass->xthickness),
+						      event->motion.y - (horiz ? widget->allocation.height + widget->style->klass->ythickness : 0),
+						      &px, &py);
+			sp_desktop_w2d_xy_point (desktop, &p, px, py);
 		        gdk_pointer_ungrab (event->button.time);
 			dragging = FALSE;
 			gtk_object_destroy (GTK_OBJECT (guide));
@@ -154,12 +176,11 @@ sp_dt_vruler_event (GtkWidget * widget, GdkEvent * event, gpointer data)
 /* Guides */
 
 gint
-sp_dt_guide_event (GnomeCanvasItem * item, GdkEvent * event, gpointer data)
+sp_dt_guide_event (SPCanvasItem * item, GdkEvent * event, gpointer data)
 {
 	static gboolean dragging = FALSE, moved = FALSE;
 	SPGuide * guide;
 	SPDesktop * desktop;
-	ArtPoint p;
 	gint ret = FALSE;
 	GString * msg;
 
@@ -170,7 +191,7 @@ sp_dt_guide_event (GnomeCanvasItem * item, GdkEvent * event, gpointer data)
 	case GDK_2BUTTON_PRESS:
 		if (event->button.button == 1) {
 			dragging = FALSE;
-			gnome_canvas_item_ungrab (item, event->button.time);
+			sp_canvas_item_ungrab (item, event->button.time);
 			sp_dt_simple_guide_dialog (guide, desktop);
 			ret = TRUE;
 		}
@@ -178,13 +199,14 @@ sp_dt_guide_event (GnomeCanvasItem * item, GdkEvent * event, gpointer data)
 	case GDK_BUTTON_PRESS:
 		if (event->button.button == 1) {
 			dragging = TRUE;
-			gnome_canvas_item_grab (item, GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK,
+			sp_canvas_item_grab (item, GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK,
 						NULL, event->button.time);
 			ret = TRUE;
 		}
 		break;
 	case GDK_MOTION_NOTIFY:
 		if (dragging) {
+			NRPointF p;
 			sp_desktop_w2d_xy_point (desktop, &p, event->motion.x, event->motion.y);
 			sp_guide_moveto (guide, p.x, p.y);
 			moved = TRUE;
@@ -196,16 +218,18 @@ sp_dt_guide_event (GnomeCanvasItem * item, GdkEvent * event, gpointer data)
 				sp_desktop_set_coordinate_status (desktop, p.x, p.y, SP_COORDINATES_UNDERLINE_X);
 				break;
 			}
+			sp_view_set_position (SP_VIEW (desktop), p.x, p.y);
 			ret=TRUE;
 		}
 		break;
 	case GDK_BUTTON_RELEASE:
 		if (dragging && event->button.button == 1) {
 			if (moved) {
-				GtkWidget * w;
+				GtkWidget *w;
 				double winx, winy;
+				NRPointF p;
 				w = GTK_WIDGET (item->canvas);
-				gnome_canvas_world_to_window (item->canvas, event->button.x, event->button.y, &winx, &winy);
+				sp_canvas_world_to_window (item->canvas, event->button.x, event->button.y, &winx, &winy);
 				sp_desktop_w2d_xy_point (desktop, &p, event->button.x, event->button.y);
 				if ((winx >= 0) && (winy >= 0) &&
 				    (winx < w->allocation.width) &&
@@ -217,13 +241,14 @@ sp_dt_guide_event (GnomeCanvasItem * item, GdkEvent * event, gpointer data)
 				moved = FALSE;
 				sp_document_done (SP_DT_DOCUMENT (desktop));
 				sp_desktop_set_coordinate_status (desktop, p.x, p.y, 0);
+				sp_view_set_position (SP_VIEW (desktop), p.x, p.y);
 			}
 			dragging = FALSE;
-			gnome_canvas_item_ungrab (item, event->button.time);
+			sp_canvas_item_ungrab (item, event->button.time);
 			ret=TRUE;
 		}
 	case GDK_ENTER_NOTIFY:
-		gnome_canvas_item_set (item, "color", guide->hicolor, NULL);
+		sp_canvas_item_set ((GtkObject *) item, "color", guide->hicolor, NULL);
 		msg = SP_PT_TO_METRIC_STRING (guide->position, SP_DEFAULT_METRIC);
 		switch (guide->orientation) {
 		case SP_GUIDE_HORIZONTAL:
@@ -237,7 +262,7 @@ sp_dt_guide_event (GnomeCanvasItem * item, GdkEvent * event, gpointer data)
 		g_string_free (msg, FALSE);
        		break;
 	case GDK_LEAVE_NOTIFY:
-		gnome_canvas_item_set (item, "color", guide->color, NULL);
+		sp_canvas_item_set ((GtkObject *) item, "color", guide->color, NULL);
 		sp_view_set_status (SP_VIEW (desktop), NULL, FALSE);
 		break;
 	default:
