@@ -4,13 +4,12 @@
  * Editable view and widget implementation
  *
  * Author:
- *   Lauris Kaplinski <lauris@ximian.com>
+ *   Lauris Kaplinski <lauris@kaplinski.com>
  *
- * Copyright (C) 1999-2001 Lauris Kaplinski
+ * Copyright (C) 1999-2002 Lauris Kaplinski
  * Copyright (C) 2000-2001 Ximian, Inc.
  *
- * Released under GNU GPL
- *
+ * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
 #define noDESKTOP_VERBOSE
@@ -19,6 +18,7 @@
 
 #include <math.h>
 #include <glib.h>
+#include <libart_lgpl/art_affine.h>
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gnome-preferences.h>
@@ -69,7 +69,9 @@ static void sp_desktop_document_resized (SPView *view, SPDocument *doc, gdouble
 
 /* Constructor */
 
-static SPView *sp_desktop_new (SPNamedView *nv, GnomeCanvas *canvas);
+static SPView *sp_desktop_new (SPNamedView *nv, SPCanvas *canvas);
+
+static void sp_desktop_prepare_shutdown (SPDesktop *dt);
 
 static void sp_dt_namedview_modified (SPNamedView *nv, guint flags, SPDesktop *desktop);
 static void sp_desktop_selection_modified (SPSelection *selection, guint flags, SPDesktop *desktop);
@@ -83,10 +85,10 @@ static void sp_desktop_zoom_update (SPDesktop * desktop);
 
 void sp_desktop_zoom (GtkEntry * caller, SPDesktopWidget *dtw);
 
-static void sp_desktop_menu_popup (GtkWidget * widget, GdkEventButton * event, gpointer data);
+static gint sp_desktop_menu_popup (GtkWidget * widget, GdkEventButton * event, gpointer data);
 
-/* fixme: These are widget forward decls, that are here, until things will be sorted out */
-static void sp_desktop_widget_update_rulers (GtkWidget * widget, SPDesktopWidget *dtw);
+/* fixme: This is here, but shouldn't in theory (Lauris) */
+static void sp_desktop_widget_update_rulers (SPDesktopWidget *dtw);
 
 SPViewClass * parent_class;
 static guint signals[LAST_SIGNAL] = { 0 };
@@ -171,27 +173,30 @@ sp_desktop_init (SPDesktop *desktop)
 static void
 sp_desktop_destroy (GtkObject *object)
 {
-	SPDesktop *desktop;
+	SPDesktop *dt;
 
-	desktop = SP_DESKTOP (object);
+	dt = SP_DESKTOP (object);
 
-	sodipodi_remove_desktop (desktop);
-
-	if (desktop->event_context) {
-		sp_event_context_finish (desktop->event_context);
-		gtk_object_destroy (GTK_OBJECT (desktop->event_context));
-		desktop->event_context = NULL;
+	if (dt->sodipodi) {
+		sodipodi_remove_desktop (dt);
+		dt->sodipodi = NULL;
 	}
 
-	if (desktop->selection) {
-		gtk_object_destroy (GTK_OBJECT (desktop->selection));
-		desktop->selection = NULL;
+	if (dt->event_context) {
+		sp_event_context_finish (dt->event_context);
+		gtk_object_destroy (GTK_OBJECT (dt->event_context));
+		dt->event_context = NULL;
 	}
 
-	if (desktop->drawing) {
-		sp_namedview_hide (desktop->namedview, desktop);
-		sp_item_hide (SP_ITEM (sp_document_root (SP_VIEW_DOCUMENT (desktop))), SP_CANVAS_ARENA (desktop->drawing)->arena);
-		desktop->drawing = NULL;
+	if (dt->selection) {
+		gtk_object_destroy (GTK_OBJECT (dt->selection));
+		dt->selection = NULL;
+	}
+
+	if (dt->drawing) {
+		sp_namedview_hide (dt->namedview, dt);
+		sp_item_hide (SP_ITEM (sp_document_root (SP_VIEW_DOCUMENT (dt))), SP_CANVAS_ARENA (dt->drawing)->arena);
+		dt->drawing = NULL;
 	}
 
 	if (GTK_OBJECT_CLASS (parent_class)->destroy)
@@ -206,7 +211,7 @@ sp_desktop_request_redraw (SPView *view)
 	dt = SP_DESKTOP (view);
 
 	if (dt->main) {
-		gtk_widget_queue_draw (GTK_WIDGET (GNOME_CANVAS_ITEM (dt->main)->canvas));
+		gtk_widget_queue_draw (GTK_WIDGET (SP_CANVAS_ITEM (dt->main)->canvas));
 	}
 }
 
@@ -219,7 +224,7 @@ sp_desktop_document_resized (SPView *view, SPDocument *doc, gdouble width, gdoub
 
 	desktop->doc2dt[5] = height;
 
-	gnome_canvas_item_affine_absolute (GNOME_CANVAS_ITEM (desktop->drawing), desktop->doc2dt);
+	sp_canvas_item_affine_absolute (SP_CANVAS_ITEM (desktop->drawing), desktop->doc2dt);
 
 	sp_ctrlrect_set_area (SP_CTRLRECT (desktop->page), 0.0, 0.0, width, height);
 }
@@ -256,10 +261,10 @@ arena_handler (SPCanvasArena *arena, NRArenaItem *item, GdkEvent *event, SPDeskt
 /* Constructor */
 
 static SPView *
-sp_desktop_new (SPNamedView *namedview, GnomeCanvas *canvas)
+sp_desktop_new (SPNamedView *namedview, SPCanvas *canvas)
 {
 	SPDesktop *desktop;
-	GnomeCanvasGroup *root, *page;
+	SPCanvasGroup *root, *page;
 	NRArenaItem *ai;
 	gdouble dw, dh;
 	SPDocument *document;
@@ -280,29 +285,24 @@ sp_desktop_new (SPNamedView *namedview, GnomeCanvas *canvas)
 	/* Setup Canvas */
 	gtk_object_set_data (GTK_OBJECT (canvas), "SPDesktop", desktop);
 
-	root = gnome_canvas_root (canvas);
+	root = sp_canvas_root (canvas);
 
-	desktop->acetate = gnome_canvas_item_new (root, GNOME_TYPE_CANVAS_ACETATE, NULL);
-	gtk_signal_connect (GTK_OBJECT (desktop->acetate), "event",
-		GTK_SIGNAL_FUNC (sp_desktop_root_handler), desktop);
+	desktop->acetate = sp_canvas_item_new (root, GNOME_TYPE_CANVAS_ACETATE, NULL);
+	gtk_signal_connect (GTK_OBJECT (desktop->acetate), "event", GTK_SIGNAL_FUNC (sp_desktop_root_handler), desktop);
 	/* Setup adminstrative layers */
-	desktop->main = (GnomeCanvasGroup *) gnome_canvas_item_new (root, GNOME_TYPE_CANVAS_GROUP, NULL);
+	desktop->main = (SPCanvasGroup *) sp_canvas_item_new (root, SP_TYPE_CANVAS_GROUP, NULL);
 	gtk_signal_connect (GTK_OBJECT (desktop->main), "event",
 		GTK_SIGNAL_FUNC (sp_desktop_root_handler), desktop);
 	/* fixme: */
-	page = (GnomeCanvasGroup *) gnome_canvas_item_new (desktop->main, GNOME_TYPE_CANVAS_GROUP, NULL);
+	page = (SPCanvasGroup *) sp_canvas_item_new (desktop->main, SP_TYPE_CANVAS_GROUP, NULL);
 
-	desktop->drawing = (GnomeCanvasGroup *) gnome_canvas_item_new (desktop->main, SP_TYPE_CANVAS_ARENA, NULL);
+	desktop->drawing = (SPCanvasGroup *) sp_canvas_item_new (desktop->main, SP_TYPE_CANVAS_ARENA, NULL);
 	gtk_signal_connect (GTK_OBJECT (desktop->drawing), "arena_event", GTK_SIGNAL_FUNC (arena_handler), desktop);
 
-	desktop->grid = (GnomeCanvasGroup *) gnome_canvas_item_new (desktop->main,
-		GNOME_TYPE_CANVAS_GROUP, NULL);
-	desktop->guides = (GnomeCanvasGroup *) gnome_canvas_item_new (desktop->main,
-		GNOME_TYPE_CANVAS_GROUP, NULL);
-	desktop->sketch = (GnomeCanvasGroup *) gnome_canvas_item_new (desktop->main,
-		GNOME_TYPE_CANVAS_GROUP, NULL);
-	desktop->controls = (GnomeCanvasGroup *) gnome_canvas_item_new (desktop->main,
-		GNOME_TYPE_CANVAS_GROUP, NULL);
+	desktop->grid = (SPCanvasGroup *) sp_canvas_item_new (desktop->main, SP_TYPE_CANVAS_GROUP, NULL);
+	desktop->guides = (SPCanvasGroup *) sp_canvas_item_new (desktop->main, SP_TYPE_CANVAS_GROUP, NULL);
+	desktop->sketch = (SPCanvasGroup *) sp_canvas_item_new (desktop->main, SP_TYPE_CANVAS_GROUP, NULL);
+	desktop->controls = (SPCanvasGroup *) sp_canvas_item_new (desktop->main, SP_TYPE_CANVAS_GROUP, NULL);
 
 	desktop->selection = sp_selection_new (desktop);
 
@@ -318,22 +318,22 @@ sp_desktop_new (SPNamedView *namedview, GnomeCanvas *canvas)
 	dw = sp_document_width (document);
 	dh = sp_document_height (document);
 
-	desktop->page = gnome_canvas_item_new (page, SP_TYPE_CTRLRECT, NULL);
+	desktop->page = sp_canvas_item_new (page, SP_TYPE_CTRLRECT, NULL);
 	sp_ctrlrect_set_area (SP_CTRLRECT (desktop->page), 0.0, 0.0, sp_document_width (document), sp_document_height (document));
 	sp_ctrlrect_set_shadow (SP_CTRLRECT (desktop->page), 5, 0x3f3f3fff);
 
 	/* Connect event for page resize */
 	desktop->doc2dt[5] = sp_document_height (document);
-	gnome_canvas_item_affine_absolute (GNOME_CANVAS_ITEM (desktop->drawing), desktop->doc2dt);
+	sp_canvas_item_affine_absolute (SP_CANVAS_ITEM (desktop->drawing), desktop->doc2dt);
 
 	/* Fixme: Setup initial zooming */
 
 	art_affine_scale (desktop->d2w, 1.0, -1.0);
 	desktop->d2w[5] = dw;
 	art_affine_invert (desktop->w2d, desktop->d2w);
-	gnome_canvas_item_affine_absolute ((GnomeCanvasItem *) desktop->main, desktop->d2w);
+	sp_canvas_item_affine_absolute ((SPCanvasItem *) desktop->main, desktop->d2w);
 
-	gnome_canvas_set_scroll_region (canvas, -SP_DESKTOP_SCROLL_LIMIT, -SP_DESKTOP_SCROLL_LIMIT, SP_DESKTOP_SCROLL_LIMIT, SP_DESKTOP_SCROLL_LIMIT);
+	sp_canvas_set_scroll_region (canvas, -SP_DESKTOP_SCROLL_LIMIT, -SP_DESKTOP_SCROLL_LIMIT, SP_DESKTOP_SCROLL_LIMIT, SP_DESKTOP_SCROLL_LIMIT);
 
 	gtk_signal_connect (GTK_OBJECT (canvas), "size-allocate", GTK_SIGNAL_FUNC (sp_desktop_size_allocate), desktop);
 	gtk_signal_connect (GTK_OBJECT (desktop->selection), "modified", GTK_SIGNAL_FUNC (sp_desktop_selection_modified), desktop);
@@ -351,15 +351,49 @@ sp_desktop_new (SPNamedView *namedview, GnomeCanvas *canvas)
 	// ?
 	// sp_active_desktop_set (desktop);
 	sodipodi_add_desktop (desktop);
+	desktop->sodipodi = SODIPODI;
 
 	return SP_VIEW (desktop);
 }
 
+static void
+sp_desktop_prepare_shutdown (SPDesktop *dt)
+{
+	if (dt->sodipodi) {
+		sodipodi_remove_desktop (dt);
+		dt->sodipodi = NULL;
+	}
+
+	if (dt->event_context) {
+		sp_event_context_finish (dt->event_context);
+		gtk_object_destroy (GTK_OBJECT (dt->event_context));
+		dt->event_context = NULL;
+	}
+
+	if (dt->selection) {
+		gtk_object_destroy (GTK_OBJECT (dt->selection));
+		dt->selection = NULL;
+	}
+
+	if (dt->drawing) {
+		sp_namedview_hide (dt->namedview, dt);
+		sp_item_hide (SP_ITEM (sp_document_root (SP_VIEW_DOCUMENT (dt))), SP_CANVAS_ARENA (dt->drawing)->arena);
+		dt->drawing = NULL;
+	}
+}
+
 static void
 sp_dt_namedview_modified (SPNamedView *nv, guint flags, SPDesktop *desktop)
 {
 	if (flags && SP_OBJECT_MODIFIED_FLAG) {
+		/* Recalculate snap distances */
 		sp_dt_update_snap_distances (desktop);
+		/* Show/hide page border */
+		if (nv->showborder) {
+			sp_canvas_item_show (desktop->page);
+		} else {
+			sp_canvas_item_hide (desktop->page);
+		}
 	}
 }
 
@@ -379,36 +413,6 @@ sp_dt_update_snap_distances (SPDesktop *desktop)
 	g_print ("Grid %g guides %g\n", desktop->gridsnap, desktop->guidesnap);
 }
 
-void
-sp_desktop_show_decorations (SPDesktop * desktop, gboolean show)
-{
-	g_assert (desktop != NULL);
-	g_assert (SP_IS_DESKTOP (desktop));
-
-	if (desktop->owner->decorations == show) return;
-
-	desktop->owner->decorations = show;
-
-	if (show) {
-		gtk_widget_show (GTK_WIDGET (desktop->owner->hscrollbar));
-		gtk_widget_show (GTK_WIDGET (desktop->owner->vscrollbar));
-		gtk_widget_show (GTK_WIDGET (desktop->owner->hruler));
-		gtk_widget_show (GTK_WIDGET (desktop->owner->vruler));
-		gtk_widget_show (GTK_WIDGET (desktop->owner->menubutton));
-		gtk_widget_show (GTK_WIDGET (desktop->owner->active));		
-		gtk_widget_show (GTK_WIDGET (desktop->owner->zoom)->parent);		
-	} else {
-		gtk_widget_hide (GTK_WIDGET (desktop->owner->hscrollbar));
-		gtk_widget_hide (GTK_WIDGET (desktop->owner->vscrollbar));
-		gtk_widget_hide (GTK_WIDGET (desktop->owner->hruler));
-		gtk_widget_hide (GTK_WIDGET (desktop->owner->vruler));
-		gtk_widget_hide (GTK_WIDGET (desktop->owner->menubutton));
-		gtk_widget_hide (GTK_WIDGET (desktop->owner->active));
-		gtk_widget_hide (GTK_WIDGET (desktop->owner->inactive));
-		gtk_widget_hide (GTK_WIDGET (desktop->owner->zoom)->parent);		
-	}
-}
-
 void
 sp_desktop_activate_guides (SPDesktop * desktop, gboolean activate)
 {
@@ -509,8 +513,8 @@ void sp_desktop_update_scrollbars (Sodipodi * sodipodi, SPSelection * selection)
   art_affine_point (&wc0, &dc0, desktop->d2w);
   art_affine_point (&wc1, &dc1, desktop->d2w);
   // enlarge to fit desktop / world coordinates 
-  gnome_canvas_window_to_world (desktop->owner->canvas, 0.0, 0.0, &p0.x, &p0.y);
-  gnome_canvas_window_to_world (desktop->owner->canvas, cw, ch, &p1.x, &p1.y);
+  sp_canvas_window_to_world (desktop->owner->canvas, 0.0, 0.0, &p0.x, &p0.y);
+  sp_canvas_window_to_world (desktop->owner->canvas, cw, ch, &p1.x, &p1.y);
   if (p0.x < wc0.x) wc0.x = p0.x;
   if (p0.y < wc0.y) wc0.y = p0.y;
   if (p1.x > wc1.x) wc1.x = p1.x;
@@ -537,11 +541,11 @@ void sp_desktop_update_scrollbars (Sodipodi * sodipodi, SPSelection * selection)
 
 
 // is called when the cavas is resized, connected in sp_desktop_new
-static 
-void sp_desktop_size_allocate (GtkWidget * widget, GtkRequisition *requisition, SPDesktop * desktop) 
+static void
+sp_desktop_size_allocate (GtkWidget * widget, GtkRequisition *requisition, SPDesktop * desktop) 
 {
-  sp_desktop_widget_update_rulers (widget, desktop->owner);
-  sp_desktop_update_scrollbars (SODIPODI, SP_DT_SELECTION (desktop));
+	sp_desktop_widget_update_rulers (desktop->owner);
+	sp_desktop_update_scrollbars (SODIPODI, SP_DT_SELECTION (desktop));
 }
 
 
@@ -607,8 +611,8 @@ sp_desktop_scroll_world (SPDesktop * desktop, gint dx, gint dy)
 	g_return_if_fail (desktop != NULL);
 	g_return_if_fail (SP_IS_DESKTOP (desktop));
 
-	gnome_canvas_get_scroll_offsets (desktop->owner->canvas, &x, &y);
-	gnome_canvas_scroll_to (desktop->owner->canvas, x - dx, y - dy);
+	sp_canvas_get_scroll_offsets (desktop->owner->canvas, &x, &y);
+	sp_canvas_scroll_to (desktop->owner->canvas, x - dx, y - dy);
 
 	//	sp_desktop_update_rulers (NULL, desktop);
 }
@@ -628,8 +632,8 @@ sp_desktop_get_visible_area (SPDesktop * desktop, ArtDRect * area)
 	cw = GTK_WIDGET (desktop->owner->canvas)->allocation.width+1;
 	ch = GTK_WIDGET (desktop->owner->canvas)->allocation.height+1;
 
-	gnome_canvas_window_to_world (desktop->owner->canvas, 0.0, 0.0, &p0.x, &p0.y);
-	gnome_canvas_window_to_world (desktop->owner->canvas, cw, ch, &p1.x, &p1.y);
+	sp_canvas_window_to_world (desktop->owner->canvas, 0.0, 0.0, &p0.x, &p0.y);
+	sp_canvas_window_to_world (desktop->owner->canvas, cw, ch, &p1.x, &p1.y);
 
 	art_affine_point (&p0, &p0, desktop->w2d);
 	art_affine_point (&p1, &p1, desktop->w2d);
@@ -645,9 +649,9 @@ sp_desktop_get_visible_area (SPDesktop * desktop, ArtDRect * area)
 
 gdouble sp_desktop_zoom_factor (SPDesktop * desktop)
 {
-  g_return_val_if_fail (SP_IS_DESKTOP (desktop), 0.0);
+	g_return_val_if_fail (SP_IS_DESKTOP (desktop), 0.0);
 
-  return sqrt (fabs(desktop->d2w[0] * desktop->d2w[3]));
+	return sqrt (fabs (desktop->d2w[0] * desktop->d2w[3]));
 }
 
 void
@@ -685,7 +689,7 @@ sp_desktop_show_region (SPDesktop * desktop, gdouble x0, gdouble y0, gdouble x1,
 	desktop->d2w[5] = p.y * scale + ch / 2;
 	art_affine_invert (desktop->w2d, desktop->d2w);
 
-	gnome_canvas_item_affine_absolute ((GnomeCanvasItem *) desktop->main, desktop->d2w);
+	sp_canvas_item_affine_absolute ((SPCanvasItem *) desktop->main, desktop->d2w);
      	sp_desktop_set_viewport (desktop, SP_DESKTOP_SCROLL_LIMIT, SP_DESKTOP_SCROLL_LIMIT);
        	sp_desktop_zoom_update (desktop);
 	sp_dt_update_snap_distances (desktop);
@@ -713,7 +717,7 @@ sp_desktop_zoom_relative (SPDesktop * desktop, gdouble zoom, gdouble cx, gdouble
         desktop->d2w[5] = -cy * desktop->d2w[3];
         art_affine_invert (desktop->w2d, desktop->d2w);
 
-	gnome_canvas_item_affine_absolute ((GnomeCanvasItem *) desktop->main, desktop->d2w);
+	sp_canvas_item_affine_absolute ((SPCanvasItem *) desktop->main, desktop->d2w);
 	sp_desktop_set_viewport (desktop, SP_DESKTOP_SCROLL_LIMIT, SP_DESKTOP_SCROLL_LIMIT);
 	sp_desktop_zoom_update (desktop);
 	sp_dt_update_snap_distances (desktop);
@@ -734,7 +738,7 @@ sp_desktop_zoom_absolute (SPDesktop * desktop, gdouble zoom, gdouble cx, gdouble
 	desktop->d2w[5] = -cy * desktop->d2w[3];
 	art_affine_invert (desktop->w2d, desktop->d2w);
 	
-	gnome_canvas_item_affine_absolute ((GnomeCanvasItem *) desktop->main, desktop->d2w);
+	sp_canvas_item_affine_absolute ((SPCanvasItem *) desktop->main, desktop->d2w);
       	sp_desktop_set_viewport (desktop, SP_DESKTOP_SCROLL_LIMIT, SP_DESKTOP_SCROLL_LIMIT);
 	sp_desktop_zoom_update (desktop);
 	sp_dt_update_snap_distances (desktop);
@@ -770,25 +774,13 @@ sp_desktop_set_viewport (SPDesktop * desktop, double x, double y)
     
     cw = GTK_WIDGET (desktop->owner->canvas)->allocation.width + 1;
     ch = GTK_WIDGET (desktop->owner->canvas)->allocation.height + 1;
-    gnome_canvas_scroll_to (desktop->owner->canvas, x - cw / 2, y - ch / 2);
+    sp_canvas_scroll_to (desktop->owner->canvas, x - cw / 2, y - ch / 2);
 
-    sp_desktop_widget_update_rulers (NULL, desktop->owner);
+    sp_desktop_widget_update_rulers (desktop->owner);
     sp_desktop_update_scrollbars (SODIPODI, SP_DT_SELECTION (desktop));
 }
 
-/* fixme: this are UI functions - find a better place for them */
-
-void
-sp_desktop_toggle_borders (GtkWidget * widget)
-{
-	SPDesktop * desktop;
-
-	desktop = SP_ACTIVE_DESKTOP;
-
-	if (desktop == NULL) return;
-
-	sp_desktop_show_decorations (desktop, !desktop->owner->decorations);
-}
+/* fixme: The idea to have underlines is good, but have to fit it into desktop/widget framework (Lauris) */
 
 /* set the coordinate statusbar underline single coordinates with undeline-mask 
  * x and y are document coordinates
@@ -799,11 +791,13 @@ sp_desktop_toggle_borders (GtkWidget * widget)
 void
 sp_desktop_set_coordinate_status (SPDesktop *desktop, gdouble x, gdouble y, guint underline)
 {
-	static gchar coord_str[40];
+#if 0
+	static guchar cstr[64];
 	gchar coord_pattern [20]= "                    ";
 	GString * x_str, * y_str;
 	gint i=0,j=0;
 
+	g_snprintf (cstr, 64, "%.2g%s, %.2g%s", x, y);
 	x_str = SP_PT_TO_STRING (x, SP_DEFAULT_METRIC);
 	y_str = SP_PT_TO_STRING (y, SP_DEFAULT_METRIC);
 	sprintf (coord_str, "%s, %s", x_str->str, y_str->str);
@@ -816,22 +810,19 @@ sp_desktop_set_coordinate_status (SPDesktop *desktop, gdouble x, gdouble y, guin
 
 	g_string_free (x_str, TRUE);
 	g_string_free (y_str, TRUE);
+#endif
 }
 
-static void
+static gint
 sp_desktop_menu_popup (GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
 	sp_event_root_menu_popup (SP_DESKTOP_WIDGET (data)->desktop, NULL, (GdkEvent *)event);
+
+	return FALSE;
 }
 
 /* SPDesktopWidget */
 
-#if 0
-#include <gtk/gtkscrollbar.h>
-#include <gtk/gtkruler.h>
-#include <libgnomeui/gnome-appbar.h>
-#endif
-
 static void sp_desktop_widget_class_init (SPDesktopWidgetClass *klass);
 static void sp_desktop_widget_init (SPDesktopWidget *widget);
 static void sp_desktop_widget_destroy (GtkObject *object);
@@ -845,6 +836,9 @@ static void sp_desktop_widget_view_status_set (SPView *view, const guchar *statu
 static void sp_dtw_desktop_activate (SPDesktop *desktop, SPDesktopWidget *dtw);
 static void sp_dtw_desktop_desactivate (SPDesktop *desktop, SPDesktopWidget *dtw);
 
+static void sp_desktop_widget_adjustment_value_changed (GtkAdjustment *adj, SPDesktopWidget *dtw);
+static void sp_desktop_widget_namedview_modified (SPNamedView *nv, guint flags, SPDesktopWidget *dtw);
+
 SPViewWidgetClass *dtw_parent_class;
 
 GtkType
@@ -925,12 +919,16 @@ sp_desktop_widget_init (SPDesktopWidget *desktop)
 	gtk_widget_show (GTK_WIDGET (desktop->vscrollbar));
 	gtk_table_attach (table, desktop->vscrollbar, 2, 3, 1, 2, GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
 	/* Canvas */
+#if 0
 	gtk_widget_push_visual (gdk_rgb_get_visual ());
 	gtk_widget_push_colormap (gdk_rgb_get_cmap ());
-	desktop->canvas = GNOME_CANVAS (gnome_canvas_new_aa ());
-	gnome_canvas_set_dither (GNOME_CANVAS (desktop->canvas), GDK_RGB_DITHER_MAX);
+#endif
+	desktop->canvas = SP_CANVAS (sp_canvas_new_aa ());
+#if 0
+	sp_canvas_set_dither (SP_CANVAS (desktop->canvas), GDK_RGB_DITHER_MAX);
 	gtk_widget_pop_colormap ();
 	gtk_widget_pop_visual ();
+#endif
 	style = gtk_style_copy (GTK_WIDGET (desktop->canvas)->style);
 	style->bg[GTK_STATE_NORMAL] = style->white;
 	gtk_widget_set_style (GTK_WIDGET (desktop->canvas), style);
@@ -953,7 +951,7 @@ sp_desktop_widget_init (SPDesktopWidget *desktop)
 	eventbox = gtk_event_box_new ();
 	gtk_widget_show (eventbox);
 	desktop->hruler = sp_hruler_new ();
-	sp_ruler_set_metric (GTK_RULER (desktop->hruler), SP_DEFAULT_METRIC);
+	sp_ruler_set_metric (GTK_RULER (desktop->hruler), SP_PT);
 	gtk_widget_show (desktop->hruler);
 	gtk_container_add (GTK_CONTAINER (eventbox), desktop->hruler);
 	gtk_table_attach (table,
@@ -972,7 +970,7 @@ sp_desktop_widget_init (SPDesktopWidget *desktop)
 	eventbox = gtk_event_box_new ();
 	gtk_widget_show (eventbox);
 	desktop->vruler = sp_vruler_new ();
-	sp_ruler_set_metric (GTK_RULER (desktop->vruler), SP_DEFAULT_METRIC);
+	sp_ruler_set_metric (GTK_RULER (desktop->vruler), SP_PT);
 	gtk_widget_show (GTK_WIDGET (desktop->vruler));
 	gtk_container_add (GTK_CONTAINER (eventbox), GTK_WIDGET (desktop->vruler));
 	gtk_table_attach (table,
@@ -1007,12 +1005,7 @@ sp_desktop_widget_init (SPDesktopWidget *desktop)
         GTK_WIDGET_UNSET_FLAGS (GTK_WIDGET(desktop->menubutton), GTK_CAN_FOCUS);
 	/* indicator for active desktop */
 	hbox = gtk_vbox_new (FALSE, 0);
-	gtk_table_attach (table,
-		GTK_WIDGET (hbox),
-		2,3,0,1,
-		FALSE,
-		FALSE,
-		0,0);
+	gtk_table_attach (table, GTK_WIDGET (hbox), 2,3,0,1, FALSE, FALSE, 0,0);
 	gtk_widget_show (hbox);
 	desktop->active = gnome_pixmap_new_from_file (SODIPODI_GLADEDIR "/dt_active.xpm");
 	gtk_box_pack_start ((GtkBox *) hbox, desktop->active, TRUE, TRUE, 0);
@@ -1082,8 +1075,8 @@ sp_desktop_widget_init (SPDesktopWidget *desktop)
 	vadj = gtk_range_get_adjustment (GTK_RANGE (desktop->vscrollbar));
 	gtk_layout_set_hadjustment (GTK_LAYOUT (desktop->canvas), hadj);
 	gtk_layout_set_vadjustment (GTK_LAYOUT (desktop->canvas), vadj);
-	gtk_signal_connect (GTK_OBJECT (hadj), "value-changed", GTK_SIGNAL_FUNC (sp_desktop_widget_update_rulers), desktop);
-	gtk_signal_connect (GTK_OBJECT (vadj), "value-changed", GTK_SIGNAL_FUNC (sp_desktop_widget_update_rulers), desktop);
+	gtk_signal_connect (GTK_OBJECT (hadj), "value-changed", GTK_SIGNAL_FUNC (sp_desktop_widget_adjustment_value_changed), desktop);
+	gtk_signal_connect (GTK_OBJECT (vadj), "value-changed", GTK_SIGNAL_FUNC (sp_desktop_widget_adjustment_value_changed), desktop);
 
 #if 1
         gtk_widget_grab_focus (GTK_WIDGET (desktop->canvas));
@@ -1121,7 +1114,7 @@ sp_desktop_widget_set_title (SPDesktopWidget *dtw)
 	window = GTK_WINDOW (gtk_object_get_data (GTK_OBJECT(dtw), "window"));
 	if (window) {
 		nv_name = sp_namedview_get_name (dtw->desktop->namedview);
-		uri = sp_document_uri (SP_VIEW_WIDGET_DOCUMENT (dtw));
+		uri = SP_DOCUMENT_NAME (SP_VIEW_WIDGET_DOCUMENT (dtw));
 		if (SPShowFullFielName) fname = uri;
 		else fname = g_basename (uri);
 		name = g_string_new ("");
@@ -1201,7 +1194,7 @@ sp_dtw_desktop_shutdown (SPView *view, SPDesktopWidget *dtw)
 			GtkWidget *dlg;
 			gchar *msg;
 			gint b;
-			msg = g_strdup_printf (_("Document %s has unsaved changes, save them?"), sp_document_uri (doc));
+			msg = g_strdup_printf (_("Document %s has unsaved changes, save them?"), SP_DOCUMENT_NAME (doc));
 			dlg = gnome_message_box_new (msg, GNOME_MESSAGE_BOX_WARNING, _("Save"), _("Don't save"), GNOME_STOCK_BUTTON_CANCEL, NULL);
 			g_free (msg);
 			b = gnome_dialog_run_and_close (GNOME_DIALOG (dlg));
@@ -1217,6 +1210,9 @@ sp_dtw_desktop_shutdown (SPView *view, SPDesktopWidget *dtw)
 			}
 		}
 	}
+
+	sp_desktop_prepare_shutdown (SP_DESKTOP (view));
+
 	return FALSE;
 }
 
@@ -1251,17 +1247,35 @@ sp_desktop_widget_new (SPNamedView *namedview)
 
 	gtk_signal_connect (GTK_OBJECT (dtw->desktop), "shutdown", GTK_SIGNAL_FUNC (sp_dtw_desktop_shutdown), dtw);
 
+	/* Listen on namedview modification */
+	gtk_signal_connect (GTK_OBJECT (namedview), "modified", sp_desktop_widget_namedview_modified, dtw);
+
+	if (dtw->decorations) {
+		if (SP_DESKTOP (dtw->desktop) == SP_ACTIVE_DESKTOP) {
+			gtk_widget_show (GTK_WIDGET (dtw->active));
+			gtk_widget_hide (GTK_WIDGET (dtw->inactive));
+		} else {
+			gtk_widget_hide (GTK_WIDGET (dtw->active));
+			gtk_widget_show (GTK_WIDGET (dtw->inactive));
+		}
+	}
+
 	return SP_VIEW_WIDGET (dtw);
 }
 
 static void
 sp_desktop_widget_view_position_set (SPView *view, gdouble x, gdouble y, SPDesktopWidget *dtw)
 {
+	guchar cstr[64];
+
 	/* fixme: */
-	GTK_RULER (dtw->hruler)->position = x;
+	GTK_RULER (dtw->hruler)->position = dtw->dt2r * (x - dtw->rx0);
 	gtk_ruler_draw_pos (GTK_RULER (dtw->hruler));
-	GTK_RULER (dtw->vruler)->position = y;
+	GTK_RULER (dtw->vruler)->position = dtw->dt2r * (y - dtw->ry0);
 	gtk_ruler_draw_pos (GTK_RULER (dtw->vruler));
+
+	g_snprintf (cstr, 64, "%6.1f, %6.1f", dtw->dt2r * (x - dtw->rx0), dtw->dt2r * (y - dtw->ry0));
+	gnome_appbar_set_status (GNOME_APPBAR (dtw->coord_status), cstr);
 }
 
 /*
@@ -1291,26 +1305,49 @@ sp_desktop_widget_view_status_set (SPView *view, const guchar *status, gboolean
 }
 
 static void
-sp_desktop_widget_update_rulers (GtkWidget *widget, SPDesktopWidget *dtw)
+sp_desktop_widget_namedview_modified (SPNamedView *nv, guint flags, SPDesktopWidget *dtw)
 {
-	ArtPoint p0, p1;
-
-	/* fixme: */
+	if (flags & SP_OBJECT_MODIFIED_FLAG) {
+		dtw->dt2r = 1.0 / nv->gridunit->unittobase;
+		dtw->rx0 = nv->gridoriginx;
+		dtw->ry0 = nv->gridoriginy;
+		sp_desktop_widget_update_rulers (dtw);
+	}
+}
 
+static void
+sp_desktop_widget_update_rulers (SPDesktopWidget *dtw)
+{
 	if (dtw->desktop) {
-		gnome_canvas_window_to_world (dtw->canvas, 0.0, 0.0, &p0.x, &p0.y);
-		gnome_canvas_window_to_world (dtw->canvas,
+		double x0, y0, x1, y1, s, e;
+		sp_canvas_window_to_world (dtw->canvas, 0.0, 0.0, &x0, &y0);
+		sp_canvas_window_to_world (dtw->canvas,
 					      ((GtkWidget *) dtw->canvas)->allocation.width,
 					      ((GtkWidget *) dtw->canvas)->allocation.height,
-					      &p1.x, &p1.y);
-		art_affine_point (&p0, &p0, dtw->desktop->w2d);
-		art_affine_point (&p1, &p1, dtw->desktop->w2d);
-
-		gtk_ruler_set_range (GTK_RULER (dtw->hruler), p0.x, p1.x, GTK_RULER (dtw->hruler)->position, p1.x);
-		gtk_ruler_set_range (GTK_RULER (dtw->vruler), p0.y, p1.y, GTK_RULER (dtw->vruler)->position, p1.y);
+					      &x1, &y1);
+		s = NR_MATRIX_DF_TRANSFORM_X ((NRMatrixD *) dtw->desktop->w2d, x0, y0);
+		e = NR_MATRIX_DF_TRANSFORM_X ((NRMatrixD *) dtw->desktop->w2d, x1, y1);
+		gtk_ruler_set_range (GTK_RULER (dtw->hruler),
+				     dtw->dt2r * (s - dtw->rx0),
+				     dtw->dt2r * (e - dtw->rx0),
+				     GTK_RULER (dtw->hruler)->position,
+				     dtw->dt2r * e);
+		s = NR_MATRIX_DF_TRANSFORM_Y ((NRMatrixD *) dtw->desktop->w2d, x0, y0);
+		e = NR_MATRIX_DF_TRANSFORM_Y ((NRMatrixD *) dtw->desktop->w2d, x1, y1);
+		gtk_ruler_set_range (GTK_RULER (dtw->vruler),
+				     dtw->dt2r * (s - dtw->ry0),
+				     dtw->dt2r * (e - dtw->ry0),
+				     GTK_RULER (dtw->vruler)->position,
+				     dtw->dt2r * e);
 	}
 }
 
+static void
+sp_desktop_widget_adjustment_value_changed (GtkAdjustment *adj, SPDesktopWidget *dtw)
+{
+	sp_desktop_widget_update_rulers (dtw);
+}
+
 /* we make the desktop window with focus active, signal is connected in interface.c */
 
 gint
@@ -1320,9 +1357,54 @@ sp_desktop_widget_set_focus (GtkWidget *widget, GdkEvent *event, SPDesktopWidget
 
 	/* give focus to canvas widget */
 	gtk_widget_grab_focus (GTK_WIDGET (dtw->canvas));
-	gnome_canvas_item_grab_focus ((GnomeCanvasItem *) dtw->desktop->main); 
+#if 0
+	sp_canvas_item_grab_focus ((SPCanvasItem *) dtw->desktop->main); 
+#endif
 
 	return FALSE;
 }
  
+void
+sp_desktop_widget_show_decorations (SPDesktopWidget *dtw, gboolean show)
+{
+	g_return_if_fail (dtw != NULL);
+	g_return_if_fail (SP_IS_DESKTOP_WIDGET (dtw));
+
+	if (!dtw->decorations != !show) {
+		dtw->decorations = show;
+		if (show) {
+			gtk_widget_show (GTK_WIDGET (dtw->hscrollbar));
+			gtk_widget_show (GTK_WIDGET (dtw->vscrollbar));
+			gtk_widget_show (GTK_WIDGET (dtw->hruler));
+			gtk_widget_show (GTK_WIDGET (dtw->vruler));
+			gtk_widget_show (GTK_WIDGET (dtw->menubutton));
+			gtk_widget_show (GTK_WIDGET (dtw->active));		
+			gtk_widget_show (GTK_WIDGET (dtw->zoom)->parent);		
+		} else {
+			gtk_widget_hide (GTK_WIDGET (dtw->hscrollbar));
+			gtk_widget_hide (GTK_WIDGET (dtw->vscrollbar));
+			gtk_widget_hide (GTK_WIDGET (dtw->hruler));
+			gtk_widget_hide (GTK_WIDGET (dtw->vruler));
+			gtk_widget_hide (GTK_WIDGET (dtw->menubutton));
+			gtk_widget_hide (GTK_WIDGET (dtw->active));
+			gtk_widget_hide (GTK_WIDGET (dtw->inactive));
+			gtk_widget_hide (GTK_WIDGET (dtw->zoom)->parent);
+		}
+	}
+}
+
+/* fixme: this are UI functions - find a better place for them */
+
+void
+sp_desktop_toggle_borders (GtkWidget * widget)
+{
+	SPDesktop * desktop;
+
+	desktop = SP_ACTIVE_DESKTOP;
+
+	if (desktop == NULL) return;
+
+	sp_desktop_widget_show_decorations (SP_DESKTOP_WIDGET (desktop->owner), !desktop->owner->decorations);
+}
+
 
