@@ -13,15 +13,26 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#define noSP_FS_VERBOSE
+
 #include <config.h>
 
 #include <string.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkwindow.h>
+#include <gtk/gtkhbox.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtkoptionmenu.h>
+#include <gtk/gtkmenuitem.h>
 
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
 
+#include <libnr/nr-values.h>
+
+#include <libart_lgpl/art_svp.h>
+#include <libart_lgpl/art_svp_wind.h>
+
 #include "../helper/art-utils.h"
 #include "../svg/svg.h"
 #include "../widgets/sp-widget.h"
@@ -47,6 +58,8 @@ static void sp_fill_style_widget_paint_mode_changed (SPPaintSelector *psel, SPPa
 static void sp_fill_style_widget_paint_dragged (SPPaintSelector *psel, SPWidget *spw);
 static void sp_fill_style_widget_paint_changed (SPPaintSelector *psel, SPWidget *spw);
 
+static void sp_fill_style_widget_fill_rule_activate (GtkWidget *w, SPWidget *spw);
+
 static void sp_fill_style_get_average_color_rgba (const GSList *objects, gfloat *c);
 static void sp_fill_style_get_average_color_cmyka (const GSList *objects, gfloat *c);
 static SPPaintSelectorMode sp_fill_style_determine_paint_selector_mode (SPStyle *style);
@@ -80,24 +93,59 @@ sp_fill_style_dialog (void)
 GtkWidget *
 sp_fill_style_widget_new (void)
 {
-	GtkWidget *spw, *psel;
+	GtkWidget *spw, *vb, *psel, *hb, *l, *om, *m, *mi;
 
 	spw = sp_widget_new_global (SODIPODI);
 
+	vb = gtk_vbox_new (FALSE, 0);
+	gtk_widget_show (vb);
+	gtk_container_add (GTK_CONTAINER (spw), vb);
+
 	psel = sp_paint_selector_new ();
 	gtk_widget_show (psel);
-	gtk_container_add (GTK_CONTAINER (spw), psel);
+	gtk_box_pack_start (GTK_BOX (vb), psel, TRUE, TRUE, 0);
 	gtk_object_set_data (GTK_OBJECT (spw), "paint-selector", psel);
 
+	gtk_signal_connect (GTK_OBJECT (psel), "mode_changed", GTK_SIGNAL_FUNC (sp_fill_style_widget_paint_mode_changed), spw);
+	gtk_signal_connect (GTK_OBJECT (psel), "dragged", GTK_SIGNAL_FUNC (sp_fill_style_widget_paint_dragged), spw);
+	gtk_signal_connect (GTK_OBJECT (psel), "changed", GTK_SIGNAL_FUNC (sp_fill_style_widget_paint_changed), spw);
+
+	hb = gtk_hbox_new (FALSE, 4);
+	gtk_widget_show (hb);
+	gtk_box_pack_start (GTK_BOX (vb), hb, FALSE, FALSE, 0);
+
+	l = gtk_label_new (_("Fill Rule"));
+	gtk_widget_show (l);
+	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
+	gtk_box_pack_start (GTK_BOX (hb), l, TRUE, TRUE, 0);
+
+	om = gtk_option_menu_new ();
+	gtk_widget_show (om);
+	gtk_box_pack_start (GTK_BOX (hb), om, FALSE, FALSE, 0);
+	gtk_object_set_data (GTK_OBJECT (spw), "fill-rule", om);
+
+	/* 0 - nonzero 1 - evenodd */
+	m = gtk_menu_new ();
+	gtk_widget_show (m);
+
+	mi = gtk_menu_item_new_with_label (_("nonzero"));
+	gtk_widget_show (mi);
+	gtk_menu_append (GTK_MENU (m), mi);
+	gtk_object_set_data (GTK_OBJECT (mi), "fill-rule", "nonzero");
+	gtk_signal_connect (GTK_OBJECT (mi), "activate", GTK_SIGNAL_FUNC (sp_fill_style_widget_fill_rule_activate), spw);
+	mi = gtk_menu_item_new_with_label (_("evenodd"));
+	gtk_widget_show (mi);
+	gtk_menu_append (GTK_MENU (m), mi);
+	gtk_object_set_data (GTK_OBJECT (mi), "fill-rule", "evenodd");
+	gtk_signal_connect (GTK_OBJECT (mi), "activate", GTK_SIGNAL_FUNC (sp_fill_style_widget_fill_rule_activate), spw);
+
+	gtk_option_menu_set_menu (GTK_OPTION_MENU (om), m);
+
 	gtk_signal_connect (GTK_OBJECT (spw), "construct", GTK_SIGNAL_FUNC (sp_fill_style_widget_construct), psel);
 	gtk_signal_connect (GTK_OBJECT (spw), "modify_selection", GTK_SIGNAL_FUNC (sp_fill_style_widget_modify_selection), psel);
 	gtk_signal_connect (GTK_OBJECT (spw), "change_selection", GTK_SIGNAL_FUNC (sp_fill_style_widget_change_selection), psel);
 	gtk_signal_connect (GTK_OBJECT (spw), "attr_changed", GTK_SIGNAL_FUNC (sp_fill_style_widget_attr_changed), psel);
 
-	gtk_signal_connect (GTK_OBJECT (psel), "mode_changed", GTK_SIGNAL_FUNC (sp_fill_style_widget_paint_mode_changed), spw);
-	gtk_signal_connect (GTK_OBJECT (psel), "dragged", GTK_SIGNAL_FUNC (sp_fill_style_widget_paint_dragged), spw);
-	gtk_signal_connect (GTK_OBJECT (psel), "changed", GTK_SIGNAL_FUNC (sp_fill_style_widget_paint_changed), spw);
-
 	sp_fill_style_widget_update (SP_WIDGET (spw), SP_ACTIVE_DESKTOP ? SP_DT_SELECTION (SP_ACTIVE_DESKTOP) : NULL);
 
 	return spw;
@@ -106,8 +154,9 @@ sp_fill_style_widget_new (void)
 static void
 sp_fill_style_widget_construct (SPWidget *spw, SPPaintSelector *psel)
 {
+#ifdef SP_FS_VERBOSE
 	g_print ("Fill style widget constructed: sodipodi %p repr %p\n", spw->sodipodi, spw->repr);
-
+#endif
 	if (spw->sodipodi) {
 		sp_fill_style_widget_update (spw, SP_ACTIVE_DESKTOP ? SP_DT_SELECTION (SP_ACTIVE_DESKTOP) : NULL);
 	} else if (spw->repr) {
@@ -142,6 +191,7 @@ static void
 sp_fill_style_widget_update (SPWidget *spw, SPSelection *sel)
 {
 	SPPaintSelector *psel;
+	GtkWidget *fillrule;
 	SPPaintSelectorMode pselmode;
 	const GSList *objects, *l;
 	SPObject *object;
@@ -149,8 +199,13 @@ sp_fill_style_widget_update (SPWidget *spw, SPSelection *sel)
 	gfloat c[5];
 	ArtDRect bbox;
 	SPLinearGradient *lg;
+	SPRadialGradient *rg;
 	gdouble ctm[6];
-	ArtPoint p0, p1;
+#if 0
+	NRPointF p0, p1, p2;
+#endif
+	NRMatrixF fctm, gs2d;
+	NRRectF fbb;
 
 	if (gtk_object_get_data (GTK_OBJECT (spw), "update")) return;
 
@@ -179,9 +234,9 @@ sp_fill_style_widget_update (SPWidget *spw, SPSelection *sel)
 			return;
 		}
 	}
-
+#ifdef SP_FS_VERBOSE
 	g_print ("FillStyleWidget: paint selector mode %d\n", pselmode);
-
+#endif
 	switch (pselmode) {
 	case SP_PAINT_SELECTOR_MODE_NONE:
 		/* No paint at all */
@@ -212,24 +267,74 @@ sp_fill_style_widget_update (SPWidget *spw, SPSelection *sel)
 			}
 		}
 		/* fixme: Probably we should set multiple mode here too */
-		sp_paint_selector_set_mode (psel, SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR);
 		sp_paint_selector_set_gradient_linear (psel, vector);
 		sp_selection_bbox_document (sel, &bbox);
 		sp_paint_selector_set_gradient_bbox (psel, bbox.x0, bbox.y0, bbox.x1, bbox.y1);
+
 		/* fixme: This is plain wrong */
 		lg = SP_LINEARGRADIENT (SP_OBJECT_STYLE_FILL_SERVER (object));
-		sp_item_invoke_bbox (SP_ITEM (object), &bbox, SP_MATRIX_D_IDENTITY);
+		sp_item_invoke_bbox (SP_ITEM (object), &bbox, (gdouble *) &NR_MATRIX_D_IDENTITY);
 		sp_item_i2doc_affine (SP_ITEM (object), ctm);
-		p0.x = lg->x1.computed;
-		p0.y = lg->y1.computed;
-		sp_lineargradient_from_position (lg, ctm, &bbox, &p0);
-		p1.x = lg->x2.computed;
-		p1.y = lg->y2.computed;
-		sp_lineargradient_from_position (lg, ctm, &bbox, &p1);
-		sp_paint_selector_set_gradient_position (psel, p0.x, p0.y, p1.x, p1.y);
+		fctm.c[0] = ctm[0];
+		fctm.c[1] = ctm[1];
+		fctm.c[2] = ctm[2];
+		fctm.c[3] = ctm[3];
+		fctm.c[4] = ctm[4];
+		fctm.c[5] = ctm[5];
+		fbb.x0 = bbox.x0;
+		fbb.y0 = bbox.y0;
+		fbb.x1 = bbox.x1;
+		fbb.y1 = bbox.y1;
+		sp_gradient_get_gs2d_matrix_f (SP_GRADIENT (lg), &fctm, &fbb, &gs2d);
+		sp_paint_selector_set_gradient_gs2d_matrix_f (psel, &gs2d);
+		sp_paint_selector_set_gradient_properties (psel, SP_GRADIENT_UNITS (lg), SP_GRADIENT_SPREAD (lg));
+		sp_paint_selector_set_lgradient_position (psel, lg->x1.computed, lg->y1.computed, lg->x2.computed, lg->y2.computed);
+		break;
+	case SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL:
+		object = SP_OBJECT (objects->data);
+		/* We know that all objects have radialgradient fill style */
+		vector = sp_gradient_get_vector (SP_GRADIENT (SP_OBJECT_STYLE_FILL_SERVER (object)), FALSE);
+		for (l = objects->next; l != NULL; l = l->next) {
+			SPObject *next;
+			next = SP_OBJECT (l->data);
+			if (sp_gradient_get_vector (SP_GRADIENT (SP_OBJECT_STYLE_FILL_SERVER (next)), FALSE) != vector) {
+				/* Multiple vectors */
+				sp_paint_selector_set_mode (psel, SP_PAINT_SELECTOR_MODE_MULTIPLE);
+				gtk_object_set_data (GTK_OBJECT (spw), "update", GINT_TO_POINTER (FALSE));
+				return;
+			}
+		}
+		/* fixme: Probably we should set multiple mode here too */
+		sp_paint_selector_set_gradient_radial (psel, vector);
+		sp_selection_bbox_document (sel, &bbox);
+		sp_paint_selector_set_gradient_bbox (psel, bbox.x0, bbox.y0, bbox.x1, bbox.y1);
+		/* fixme: This is plain wrong */
+		rg = SP_RADIALGRADIENT (SP_OBJECT_STYLE_FILL_SERVER (object));
+		sp_item_invoke_bbox (SP_ITEM (object), &bbox, (gdouble *) &NR_MATRIX_D_IDENTITY);
+		sp_item_i2doc_affine (SP_ITEM (object), ctm);
+		fctm.c[0] = ctm[0];
+		fctm.c[1] = ctm[1];
+		fctm.c[2] = ctm[2];
+		fctm.c[3] = ctm[3];
+		fctm.c[4] = ctm[4];
+		fctm.c[5] = ctm[5];
+		fbb.x0 = bbox.x0;
+		fbb.y0 = bbox.y0;
+		fbb.x1 = bbox.x1;
+		fbb.y1 = bbox.y1;
+		sp_gradient_get_gs2d_matrix_f (SP_GRADIENT (rg), &fctm, &fbb, &gs2d);
+		sp_paint_selector_set_gradient_gs2d_matrix_f (psel, &gs2d);
+		sp_paint_selector_set_gradient_properties (psel, SP_GRADIENT_UNITS (rg), SP_GRADIENT_SPREAD (rg));
+		sp_paint_selector_set_rgradient_position (psel, rg->cx.computed, rg->cy.computed, rg->fx.computed, rg->fy.computed, rg->r.computed);
+		break;
+	default:
+		sp_paint_selector_set_mode (psel, SP_PAINT_SELECTOR_MODE_MULTIPLE);
 		break;
 	}
 
+	fillrule = gtk_object_get_data (GTK_OBJECT (spw), "fill-rule");
+	gtk_option_menu_set_history (GTK_OPTION_MENU (fillrule), (SP_OBJECT_STYLE (object)->fill_rule.computed == ART_WIND_RULE_NONZERO) ? 0 : 1);
+
 	gtk_object_set_data (GTK_OBJECT (spw), "update", GINT_TO_POINTER (FALSE));
 }
 
@@ -237,6 +342,7 @@ static void
 sp_fill_style_widget_update_repr (SPWidget *spw, SPRepr *repr)
 {
 	SPPaintSelector *psel;
+	GtkWidget *fillrule;
 	SPPaintSelectorMode pselmode;
 	SPStyle *style;
 	gfloat c[5];
@@ -251,9 +357,9 @@ sp_fill_style_widget_update_repr (SPWidget *spw, SPRepr *repr)
 	sp_style_read_from_repr (style, repr);
 
 	pselmode = sp_fill_style_determine_paint_selector_mode (style);
-
+#ifdef SP_FS_VERBOSE
 	g_print ("FillStyleWidget: paint selector mode %d\n", pselmode);
-
+#endif
 	switch (pselmode) {
 	case SP_PAINT_SELECTOR_MODE_NONE:
 		/* No paint at all */
@@ -273,8 +379,13 @@ sp_fill_style_widget_update_repr (SPWidget *spw, SPRepr *repr)
 		break;
 	case SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR:
 		break;
+	default:
+		break;
 	}
 
+	fillrule = gtk_object_get_data (GTK_OBJECT (spw), "fill-rule");
+	gtk_option_menu_set_history (GTK_OPTION_MENU (fillrule), (style->fill_rule.computed == ART_WIND_RULE_NONZERO) ? 0 : 1);
+
 	sp_style_unref (style);
 
 	gtk_object_set_data (GTK_OBJECT (spw), "update", GINT_TO_POINTER (FALSE));
@@ -300,9 +411,9 @@ sp_fill_style_widget_paint_dragged (SPPaintSelector *psel, SPWidget *spw)
 
 	if (!spw->sodipodi) return;
 	if (gtk_object_get_data (GTK_OBJECT (spw), "update")) return;
-
+#ifdef SP_FS_VERBOSE
 	g_print ("FillStyleWidget: paint dragged\n");
-
+#endif
 	switch (psel->mode) {
 	case SP_PAINT_SELECTOR_MODE_EMPTY:
 	case SP_PAINT_SELECTOR_MODE_MULTIPLE:
@@ -328,32 +439,20 @@ sp_fill_style_widget_paint_dragged (SPPaintSelector *psel, SPWidget *spw)
 		vector = sp_gradient_ensure_vector_normalized (vector);
 		items = sp_widget_get_item_list (spw);
 		for (i = items; i != NULL; i = i->next) {
-			SPLinearGradient *lg;
-			gfloat p[4];
-			ArtDRect bbox;
-			gdouble ctm[6];
-			ArtPoint p0, p1;
-			sp_item_force_fill_lineargradient_vector (SP_ITEM (i->data), vector);
-
-			/* This gives us position in document coordinates */
-			sp_paint_selector_get_gradient_position_floatv (psel, p);
-
-			sp_item_invoke_bbox (SP_ITEM (i->data), &bbox, SP_MATRIX_D_IDENTITY);
-			sp_item_i2doc_affine (SP_ITEM (i->data), ctm);
-
-			lg = SP_LINEARGRADIENT (SP_OBJECT_STYLE_FILL_SERVER (i->data));
-			p0.x = p[0];
-			p0.y = p[1];
-			sp_lineargradient_to_position (lg, ctm, &bbox, &p0);
-			p1.x = p[2];
-			p1.y = p[3];
-			sp_lineargradient_to_position (lg, ctm, &bbox, &p1);
-
-			sp_lineargradient_set_position (lg, p0.x, p0.y, p1.x, p1.y);
+			SPGradient *lg;
+			lg = sp_item_force_fill_lineargradient_vector (SP_ITEM (i->data), vector);
+			sp_paint_selector_write_lineargradient (psel, SP_LINEARGRADIENT (lg), SP_ITEM (i->data));
 		}
 		break;
 	case SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL:
-		g_warning ("file %s: line %d: Paint %d should not emit 'dragged'", __FILE__, __LINE__, psel->mode);
+		vector = sp_paint_selector_get_gradient_vector (psel);
+		vector = sp_gradient_ensure_vector_normalized (vector);
+		items = sp_widget_get_item_list (spw);
+		for (i = items; i != NULL; i = i->next) {
+			SPGradient *rg;
+			rg = sp_item_force_fill_radialgradient_vector (SP_ITEM (i->data), vector);
+			sp_paint_selector_write_radialgradient (psel, SP_RADIALGRADIENT (rg), SP_ITEM (i->data));
+		}
 		break;
 	default:
 		g_warning ("file %s: line %d: Paint selector should not be in mode %d", __FILE__, __LINE__, psel->mode);
@@ -372,10 +471,12 @@ sp_fill_style_widget_paint_changed (SPPaintSelector *psel, SPWidget *spw)
 	guchar b[64];
 
 	if (gtk_object_get_data (GTK_OBJECT (spw), "update")) return;
-
+#ifdef SP_FS_VERBOSE
 	g_print ("FillStyleWidget: paint changed\n");
-
+#endif
 	if (spw->sodipodi) {
+		/* fixme: */
+		if (!SP_WIDGET_DOCUMENT (spw)) return;
 		reprs = NULL;
 		items = sp_widget_get_item_list (spw);
 		for (i = items; i != NULL; i = i->next) {
@@ -443,39 +544,35 @@ sp_fill_style_widget_paint_changed (SPPaintSelector *psel, SPWidget *spw)
 			} else {
 				vector = sp_gradient_ensure_vector_normalized (vector);
 				for (i = items; i != NULL; i = i->next) {
-					SPLinearGradient *lg;
-					gfloat p[4];
-					ArtDRect bbox;
-					gdouble ctm[6];
-					ArtPoint p0, p1;
-
-					sp_item_force_fill_lineargradient_vector (SP_ITEM (i->data), vector);
-
-					/* This gives us position in document coordinates */
-					sp_paint_selector_get_gradient_position_floatv (psel, p);
-
-					sp_item_invoke_bbox (SP_ITEM (i->data), &bbox, SP_MATRIX_D_IDENTITY);
-					sp_item_i2doc_affine (SP_ITEM (i->data), ctm);
-
-					lg = SP_LINEARGRADIENT (SP_OBJECT_STYLE_FILL_SERVER (i->data));
-					p0.x = p[0];
-					p0.y = p[1];
-					sp_lineargradient_to_position (lg, ctm, &bbox, &p0);
-					p1.x = p[2];
-					p1.y = p[3];
-					sp_lineargradient_to_position (lg, ctm, &bbox, &p1);
-
-					sp_repr_set_double (SP_OBJECT_REPR (lg), "x1", p0.x);
-					sp_repr_set_double (SP_OBJECT_REPR (lg), "y1", p0.y);
-					sp_repr_set_double (SP_OBJECT_REPR (lg), "x2", p1.x);
-					sp_repr_set_double (SP_OBJECT_REPR (lg), "y2", p1.y);
+					SPGradient *lg;
+					lg = sp_item_force_fill_lineargradient_vector (SP_ITEM (i->data), vector);
+					sp_paint_selector_write_lineargradient (psel, SP_LINEARGRADIENT (lg), SP_ITEM (i->data));
+					sp_object_invoke_write (SP_OBJECT (lg), SP_OBJECT_REPR (lg), SP_OBJECT_WRITE_SODIPODI);
 				}
 			}
 			sp_document_done (SP_WIDGET_DOCUMENT (spw));
 		}
 		break;
 	case SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL:
-		g_warning ("file %s: line %d: Paint %d should not emit 'changed'", __FILE__, __LINE__, psel->mode);
+		if (items) {
+			vector = sp_paint_selector_get_gradient_vector (psel);
+			if (!vector) {
+				/* No vector in paint selector should mean that we just changed mode */
+				vector = sp_document_default_gradient_vector (SP_WIDGET_DOCUMENT (spw));
+				for (i = items; i != NULL; i = i->next) {
+					sp_item_force_fill_radialgradient_vector (SP_ITEM (i->data), vector);
+				}
+			} else {
+				vector = sp_gradient_ensure_vector_normalized (vector);
+				for (i = items; i != NULL; i = i->next) {
+					SPGradient *rg;
+					rg = sp_item_force_fill_radialgradient_vector (SP_ITEM (i->data), vector);
+					sp_paint_selector_write_radialgradient (psel, SP_RADIALGRADIENT (rg), SP_ITEM (i->data));
+					sp_object_invoke_write (SP_OBJECT (rg), SP_OBJECT_REPR (rg), SP_OBJECT_WRITE_SODIPODI);
+				}
+			}
+			sp_document_done (SP_WIDGET_DOCUMENT (spw));
+		}
 		break;
 	default:
 		g_warning ("file %s: line %d: Paint selector should not be in mode %d", __FILE__, __LINE__, psel->mode);
@@ -485,6 +582,38 @@ sp_fill_style_widget_paint_changed (SPPaintSelector *psel, SPWidget *spw)
 	g_slist_free (reprs);
 }
 
+static void
+sp_fill_style_widget_fill_rule_activate (GtkWidget *w, SPWidget *spw)
+{
+	const GSList *items, *i, *r;
+	GSList *reprs;
+	SPCSSAttr *css;
+
+	if (gtk_object_get_data (GTK_OBJECT (spw), "update")) return;
+
+	if (spw->sodipodi) {
+		reprs = NULL;
+		items = sp_widget_get_item_list (spw);
+		for (i = items; i != NULL; i = i->next) {
+			reprs = g_slist_prepend (reprs, SP_OBJECT_REPR (i->data));
+		}
+	} else {
+		reprs = g_slist_prepend (NULL, spw->repr);
+		items = NULL;
+	}
+
+	css = sp_repr_css_attr_new ();
+	sp_repr_css_set_property (css, "fill-rule", gtk_object_get_data (GTK_OBJECT (w), "fill-rule"));
+	for (r = reprs; r != NULL; r = r->next) {
+		sp_repr_css_change_recursive ((SPRepr *) r->data, css, "style");
+	}
+	sp_repr_css_attr_unref (css);
+	if (spw->sodipodi) sp_document_done (SP_WIDGET_DOCUMENT (spw));
+
+	g_slist_free (reprs);
+}
+
+
 static void
 sp_fill_style_get_average_color_rgba (const GSList *objects, gfloat *c)
 {
@@ -574,6 +703,8 @@ sp_fill_style_determine_paint_selector_mode (SPStyle *style)
 	case SP_PAINT_TYPE_PAINTSERVER:
 		if (SP_IS_LINEARGRADIENT (SP_STYLE_FILL_SERVER (style))) {
 			return SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR;
+		} else if (SP_IS_RADIALGRADIENT (SP_STYLE_FILL_SERVER (style))) {
+			return SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL;
 		}
 		g_warning ("file %s: line %d: Unknown paintserver", __FILE__, __LINE__);
 		return SP_PAINT_SELECTOR_MODE_NONE;
