@@ -16,6 +16,7 @@
 #include <config.h>
 
 #include <glib.h>
+#include <libart_lgpl/art_affine.h>
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
 #include <gtk/gtksignal.h>
@@ -144,13 +145,13 @@ sp_object_properties_dialog (void)
 		/* Modify style selector */
 		hs = gtk_hseparator_new ();
 		gtk_widget_show (hs);
-		gtk_box_pack_start (GTK_BOX (vb), hs, TRUE, TRUE, 0);
+		gtk_box_pack_start (GTK_BOX (vb), hs, FALSE, FALSE, 0);
 
 		hb = gtk_hbox_new (FALSE, 4);
 		gtk_widget_show (hb);
-		gtk_box_pack_start (GTK_BOX (vb), hb, TRUE, TRUE, 0);
+		gtk_box_pack_start (GTK_BOX (vb), hb, FALSE, FALSE, 0);
 
-		l = gtk_label_new (_("Style to change:"));
+		l = gtk_label_new (_("Apply to:"));
 		gtk_widget_show (l);
 		gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
 		gtk_box_pack_start (GTK_BOX (hb), l, TRUE, TRUE, 4);
