@@ -12,6 +12,8 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#define noSP_SS_VERBOSE
+
 #include <config.h>
 
 #include <string.h>
@@ -19,9 +21,16 @@
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
 
+#include <libnr/nr-values.h>
+
 #include <gtk/gtksignal.h>
 #include <gtk/gtkadjustment.h>
 #include <gtk/gtkwindow.h>
+#include <gtk/gtkframe.h>
+#include <gtk/gtktable.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtkhbox.h>
+#include <gtk/gtkspinbutton.h>
 #include <gtk/gtkradiobutton.h>
 #include <libgnomeui/gnome-stock.h>
 
@@ -29,7 +38,8 @@
 #include "../helper/unit-menu.h"
 #include "../svg/svg.h"
 #include "../widgets/sp-widget.h"
-#include "../widgets/paint-selector.h"
+#include <widgets/paint-selector.h>
+#include <widgets/dash-selector.h>
 #include "../style.h"
 #include "../gradient-chemistry.h"
 #include "../document.h"
@@ -87,8 +97,9 @@ sp_stroke_style_paint_widget_new (void)
 static void
 sp_stroke_style_paint_construct (SPWidget *spw, SPPaintSelector *psel)
 {
+#ifdef SP_SS_VERBOSE
 	g_print ("Stroke style widget constructed: sodipodi %p repr %p\n", spw->sodipodi, spw->repr);
-
+#endif
 	if (spw->sodipodi) {
 		sp_stroke_style_paint_update (spw, SP_ACTIVE_DESKTOP ? SP_DT_SELECTION (SP_ACTIVE_DESKTOP) : NULL);
 	} else if (spw->repr) {
@@ -130,8 +141,13 @@ sp_stroke_style_paint_update (SPWidget *spw, SPSelection *sel)
 	gfloat c[5];
 	ArtDRect bbox;
 	SPLinearGradient *lg;
+	SPRadialGradient *rg;
 	gdouble ctm[6];
-	ArtPoint p0, p1;
+#if 0
+	NRPointF p0, p1;
+#endif
+	NRMatrixF fctm, gs2d;
+	NRRectF fbb;
 
 	if (gtk_object_get_data (GTK_OBJECT (spw), "update")) return;
 
@@ -160,9 +176,9 @@ sp_stroke_style_paint_update (SPWidget *spw, SPSelection *sel)
 			return;
 		}
 	}
-
+#ifdef SP_SS_VERBOSE
 	g_print ("StrokeStyleWidget: paint selector mode %d\n", pselmode);
-
+#endif
 	switch (pselmode) {
 	case SP_PAINT_SELECTOR_MODE_NONE:
 		/* No paint at all */
@@ -199,15 +215,62 @@ sp_stroke_style_paint_update (SPWidget *spw, SPSelection *sel)
 		sp_paint_selector_set_gradient_bbox (psel, bbox.x0, bbox.y0, bbox.x1, bbox.y1);
 		/* fixme: This is plain wrong */
 		lg = SP_LINEARGRADIENT (SP_OBJECT_STYLE_STROKE_SERVER (object));
-		sp_item_invoke_bbox (SP_ITEM (object), &bbox, SP_MATRIX_D_IDENTITY);
+		sp_item_invoke_bbox (SP_ITEM (object), &bbox, NR_MATRIX_D_IDENTITY.c);
 		sp_item_i2doc_affine (SP_ITEM (object), ctm);
-		p0.x = lg->x1.computed;
-		p0.y = lg->y1.computed;
-		sp_lineargradient_from_position (lg, ctm, &bbox, &p0);
-		p1.x = lg->x2.computed;
-		p1.y = lg->y2.computed;
-		sp_lineargradient_from_position (lg, ctm, &bbox, &p1);
-		sp_paint_selector_set_gradient_position (psel, p0.x, p0.y, p1.x, p1.y);
+		fctm.c[0] = ctm[0];
+		fctm.c[1] = ctm[1];
+		fctm.c[2] = ctm[2];
+		fctm.c[3] = ctm[3];
+		fctm.c[4] = ctm[4];
+		fctm.c[5] = ctm[5];
+		fbb.x0 = bbox.x0;
+		fbb.y0 = bbox.y0;
+		fbb.x1 = bbox.x1;
+		fbb.y1 = bbox.y1;
+		sp_gradient_get_gs2d_matrix_f (SP_GRADIENT (lg), &fctm, &fbb, &gs2d);
+		sp_paint_selector_set_gradient_gs2d_matrix_f (psel, &gs2d);
+		sp_paint_selector_set_gradient_properties (psel, SP_GRADIENT_UNITS (lg), SP_GRADIENT_SPREAD (lg));
+		sp_paint_selector_set_lgradient_position (psel, lg->x1.computed, lg->y1.computed, lg->x2.computed, lg->y2.computed);
+		break;
+	case SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL:
+		object = SP_OBJECT (objects->data);
+		/* We know that all objects have radialgradient stroke style */
+		vector = sp_gradient_get_vector (SP_GRADIENT (SP_OBJECT_STYLE_STROKE_SERVER (object)), FALSE);
+		for (l = objects->next; l != NULL; l = l->next) {
+			SPObject *next;
+			next = SP_OBJECT (l->data);
+			if (sp_gradient_get_vector (SP_GRADIENT (SP_OBJECT_STYLE_STROKE_SERVER (next)), FALSE) != vector) {
+				/* Multiple vectors */
+				sp_paint_selector_set_mode (psel, SP_PAINT_SELECTOR_MODE_MULTIPLE);
+				gtk_object_set_data (GTK_OBJECT (spw), "update", GINT_TO_POINTER (FALSE));
+				return;
+			}
+		}
+		/* fixme: Probably we should set multiple mode here too */
+		sp_paint_selector_set_gradient_radial (psel, vector);
+		sp_selection_bbox_document (sel, &bbox);
+		sp_paint_selector_set_gradient_bbox (psel, bbox.x0, bbox.y0, bbox.x1, bbox.y1);
+		/* fixme: This is plain wrong */
+		rg = SP_RADIALGRADIENT (SP_OBJECT_STYLE_STROKE_SERVER (object));
+		sp_item_invoke_bbox (SP_ITEM (object), &bbox, (gdouble *) &NR_MATRIX_D_IDENTITY);
+		sp_item_i2doc_affine (SP_ITEM (object), ctm);
+		fctm.c[0] = ctm[0];
+		fctm.c[1] = ctm[1];
+		fctm.c[2] = ctm[2];
+		fctm.c[3] = ctm[3];
+		fctm.c[4] = ctm[4];
+		fctm.c[5] = ctm[5];
+		fbb.x0 = bbox.x0;
+		fbb.y0 = bbox.y0;
+		fbb.x1 = bbox.x1;
+		fbb.y1 = bbox.y1;
+		sp_gradient_get_gs2d_matrix_f (SP_GRADIENT (rg), &fctm, &fbb, &gs2d);
+		sp_paint_selector_set_gradient_gs2d_matrix_f (psel, &gs2d);
+		sp_paint_selector_set_gradient_properties (psel, SP_GRADIENT_UNITS (rg), SP_GRADIENT_SPREAD (rg));
+		sp_paint_selector_set_rgradient_position (psel, rg->cx.computed, rg->cy.computed, rg->fx.computed, rg->fy.computed, rg->r.computed);
+		break;
+	default:
+		sp_paint_selector_set_mode (psel, SP_PAINT_SELECTOR_MODE_MULTIPLE);
 		break;
 	}
 
@@ -232,9 +295,9 @@ sp_stroke_style_paint_update_repr (SPWidget *spw, SPRepr *repr)
 	sp_style_read_from_repr (style, repr);
 
 	pselmode = sp_stroke_style_determine_paint_selector_mode (style);
-
+#ifdef SP_SS_VERBOSE
 	g_print ("StrokeStyleWidget: paint selector mode %d\n", pselmode);
-
+#endif
 	switch (pselmode) {
 	case SP_PAINT_SELECTOR_MODE_NONE:
 		/* No paint at all */
@@ -255,6 +318,8 @@ sp_stroke_style_paint_update_repr (SPWidget *spw, SPRepr *repr)
 	case SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR:
 		/* fixme: Think about it (Lauris) */
 		break;
+	default:
+		break;
 	}
 
 	sp_style_unref (style);
@@ -281,9 +346,9 @@ sp_stroke_style_paint_dragged (SPPaintSelector *psel, SPWidget *spw)
 	gfloat c[5];
 
 	if (gtk_object_get_data (GTK_OBJECT (spw), "update")) return;
-
+#ifdef SP_SS_VERBOSE
 	g_print ("StrokeStyleWidget: paint dragged\n");
-
+#endif
 	switch (psel->mode) {
 	case SP_PAINT_SELECTOR_MODE_EMPTY:
 	case SP_PAINT_SELECTOR_MODE_MULTIPLE:
@@ -309,31 +374,20 @@ sp_stroke_style_paint_dragged (SPPaintSelector *psel, SPWidget *spw)
 		vector = sp_gradient_ensure_vector_normalized (vector);
 		items = sp_widget_get_item_list (spw);
 		for (i = items; i != NULL; i = i->next) {
-			SPLinearGradient *lg;
-			gfloat p[4];
-			ArtDRect bbox;
-			gdouble ctm[6];
-			ArtPoint p0, p1;
-			sp_item_force_stroke_lineargradient_vector (SP_ITEM (i->data), vector);
-			/* This gives us position in document coordinates */
-			sp_paint_selector_get_gradient_position_floatv (psel, p);
-
-			sp_item_invoke_bbox (SP_ITEM (i->data), &bbox, SP_MATRIX_D_IDENTITY);
-			sp_item_i2doc_affine (SP_ITEM (i->data), ctm);
-
-			lg = SP_LINEARGRADIENT (SP_OBJECT_STYLE_STROKE_SERVER (i->data));
-			p0.x = p[0];
-			p0.y = p[1];
-			sp_lineargradient_to_position (lg, ctm, &bbox, &p0);
-			p1.x = p[2];
-			p1.y = p[3];
-			sp_lineargradient_to_position (lg, ctm, &bbox, &p1);
-
-			sp_lineargradient_set_position (lg, p0.x, p0.y, p1.x, p1.y);
+			SPGradient *lg;
+			lg = sp_item_force_stroke_lineargradient_vector (SP_ITEM (i->data), vector);
+			sp_paint_selector_write_lineargradient (psel, SP_LINEARGRADIENT (lg), SP_ITEM (i->data));
 		}
 		break;
 	case SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL:
-		g_warning ("file %s: line %d: Paint %d should not emit 'dragged'", __FILE__, __LINE__, psel->mode);
+		vector = sp_paint_selector_get_gradient_vector (psel);
+		vector = sp_gradient_ensure_vector_normalized (vector);
+		items = sp_widget_get_item_list (spw);
+		for (i = items; i != NULL; i = i->next) {
+			SPGradient *rg;
+			rg = sp_item_force_stroke_radialgradient_vector (SP_ITEM (i->data), vector);
+			sp_paint_selector_write_radialgradient (psel, SP_RADIALGRADIENT (rg), SP_ITEM (i->data));
+		}
 		break;
 	default:
 		g_warning ("file %s: line %d: Paint selector should not be in mode %d", __FILE__, __LINE__, psel->mode);
@@ -352,10 +406,12 @@ sp_stroke_style_paint_changed (SPPaintSelector *psel, SPWidget *spw)
 	guchar b[64];
 
 	if (gtk_object_get_data (GTK_OBJECT (spw), "update")) return;
-
+#ifdef SP_SS_VERBOSE
 	g_print ("StrokeStyleWidget: paint changed\n");
-
+#endif
 	if (spw->sodipodi) {
+		/* fixme: */
+		if (!SP_WIDGET_DOCUMENT (spw)) return;
 		reprs = NULL;
 		items = sp_widget_get_item_list (spw);
 		for (i = items; i != NULL; i = i->next) {
@@ -423,39 +479,35 @@ sp_stroke_style_paint_changed (SPPaintSelector *psel, SPWidget *spw)
 			} else {
 				vector = sp_gradient_ensure_vector_normalized (vector);
 				for (i = items; i != NULL; i = i->next) {
-					SPLinearGradient *lg;
-					gfloat p[4];
-					ArtDRect bbox;
-					gdouble ctm[6];
-					ArtPoint p0, p1;
-
-					sp_item_force_stroke_lineargradient_vector (SP_ITEM (i->data), vector);
-
-					/* This gives us position in document coordinates */
-					sp_paint_selector_get_gradient_position_floatv (psel, p);
-
-					sp_item_invoke_bbox (SP_ITEM (i->data), &bbox, SP_MATRIX_D_IDENTITY);
-					sp_item_i2doc_affine (SP_ITEM (i->data), ctm);
-
-					lg = SP_LINEARGRADIENT (SP_OBJECT_STYLE_STROKE_SERVER (i->data));
-					p0.x = p[0];
-					p0.y = p[1];
-					sp_lineargradient_to_position (lg, ctm, &bbox, &p0);
-					p1.x = p[2];
-					p1.y = p[3];
-					sp_lineargradient_to_position (lg, ctm, &bbox, &p1);
-
-					sp_repr_set_double (SP_OBJECT_REPR (lg), "x1", p0.x);
-					sp_repr_set_double (SP_OBJECT_REPR (lg), "y1", p0.y);
-					sp_repr_set_double (SP_OBJECT_REPR (lg), "x2", p1.x);
-					sp_repr_set_double (SP_OBJECT_REPR (lg), "y2", p1.y);
+					SPGradient *lg;
+					lg = sp_item_force_stroke_lineargradient_vector (SP_ITEM (i->data), vector);
+					sp_paint_selector_write_lineargradient (psel, SP_LINEARGRADIENT (lg), SP_ITEM (i->data));
+					sp_object_invoke_write (SP_OBJECT (lg), SP_OBJECT_REPR (lg), SP_OBJECT_WRITE_SODIPODI);
 				}
 			}
 			sp_document_done (SP_WIDGET_DOCUMENT (spw));
 		}
 		break;
 	case SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL:
-		g_warning ("file %s: line %d: Paint %d should not emit 'changed'", __FILE__, __LINE__, psel->mode);
+		if (items) {
+			vector = sp_paint_selector_get_gradient_vector (psel);
+			if (!vector) {
+				/* No vector in paint selector should mean that we just changed mode */
+				vector = sp_document_default_gradient_vector (SP_WIDGET_DOCUMENT (spw));
+				for (i = items; i != NULL; i = i->next) {
+					sp_item_force_stroke_radialgradient_vector (SP_ITEM (i->data), vector);
+				}
+			} else {
+				vector = sp_gradient_ensure_vector_normalized (vector);
+				for (i = items; i != NULL; i = i->next) {
+					SPGradient *lg;
+					lg = sp_item_force_stroke_radialgradient_vector (SP_ITEM (i->data), vector);
+					sp_paint_selector_write_radialgradient (psel, SP_RADIALGRADIENT (lg), SP_ITEM (i->data));
+					sp_object_invoke_write (SP_OBJECT (lg), SP_OBJECT_REPR (lg), SP_OBJECT_WRITE_SODIPODI);
+				}
+			}
+			sp_document_done (SP_WIDGET_DOCUMENT (spw));
+		}
 		break;
 	default:
 		g_warning ("file %s: line %d: Paint selector should not be in mode %d", __FILE__, __LINE__, psel->mode);
@@ -479,11 +531,12 @@ static void sp_stroke_style_set_join_buttons (SPWidget *spw, GtkWidget *active);
 static void sp_stroke_style_set_cap_buttons (SPWidget *spw, GtkWidget *active);
 static void sp_stroke_style_width_changed (GtkAdjustment *adj, SPWidget *spw);
 static void sp_stroke_style_any_toggled (GtkToggleButton *tb, SPWidget *spw);
+static void sp_stroke_style_line_dash_changed (SPDashSelector *dsel, SPWidget *spw);
 
 GtkWidget *
 sp_stroke_style_line_widget_new (void)
 {
-	GtkWidget *spw, *f, *t, *l, *hb, *sb, *us, *tb, *px;
+	GtkWidget *spw, *f, *t, *l, *hb, *sb, *us, *tb, *px, *ds;
 	GtkObject *a;
 
 	spw = sp_widget_new_global (SODIPODI);
@@ -609,6 +662,18 @@ sp_stroke_style_line_widget_new (void)
 	gtk_widget_show (px);
 	gtk_container_add (GTK_CONTAINER (tb), px);
 
+	/* Dash */
+	l = gtk_label_new (_("Pattern:"));
+	gtk_widget_show (l);
+	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
+	gtk_table_attach (GTK_TABLE (t), l, 0, 1, 3, 4, GTK_FILL, 0, 4, 0);
+
+	ds = sp_dash_selector_new (sodipodi_get_repr (SODIPODI, "palette.dashes"));
+	gtk_widget_show (ds);
+	gtk_table_attach (GTK_TABLE (t), ds, 1, 4, 3, 4, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+	gtk_object_set_data (GTK_OBJECT (spw), "dash", ds);
+	gtk_signal_connect (GTK_OBJECT (ds), "changed", GTK_SIGNAL_FUNC (sp_stroke_style_line_dash_changed), spw);
+
 	gtk_signal_connect (GTK_OBJECT (spw), "construct", GTK_SIGNAL_FUNC (sp_stroke_style_line_construct), NULL);
 	gtk_signal_connect (GTK_OBJECT (spw), "modify_selection", GTK_SIGNAL_FUNC (sp_stroke_style_line_modify_selection), NULL);
 	gtk_signal_connect (GTK_OBJECT (spw), "change_selection", GTK_SIGNAL_FUNC (sp_stroke_style_line_change_selection), NULL);
@@ -622,8 +687,9 @@ sp_stroke_style_line_widget_new (void)
 static void
 sp_stroke_style_line_construct (SPWidget *spw, gpointer data)
 {
+#ifdef SP_SS_VERBOSE
 	g_print ("Stroke style widget constructed: sodipodi %p repr %p\n", spw->sodipodi, spw->repr);
-
+#endif
 	if (spw->sodipodi) {
 		sp_stroke_style_line_update (spw, SP_ACTIVE_DESKTOP ? SP_DT_SELECTION (SP_ACTIVE_DESKTOP) : NULL);
 	} else if (spw->repr) {
@@ -657,11 +723,12 @@ sp_stroke_style_line_attr_changed (SPWidget *spw, const guchar *key, const gucha
 static void
 sp_stroke_style_line_update (SPWidget *spw, SPSelection *sel)
 {
-	GtkWidget *sset, *units;
+	GtkWidget *sset, *units, *dsel;
 	GtkObject *width;
 	const SPUnit *unit;
 	const GSList *objects, *l;
 	SPObject *object;
+	SPStyle *style;
 	gdouble avgwidth;
 	gboolean stroked;
 	gint jointype, captype;
@@ -674,6 +741,7 @@ sp_stroke_style_line_update (SPWidget *spw, SPSelection *sel)
 	sset = gtk_object_get_data (GTK_OBJECT (spw), "stroke");
 	width = gtk_object_get_data (GTK_OBJECT (spw), "width");
 	units = gtk_object_get_data (GTK_OBJECT (spw), "units");
+	dsel = gtk_object_get_data (GTK_OBJECT (spw), "dash");
 
 	if (!sel || sp_selection_is_empty (sel)) {
 		/* No objects, set empty */
@@ -693,7 +761,9 @@ sp_stroke_style_line_update (SPWidget *spw, SPSelection *sel)
 		sp_item_i2d_affine (SP_ITEM (l->data), i2d);
 		object = SP_OBJECT (l->data);
 		dist = sp_distance_d_matrix_d_transform (object->style->stroke_width.computed, i2d);
+#ifdef SP_SS_VERBOSE
 		g_print ("%g in user is %g on desktop\n", object->style->stroke_width.computed, dist);
+#endif
 		avgwidth += dist;
 		if (object->style->stroke.type == SP_PAINT_TYPE_NONE) stroked = FALSE;
 	}
@@ -712,6 +782,7 @@ sp_stroke_style_line_update (SPWidget *spw, SPSelection *sel)
 
 	/* Join & Cap */
 	object = SP_OBJECT (objects->data);
+	style = SP_OBJECT_STYLE (object);
 	jointype = object->style->stroke_linejoin.value;
 	captype = object->style->stroke_linecap.value;
 
@@ -736,7 +807,6 @@ sp_stroke_style_line_update (SPWidget *spw, SPSelection *sel)
 		tb = NULL;
 		break;
 	}
-
 	sp_stroke_style_set_join_buttons (spw, tb);
 
 	switch (captype) {
@@ -753,9 +823,21 @@ sp_stroke_style_line_update (SPWidget *spw, SPSelection *sel)
 		tb = NULL;
 		break;
 	}
-
 	sp_stroke_style_set_cap_buttons (spw, tb);
 
+	/* Dash */
+	if (style->stroke_dash.n_dash > 0) {
+		double d[64];
+		int len, i;
+		len = MIN (style->stroke_dash.n_dash, 64);
+		for (i = 0; i < len; i++) {
+			d[i] = style->stroke_dash.dash[i] / style->stroke_width.computed;
+		}
+		sp_dash_selector_set_dash (SP_DASH_SELECTOR (dsel), len, d, style->stroke_dash.offset / style->stroke_width.computed);
+	} else {
+		sp_dash_selector_set_dash (SP_DASH_SELECTOR (dsel), 0, NULL, 0.0);
+	}
+
 	gtk_widget_set_sensitive (sset, TRUE);
 
 	gtk_object_set_data (GTK_OBJECT (spw), "update", GINT_TO_POINTER (FALSE));
@@ -764,7 +846,7 @@ sp_stroke_style_line_update (SPWidget *spw, SPSelection *sel)
 static void
 sp_stroke_style_line_update_repr (SPWidget *spw, SPRepr *repr)
 {
-	GtkWidget *sset, *units;
+	GtkWidget *sset, *units, *dsel;
 	GtkObject *width;
 	SPStyle *style;
 	const SPUnit *unit;
@@ -778,6 +860,7 @@ sp_stroke_style_line_update_repr (SPWidget *spw, SPRepr *repr)
 	sset = gtk_object_get_data (GTK_OBJECT (spw), "stroke");
 	width = gtk_object_get_data (GTK_OBJECT (spw), "width");
 	units = gtk_object_get_data (GTK_OBJECT (spw), "units");
+	dsel = gtk_object_get_data (GTK_OBJECT (spw), "dash");
 
 	style = sp_style_new ();
 	sp_style_read_from_repr (style, repr);
@@ -827,6 +910,19 @@ sp_stroke_style_line_update_repr (SPWidget *spw, SPRepr *repr)
 	}
 	sp_stroke_style_set_cap_buttons (spw, tb);
 
+	/* Dash */
+	if (style->stroke_dash.n_dash > 0) {
+		double d[64];
+		int len, i;
+		len = MIN (style->stroke_dash.n_dash, 64);
+		for (i = 0; i < len; i++) {
+			d[i] = style->stroke_dash.dash[i] / style->stroke_width.computed;
+		}
+		sp_dash_selector_set_dash (SP_DASH_SELECTOR (dsel), len, d, style->stroke_dash.offset / style->stroke_width.computed);
+	} else {
+		sp_dash_selector_set_dash (SP_DASH_SELECTOR (dsel), 0, NULL, 0.0);
+	}
+
 	gtk_widget_set_sensitive (sset, TRUE);
 
 	sp_style_unref (style);
@@ -835,19 +931,47 @@ sp_stroke_style_line_update_repr (SPWidget *spw, SPRepr *repr)
 }
 
 static void
-sp_stroke_style_width_changed (GtkAdjustment *adj, SPWidget *spw)
+sp_stroke_style_set_scaled_dash (SPCSSAttr *css, int ndash, double *dash, double offset, double scale)
 {
+	if (ndash > 0) {
+		unsigned char c[1024];
+		int i, pos;
+		pos = 0;
+		for (i = 0; i < ndash; i++) {
+			pos += g_snprintf (c + pos, 1022 - pos, "%g", dash[i] * scale);
+			if ((i < (ndash - 1)) && (pos < 1020)) {
+				c[pos] = ',';
+				pos += 1;
+			}
+		}
+		c[pos] = 0;
+		sp_repr_css_set_property (css, "stroke-dasharray", c);
+		g_snprintf (c, 1024, "%g", offset * scale);
+		sp_repr_css_set_property (css, "stroke-dashoffset", c);
+	} else {
+		sp_repr_css_set_property (css, "stroke-dasharray", "none");
+		sp_repr_css_set_property (css, "stroke-dashoffset", NULL);
+	}
+}
+
+static void
+sp_stroke_style_scale_line (SPWidget *spw)
+{
+	GtkAdjustment *wadj;
 	SPUnitSelector *us;
+	SPDashSelector *dsel;
 	const GSList *items, *i, *r;
 	GSList *reprs;
 	SPCSSAttr *css;
 	guchar c[32];
 
-	if (gtk_object_get_data (GTK_OBJECT (spw), "update")) return;
-
+	wadj = gtk_object_get_data (GTK_OBJECT (spw), "width");
 	us = gtk_object_get_data (GTK_OBJECT (spw), "units");
+	dsel = gtk_object_get_data (GTK_OBJECT (spw), "dash");
 
 	if (spw->sodipodi) {
+		/* fixme: */
+		if (!SP_WIDGET_DOCUMENT (spw)) return;
 		reprs = NULL;
 		items = sp_widget_get_item_list (spw);
 		for (i = items; i != NULL; i = i->next) {
@@ -863,25 +987,37 @@ sp_stroke_style_width_changed (GtkAdjustment *adj, SPWidget *spw)
 
 	if (items) {
 		for (i = items; i != NULL; i = i->next) {
-			gdouble i2d[6], d2i[6];
-			gdouble length, dist;
-			length = adj->value;
+			double i2d[6], d2i[6];
+			double length, dist;
+			double *dash, offset;
+			int ndash;
+			length = wadj->value;
+			sp_dash_selector_get_dash (dsel, &ndash, &dash, &offset);
+			/* Set stroke width */
 			sp_convert_distance (&length, sp_unit_selector_get_unit (us), SP_PS_UNIT);
 			sp_item_i2d_affine (SP_ITEM (i->data), i2d);
 			art_affine_invert (d2i, i2d);
 			dist = sp_distance_d_matrix_d_transform (length, d2i);
 			g_snprintf (c, 32, "%g", sp_distance_d_matrix_d_transform (length, d2i));
 			sp_repr_css_set_property (css, "stroke-width", c);
+			/* Set dash */
+			sp_stroke_style_set_scaled_dash (css, ndash, dash, offset, dist);
 			sp_repr_css_change_recursive (SP_OBJECT_REPR (i->data), css, "style");
+			if (dash) g_free (dash);
 		}
 	} else {
 		for (r = reprs; r != NULL; r = r->next) {
-			gdouble length;
-			length = adj->value;
+			double length;
+			double *dash, offset;
+			int ndash;
+			length = wadj->value;
+			sp_dash_selector_get_dash (dsel, &ndash, &dash, &offset);
 			sp_convert_distance (&length, sp_unit_selector_get_unit (us), SP_PS_UNIT);
 			g_snprintf (c, 32, "%g", length * 1.25);
 			sp_repr_css_set_property (css, "stroke-width", c);
+			sp_stroke_style_set_scaled_dash (css, ndash, dash, offset, length);
 			sp_repr_css_change_recursive ((SPRepr *) r->data, css, "style");
+			if (dash) g_free (dash);
 		}
 	}
 
@@ -891,6 +1027,22 @@ sp_stroke_style_width_changed (GtkAdjustment *adj, SPWidget *spw)
 	g_slist_free (reprs);
 }
 
+static void
+sp_stroke_style_width_changed (GtkAdjustment *adj, SPWidget *spw)
+{
+	if (gtk_object_get_data (GTK_OBJECT (spw), "update")) return;
+
+	sp_stroke_style_scale_line (spw);
+}
+
+static void
+sp_stroke_style_line_dash_changed (SPDashSelector *dsel, SPWidget *spw)
+{
+	if (gtk_object_get_data (GTK_OBJECT (spw), "update")) return;
+
+	sp_stroke_style_scale_line (spw);
+}
+
 static void
 sp_stroke_style_any_toggled (GtkToggleButton *tb, SPWidget *spw)
 {
@@ -1032,8 +1184,9 @@ sp_stroke_style_determine_paint_selector_mode (SPStyle *style)
 	case SP_PAINT_TYPE_PAINTSERVER:
 		if (SP_IS_LINEARGRADIENT (SP_STYLE_STROKE_SERVER (style))) {
 			return SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR;
+		} else if (SP_IS_RADIALGRADIENT (SP_STYLE_STROKE_SERVER (style))) {
+			return SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL;
 		}
-		g_warning ("file %s: line %d: Unknown paintserver", __FILE__, __LINE__);
 		return SP_PAINT_SELECTOR_MODE_NONE;
 	default:
 		g_warning ("file %s: line %d: Unknown paint type %d", __FILE__, __LINE__, style->stroke.type);
