@@ -14,17 +14,26 @@
 
 #include <config.h>
 
+#include <string.h>
+#include <libnrtype/nr-type-directory.h>
+
 #include <glib.h>
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkwindow.h>
+#include <gtk/gtknotebook.h>
 #include <gtk/gtkvbox.h>
+#include <gtk/gtkhbox.h>
+#include <gtk/gtkframe.h>
+#include <gtk/gtktable.h>
 #include <gtk/gtktext.h>
+#include <gtk/gtkradiobutton.h>
+#include <gtk/gtkhseparator.h>
 #include <libgnomeui/gnome-stock.h>
-#include <libgnomeprint/gnome-font-dialog.h>
 #include <gal/widgets/e-unicode.h>
 
+#include "../widgets/font-selector.h"
 #include "../forward.h"
 #include "../sodipodi.h"
 #include "../document.h"
@@ -45,7 +54,7 @@ static void sp_text_edit_dialog_close (GtkButton *button, GtkWidget *dlg);
 static void sp_text_edit_dialog_read_selection (GtkWidget *dlg, gboolean style, gboolean content);
 
 static void sp_text_edit_dialog_text_changed (GtkText *txt, GtkWidget *dlg);
-static void sp_text_edit_dialog_font_changed (GnomeFontSelection *fontsel, GnomeFont *font, GtkWidget *dlg);
+static void sp_text_edit_dialog_font_changed (SPFontSelector *fontsel, NRFont *font, GtkWidget *dlg);
 static void sp_text_edit_dialog_any_toggled (GtkToggleButton *tb, GtkWidget *dlg);
 
 static SPText *sp_ted_get_selected_text_item (void);
@@ -75,6 +84,7 @@ sp_text_edit_dialog (void)
 
 		dlg = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 		gtk_window_set_title (GTK_WINDOW (dlg), _("Text properties"));
+		gtk_window_set_policy (GTK_WINDOW (dlg), TRUE, TRUE, FALSE);
 		gtk_signal_connect (GTK_OBJECT (dlg), "destroy", GTK_SIGNAL_FUNC (sp_text_edit_dialog_destroy), dlg);
 		gtk_signal_connect (GTK_OBJECT (dlg), "delete_event", GTK_SIGNAL_FUNC (sp_text_edit_dialog_delete), dlg);
 
@@ -104,7 +114,7 @@ sp_text_edit_dialog (void)
 		gtk_widget_show (hb);
 		gtk_box_pack_start (GTK_BOX (vb), hb, TRUE, TRUE, 0);
 
-		fontsel = gnome_font_selection_new ();
+		fontsel = sp_font_selector_new ();
 		gtk_widget_show (fontsel);
 		gtk_signal_connect (GTK_OBJECT (fontsel), "font_set", GTK_SIGNAL_FUNC (sp_text_edit_dialog_font_changed), dlg);
 		gtk_box_pack_start (GTK_BOX (hb), fontsel, TRUE, TRUE, 0);
@@ -176,7 +186,7 @@ sp_text_edit_dialog (void)
 		gtk_table_attach (GTK_TABLE (tbl), b, 2, 3, 1, 2, 0, 0, 0, 0);
 		gtk_object_set_data (GTK_OBJECT (dlg), "writing_mode_tb", b);
 
-		preview = gnome_font_preview_new ();
+		preview = sp_font_preview_new ();
 		gtk_widget_show (preview);
 		gtk_box_pack_start (GTK_BOX (vb), preview, TRUE, TRUE, 4);
 		gtk_object_set_data (GTK_OBJECT (dlg), "preview", preview);
@@ -258,10 +268,8 @@ sp_text_edit_dialog_update_object (SPText *text, SPRepr *repr)
 	if (repr) {
 		GtkWidget *fontsel, *preview, *b;
 		SPCSSAttr *css;
-		GnomeFont *font;
-		gint weight;
-		const guchar *val;
-		guchar c[64];
+		NRFont *font;
+		guchar c[256];
 
 		fontsel = gtk_object_get_data (GTK_OBJECT (dlg), "fontsel");
 		preview = gtk_object_get_data (GTK_OBJECT (dlg), "preview");
@@ -269,25 +277,18 @@ sp_text_edit_dialog_update_object (SPText *text, SPRepr *repr)
 		css = sp_repr_css_attr_new ();
 
 		/* font */
-		font = gnome_font_selection_get_font (GNOME_FONT_SELECTION (fontsel));
-		val = gnome_font_get_family_name (font);
-		sp_repr_css_set_property (css, "font-family", val);
-		weight = gnome_font_get_weight_code (font);
-		if (weight < GNOME_FONT_SEMI) {
-			val = "normal";
-		} else {
-			val = "bold";
-		}
-		sp_repr_css_set_property (css, "font-weight", val);
-		if (gnome_font_is_italic (font)) {
-			val = "italic";
-		} else {
-			val = "normal";
-		}
-		sp_repr_css_set_property (css, "font-style", val);
-		snprintf (c, 64, "%g", gnome_font_get_size (font));
+		font = sp_font_selector_get_font (SP_FONT_SELECTOR (fontsel));
+		nr_typeface_family_name_get (NR_FONT_TYPEFACE (font), c, 256);
+		sp_repr_css_set_property (css, "font-family", c);
+		nr_typeface_attribute_get (NR_FONT_TYPEFACE (font), "weight", c, 256);
+		g_strdown (c);
+		sp_repr_css_set_property (css, "font-weight", c);
+		nr_typeface_attribute_get (NR_FONT_TYPEFACE (font), "style", c, 256);
+		g_strdown (c);
+		sp_repr_css_set_property (css, "font-style", c);
+		snprintf (c, 64, "%g", NR_FONT_SIZE (font));
 		sp_repr_css_set_property (css, "font-size", c);
-		gnome_font_unref (font);
+		nr_font_unref (font);
 		/* Layout */
 		b = gtk_object_get_data (GTK_OBJECT (dlg), "text_anchor_start");
 		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (b))) {
@@ -369,6 +370,78 @@ sp_text_edit_dialog_close (GtkButton *button, GtkWidget *dlg)
 	gtk_object_destroy (GTK_OBJECT (dlg));
 }
 
+static guchar *
+sp_text_edit_dialog_font_style_to_lookup (SPStyle *style)
+{
+	static guchar c[256];
+	guchar *wstr, *sstr, *p;
+
+	switch (style->font_weight.computed) {
+	case SP_CSS_FONT_WEIGHT_100:
+		wstr = "extra light";
+		break;
+	case SP_CSS_FONT_WEIGHT_200:
+		wstr = "thin";
+		break;
+	case SP_CSS_FONT_WEIGHT_300:
+		wstr = "light";
+		break;
+	case SP_CSS_FONT_WEIGHT_400:
+	case SP_CSS_FONT_WEIGHT_NORMAL:
+		wstr = NULL;
+		break;
+	case SP_CSS_FONT_WEIGHT_500:
+		wstr = "medium";
+		break;
+	case SP_CSS_FONT_WEIGHT_600:
+		wstr = "semi";
+		break;
+	case SP_CSS_FONT_WEIGHT_700:
+	case SP_CSS_FONT_WEIGHT_BOLD:
+		wstr = "bold";
+		break;
+	case SP_CSS_FONT_WEIGHT_800:
+		wstr = "heavy";
+		break;
+	case SP_CSS_FONT_WEIGHT_900:
+		wstr = "black";
+		break;
+	default:
+		wstr = NULL;
+		break;
+	}
+
+	switch (style->font_style.computed) {
+	case SP_CSS_FONT_STYLE_NORMAL:
+		sstr = NULL;
+		break;
+	case SP_CSS_FONT_STYLE_ITALIC:
+		sstr = "italic";
+		break;
+	case SP_CSS_FONT_STYLE_OBLIQUE:
+		sstr = "oblique";
+		break;
+	default:
+		sstr = NULL;
+		break;
+	}
+
+	p = c;
+	if (wstr) {
+		if (p != c) *p++ = ' ';
+		memcpy (p, wstr, strlen (wstr));
+		p += strlen (wstr);
+	}
+	if (sstr) {
+		if (p != c) *p++ = ' ';
+		memcpy (p, sstr, strlen (sstr));
+		p += strlen (sstr);
+	}
+	*p = '\0';
+
+	return c;
+}
+
 static void
 sp_text_edit_dialog_read_selection (GtkWidget *dlg, gboolean dostyle, gboolean docontent)
 {
@@ -397,12 +470,20 @@ sp_text_edit_dialog_read_selection (GtkWidget *dlg, gboolean dostyle, gboolean d
 			guchar *str;
 			str = sp_text_get_string_multiline (text);
 			if (str && *str) {
-				e_utf8_gtk_editable_set_text (GTK_EDITABLE (textw), str);
-				gnome_font_preview_set_phrase (GNOME_FONT_PREVIEW (preview), str);
+				gchar *gtkstr;
+				int pos;
+				pos = 0;
+				gtk_text_freeze (GTK_TEXT (textw));
+				gtk_editable_delete_text (GTK_EDITABLE (textw), 0, -1);
+				gtkstr = e_utf8_to_gtk_string (textw, str);
+				gtk_editable_insert_text (GTK_EDITABLE (textw), gtkstr, strlen (gtkstr), &pos);
+				g_free (gtkstr);
+				gtk_text_thaw (GTK_TEXT (textw));
+				sp_font_preview_set_phrase (SP_FONT_PREVIEW (preview), str);
 				g_free (str);
 			} else {
 				gtk_editable_delete_text (GTK_EDITABLE (textw), 0, -1);
-				gnome_font_preview_set_phrase (GNOME_FONT_PREVIEW (preview), NULL);
+				sp_font_preview_set_phrase (SP_FONT_PREVIEW (preview), NULL);
 			}
 		}
 	} else {
@@ -422,17 +503,20 @@ sp_text_edit_dialog_read_selection (GtkWidget *dlg, gboolean dostyle, gboolean d
 	}
 
 	if (dostyle) {
-		GnomeFont *font;
+		NRTypeFace *tf;
+		NRFont *font;
 		GtkWidget *b;
+		guchar *c;
+
+		c = sp_text_edit_dialog_font_style_to_lookup (style);
+		tf = nr_type_directory_lookup_fuzzy (style->text->font_family.value, c);
+		font = nr_font_new_default (tf, NR_TYPEFACE_METRICS_HORIZONTAL, style->font_size.computed);
+		nr_typeface_unref (tf);
 
-		font = gnome_font_new_closest (style->text->font_family.value,
-					       sp_text_font_weight_to_gp (style->font_weight.computed),
-					       sp_text_font_italic_to_gp (style->font_style.computed),
-					       style->font_size.computed);
 		if (font) {
-			gnome_font_selection_set_font (GNOME_FONT_SELECTION (fontsel), font);
-			gnome_font_preview_set_font (GNOME_FONT_PREVIEW (preview), font);
-			gnome_font_unref (font);
+			sp_font_selector_set_font (SP_FONT_SELECTOR (fontsel), font);
+			sp_font_preview_set_font (SP_FONT_PREVIEW (preview), font);
+			nr_font_unref (font);
 		}
 
 		if (style->text_anchor.computed == SP_CSS_TEXT_ANCHOR_START) {
@@ -472,9 +556,9 @@ sp_text_edit_dialog_text_changed (GtkText *txt, GtkWidget *dlg)
 
 	str = e_utf8_gtk_editable_get_text (GTK_EDITABLE (textw));
 	if (str && *str) {
-		gnome_font_preview_set_phrase (GNOME_FONT_PREVIEW (preview), str);
+		sp_font_preview_set_phrase (SP_FONT_PREVIEW (preview), str);
 	} else {
-		gnome_font_preview_set_phrase (GNOME_FONT_PREVIEW (preview), NULL);
+		sp_font_preview_set_phrase (SP_FONT_PREVIEW (preview), NULL);
 	}
 	if (str) g_free (str);
 
@@ -485,7 +569,7 @@ sp_text_edit_dialog_text_changed (GtkText *txt, GtkWidget *dlg)
 }
 
 static void
-sp_text_edit_dialog_font_changed (GnomeFontSelection *fontsel, GnomeFont *font, GtkWidget *dlg)
+sp_text_edit_dialog_font_changed (SPFontSelector *fsel, NRFont *font, GtkWidget *dlg)
 {
 	GtkWidget *preview, *apply, *def;
 	SPText *text;
@@ -498,7 +582,7 @@ sp_text_edit_dialog_font_changed (GnomeFontSelection *fontsel, GnomeFont *font,
 	apply = gtk_object_get_data (GTK_OBJECT (dlg), "apply");
 	def = gtk_object_get_data (GTK_OBJECT (dlg), "default");
 
-	gnome_font_preview_set_font (GNOME_FONT_PREVIEW (preview), font);
+	sp_font_preview_set_font (SP_FONT_PREVIEW (preview), font);
 
 	if (text) {
 		gtk_widget_set_sensitive (apply, TRUE);
