@@ -5,12 +5,12 @@
  * RGBA display list system for sodipodi
  *
  * Author:
- *   Lauris Kaplinski <lauris@ximian.com>
+ *   Lauris Kaplinski <lauris@kaplinski.com>
  *
- * Copyright (C) 2001 Lauris Kaplinski and Ximian, Inc.
- *
- * Released under GNU GPL
+ * Copyright (C) 2001-2002 Lauris Kaplinski
+ * Copyright (C) 2001 Ximian, Inc.
  *
+ * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
 typedef struct _SPCanvasArena SPCanvasArena;
@@ -22,11 +22,11 @@ typedef struct _SPCanvasArenaClass SPCanvasArenaClass;
 #define SP_IS_CANVAS_ARENA(obj) (GTK_CHECK_TYPE ((obj), SP_TYPE_CANVAS_ARENA))
 #define SP_IS_CANVAS_ARENA_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), SP_TYPE_CANVAS_ARENA))
 
-#include <libgnomeui/gnome-canvas.h>
+#include "../helper/sp-canvas.h"
 #include "nr-arena-item.h"
 
 struct _SPCanvasArena {
-	GnomeCanvasItem item;
+	SPCanvasItem item;
 
 	guint cursor : 1;
 	guint sticky : 1;
@@ -43,7 +43,7 @@ struct _SPCanvasArena {
 };
 
 struct _SPCanvasArenaClass {
-	GnomeCanvasItemClass parent_class;
+	SPCanvasItemClass parent_class;
 
 	gint (* arena_event) (SPCanvasArena *carena, NRArenaItem *item, GdkEvent *event);
 };
