@@ -15,6 +15,8 @@
 
 #include <math.h>
 #include <string.h>
+#include <libnr/nr-rect.h>
+#include <libnr/nr-matrix.h>
 #include <libart_lgpl/art_misc.h>
 #include <libart_lgpl/art_bpath.h>
 #include <libart_lgpl/art_vpath.h>
@@ -36,9 +38,8 @@ static void nr_arena_glyphs_class_init (NRArenaGlyphsClass *klass);
 static void nr_arena_glyphs_init (NRArenaGlyphs *glyphs);
 static void nr_arena_glyphs_destroy (GtkObject *object);
 
-static guint nr_arena_glyphs_update (NRArenaItem *item, NRIRect *area, NRGC *gc, guint state, guint reset);
-static guint nr_arena_glyphs_render (NRArenaItem *item, NRIRect *area, NRBuffer *b);
-static guint nr_arena_glyphs_clip (NRArenaItem *item, NRIRect *area, NRBuffer *b);
+static guint nr_arena_glyphs_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint state, guint reset);
+static guint nr_arena_glyphs_clip (NRArenaItem *item, NRRectL *area, NRBuffer *b);
 static NRArenaItem *nr_arena_glyphs_pick (NRArenaItem *item, gdouble x, gdouble y, gdouble delta, gboolean sticky);
 
 static NRArenaItemClass *glyphs_parent_class;
@@ -75,7 +76,6 @@ nr_arena_glyphs_class_init (NRArenaGlyphsClass *klass)
 	object_class->destroy = nr_arena_glyphs_destroy;
 
 	item_class->update = nr_arena_glyphs_update;
-	item_class->render = nr_arena_glyphs_render;
 	item_class->clip = nr_arena_glyphs_clip;
 	item_class->pick = nr_arena_glyphs_pick;
 }
@@ -85,11 +85,7 @@ nr_arena_glyphs_init (NRArenaGlyphs *glyphs)
 {
 	glyphs->curve = NULL;
 	glyphs->style = NULL;
-#if 0
-	glyphs->fill_painter = NULL;
-	glyphs->stroke_painter = NULL;
-#endif
-	glyphs->fill_svp = NULL;
+
 	glyphs->stroke_svp = NULL;
 }
 
@@ -100,27 +96,18 @@ nr_arena_glyphs_destroy (GtkObject *object)
 
 	glyphs = NR_ARENA_GLYPHS (object);
 
-	if (glyphs->fill_svp) {
-		art_svp_free (glyphs->fill_svp);
-		glyphs->fill_svp = NULL;
-	}
-
 	if (glyphs->stroke_svp) {
 		art_svp_free (glyphs->stroke_svp);
 		glyphs->stroke_svp = NULL;
 	}
 
-#if 0
-	if (glyphs->fill_painter) {
-		sp_painter_free (glyphs->fill_painter);
-		glyphs->fill_painter = NULL;
+	if (glyphs->rfont) {
+		glyphs->rfont = nr_rasterfont_unref (glyphs->rfont);
 	}
 
-	if (glyphs->stroke_painter) {
-		sp_painter_free (glyphs->stroke_painter);
-		glyphs->stroke_painter = NULL;
+	if (glyphs->font) {
+		glyphs->font = nr_font_unref (glyphs->font);
 	}
-#endif
 
 	if (glyphs->style) {
 		sp_style_unref (glyphs->style);
@@ -128,8 +115,7 @@ nr_arena_glyphs_destroy (GtkObject *object)
 	}
 
 	if (glyphs->curve) {
-		sp_curve_unref (glyphs->curve);
-		glyphs->curve = NULL;
+		glyphs->curve = sp_curve_unref (glyphs->curve);
 	}
 
 	if (GTK_OBJECT_CLASS (glyphs_parent_class)->destroy)
@@ -137,9 +123,11 @@ nr_arena_glyphs_destroy (GtkObject *object)
 }
 
 static guint
-nr_arena_glyphs_update (NRArenaItem *item, NRIRect *area, NRGC *gc, guint state, guint reset)
+nr_arena_glyphs_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint state, guint reset)
 {
 	NRArenaGlyphs *glyphs;
+	NRRasterFont *rfont;
+	NRMatrixF t;
 	ArtBpath *abp;
 	ArtVpath *vp, *pvp;
 	ArtDRect bbox;
@@ -148,67 +136,68 @@ nr_arena_glyphs_update (NRArenaItem *item, NRIRect *area, NRGC *gc, guint state,
 
 	/* Request repaint old area if needed */
 	/* fixme: Think about it a bit (Lauris) */
-	if (!nr_irect_is_empty (&item->bbox)) {
+	if (!nr_rect_l_test_empty (&item->bbox)) {
 		nr_arena_request_render_rect (item->arena, &item->bbox);
-		nr_irect_set_empty (&item->bbox);
+		nr_rect_l_set_empty (&item->bbox);
 	}
 
 	/* Release state data */
-	if (glyphs->fill_svp) {
-		art_svp_free (glyphs->fill_svp);
-		glyphs->fill_svp = NULL;
-	}
 	if (glyphs->stroke_svp) {
 		art_svp_free (glyphs->stroke_svp);
 		glyphs->stroke_svp = NULL;
 	}
-#if 0
-	if (glyphs->fill_painter) {
-		sp_painter_free (glyphs->fill_painter);
-		glyphs->fill_painter = NULL;
-	}
-	if (glyphs->stroke_painter) {
-		sp_painter_free (glyphs->stroke_painter);
-		glyphs->stroke_painter = NULL;
-	}
-#endif
 
-	if (!glyphs->curve || !glyphs->style) return NR_ARENA_ITEM_STATE_ALL;
+	if (!glyphs->font || !glyphs->curve || !glyphs->style) return NR_ARENA_ITEM_STATE_ALL;
 	if ((glyphs->style->fill.type == SP_PAINT_TYPE_NONE) && (glyphs->style->stroke.type == SP_PAINT_TYPE_NONE)) return NR_ARENA_ITEM_STATE_ALL;
 
-	/* Build state data */
-	abp = art_bpath_affine_transform (glyphs->curve->bpath, gc->affine);
-	vp = art_bez_path_to_vec (abp, 0.25);
-	art_free (abp);
-	pvp = art_vpath_perturb (vp);
-	art_free (vp);
+	bbox.x0 = bbox.y0 = bbox.x1 = bbox.y1 = 0.0;
 
 	if (glyphs->style->fill.type != SP_PAINT_TYPE_NONE) {
-		ArtSVP *svpa, *svpb;
-		svpa = art_svp_from_vpath (pvp);
-		svpb = art_svp_uncross (svpa);
-		art_svp_free (svpa);
-		glyphs->fill_svp = art_svp_rewind_uncrossed (svpb, glyphs->style->fill_rule.value);
-		art_svp_free (svpb);
+		NRRectF area;
+
+		t.c[0] = gc->affine[0];
+		t.c[1] = gc->affine[1];
+		t.c[2] = gc->affine[2];
+		t.c[3] = gc->affine[3];
+		t.c[4] = gc->affine[4];
+		t.c[5] = gc->affine[5];
+		nr_matrix_multiply_fff (&t, &glyphs->transform, &t);
+		rfont = nr_rasterfont_new (glyphs->font, &t);
+		if (glyphs->rfont) glyphs->rfont = nr_rasterfont_unref (glyphs->rfont);
+		glyphs->rfont = rfont;
+		glyphs->x = t.c[4];
+		glyphs->y = t.c[5];
+		nr_rasterfont_glyph_area_get (rfont, glyphs->glyph, &area);
+		bbox.x0 = area.x0 + glyphs->x;
+		bbox.y0 = area.y0 + glyphs->y;
+		bbox.x1 = area.x1 + glyphs->x;
+		bbox.y1 = area.y1 + glyphs->y;
 	}
 
 	if (glyphs->style->stroke.type != SP_PAINT_TYPE_NONE) {
-		gdouble width;
-		width = sp_distance_d_matrix_d_transform (glyphs->style->stroke_width.computed, gc->affine);
-		if (width > 0.125) {
-			glyphs->stroke_svp = art_svp_vpath_stroke (pvp,
-								   glyphs->style->stroke_linejoin.value,
-								   glyphs->style->stroke_linecap.value,
-								   width,
-								   glyphs->style->stroke_miterlimit.value, 0.25);
+		/* Build state data */
+		abp = art_bpath_affine_transform (glyphs->curve->bpath, gc->affine);
+		vp = art_bez_path_to_vec (abp, 0.25);
+		art_free (abp);
+		pvp = art_vpath_perturb (vp);
+		art_free (vp);
+
+		if (glyphs->style->stroke.type != SP_PAINT_TYPE_NONE) {
+			gdouble width;
+			width = sp_distance_d_matrix_d_transform (glyphs->style->stroke_width.computed, gc->affine);
+			if (width > 0.125) {
+				glyphs->stroke_svp = art_svp_vpath_stroke (pvp,
+									   glyphs->style->stroke_linejoin.value,
+									   glyphs->style->stroke_linecap.value,
+									   width,
+									   glyphs->style->stroke_miterlimit.value, 0.25);
+			}
 		}
+
+		art_free (pvp);
 	}
 
-	art_free (pvp);
-
-	bbox.x0 = bbox.y0 = bbox.x1 = bbox.y1 = 0.0;
 	if (glyphs->stroke_svp) art_drect_svp_union (&bbox, glyphs->stroke_svp);
-	if (glyphs->fill_svp) art_drect_svp_union (&bbox, glyphs->fill_svp);
 	if (art_drect_empty (&bbox)) return NR_ARENA_ITEM_STATE_ALL;
 
 	item->bbox.x0 = bbox.x0 - 1.0;
@@ -217,127 +206,25 @@ nr_arena_glyphs_update (NRArenaItem *item, NRIRect *area, NRGC *gc, guint state,
 	item->bbox.y1 = bbox.y1 + 1.0;
 	nr_arena_request_render_rect (item->arena, &item->bbox);
 
-#if 0
-	if (glyphs->style->fill.type == SP_PAINT_TYPE_PAINTSERVER) {
-		/* fixme: This is probably not correct as bbox has to be the one of fill */
-		g_warning ("Glyphs gradient fill: Implement this");
-		glyphs->fill_painter = sp_paint_server_painter_new (SP_STYLE_FILL_SERVER (glyphs->style), gc->affine, &bbox);
-	}
-	if (glyphs->style->stroke.type == SP_PAINT_TYPE_PAINTSERVER) {
-		/* fixme: This is probably not correct as bbox has to be the one of fill */
-		glyphs->stroke_painter = sp_paint_server_painter_new (SP_STYLE_STROKE_SERVER (glyphs->style), gc->affine, &bbox);
-	}
-#endif
-
 	return NR_ARENA_ITEM_STATE_ALL;
 }
 
 static guint
-nr_arena_glyphs_render (NRArenaItem *item, NRIRect *area, NRBuffer *b)
+nr_arena_glyphs_clip (NRArenaItem *item, NRRectL *area, NRBuffer *b)
 {
+	NRArenaGlyphs *glyphs;
+
+	glyphs = NR_ARENA_GLYPHS (item);
+
+	if (!glyphs->font || !glyphs->curve) return item->state;
+
 #if 0
-	NRArenaGlyphs *glyphs;
-	SPStyle *style;
-
-	glyphs = NR_ARENA_GLYPHS (item);
-
-	if (!glyphs->curve) return item->state;
-	if (!glyphs->style) return item->state;
-
-	style = glyphs->style;
-
-	if (glyphs->fill_svp) {
-		NRBuffer *m;
-		guint32 rgba;
-
-		m = nr_buffer_get (NR_IMAGE_A8, area->x1 - area->x0, area->y1 - area->y0, TRUE, TRUE);
-		art_gray_svp_aa (glyphs->fill_svp, area->x0, area->y0, area->x1, area->y1, m->px, m->rs);
-		m->empty = FALSE;
-
-		switch (style->fill.type) {
-		case SP_PAINT_TYPE_COLOR:
-			rgba = sp_color_get_rgba32_falpha (&style->fill.value.color,
-							   SP_SCALE24_TO_FLOAT (style->fill_opacity.value) *
-							   SP_SCALE24_TO_FLOAT (style->opacity.value));
-			nr_render_buf_mask_rgba32 (b, 0, 0, area->x1 - area->x0, area->y1 - area->y0, m, 0, 0, rgba);
-			b->empty = FALSE;
-			break;
-		case SP_PAINT_TYPE_PAINTSERVER:
-			if (glyphs->fill_painter) {
-				NRBuffer *pb;
-				/* Need separate gradient buffer */
-				pb = nr_buffer_get (NR_IMAGE_R8G8B8A8, area->x1 - area->x0, area->y1 - area->y0, TRUE, FALSE);
-				glyphs->fill_painter->fill (glyphs->fill_painter, pb->px, area->x0, area->y0, pb->w, pb->h, pb->rs);
-				pb->empty = FALSE;
-				/* Composite */
-				nr_render_buf_buf_mask (b, 0, 0, area->x1 - area->x0, area->y1 - area->y0,
-							pb, 0, 0,
-							m, 0, 0);
-				b->empty = FALSE;
-				nr_buffer_free (pb);
-			}
-			break;
-		default:
-			break;
-		}
-		nr_buffer_free (m);
-	}
-
-	if (glyphs->stroke_svp) {
-		NRBuffer *m;
-		guint32 rgba;
-
-		m = nr_buffer_get (NR_IMAGE_A8, area->x1 - area->x0, area->y1 - area->y0, TRUE, TRUE);
-		art_gray_svp_aa (glyphs->stroke_svp, area->x0, area->y0, area->x1, area->y1, m->px, m->rs);
-		m->empty = FALSE;
-
-		switch (style->stroke.type) {
-		case SP_PAINT_TYPE_COLOR:
-			rgba = sp_color_get_rgba32_falpha (&style->stroke.value.color,
-							   SP_SCALE24_TO_FLOAT (style->stroke_opacity.value) *
-							   SP_SCALE24_TO_FLOAT (style->opacity.value));
-			nr_render_buf_mask_rgba32 (b, 0, 0, area->x1 - area->x0, area->y1 - area->y0, m, 0, 0, rgba);
-			b->empty = FALSE;
-			nr_buffer_free (m);
-			break;
-		case SP_PAINT_TYPE_PAINTSERVER:
-			if (glyphs->stroke_painter) {
-				NRBuffer *pb;
-				/* Need separate gradient buffer */
-				pb = nr_buffer_get (NR_IMAGE_R8G8B8A8, area->x1 - area->x0, area->y1 - area->y0, TRUE, FALSE);
-				glyphs->stroke_painter->fill (glyphs->stroke_painter, pb->px, area->x0, area->y0, pb->w, pb->h, pb->rs);
-				pb->empty = FALSE;
-				/* Composite */
-				nr_render_buf_buf_mask (b, 0, 0, area->x1 - area->x0, area->y1 - area->y0,
-							pb, 0, 0,
-							m, 0, 0);
-				b->empty = FALSE;
-				nr_buffer_free (pb);
-			}
-			break;
-		default:
-			break;
-		}
-		nr_buffer_free (m);
-	}
-#endif
-
-	return item->state;
-}
-
-static guint
-nr_arena_glyphs_clip (NRArenaItem *item, NRIRect *area, NRBuffer *b)
-{
-	NRArenaGlyphs *glyphs;
-
-	glyphs = NR_ARENA_GLYPHS (item);
-
-	if (!glyphs->curve) return item->state;
-
+	/* fixme: Implement */
 	if (glyphs->fill_svp) {
 		art_gray_svp_aa (glyphs->fill_svp, area->x0, area->y0, area->x1, area->y1, b->px, b->rs);
 		b->empty = FALSE;
 	}
+#endif
 
 	return item->state;
 }
@@ -349,19 +236,16 @@ nr_arena_glyphs_pick (NRArenaItem *item, gdouble x, gdouble y, gdouble delta, gb
 
 	glyphs = NR_ARENA_GLYPHS (item);
 
-	if (!glyphs->curve) return NULL;
+	if (!glyphs->font || !glyphs->curve) return NULL;
 	if (!glyphs->style) return NULL;
 
-	if (glyphs->fill_svp && (glyphs->style->fill.type != SP_PAINT_TYPE_NONE)) {
-		if (art_svp_point_wind (glyphs->fill_svp, x, y)) return item;
-	}
+	/* fixme: */
+	if ((x >= item->bbox.x0) && (y >= item->bbox.y0) && (x < item->bbox.x1) && (y < item->bbox.y1)) return item;
+
 	if (glyphs->stroke_svp && (glyphs->style->stroke.type != SP_PAINT_TYPE_NONE)) {
 		if (art_svp_point_wind (glyphs->stroke_svp, x, y)) return item;
 	}
 	if (delta > 1e-3) {
-		if (glyphs->fill_svp && (glyphs->style->fill.type != SP_PAINT_TYPE_NONE)) {
-			if (art_svp_point_dist (glyphs->fill_svp, x, y) <= delta) return item;
-		}
 		if (glyphs->stroke_svp && (glyphs->style->stroke.type != SP_PAINT_TYPE_NONE)) {
 			if (art_svp_point_dist (glyphs->stroke_svp, x, y) <= delta) return item;
 		}
@@ -371,7 +255,7 @@ nr_arena_glyphs_pick (NRArenaItem *item, gdouble x, gdouble y, gdouble delta, gb
 }
 
 void
-nr_arena_glyphs_set_path (NRArenaGlyphs *glyphs, SPCurve *curve, gboolean private, const gdouble *affine)
+nr_arena_glyphs_set_path (NRArenaGlyphs *glyphs, SPCurve *curve, gboolean private, NRFont *font, gint glyph, const NRMatrixF *transform)
 {
 	g_return_if_fail (glyphs != NULL);
 	g_return_if_fail (NR_IS_ARENA_GLYPHS (glyphs));
@@ -384,17 +268,30 @@ nr_arena_glyphs_set_path (NRArenaGlyphs *glyphs, SPCurve *curve, gboolean privat
 	}
 
 	if (curve) {
-		if (affine) {
+		if (transform) {
 			ArtBpath *abp;
-			abp = art_bpath_affine_transform (curve->bpath, affine);
+			double a[6];
+			a[0] = transform->c[0];
+			a[1] = transform->c[1];
+			a[2] = transform->c[2];
+			a[3] = transform->c[3];
+			a[4] = transform->c[4];
+			a[5] = transform->c[5];
+			abp = art_bpath_affine_transform (curve->bpath, a);
 			curve = sp_curve_new_from_bpath (abp);
 			glyphs->curve = curve;
+			glyphs->transform = *transform;
 		} else {
 			glyphs->curve = curve;
 			sp_curve_ref (curve);
+			nr_matrix_f_set_identity (&glyphs->transform);
 		}
 	}
 
+	if (glyphs->font) glyphs->font = nr_font_unref (glyphs->font);
+	if (font) glyphs->font = nr_font_ref (font);
+	glyphs->glyph = glyph;
+
 	nr_arena_item_request_update (NR_ARENA_ITEM (glyphs), NR_ARENA_ITEM_STATE_ALL, FALSE);
 }
 
@@ -412,42 +309,29 @@ nr_arena_glyphs_set_style (NRArenaGlyphs *glyphs, SPStyle *style)
 }
 
 static guint
-nr_arena_glyphs_fill_mask (NRArenaGlyphs *glyphs, NRIRect *area, NRBuffer *b)
+nr_arena_glyphs_fill_mask (NRArenaGlyphs *glyphs, NRRectL *area, NRPixBlock *m)
 {
 	NRArenaItem *item;
 
+	/* fixme: area == m->area, so merge these */
+
 	item = NR_ARENA_ITEM (glyphs);
 
-	if (glyphs->fill_svp && nr_irect_do_intersect (area, &item->bbox)) {
-		NRBuffer *gb;
-		gint x, y;
-		gb = nr_buffer_get (NR_IMAGE_A8, area->x1 - area->x0, area->y1 - area->y0, TRUE, TRUE);
-		art_gray_svp_aa (glyphs->fill_svp, area->x0, area->y0, area->x1, area->y1, gb->px, gb->rs);
-		for (y = area->y0; y < area->y1; y++) {
-			guchar *d, *s;
-			d = b->px + (y - area->y0) * b->rs;
-			s = gb->px + (y - area->y0) * b->rs;
-			for (x = area->x0; x < area->x1; x++) {
-				*d = (*d) + ((255 - *d) * (*s) / 255);
-				d += 1;
-				s += 1;
-			}
-		}
-		nr_buffer_free (gb);
-		b->empty = FALSE;
+	if (glyphs->rfont && nr_rect_l_test_intersect (area, &item->bbox)) {
+		nr_rasterfont_glyph_mask_render (glyphs->rfont, glyphs->glyph, m, glyphs->x, glyphs->y);
 	}
 
 	return item->state;
 }
 
 static guint
-nr_arena_glyphs_stroke_mask (NRArenaGlyphs *glyphs, NRIRect *area, NRBuffer *b)
+nr_arena_glyphs_stroke_mask (NRArenaGlyphs *glyphs, NRRectL *area, NRBuffer *b)
 {
 	NRArenaItem *item;
 
 	item = NR_ARENA_ITEM (glyphs);
 
-	if (glyphs->stroke_svp && nr_irect_do_intersect (area, &item->bbox)) {
+	if (glyphs->stroke_svp && nr_rect_l_test_intersect (area, &item->bbox)) {
 		NRBuffer *gb;
 		gint x, y;
 		gb = nr_buffer_get (NR_IMAGE_A8, area->x1 - area->x0, area->y1 - area->y0, TRUE, TRUE);
@@ -473,8 +357,8 @@ static void nr_arena_glyphs_group_class_init (NRArenaGlyphsGroupClass *klass);
 static void nr_arena_glyphs_group_init (NRArenaGlyphsGroup *group);
 static void nr_arena_glyphs_group_destroy (GtkObject *object);
 
-static guint nr_arena_glyphs_group_update (NRArenaItem *item, NRIRect *area, NRGC *gc, guint state, guint reset);
-static guint nr_arena_glyphs_group_render (NRArenaItem *item, NRIRect *area, NRBuffer *b);
+static guint nr_arena_glyphs_group_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint state, guint reset);
+static guint nr_arena_glyphs_group_render (NRArenaItem *item, NRRectL *area, NRBuffer *b);
 static NRArenaItem *nr_arena_glyphs_group_pick (NRArenaItem *item, gdouble x, gdouble y, gdouble delta, gboolean sticky);
 
 static NRArenaGroupClass *group_parent_class;
@@ -553,7 +437,7 @@ nr_arena_glyphs_group_destroy (GtkObject *object)
 }
 
 static guint
-nr_arena_glyphs_group_update (NRArenaItem *item, NRIRect *area, NRGC *gc, guint state, guint reset)
+nr_arena_glyphs_group_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint state, guint reset)
 {
 	NRArenaGlyphsGroup *group;
 
@@ -586,7 +470,7 @@ nr_arena_glyphs_group_update (NRArenaItem *item, NRIRect *area, NRGC *gc, guint
 /* This sucks - as soon, as we have inheritable renderprops, do something with that opacity */
 
 static guint
-nr_arena_glyphs_group_render (NRArenaItem *item, NRIRect *area, NRBuffer *b)
+nr_arena_glyphs_group_render (NRArenaItem *item, NRRectL *area, NRBuffer *b)
 {
 	NRArenaGroup *group;
 	NRArenaGlyphsGroup *ggroup;
@@ -602,17 +486,22 @@ nr_arena_glyphs_group_render (NRArenaItem *item, NRIRect *area, NRBuffer *b)
 
 	/* Fill */
 	if (style->fill.type != SP_PAINT_TYPE_NONE) {
+		NRPixBlock mb;
 		NRBuffer *m;
 		guint32 rgba;
 		m = nr_buffer_get (NR_IMAGE_A8, area->x1 - area->x0, area->y1 - area->y0, TRUE, TRUE);
+		nr_pixblock_setup_extern (&mb, NR_PIXBLOCK_MODE_A8, area->x0, area->y0, area->x1, area->y1, m->px, m->rs, FALSE, FALSE);
 		/* Render children fill mask */
 		for (child = group->children; child != NULL; child = child->next) {
-			ret = nr_arena_glyphs_fill_mask (NR_ARENA_GLYPHS (child), area, m);
+			ret = nr_arena_glyphs_fill_mask (NR_ARENA_GLYPHS (child), area, &mb);
 			if (!(ret & NR_ARENA_ITEM_STATE_RENDER)) {
+				nr_pixblock_release (&mb);
 				nr_buffer_free (m);
 				return ret;
 			}
 		}
+		m->empty = mb.empty;
+		nr_pixblock_release (&mb);
 		/* Composite into buffer */
 		switch (style->fill.type) {
 		case SP_PAINT_TYPE_COLOR:
@@ -722,30 +611,30 @@ nr_arena_glyphs_group_clear (NRArenaGlyphsGroup *sg)
 }
 
 void
-nr_arena_glyphs_group_add_component (NRArenaGlyphsGroup *sg, SPCurve *curve, gboolean private, const gdouble *affine)
+nr_arena_glyphs_group_add_component (NRArenaGlyphsGroup *sg, NRFont *font, int glyph, const NRMatrixF *transform)
 {
 	NRArenaGroup *group;
-	NRArenaItem *new;
+	NRBPath bpath;
 
 	group = NR_ARENA_GROUP (sg);
 
-	nr_arena_item_request_render (NR_ARENA_ITEM (group));
+	if (nr_font_glyph_outline_get (font, glyph, &bpath, FALSE) && bpath.path) {
+		SPCurve *curve;
 
-	new = nr_arena_item_new (NR_ARENA_ITEM (group)->arena, NR_TYPE_ARENA_GLYPHS);
-	nr_arena_item_append_child (NR_ARENA_ITEM (group), new);
-	nr_arena_item_unref (new);
-	nr_arena_glyphs_set_path (NR_ARENA_GLYPHS (new), curve, private, affine);
-	nr_arena_glyphs_set_style (NR_ARENA_GLYPHS (new), sg->style);
-}
+		nr_arena_item_request_render (NR_ARENA_ITEM (group));
 
-void
-nr_arena_glyphs_group_set_component (NRArenaGlyphsGroup *sg, SPCurve *curve, gboolean private, const gdouble *affine)
-{
-	g_return_if_fail (sg != NULL);
-	g_return_if_fail (NR_IS_ARENA_GLYPHS_GROUP (sg));
+		curve = sp_curve_new_from_foreign_bpath (bpath.path);
+		if (curve) {
+			NRArenaItem *new;
+			new = nr_arena_item_new (NR_ARENA_ITEM (group)->arena, NR_TYPE_ARENA_GLYPHS);
+			nr_arena_item_append_child (NR_ARENA_ITEM (group), new);
+			nr_arena_item_unref (new);
+			nr_arena_glyphs_set_path (NR_ARENA_GLYPHS (new), curve, FALSE, font, glyph, transform);
+			nr_arena_glyphs_set_style (NR_ARENA_GLYPHS (new), sg->style);
+			sp_curve_unref (curve);
+		}
+	}
 
-	nr_arena_glyphs_group_clear (sg);
-	nr_arena_glyphs_group_add_component (sg, curve, private, affine);
 }
 
 void
@@ -780,7 +669,13 @@ nr_arena_glyphs_group_set_paintbox (NRArenaGlyphsGroup *gg, const ArtDRect *pbox
 	g_return_if_fail (pbox->x1 > pbox->x0);
 	g_return_if_fail (pbox->y1 > pbox->y0);
 
-	memcpy (&gg->paintbox, pbox, sizeof (ArtDRect));
+	if ((pbox->x0 < pbox->x1) && (pbox->y0 < pbox->y1)) {
+		memcpy (&gg->paintbox, pbox, sizeof (ArtDRect));
+	} else {
+		/* fixme: We kill warning, although not sure what to do here (Lauris) */
+		gg->paintbox.x0 = gg->paintbox.y0 = 0.0;
+		gg->paintbox.x1 = gg->paintbox.y1 = 256.0;
+	}
 
 	nr_arena_item_request_update (NR_ARENA_ITEM (gg), NR_ARENA_ITEM_STATE_ALL, FALSE);
 }
