@@ -4,17 +4,20 @@
  * RGBA display list system for sodipodi
  *
  * Author:
- *   Lauris Kaplinski <lauris@ximian.com>
+ *   Lauris Kaplinski <lauris@kaplinski.com>
  *
- * Copyright (C) 2001 Lauris Kaplinski and Ximian, Inc.
- *
- * Released under GNU GPL
+ * Copyright (C) 2001-2002 Lauris Kaplinski
+ * Copyright (C) 2001 Ximian, Inc.
  *
+ * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
 
 #include <math.h>
 #include <string.h>
+#include <libnr/nr-rect.h>
+#include <libnr/nr-matrix.h>
+#include <libnr/nr-path.h>
 #include <libart_lgpl/art_misc.h>
 #include <libart_lgpl/art_bpath.h>
 #include <libart_lgpl/art_vpath.h>
@@ -36,9 +39,9 @@ static void nr_arena_shape_class_init (NRArenaShapeClass *klass);
 static void nr_arena_shape_init (NRArenaShape *shape);
 static void nr_arena_shape_destroy (GtkObject *object);
 
-static guint nr_arena_shape_update (NRArenaItem *item, NRIRect *area, NRGC *gc, guint state, guint reset);
-static guint nr_arena_shape_render (NRArenaItem *item, NRIRect *area, NRBuffer *b);
-static guint nr_arena_shape_clip (NRArenaItem *item, NRIRect *area, NRBuffer *b);
+static guint nr_arena_shape_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint state, guint reset);
+static guint nr_arena_shape_render (NRArenaItem *item, NRRectL *area, NRBuffer *b);
+static guint nr_arena_shape_clip (NRArenaItem *item, NRRectL *area, NRBuffer *b);
 static NRArenaItem *nr_arena_shape_pick (NRArenaItem *item, gdouble x, gdouble y, gdouble delta, gboolean sticky);
 
 static NRArenaItemClass *shape_parent_class;
@@ -87,6 +90,8 @@ nr_arena_shape_init (NRArenaShape *shape)
 	shape->style = NULL;
 	shape->paintbox.x0 = shape->paintbox.y0 = 0.0;
 	shape->paintbox.x1 = shape->paintbox.y1 = 256.0;
+
+	nr_matrix_d_set_identity (&shape->ctm);
 	shape->fill_painter = NULL;
 	shape->stroke_painter = NULL;
 	shape->fill_svp = NULL;
@@ -135,26 +140,60 @@ nr_arena_shape_destroy (GtkObject *object)
 }
 
 static guint
-nr_arena_shape_update (NRArenaItem *item, NRIRect *area, NRGC *gc, guint state, guint reset)
+nr_arena_shape_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint state, guint reset)
 {
 	NRArenaShape *shape;
+	SPStyle *style;
 	ArtBpath *abp;
 	ArtVpath *vp, *pvp;
 	ArtDRect bbox;
 
 	shape = NR_ARENA_SHAPE (item);
+	style = shape->style;
+
+	if (!(state & NR_ARENA_ITEM_STATE_RENDER)) {
+		/* We do not have to create rendering structures */
+		memcpy (shape->ctm.c, gc->affine, 6 * sizeof (double));
+		if (state & NR_ARENA_ITEM_STATE_BBOX) {
+			if (shape->curve) {
+				NRMatrixF ctm;
+				NRRectF bbox;
+				NRBPath bp;
+				/* fixme: */
+				bbox.x0 = bbox.y0 = NR_HUGE_F;
+				bbox.x1 = bbox.y1 = -NR_HUGE_F;
+				ctm.c[0] = gc->affine[0];
+				ctm.c[1] = gc->affine[1];
+				ctm.c[2] = gc->affine[2];
+				ctm.c[3] = gc->affine[3];
+				ctm.c[4] = gc->affine[4];
+				ctm.c[5] = gc->affine[5];
+				bp.path = shape->curve->bpath;
+				nr_path_matrix_f_bbox_f_union (&bp, &ctm, &bbox, 1.0);
+				item->bbox.x0 = bbox.x0 - 1.0;
+				item->bbox.y0 = bbox.y0 - 1.0;
+				item->bbox.x1 = bbox.x1 + 1.9999;
+				item->bbox.y1 = bbox.y1 + 1.9999;
+			}
+		}
+		return (state | item->state);
+	}
 
 	/* Request repaint old area if needed */
 	/* fixme: Think about it a bit (Lauris) */
-	if (!nr_irect_is_empty (&item->bbox)) {
+	/* fixme: Thios is only needed, if actually rendered/had svp (Lauris) */
+	if (!nr_rect_l_test_empty (&item->bbox)) {
 		nr_arena_request_render_rect (item->arena, &item->bbox);
-		nr_irect_set_empty (&item->bbox);
+		nr_rect_l_set_empty (&item->bbox);
 	}
 
 	/* Release state data */
-	if (shape->fill_svp) {
-		art_svp_free (shape->fill_svp);
-		shape->fill_svp = NULL;
+	if (TRUE || !nr_matrix_d_test_transform_equal ((NRMatrixD *) gc->affine, &shape->ctm, NR_EPSILON_D)) {
+		/* Concept test */
+		if (shape->fill_svp) {
+			art_svp_free (shape->fill_svp);
+			shape->fill_svp = NULL;
+		}
 	}
 	if (shape->stroke_svp) {
 		art_svp_free (shape->stroke_svp);
@@ -170,37 +209,70 @@ nr_arena_shape_update (NRArenaItem *item, NRIRect *area, NRGC *gc, guint state,
 	}
 
 	if (!shape->curve || !shape->style) return NR_ARENA_ITEM_STATE_ALL;
+	if (sp_curve_is_empty (shape->curve)) return NR_ARENA_ITEM_STATE_ALL;
 	if ((shape->style->fill.type == SP_PAINT_TYPE_NONE) && (shape->style->stroke.type == SP_PAINT_TYPE_NONE)) return NR_ARENA_ITEM_STATE_ALL;
 
 	/* Build state data */
-	abp = art_bpath_affine_transform (shape->curve->bpath, gc->affine);
-	vp = art_bez_path_to_vec (abp, 0.25);
-	art_free (abp);
-	pvp = art_vpath_perturb (vp);
-	art_free (vp);
-
 	if (shape->style->fill.type != SP_PAINT_TYPE_NONE) {
-		ArtSVP *svpa, *svpb;
-		svpa = art_svp_from_vpath (pvp);
-		svpb = art_svp_uncross (svpa);
-		art_svp_free (svpa);
-		shape->fill_svp = art_svp_rewind_uncrossed (svpb, shape->style->fill_rule.value);
-		art_svp_free (svpb);
-	}
-
-	if (shape->style->stroke.type != SP_PAINT_TYPE_NONE) {
-		gdouble width;
-		width = sp_distance_d_matrix_d_transform (shape->style->stroke_width.computed, gc->affine);
-		if (width > 0.125) {
-			shape->stroke_svp = art_svp_vpath_stroke (pvp,
-								  shape->style->stroke_linejoin.value,
-								  shape->style->stroke_linecap.value,
-								  width,
-								  shape->style->stroke_miterlimit.value, 0.25);
+		if ((shape->curve->end > 2) || (shape->curve->bpath[1].code == ART_CURVETO)) {
+			if (TRUE || !shape->fill_svp) {
+				ArtSVP *svpa, *svpb;
+				abp = art_bpath_affine_transform (shape->curve->bpath, gc->affine);
+				vp = sp_vpath_from_bpath_transform_closepath (abp, NULL, TRUE, 0.25);
+				art_free (abp);
+				pvp = art_vpath_perturb (vp);
+				art_free (vp);
+				svpa = art_svp_from_vpath (pvp);
+				art_free (pvp);
+				svpb = art_svp_uncross (svpa);
+				art_svp_free (svpa);
+				shape->fill_svp = art_svp_rewind_uncrossed (svpb, shape->style->fill_rule.value);
+				art_svp_free (svpb);
+			} else if (!NR_DF_TEST_CLOSE (gc->affine[4], shape->ctm.c[4], NR_EPSILON_D) ||
+				   !NR_DF_TEST_CLOSE (gc->affine[5], shape->ctm.c[5], NR_EPSILON_D)) {
+				ArtSVP *svpa;
+				/* Concept test */
+				svpa = art_svp_translate (shape->fill_svp, gc->affine[4] - shape->ctm.c[4], gc->affine[5] - shape->ctm.c[5]);
+				art_svp_free (shape->fill_svp);
+				shape->fill_svp = svpa;
+			}
+			memcpy (shape->ctm.c, gc->affine, 6 * sizeof (double));
 		}
 	}
 
-	art_free (pvp);
+	if (style->stroke.type != SP_PAINT_TYPE_NONE) {
+		double width, scale, dlen;
+		int i;
+		abp = art_bpath_affine_transform (shape->curve->bpath, gc->affine);
+		vp = art_bez_path_to_vec (abp, 0.25);
+		art_free (abp);
+		pvp = art_vpath_perturb (vp);
+		art_free (vp);
+		scale = sp_distance_d_matrix_d_transform (1.0, gc->affine);
+		width = MAX (0.125, style->stroke_width.computed * scale);
+		dlen = 0.0;
+		for (i = 0; i < style->stroke_dash.n_dash; i++) dlen += style->stroke_dash.dash[i] * scale;
+		if (dlen >= 1.0) {
+			ArtVpathDash dash;
+			int i;
+			dash.offset = style->stroke_dash.offset * scale;
+			dash.n_dash = style->stroke_dash.n_dash;
+			dash.dash = g_new (double, dash.n_dash);
+			for (i = 0; i < dash.n_dash; i++) {
+				dash.dash[i] = style->stroke_dash.dash[i] * scale;
+			}
+			vp = art_vpath_dash (pvp, &dash);
+			art_free (pvp);
+			pvp = vp;
+			g_free (dash.dash);
+		}
+		shape->stroke_svp = art_svp_vpath_stroke (pvp,
+							  shape->style->stroke_linejoin.value,
+							  shape->style->stroke_linecap.value,
+							  width,
+							  shape->style->stroke_miterlimit.value, 0.25);
+		art_free (pvp);
+	}
 
 	bbox.x0 = bbox.y0 = bbox.x1 = bbox.y1 = 0.0;
 	if (shape->stroke_svp) art_drect_svp_union (&bbox, shape->stroke_svp);
@@ -226,7 +298,7 @@ nr_arena_shape_update (NRArenaItem *item, NRIRect *area, NRGC *gc, guint state,
 }
 
 static guint
-nr_arena_shape_render (NRArenaItem *item, NRIRect *area, NRBuffer *b)
+nr_arena_shape_render (NRArenaItem *item, NRRectL *area, NRBuffer *b)
 {
 	NRArenaShape *shape;
 	SPStyle *style;
@@ -316,7 +388,7 @@ nr_arena_shape_render (NRArenaItem *item, NRIRect *area, NRBuffer *b)
 }
 
 static guint
-nr_arena_shape_clip (NRArenaItem *item, NRIRect *area, NRBuffer *b)
+nr_arena_shape_clip (NRArenaItem *item, NRRectL *area, NRBuffer *b)
 {
 	NRArenaShape *shape;
 
@@ -342,18 +414,49 @@ nr_arena_shape_pick (NRArenaItem *item, gdouble x, gdouble y, gdouble delta, gbo
 	if (!shape->curve) return NULL;
 	if (!shape->style) return NULL;
 
-	if (shape->fill_svp && (shape->style->fill.type != SP_PAINT_TYPE_NONE)) {
-		if (art_svp_point_wind (shape->fill_svp, x, y)) return item;
-	}
-	if (shape->stroke_svp && (shape->style->stroke.type != SP_PAINT_TYPE_NONE)) {
-		if (art_svp_point_wind (shape->stroke_svp, x, y)) return item;
-	}
-	if (delta > 1e-3) {
+	if (item->state & NR_ARENA_ITEM_STATE_RENDER) {
 		if (shape->fill_svp && (shape->style->fill.type != SP_PAINT_TYPE_NONE)) {
-			if (art_svp_point_dist (shape->fill_svp, x, y) <= delta) return item;
+			if (art_svp_point_wind (shape->fill_svp, x, y)) return item;
 		}
 		if (shape->stroke_svp && (shape->style->stroke.type != SP_PAINT_TYPE_NONE)) {
-			if (art_svp_point_dist (shape->stroke_svp, x, y) <= delta) return item;
+			if (art_svp_point_wind (shape->stroke_svp, x, y)) return item;
+		}
+		if (delta > 1e-3) {
+			if (shape->fill_svp && (shape->style->fill.type != SP_PAINT_TYPE_NONE)) {
+				if (art_svp_point_dist (shape->fill_svp, x, y) <= delta) return item;
+			}
+			if (shape->stroke_svp && (shape->style->stroke.type != SP_PAINT_TYPE_NONE)) {
+				if (art_svp_point_dist (shape->stroke_svp, x, y) <= delta) return item;
+			}
+		}
+	} else {
+		NRMatrixF t;
+		NRPointF pt;
+		NRBPath bp;
+		float dist;
+		int wind;
+		pt.x = x;
+		pt.y = y;
+		t.c[0] = shape->ctm.c[0];
+		t.c[1] = shape->ctm.c[1];
+		t.c[2] = shape->ctm.c[2];
+		t.c[3] = shape->ctm.c[3];
+		t.c[4] = shape->ctm.c[4];
+		t.c[5] = shape->ctm.c[5];
+		bp.path = shape->curve->bpath;
+		dist = NR_HUGE_F;
+		wind = 0;
+		nr_path_matrix_f_point_f_bbox_wind_distance (&bp, &t, &pt, NULL, &wind, &dist, NR_EPSILON_F);
+		if (shape->style->fill.type != SP_PAINT_TYPE_NONE) {
+			if (!shape->style->fill_rule.value) {
+				if (wind != 0) return item;
+			} else {
+				if (wind & 0x1) return item;
+			}
+		}
+		if (shape->style->stroke.type != SP_PAINT_TYPE_NONE) {
+			/* fixme: We do not take stroke width into account here (Lauris) */
+			if (dist < delta) return item;
 		}
 	}
 
