@@ -5,12 +5,12 @@
  * RGBA display list system for sodipodi
  *
  * Author:
- *   Lauris Kaplinski <lauris@ximian.com>
+ *   Lauris Kaplinski <lauris@kaplinski.com>
  *
- * Copyright (C) 2001 Lauris Kaplinski and Ximian, Inc.
- *
- * Released under GNU GPL
+ * Copyright (C) 2001-2002 Lauris Kaplinski
+ * Copyright (C) 2001 Ximian, Inc.
  *
+ * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
 #define NR_TYPE_ARENA_SHAPE (nr_arena_shape_get_type ())
@@ -31,6 +31,7 @@ struct _NRArenaShape {
 	SPStyle *style;
 	ArtDRect paintbox;
 	/* State data */
+	NRMatrixD ctm;
 	SPPainter *fill_painter;
 	SPPainter *stroke_painter;
 	ArtSVP *fill_svp;
