@@ -25,12 +25,6 @@ typedef enum {
 	SPXMaxYMax
 } SPAspect;
 
-#define SP_TYPE_DOCUMENT (sp_document_get_type ())
-#define SP_DOCUMENT(obj) (GTK_CHECK_CAST ((obj), SP_TYPE_DOCUMENT, SPDocument))
-#define SP_DOCUMENT_CLASS(klass) (GTK_CHECK_CLASS_CAST ((klass), SP_TYPE_DOCUMENT, SPItemClass))
-#define SP_IS_DOCUMENT(obj) (GTK_CHECK_TYPE ((obj), SP_TYPE_DOCUMENT))
-#define SP_IS_DOCUMENT_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), SP_TYPE_DOCUMENT))
-
 #include <gtk/gtktypeutils.h>
 #include <gtk/gtkobject.h>
 #include <libart_lgpl/art_rect.h>
@@ -42,10 +36,16 @@ typedef struct _SPDocumentPrivate SPDocumentPrivate;
 struct _SPDocument {
 	GtkObject object;
 
+	guint public : 1;
+
 	SPReprDoc *rdoc; /* Our SPReprDoc */
 	SPRepr *rroot; /* Root element of SPReprDoc */
 	SPObject *root; /* Our SPRoot */
 
+	guchar *uri; /* URI string or NULL */
+	guchar *base;
+	guchar *name;
+
 	/* fixme: remove this */
 	SPDocumentPrivate * private;
 
@@ -63,16 +63,13 @@ struct _SPDocumentClass {
 	void (* resized) (SPDocument *document, gdouble width, gdouble height);
 };
 
-GtkType sp_document_get_type (void);
-
 /*
- * Constructor
- *
  * Fetches document from URI, or creates new, if NULL
+ * Public document appear in document list
  */
 
-SPDocument * sp_document_new (const gchar * uri);
-SPDocument * sp_document_new_from_mem (const gchar * buffer, gint length);
+SPDocument *sp_document_new (const gchar *uri, gboolean public);
+SPDocument *sp_document_new_from_mem (const gchar *buffer, gint length, gboolean public);
 
 SPDocument *sp_document_ref (SPDocument *doc);
 SPDocument *sp_document_unref (SPDocument *doc);
@@ -84,13 +81,13 @@ SPDocument *sp_document_unref (SPDocument *doc);
 #define sp_document_repr_doc(d) (SP_DOCUMENT (d)->rdoc)
 #define sp_document_repr_root(d) (SP_DOCUMENT (d)->rroot)
 #define sp_document_root(d) (SP_DOCUMENT (d)->root)
-#define SP_DOCUMENT_ROOT sp_document_root
+#define SP_DOCUMENT_ROOT(d)  (SP_DOCUMENT (d)->root)
 
 gdouble sp_document_width (SPDocument * document);
 gdouble sp_document_height (SPDocument * document);
-#define SP_DOCUMENT_URI(d) sp_document_uri (d)
-const gchar * sp_document_uri (SPDocument * document);
-const gchar * sp_document_base (SPDocument * document);
+#define SP_DOCUMENT_URI(d) (SP_DOCUMENT (d)->uri)
+#define SP_DOCUMENT_NAME(d) (SP_DOCUMENT (d)->name)
+#define SP_DOCUMENT_BASE(d) (SP_DOCUMENT (d)->base)
 
 const GSList * sp_document_namedview_list (SPDocument * document);
 SPNamedView * sp_document_namedview (SPDocument * document, const gchar * name);
