@@ -20,6 +20,7 @@
 #include <string.h>
 #include <glib.h>
 #include <gdk/gdkkeysyms.h>
+#include <gtk/gtksignal.h>
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
 #include "xml/repr.h"
@@ -44,7 +45,7 @@
 
 #define TOLERANCE 1.0
 
-#define SPDC_EVENT_MASK (GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK)
+#define SPDC_EVENT_MASK (GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK)
 
 /* Drawing anchors */
 
@@ -54,7 +55,7 @@ struct _SPDrawAnchor {
 	guint start : 1;
 	guint active : 1;
 	ArtPoint dp, wp;
-	GnomeCanvasItem *ctrl;
+	SPCanvasItem *ctrl;
 };
 
 static void sp_draw_context_class_init (SPDrawContextClass *klass);
@@ -150,9 +151,11 @@ sp_draw_context_finalize (GtkObject *object)
 
 	dc = SP_DRAW_CONTEXT (object);
 
-	if (dc->grab) gnome_canvas_item_ungrab (dc->grab, gdk_time_get ());
+	if (dc->grab) sp_canvas_item_ungrab (dc->grab, gdk_time_get ());
 
-	gtk_signal_disconnect_by_data (GTK_OBJECT (dc->selection), dc);
+	if (dc->selection) {
+		gtk_signal_disconnect_by_data (GTK_OBJECT (dc->selection), dc);
+	}
 
 	spdc_free_colors (dc);
 
@@ -205,7 +208,7 @@ sp_draw_context_finish (SPEventContext *ec)
 	dc = SP_DRAW_CONTEXT (ec);
 
 	if (dc->grab) {
-		gnome_canvas_item_ungrab (dc->grab, gdk_time_get ());
+		sp_canvas_item_ungrab (dc->grab, gdk_time_get ());
 	}
 
 	if (dc->selection) {
@@ -511,7 +514,7 @@ spdc_flush_white (SPDrawContext *dc, SPCurve *gc)
 		c = sp_curve_transform (c, d2item);
 	} else {
 		gdouble d2item[6];
-		sp_desktop_d2doc_affine (SP_EVENT_CONTEXT_DESKTOP (dc), d2item);
+		sp_desktop_dt2root_affine (SP_EVENT_CONTEXT_DESKTOP (dc), (NRMatrixD *) d2item);
 		c = sp_curve_transform (c, d2item);
 	}
 
@@ -607,7 +610,7 @@ fit_and_split (SPDrawContext * dc)
 		sp_canvas_bpath_set_bpath (SP_CANVAS_BPATH (dc->red_bpath), dc->red_curve);
 	} else {
 		SPCurve *curve;
-		GnomeCanvasItem *cshape;
+		SPCanvasItem *cshape;
 		/* Fit and draw and copy last point */
 		g_assert (!sp_curve_empty (dc->red_curve));
 		sp_curve_append_continuous (dc->green_curve, dc->red_curve, 1e-9);
@@ -719,6 +722,7 @@ static SPDrawAnchor *
 sp_draw_anchor_new (SPDrawContext *dc, SPCurve *curve, gboolean start, gdouble dx, gdouble dy)
 {
 	SPDrawAnchor *a;
+	NRPointF fp;
 
 	a = g_new (SPDrawAnchor, 1);
 
@@ -728,8 +732,10 @@ sp_draw_anchor_new (SPDrawContext *dc, SPCurve *curve, gboolean start, gdouble d
 	a->active = FALSE;
 	a->dp.x = dx;
 	a->dp.y = dy;
-	sp_desktop_d2w_xy_point (SP_EVENT_CONTEXT_DESKTOP (dc), &a->wp, dx, dy);
-	a->ctrl = gnome_canvas_item_new (SP_DT_CONTROLS (SP_EVENT_CONTEXT_DESKTOP (dc)), SP_TYPE_CTRL,
+	sp_desktop_d2w_xy_point (SP_EVENT_CONTEXT_DESKTOP (dc), &fp, dx, dy);
+	a->wp.x = fp.x;
+	a->wp.y = fp.y;
+	a->ctrl = sp_canvas_item_new (SP_DT_CONTROLS (SP_EVENT_CONTEXT_DESKTOP (dc)), SP_TYPE_CTRL,
 					 "size", 4.0,
 					 "filled", 0,
 					 "fill_color", 0xff00007f,
@@ -759,14 +765,14 @@ sp_draw_anchor_test (SPDrawAnchor *anchor, gdouble wx, gdouble wy, gboolean acti
 {
 	if (activate && (fabs (wx - anchor->wp.x) <= A_SNAP) && (fabs (wy - anchor->wp.y) <= A_SNAP)) {
 		if (!anchor->active) {
-			gnome_canvas_item_set (anchor->ctrl, "filled", TRUE, NULL);
+			sp_canvas_item_set ((GtkObject *) anchor->ctrl, "filled", TRUE, NULL);
 			anchor->active = TRUE;
 		}
 		return anchor;
 	}
 
 	if (anchor->active) {
-		gnome_canvas_item_set (anchor->ctrl, "filled", FALSE, NULL);
+		sp_canvas_item_set ((GtkObject *) anchor->ctrl, "filled", FALSE, NULL);
 		anchor->active = FALSE;
 	}
 	return NULL;
@@ -848,6 +854,7 @@ sp_pencil_context_root_handler (SPEventContext *ec, GdkEvent *event)
 	ArtPoint p;
 	gint ret;
 	SPDrawAnchor *anchor;
+	NRPointF fp;
 
 	dc = SP_DRAW_CONTEXT (ec);
 	pc = SP_PENCIL_CONTEXT (ec);
@@ -858,11 +865,16 @@ sp_pencil_context_root_handler (SPEventContext *ec, GdkEvent *event)
 	switch (event->type) {
 	case GDK_BUTTON_PRESS:
 		if (event->button.button == 1) {
+			NRPointF fp;
+#if 0
 			/* Grab mouse, so release will not pass unnoticed */
-			dc->grab = GNOME_CANVAS_ITEM (dt->acetate);
-			gnome_canvas_item_grab (dc->grab, SPDC_EVENT_MASK, NULL, event->button.time);
+			dc->grab = SP_CANVAS_ITEM (dt->acetate);
+			sp_canvas_item_grab (dc->grab, SPDC_EVENT_MASK, NULL, event->button.time);
+#endif
 			/* Find desktop coordinates */
-			sp_desktop_w2d_xy_point (dt, &p, event->button.x, event->button.y);
+			sp_desktop_w2d_xy_point (dt, &fp, event->button.x, event->button.y);
+			p.x = fp.x;
+			p.y = fp.y;
 			/* Test, whether we hit any anchor */
 			anchor = test_inside (dc, event->button.x, event->button.y);
 
@@ -884,8 +896,17 @@ sp_pencil_context_root_handler (SPEventContext *ec, GdkEvent *event)
 		}
 		break;
 	case GDK_MOTION_NOTIFY:
+#if 1
+		if ((event->motion.state & GDK_BUTTON1_MASK) && !dc->grab) {
+			/* Grab mouse, so release will not pass unnoticed */
+			dc->grab = SP_CANVAS_ITEM (dt->acetate);
+			sp_canvas_item_grab (dc->grab, SPDC_EVENT_MASK, NULL, event->button.time);
+		}
+#endif
 		/* Find desktop coordinates */
-		sp_desktop_w2d_xy_point (dt, &p, event->motion.x, event->motion.y);
+		sp_desktop_w2d_xy_point (dt, &fp, event->motion.x, event->motion.y);
+		p.x = fp.x;
+		p.y = fp.y;
 		/* Test, whether we hit any anchor */
 		anchor = test_inside (dc, event->button.x, event->button.y);
 
@@ -920,8 +941,11 @@ sp_pencil_context_root_handler (SPEventContext *ec, GdkEvent *event)
 		break;
 	case GDK_BUTTON_RELEASE:
 		if (event->button.button == 1) {
+			NRPointF fp;
 			/* Find desktop coordinates */
-			sp_desktop_w2d_xy_point (dt, &p, event->motion.x, event->motion.y);
+			sp_desktop_w2d_xy_point (dt, &fp, event->motion.x, event->motion.y);
+			p.x = fp.x;
+			p.y = fp.y;
 			/* Test, whether we hit any anchor */
 			anchor = test_inside (dc, event->button.x, event->button.y);
 
@@ -952,6 +976,8 @@ sp_pencil_context_root_handler (SPEventContext *ec, GdkEvent *event)
 				/* Write curves to object */
 				g_print ("Finishing freehand\n");
 				spdc_concat_colors_and_flush (dc, FALSE);
+				dc->sa = NULL;
+				dc->ea = NULL;
 				if (dc->green_anchor) {
 					dc->green_anchor = sp_draw_anchor_destroy (dc->green_anchor);
 				}
@@ -961,8 +987,13 @@ sp_pencil_context_root_handler (SPEventContext *ec, GdkEvent *event)
 			default:
 				break;
 			}
-			/* Release grab now */
-			gnome_canvas_item_ungrab (dc->grab, event->button.time);
+#if 1
+			if (dc->grab) {
+				/* Release grab now */
+				sp_canvas_item_ungrab (dc->grab, event->button.time);
+				dc->grab = NULL;
+			}
+#endif
 			dc->grab = NULL;
 			ret = TRUE;
 		}
@@ -1065,6 +1096,8 @@ spdc_finish_endpoint (SPPencilContext *pc, ArtPoint *p, gboolean snap, guint sta
 			/* Write curves to object */
 			g_print ("Finishing real red curve\n");
 			spdc_concat_colors_and_flush (dc, FALSE);
+			dc->sa = NULL;
+			dc->ea = NULL;
 		}
 	}
 }
@@ -1181,19 +1214,19 @@ sp_pen_context_setup (SPEventContext *ec)
 		SP_EVENT_CONTEXT_CLASS (pen_parent_class)->setup (ec);
 
 	/* Pen indicators */
-	pc->c0 = gnome_canvas_item_new (SP_DT_CONTROLS (SP_EVENT_CONTEXT_DESKTOP (ec)), SP_TYPE_CTRL, "shape", SP_CTRL_SHAPE_CIRCLE,
+	pc->c0 = sp_canvas_item_new (SP_DT_CONTROLS (SP_EVENT_CONTEXT_DESKTOP (ec)), SP_TYPE_CTRL, "shape", SP_CTRL_SHAPE_CIRCLE,
+				     "size", 4.0, "filled", 0, "fill_color", 0xff00007f, "stroked", 1, "stroke_color", 0x0000ff7f, NULL);
+	pc->c1 = sp_canvas_item_new (SP_DT_CONTROLS (SP_EVENT_CONTEXT_DESKTOP (ec)), SP_TYPE_CTRL, "shape", SP_CTRL_SHAPE_CIRCLE,
 					"size", 4.0, "filled", 0, "fill_color", 0xff00007f, "stroked", 1, "stroke_color", 0x0000ff7f, NULL);
-	pc->c1 = gnome_canvas_item_new (SP_DT_CONTROLS (SP_EVENT_CONTEXT_DESKTOP (ec)), SP_TYPE_CTRL, "shape", SP_CTRL_SHAPE_CIRCLE,
-					"size", 4.0, "filled", 0, "fill_color", 0xff00007f, "stroked", 1, "stroke_color", 0x0000ff7f, NULL);
-	pc->cl0 = gnome_canvas_item_new (SP_DT_CONTROLS (SP_EVENT_CONTEXT_DESKTOP (ec)), SP_TYPE_CTRLLINE, NULL);
+	pc->cl0 = sp_canvas_item_new (SP_DT_CONTROLS (SP_EVENT_CONTEXT_DESKTOP (ec)), SP_TYPE_CTRLLINE, NULL);
 	sp_ctrlline_set_rgba32 (SP_CTRLLINE (pc->cl0), 0x0000007f);
-	pc->cl1 = gnome_canvas_item_new (SP_DT_CONTROLS (SP_EVENT_CONTEXT_DESKTOP (ec)), SP_TYPE_CTRLLINE, NULL);
+	pc->cl1 = sp_canvas_item_new (SP_DT_CONTROLS (SP_EVENT_CONTEXT_DESKTOP (ec)), SP_TYPE_CTRLLINE, NULL);
 	sp_ctrlline_set_rgba32 (SP_CTRLLINE (pc->cl1), 0x0000007f);
 
-	gnome_canvas_item_hide (pc->c0);
-	gnome_canvas_item_hide (pc->c1);
-	gnome_canvas_item_hide (pc->cl0);
-	gnome_canvas_item_hide (pc->cl1);
+	sp_canvas_item_hide (pc->c0);
+	sp_canvas_item_hide (pc->c1);
+	sp_canvas_item_hide (pc->cl0);
+	sp_canvas_item_hide (pc->cl1);
 
 	sp_event_context_read (ec, "mode");
 }
@@ -1202,6 +1235,9 @@ static void
 sp_pen_context_finish (SPEventContext *ec)
 {
 	spdc_pen_finish (SP_PEN_CONTEXT (ec), FALSE);
+
+	if (SP_EVENT_CONTEXT_CLASS (pen_parent_class)->finish)
+		SP_EVENT_CONTEXT_CLASS (pen_parent_class)->finish (ec);
 }
 
 static void
@@ -1229,6 +1265,7 @@ sp_pen_context_root_handler (SPEventContext *ec, GdkEvent *event)
 	ArtPoint p;
 	gint ret;
 	SPDrawAnchor *anchor;
+	NRPointF fp;
 
 	dc = SP_DRAW_CONTEXT (ec);
 	pc = SP_PEN_CONTEXT (ec);
@@ -1239,11 +1276,15 @@ sp_pen_context_root_handler (SPEventContext *ec, GdkEvent *event)
 	switch (event->type) {
 	case GDK_BUTTON_PRESS:
 		if (event->button.button == 1) {
+#if 0
 			/* Grab mouse, so release will not pass unnoticed */
-			dc->grab = GNOME_CANVAS_ITEM (dt->acetate);
-			gnome_canvas_item_grab (dc->grab, SPDC_EVENT_MASK, NULL, event->button.time);
+			dc->grab = SP_CANVAS_ITEM (dt->acetate);
+			sp_canvas_item_grab (dc->grab, SPDC_EVENT_MASK, NULL, event->button.time);
+#endif
 			/* Find desktop coordinates */
-			sp_desktop_w2d_xy_point (dt, &p, event->button.x, event->button.y);
+			sp_desktop_w2d_xy_point (dt, &fp, event->button.x, event->button.y);
+			p.x = fp.x;
+			p.y = fp.y;
 			/* Test, whether we hit any anchor */
 			anchor = test_inside (dc, event->button.x, event->button.y);
 
@@ -1308,8 +1349,17 @@ sp_pen_context_root_handler (SPEventContext *ec, GdkEvent *event)
 		}
 		break;
 	case GDK_MOTION_NOTIFY:
+#if 1
+		if ((event->motion.state & GDK_BUTTON1_MASK) && !dc->grab) {
+			/* Grab mouse, so release will not pass unnoticed */
+			dc->grab = SP_CANVAS_ITEM (dt->acetate);
+			sp_canvas_item_grab (dc->grab, SPDC_EVENT_MASK, NULL, event->button.time);
+		}
+#endif
 		/* Find desktop coordinates */
-		sp_desktop_w2d_xy_point (dt, &p, event->motion.x, event->motion.y);
+		sp_desktop_w2d_xy_point (dt, &fp, event->motion.x, event->motion.y);
+		p.x = fp.x;
+		p.y = fp.y;
 		/* Test, whether we hit any anchor */
 		anchor = test_inside (dc, event->button.x, event->button.y);
 
@@ -1369,7 +1419,9 @@ sp_pen_context_root_handler (SPEventContext *ec, GdkEvent *event)
 	case GDK_BUTTON_RELEASE:
 		if (event->button.button == 1) {
 			/* Find desktop coordinates */
-			sp_desktop_w2d_xy_point (dt, &p, event->motion.x, event->motion.y);
+			sp_desktop_w2d_xy_point (dt, &fp, event->motion.x, event->motion.y);
+			p.x = fp.x;
+			p.y = fp.y;
 			/* Test, whether we hit any anchor */
 			anchor = test_inside (dc, event->button.x, event->button.y);
 
@@ -1435,12 +1487,20 @@ sp_pen_context_root_handler (SPEventContext *ec, GdkEvent *event)
 			default:
 				break;
 			}
-			/* Release grab now */
-			gnome_canvas_item_ungrab (dc->grab, event->button.time);
-			dc->grab = NULL;
+#if 1
+			if (dc->grab) {
+				/* Release grab now */
+				sp_canvas_item_ungrab (dc->grab, event->button.time);
+				dc->grab = NULL;
+			}
+#endif
 			ret = TRUE;
 		}
 		break;
+	case GDK_2BUTTON_PRESS:
+		spdc_pen_finish (pc, FALSE);
+		ret = TRUE;
+		break;
 	case GDK_KEY_PRESS:
 		/* fixme: */
 		switch (event->key.keyval) {
@@ -1451,10 +1511,10 @@ sp_pen_context_root_handler (SPEventContext *ec, GdkEvent *event)
 		case GDK_Escape:
 			pc->state = SP_PEN_CONTEXT_STOP;
 			spdc_reset_colors (dc);
-			gnome_canvas_item_hide (pc->c0);
-			gnome_canvas_item_hide (pc->c1);
-			gnome_canvas_item_hide (pc->cl0);
-			gnome_canvas_item_hide (pc->cl1);
+			sp_canvas_item_hide (pc->c0);
+			sp_canvas_item_hide (pc->c1);
+			sp_canvas_item_hide (pc->cl0);
+			sp_canvas_item_hide (pc->cl1);
 			ret = TRUE;
 			break;
 		case GDK_BackSpace:
@@ -1462,10 +1522,10 @@ sp_pen_context_root_handler (SPEventContext *ec, GdkEvent *event)
 				/* Same as cancel */
 				pc->state = SP_PEN_CONTEXT_STOP;
 				spdc_reset_colors (dc);
-				gnome_canvas_item_hide (pc->c0);
-				gnome_canvas_item_hide (pc->c1);
-				gnome_canvas_item_hide (pc->cl0);
-				gnome_canvas_item_hide (pc->cl1);
+				sp_canvas_item_hide (pc->c0);
+				sp_canvas_item_hide (pc->c1);
+				sp_canvas_item_hide (pc->cl0);
+				sp_canvas_item_hide (pc->cl1);
 				ret = TRUE;
 				break;
 			} else {
@@ -1497,10 +1557,10 @@ sp_pen_context_root_handler (SPEventContext *ec, GdkEvent *event)
 				}
 				dc->npoints = 2;
 				sp_curve_backspace (dc->green_curve);
-				gnome_canvas_item_hide (pc->c0);
-				gnome_canvas_item_hide (pc->c1);
-				gnome_canvas_item_hide (pc->cl0);
-				gnome_canvas_item_hide (pc->cl1);
+				sp_canvas_item_hide (pc->c0);
+				sp_canvas_item_hide (pc->c1);
+				sp_canvas_item_hide (pc->cl0);
+				sp_canvas_item_hide (pc->cl1);
 				pc->state = SP_PEN_CONTEXT_POINT;
 				spdc_pen_set_point (pc, &pt, event->motion.state);
 				ret = TRUE;
@@ -1554,19 +1614,19 @@ spdc_pen_set_ctrl (SPPenContext *pc, ArtPoint *p, guint state)
 
 	dc = SP_DRAW_CONTEXT (pc);
 
-	gnome_canvas_item_show (pc->c1);
-	gnome_canvas_item_show (pc->cl1);
+	sp_canvas_item_show (pc->c1);
+	sp_canvas_item_show (pc->cl1);
 
 	if (dc->npoints == 2) {
 		dc->p[1] = *p;
-		gnome_canvas_item_hide (pc->c0);
-		gnome_canvas_item_hide (pc->cl0);
+		sp_canvas_item_hide (pc->c0);
+		sp_canvas_item_hide (pc->cl0);
 		sp_ctrl_moveto (SP_CTRL (pc->c1), dc->p[1].x, dc->p[1].y);
 		sp_ctrlline_set_coords (SP_CTRLLINE (pc->cl1), dc->p[0].x, dc->p[0].y, dc->p[1].x, dc->p[1].y);
 	} else if (dc->npoints == 5) {
 		dc->p[4] = *p;
-		gnome_canvas_item_show (pc->c0);
-		gnome_canvas_item_show (pc->cl0);
+		sp_canvas_item_show (pc->c0);
+		sp_canvas_item_show (pc->cl0);
 		if (((pc->mode == SP_PEN_CONTEXT_MODE_CLICK) && (state & GDK_CONTROL_MASK)) ||
 		    ((pc->mode == SP_PEN_CONTEXT_MODE_DRAG) && !(state & GDK_MOD1_MASK))) {
 			gdouble dx, dy;
@@ -1597,7 +1657,7 @@ spdc_pen_finish_segment (SPPenContext *pc, ArtPoint *p, guint state)
 
 	if (!sp_curve_empty (dc->red_curve)) {
 		SPCurve *curve;
-		GnomeCanvasItem *cshape;
+		SPCanvasItem *cshape;
 
 		sp_curve_append_continuous (dc->green_curve, dc->red_curve, 1e-9);
 		curve = sp_curve_copy (dc->red_curve);
@@ -1627,14 +1687,16 @@ spdc_pen_finish (SPPenContext *pc, gboolean closed)
 
 	sp_curve_reset (dc->red_curve);
 	spdc_concat_colors_and_flush (dc, closed);
+	dc->sa = NULL;
+	dc->ea = NULL;
 
 	dc->npoints = 0;
 	pc->state = SP_PEN_CONTEXT_POINT;
 
-	gnome_canvas_item_hide (pc->c0);
-	gnome_canvas_item_hide (pc->c1);
-	gnome_canvas_item_hide (pc->cl0);
-	gnome_canvas_item_hide (pc->cl1);
+	sp_canvas_item_hide (pc->c0);
+	sp_canvas_item_hide (pc->c1);
+	sp_canvas_item_hide (pc->cl0);
+	sp_canvas_item_hide (pc->cl1);
 
 	if (dc->green_anchor) {
 		dc->green_anchor = sp_draw_anchor_destroy (dc->green_anchor);
