@@ -36,6 +36,12 @@
 #include <glib.h>
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
+#include <gtk/gtkmain.h>
+#include <gtk/gtksignal.h>
+#include <gtk/gtktable.h>
+#include <gtk/gtkbutton.h>
+#include <gtk/gtkspinbutton.h>
+#include <gtk/gtklabel.h>
 #include "svg/svg.h"
 #include "helper/curve.h"
 #include "helper/canvas-bpath.h"
@@ -65,6 +71,10 @@
 #define DYNA_MIN_WIDTH 1.0
 #endif
 
+#define DRAG_MIN 0.0
+#define DRAG_DEFAULT 0.5
+#define DRAG_MAX 1.0
+
 static void sp_dyna_draw_context_class_init (SPDynaDrawContextClass *klass);
 static void sp_dyna_draw_context_init (SPDynaDrawContext *ddc);
 static void sp_dyna_draw_context_destroy (GtkObject *object);
@@ -168,12 +178,12 @@ sp_dyna_draw_context_init (SPDynaDrawContext *ddc)
 
 #ifdef NORMALIZED_COORDINATE
 	ddc->mass = 0.3;
-	ddc->drag = 0.5;
+	ddc->drag = DRAG_DEFAULT;
 	ddc->angle = 30.0;
 	ddc->width = 0.2;
 #else
 	ddc->mass = 0.2;
-	ddc->drag = 0.5;
+	ddc->drag = DRAG_DEFAULT;
 	ddc->angle = 30.0;
 	ddc->width = 0.1;
 #endif
@@ -222,7 +232,7 @@ sp_dyna_draw_context_setup (SPEventContext *ec)
 	ddc->cal2 = sp_curve_new_sized (32);
 
 	/* style should be changed when dc->use_calligraphc is touched */  
-	ddc->currentshape = gnome_canvas_item_new (SP_DT_SKETCH (ec->desktop), SP_TYPE_CANVAS_BPATH, NULL);
+	ddc->currentshape = sp_canvas_item_new (SP_DT_SKETCH (ec->desktop), SP_TYPE_CANVAS_BPATH, NULL);
 	sp_canvas_bpath_set_fill (SP_CANVAS_BPATH (ddc->currentshape), DDC_RED_RGBA, ART_WIND_RULE_ODDEVEN);
 	sp_canvas_bpath_set_stroke (SP_CANVAS_BPATH (ddc->currentshape), 0x00000000, 1.0, ART_PATH_STROKE_JOIN_MITER, ART_PATH_STROKE_CAP_BUTT);
 	/* fixme: Cannot we cascade it to root more clearly? */
@@ -246,8 +256,8 @@ sp_dyna_draw_context_set (SPEventContext *ec, const guchar *key, const guchar *v
 		dval = (val) ? atof (val) : 0.2;
 		ddc->mass = CLAMP (dval, -1000.0, 1000.0);
 	} else if (!strcmp (key, "drag")) {
-		dval = (val) ? atof (val) : 0.5;
-		ddc->drag = CLAMP (dval, -1000.0, 1000.0);
+		dval = (val) ? atof (val) : DRAG_DEFAULT;
+		ddc->drag = CLAMP (dval, DRAG_MIN, DRAG_MAX);
 	} else if (!strcmp (key, "angle")) {
 		dval = (val) ? atof (val) : 0.5;
 		dval = fmod (dval, 360.0);
@@ -458,20 +468,23 @@ sp_dyna_draw_timeout_handler (gpointer data)
 {
   SPDynaDrawContext *dc;
   SPDesktop *desktop;
-  GnomeCanvas *canvas;
+  SPCanvas *canvas;
   gint x, y;
   ArtPoint p;
+  NRPointF fp;
 
   dc = SP_DYNA_DRAW_CONTEXT (data);
   desktop = SP_EVENT_CONTEXT(dc)->desktop;
-  canvas = GNOME_CANVAS (SP_DT_CANVAS (desktop));
+  canvas = SP_CANVAS (SP_DT_CANVAS (desktop));
 
   dc->dragging = TRUE;
   dc->dynahand = TRUE;
   
   gtk_widget_get_pointer (GTK_WIDGET(canvas), &x, &y);
-  gnome_canvas_window_to_world (canvas, (double)x, (double)y, &p.x, &p.y);
-  sp_desktop_w2d_xy_point (desktop, &p, p.x, p.y);
+  sp_canvas_window_to_world (canvas, (double)x, (double)y, &p.x, &p.y);
+  sp_desktop_w2d_xy_point (desktop, &fp, p.x, p.y);
+  p.x = fp.x;
+  p.y = fp.y;
 /*    g_print ("(%d %d)=>(%g %g)\n", x, y, p.x, p.y); */
   if (! sp_dyna_draw_apply (dc, p.x, p.y))
     return TRUE;
@@ -492,144 +505,135 @@ gint
 sp_dyna_draw_context_root_handler (SPEventContext * event_context,
 				   GdkEvent * event)
 {
-  SPDynaDrawContext *dc;
-  SPDesktop *desktop;
-  ArtPoint p;
-  gint ret;
+	SPDynaDrawContext *dc;
+	SPDesktop *desktop;
+	ArtPoint p;
+	gint ret;
+	NRPointF fp;
 
-  dc = SP_DYNA_DRAW_CONTEXT (event_context);
-  desktop = event_context->desktop;
+	dc = SP_DYNA_DRAW_CONTEXT (event_context);
+	desktop = event_context->desktop;
 
-  ret = FALSE;
+	ret = FALSE;
 
-  switch (event->type)
-    {
-    case GDK_BUTTON_PRESS:
-      if (event->button.button == 1)
-	{
-	  sp_desktop_w2d_xy_point (desktop, &p, event->button.x,
-				   event->button.y);
-	  sp_dyna_draw_reset (dc, p.x, p.y);
-          sp_dyna_draw_apply (dc, p.x, p.y);
-          sp_dyna_draw_get_curr_vpoint (dc, &p.x, &p.y);
+	switch (event->type) {
+	case GDK_BUTTON_PRESS:
+		if (event->button.button == 1) {
+			sp_desktop_w2d_xy_point (desktop, &fp, event->button.x, event->button.y);
+			p.x = fp.x;
+			p.y = fp.y;
+			sp_dyna_draw_reset (dc, p.x, p.y);
+			sp_dyna_draw_apply (dc, p.x, p.y);
+			sp_dyna_draw_get_curr_vpoint (dc, &p.x, &p.y);
 
-	  sp_desktop_free_snap (desktop, &p);
-	  sp_curve_reset (dc->accumulated);
-	  if (dc->repr) {
-		  dc->repr = NULL;
-	  }
+			sp_desktop_free_snap (desktop, &p);
+			sp_curve_reset (dc->accumulated);
+			if (dc->repr) {
+				dc->repr = NULL;
+			}
 
-	  /* initialize first point */
-          dc->npoints = 0;
+			/* initialize first point */
+			dc->npoints = 0;
          
-          if (dc->use_timeout)
-            gnome_canvas_item_grab (GNOME_CANVAS_ITEM (desktop->acetate),
-                                    GDK_BUTTON_RELEASE_MASK |
-                                    GDK_BUTTON_PRESS_MASK, NULL,
-                                    event->button.time);
-          else
-            gnome_canvas_item_grab (GNOME_CANVAS_ITEM (desktop->acetate),
-                                    GDK_BUTTON_RELEASE_MASK |
-                                    GDK_POINTER_MOTION_MASK |
-                                    GDK_BUTTON_PRESS_MASK, NULL,
-                                    event->button.time);
+			if (dc->use_timeout) {
+				sp_canvas_item_grab (SP_CANVAS_ITEM (desktop->acetate),
+							GDK_BUTTON_RELEASE_MASK |
+							GDK_BUTTON_PRESS_MASK, NULL,
+							event->button.time);
+			} else {
+				sp_canvas_item_grab (SP_CANVAS_ITEM (desktop->acetate),
+							GDK_BUTTON_RELEASE_MASK |
+							GDK_POINTER_MOTION_MASK |
+							GDK_BUTTON_PRESS_MASK, NULL,
+							event->button.time);
+			}
 
-          if (dc->use_timeout && !dc->timer_id)
-            dc->timer_id = gtk_timeout_add (SAMPLE_TIMEOUT, sp_dyna_draw_timeout_handler, dc);
-	  ret = TRUE;
-	}
-      break;
-    case GDK_MOTION_NOTIFY:
-      if (!dc->use_timeout && (event->motion.state & GDK_BUTTON1_MASK))
-	{
-	  dc->dragging = TRUE;
-	  dc->dynahand = TRUE;
+			if (dc->use_timeout && !dc->timer_id) {
+				dc->timer_id = gtk_timeout_add (SAMPLE_TIMEOUT, sp_dyna_draw_timeout_handler, dc);
+			}
+			ret = TRUE;
+		}
+		break;
+	case GDK_MOTION_NOTIFY:
+		if (!dc->use_timeout && (event->motion.state & GDK_BUTTON1_MASK)) {
+			NRPointF fp;
 
-          sp_desktop_w2d_xy_point (desktop, &p, event->motion.x,
-                                   event->motion.y);
+			dc->dragging = TRUE;
+			dc->dynahand = TRUE;
+
+			sp_desktop_w2d_xy_point (desktop, &fp, event->motion.x, event->motion.y);
+			p.x = fp.x;
+			p.y = fp.y;
 #if 0
-          g_print ("(%g %g)=>(%g %g)\n", event->motion.x, event->motion.y, p.x, p.y);
+			g_print ("(%g %g)=>(%g %g)\n", event->motion.x, event->motion.y, p.x, p.y);
 #endif
-	  if (! sp_dyna_draw_apply (dc, p.x, p.y))
-            {
-              ret = TRUE;
-              break;
-            }
-          sp_dyna_draw_get_curr_vpoint (dc, &p.x, &p.y);
+			if (! sp_dyna_draw_apply (dc, p.x, p.y)) {
+				ret = TRUE;
+				break;
+			}
+			sp_dyna_draw_get_curr_vpoint (dc, &p.x, &p.y);
 
-	  sp_desktop_free_snap (desktop, &p);
+			sp_desktop_free_snap (desktop, &p);
 
-	  if ((dc->curx != dc->lastx) || (dc->cury != dc->lasty))
-	    {
-              sp_dyna_draw_brush (dc);
-              g_assert (dc->npoints > 0);
-              fit_and_split (dc, FALSE);
-	    }
-	  ret = TRUE;
-	}
-      break;
+			if ((dc->curx != dc->lastx) || (dc->cury != dc->lasty)) {
+				sp_dyna_draw_brush (dc);
+				g_assert (dc->npoints > 0);
+				fit_and_split (dc, FALSE);
+			}
+			ret = TRUE;
+		}
+		break;
 
-    case GDK_BUTTON_RELEASE:
-      if (event->button.button == 1 &&
-          dc->use_timeout && dc->timer_id != 0)
-        {
-          gtk_timeout_remove (dc->timer_id);
-          dc->timer_id = 0;
-        }
-      if (dc->dragging && event->button.button == 1)
-	{
-	  dc->dragging = FALSE;
+	case GDK_BUTTON_RELEASE:
+		if (event->button.button == 1 &&
+		    dc->use_timeout && dc->timer_id != 0) {
+			gtk_timeout_remove (dc->timer_id);
+			dc->timer_id = 0;
+		}
+		if (dc->dragging && event->button.button == 1) {
+			dc->dragging = FALSE;
 
 /*            g_print ("[release]\n"); */
-	  if (dc->dynahand)
-	    {
-	      dc->dynahand = FALSE;
-	      /* Remove all temporary line segments */
-	      while (dc->segments)
-		{
-		  gtk_object_destroy (GTK_OBJECT (dc->segments->data));
-		  dc->segments = g_slist_remove (dc->segments, dc->segments->data);
+			if (dc->dynahand) {
+				dc->dynahand = FALSE;
+				/* Remove all temporary line segments */
+				while (dc->segments) {
+					gtk_object_destroy (GTK_OBJECT (dc->segments->data));
+					dc->segments = g_slist_remove (dc->segments, dc->segments->data);
+				}
+				/* Create object */
+				fit_and_split (dc, TRUE);
+				if (dc->use_calligraphic) {
+					accumulate_calligraphic (dc);
+				} else {
+					concat_current_line (dc);
+				}
+				set_to_accumulated (dc); /* temporal implementation */
+				if (dc->use_calligraphic /* || dc->cinside*/) {
+					/* reset accumulated curve */
+					sp_curve_reset (dc->accumulated);
+					clear_current (dc);
+					if (dc->repr) {
+						dc->repr = NULL;
+					}
+				}
+				
+			}
+
+			sp_canvas_item_ungrab (SP_CANVAS_ITEM (desktop->acetate), event->button.time);
+			ret = TRUE;
 		}
-	      /* Create object */
-              fit_and_split (dc, TRUE);
-              if (dc->use_calligraphic)
-                {
-                  accumulate_calligraphic (dc);
-                }
-              else
-                {
-                  concat_current_line (dc);
-                }
-              set_to_accumulated (dc); /* temporal implementation */
-              if (dc->use_calligraphic /* || dc->cinside*/)
-                {
-                  /* reset accumulated curve */
-                  sp_curve_reset (dc->accumulated);
-                  clear_current (dc);
-                  if (dc->repr)
-		    {
-		      dc->repr = NULL;
-		    }
-                }
-              
-	    }
-
-	  gnome_canvas_item_ungrab (GNOME_CANVAS_ITEM (desktop->acetate),
-				    event->button.time);
-	  ret = TRUE;
+		break;
+	default:
+		break;
 	}
-      break;
-    default:
-      break;
-    }
 
-  if (!ret)
-    {
-      if (SP_EVENT_CONTEXT_CLASS (parent_class)->root_handler)
-	ret = SP_EVENT_CONTEXT_CLASS (parent_class)-> root_handler (event_context, event);
-    }
+	if (!ret) {
+		if (SP_EVENT_CONTEXT_CLASS (parent_class)->root_handler)
+			ret = SP_EVENT_CONTEXT_CLASS (parent_class)-> root_handler (event_context, event);
+	}
 
-  return ret;
+	return ret;
 }
 
 static void
@@ -688,7 +692,7 @@ set_to_accumulated (SPDynaDrawContext * dc)
 	  sp_repr_unref (dc->repr);
 	  sp_selection_set_repr (SP_DT_SELECTION (desktop), dc->repr);
 	}
-      sp_desktop_d2doc_affine (desktop, d2doc);
+      sp_desktop_dt2root_affine (desktop, (NRMatrixD *) d2doc);
       abp = art_bpath_affine_transform (sp_curve_first_bpath (dc->accumulated), d2doc);
       str = sp_svg_write_path (abp);
       g_assert (str != NULL);
@@ -807,7 +811,7 @@ fit_and_split_line (SPDynaDrawContext *dc,
 	else
 	{
 		SPCurve *curve;
-		GnomeCanvasItem *cbp;
+		SPCanvasItem *cbp;
 		/* Fit and draw and copy last point */
 #ifdef DYNA_DRAW_VERBOSE
 		g_print ("[%d]Yup\n", dc->npoints);
@@ -815,7 +819,7 @@ fit_and_split_line (SPDynaDrawContext *dc,
 		g_assert (!sp_curve_empty (dc->currentcurve));
 		concat_current_line (dc);
 
-		cbp = gnome_canvas_item_new (SP_DT_SKETCH (SP_EVENT_CONTEXT (dc)->desktop), SP_TYPE_CANVAS_BPATH, NULL);
+		cbp = sp_canvas_item_new (SP_DT_SKETCH (SP_EVENT_CONTEXT (dc)->desktop), SP_TYPE_CANVAS_BPATH, NULL);
 		curve = sp_curve_copy (dc->currentcurve);
 		sp_canvas_bpath_set_bpath (SP_CANVAS_BPATH (cbp), curve);
 		sp_curve_unref (curve);
@@ -934,11 +938,11 @@ fit_and_split_calligraphics (SPDynaDrawContext *dc, gboolean release)
 		g_print ("[%d]Yup\n", dc->npoints);
 #endif
 		if (!release) {
-			GnomeCanvasItem *cbp;
+			SPCanvasItem *cbp;
 
 			g_assert (!sp_curve_empty (dc->currentcurve));
 
-			cbp = gnome_canvas_item_new (SP_DT_SKETCH (SP_EVENT_CONTEXT (dc)->desktop), SP_TYPE_CANVAS_BPATH, NULL);
+			cbp = sp_canvas_item_new (SP_DT_SKETCH (SP_EVENT_CONTEXT (dc)->desktop), SP_TYPE_CANVAS_BPATH, NULL);
 			curve = sp_curve_copy (dc->currentcurve);
 			sp_canvas_bpath_set_bpath (SP_CANVAS_BPATH (cbp), curve);
 			sp_curve_unref (curve);
@@ -1009,7 +1013,7 @@ sp_ddc_defaults (GtkWidget *widget, GtkObject *obj)
 	adj = gtk_object_get_data (obj, "mass");
 	gtk_adjustment_set_value (adj, 0.3);
 	adj = gtk_object_get_data (obj, "drag");
-	gtk_adjustment_set_value (adj, 0.5);
+	gtk_adjustment_set_value (adj, DRAG_DEFAULT);
 	adj = gtk_object_get_data (obj, "angle");
 	gtk_adjustment_set_value (adj, 30.0);
 	adj = gtk_object_get_data (obj, "width");
@@ -1046,7 +1050,7 @@ sp_dyna_draw_context_config_widget (SPEventContext *ec)
 	gtk_widget_show (l);
 	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
 	gtk_table_attach (GTK_TABLE (tbl), l, 0, 1, 1, 2, 0, 0, 0, 0);
-	a = gtk_adjustment_new (ddc->drag, 0.0, 10.0, 0.01, 0.1, 0.1);
+	a = gtk_adjustment_new (ddc->drag, DRAG_MIN, DRAG_MAX, 0.01, 0.1, 0.1);
 	gtk_object_set_data (GTK_OBJECT (tbl), "drag", a);
 	sb = gtk_spin_button_new (GTK_ADJUSTMENT (a), 0.1, 2);
 	gtk_widget_show (sb);
