@@ -1,13 +1,16 @@
-#ifndef _FORWARD_H_
-#define _FORWARD_H_
+#ifndef __FORWARD_H__
+#define __FORWARD_H__
 
 /*
  * Forward declarations of most used objects
  *
- * Author: Lauris Kaplinski <lauris@ximian.com>
+ * Author:
+ *   Lauris Kaplinski <lauris@kaplinski.com>
  *
+ * Copyright (C) 2001-2002 Lauris Kaplinski
  * Copyright (C) 2001 Ximian, Inc.
  *
+ * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
 /* Generic containers */
@@ -15,9 +18,17 @@
 typedef struct _Sodipodi Sodipodi;
 typedef struct _SodipodiClass SodipodiClass;
 
+/* Editing window */
+
 typedef struct _SPDesktop SPDesktop;
 typedef struct _SPDesktopClass SPDesktopClass;
 
+#define SP_TYPE_DESKTOP (sp_desktop_get_type ())
+#define SP_DESKTOP(o) (GTK_CHECK_CAST ((o), SP_TYPE_DESKTOP, SPDesktop))
+#define SP_IS_DESKTOP(o) (GTK_CHECK_TYPE ((o), SP_TYPE_DESKTOP))
+
+unsigned int sp_desktop_get_type (void);
+
 typedef struct _SPSelection SPSelection;
 typedef struct _SPSelectionClass SPSelectionClass;
 
@@ -29,11 +40,23 @@ typedef struct _SPEventContextClass SPEventContextClass;
 typedef struct _SPDocument SPDocument;
 typedef struct _SPDocumentClass SPDocumentClass;
 
+#define SP_TYPE_DOCUMENT (sp_document_get_type ())
+#define SP_DOCUMENT(obj) (GTK_CHECK_CAST ((obj), SP_TYPE_DOCUMENT, SPDocument))
+#define SP_IS_DOCUMENT(obj) (GTK_CHECK_TYPE ((obj), SP_TYPE_DOCUMENT))
+
+unsigned int sp_document_get_type (void);
+
 /* Objects */
 
 typedef struct _SPObject SPObject;
 typedef struct _SPObjectClass SPObjectClass;
 
+#define SP_TYPE_OBJECT (sp_object_get_type ())
+#define SP_OBJECT(obj) (GTK_CHECK_CAST ((obj), SP_TYPE_OBJECT, SPObject))
+#define SP_IS_OBJECT(obj) (GTK_CHECK_TYPE ((obj), SP_TYPE_OBJECT))
+
+unsigned int sp_object_get_type (void);
+
 typedef struct _SPItem SPItem;
 typedef struct _SPItemClass SPItemClass;
 
@@ -64,6 +87,9 @@ typedef struct _SPPathClass SPPathClass;
 typedef struct _SPShape SPShape;
 typedef struct _SPShapeClass SPShapeClass;
 
+typedef struct _SPPolygon SPPolygon;
+typedef struct _SPPolygonClass SPPolygonClass;
+
 typedef struct _SPEllipse SPEllipse;
 typedef struct _SPEllipseClass SPEllipseClass;
 
@@ -100,6 +126,8 @@ typedef struct _SPLinearGradientClass SPLinearGradientClass;
 typedef struct _SPRadialGradient SPRadialGradient;
 typedef struct _SPRadialGradientClass SPRadialGradientClass;
 
+typedef struct _SPPattern SPPattern;
+
 typedef struct _SPClipPath SPClipPath;
 typedef struct _SPClipPathClass SPClipPathClass;
 
@@ -115,4 +143,6 @@ typedef struct _SPStyle SPStyle;
 
 typedef struct _SPEvent SPEvent;
 
+typedef struct _SPPrintContext SPPrintContext;
+
 #endif
