@@ -31,11 +31,11 @@ typedef struct _SPCanvasBPathClass SPCanvasBPathClass;
 #include <libart_lgpl/art_svp.h>
 #include <libart_lgpl/art_svp_wind.h>
 #include <libart_lgpl/art_svp_vpath_stroke.h>
-#include <libgnomeui/gnome-canvas.h>
+#include "sp-canvas.h"
 #include "curve.h"
 
 struct _SPCanvasBPath {
-	GnomeCanvasItem item;
+	SPCanvasItem item;
 
 	/* Line def */
 	SPCurve *curve;
@@ -57,12 +57,12 @@ struct _SPCanvasBPath {
 };
 
 struct _SPCanvasBPathClass {
-	GnomeCanvasItemClass parent_class;
+	SPCanvasItemClass parent_class;
 };
 
 GtkType sp_canvas_bpath_get_type (void);
 
-GnomeCanvasItem *sp_canvas_bpath_new (GnomeCanvasGroup *parent, SPCurve *curve);
+SPCanvasItem *sp_canvas_bpath_new (SPCanvasGroup *parent, SPCurve *curve);
 
 void sp_canvas_bpath_set_bpath (SPCanvasBPath *cbp, SPCurve *curve);
 void sp_canvas_bpath_set_fill (SPCanvasBPath *cbp, guint32 rgba, ArtWindRule rule);
