@@ -66,7 +66,7 @@ SPCurve *
 sp_curve_new_from_bpath (ArtBpath *bpath)
 {
 	SPCurve *curve;
-
+	gint i;
 	g_return_val_if_fail (bpath != NULL, NULL);
 
 	if (!sp_bpath_good (bpath)) {
@@ -83,6 +83,8 @@ sp_curve_new_from_bpath (ArtBpath *bpath)
 	curve->bpath = bpath;
 	curve->length = sp_bpath_length (bpath);
 	curve->end = curve->length - 1;
+	for (i = curve->end; i > 0; i--) if ((curve->bpath[i].code == ART_MOVETO) || (curve->bpath[i].code == ART_MOVETO_OPEN)) break;
+	curve->substart = i;
 	curve->sbpath = FALSE;
 	curve->hascpt = FALSE;
 	curve->posset = FALSE;
@@ -545,6 +547,9 @@ sp_curve_closepath (SPCurve * curve)
 		sp_curve_lineto (curve, bs->x3, bs->y3);
 	}
 
+	bs = curve->bpath + curve->substart;
+	be = curve->bpath + curve->end - 1;
+
 	bs->code = ART_MOVETO;
 
 	curve->closed = TRUE;
@@ -635,7 +640,7 @@ sp_curve_reverse (SPCurve *curve)
 
   new_curve = sp_curve_new_sized (curve->length);
 
-  g_assert (bs->code == ART_MOVETO_OPEN);
+  g_assert (bs->code == ART_MOVETO_OPEN || bs->code == ART_MOVETO);
   g_assert ((be+1)->code == ART_END);
 
   sp_curve_moveto (new_curve, be->x3, be->y3);
@@ -842,8 +847,8 @@ sp_bpath_clean (ArtBpath *bpath)
 			*np++ = *bp++;
 			while ((bp->code == ART_LINETO) || (bp->code == ART_CURVETO)) *np++ = *bp++;
 		} else {
-			*bp++;
-			while ((bp->code == ART_LINETO) || (bp->code == ART_CURVETO)) *bp++;
+			bp++;
+			while ((bp->code == ART_LINETO) || (bp->code == ART_CURVETO)) bp++;
 		}
 	}
 
