@@ -19,6 +19,7 @@
 #include <glib.h>
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 #include <libgnome/gnome-mime.h>
 #include <libgnomeui/gnome-stock.h>
 #include <libgnomeui/gnome-messagebox.h>
@@ -50,6 +51,7 @@
 #include "dialogs/display-settings.h"
 #include "dialogs/tool-options.h"
 #include "dialogs/tool-attributes.h"
+#include "dialogs/node-edit.h"
 
 void fake_dialogs (void);
 
@@ -84,6 +86,7 @@ sp_create_window (SPViewWidget *vw, gboolean editable)
 
 	w = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	gtk_object_set_data (GTK_OBJECT (vw), "window", w);
+	gtk_object_set_data (GTK_OBJECT (SP_VIEW_WIDGET_VIEW (vw)), "window", w);
 
 	/* fixme: */
 	if (editable) {
@@ -152,24 +155,35 @@ sp_ui_new_view_preview (GtkWidget *widget)
 void
 sp_ui_close_view (GtkWidget * widget)
 {
-	gpointer w;
+	GtkObject *w;
 
 	if (SP_ACTIVE_DESKTOP == NULL) return;
 
-	if (sp_ui_delete (NULL, NULL, SP_VIEW (SP_ACTIVE_DESKTOP))) return;
-
+	if (sp_view_shutdown (SP_VIEW (SP_ACTIVE_DESKTOP))) return;
 	w = gtk_object_get_data (GTK_OBJECT (SP_ACTIVE_DESKTOP), "window");
-
-	if (w) gtk_object_destroy (GTK_OBJECT (w));
+	gtk_object_destroy (w);
 
 	/*
 	 * We have to fake dialog initialization, or corresponding code is
 	 * not compiled into application.
 	 */
-
+	/* fixme: Remove this if not needed anymore (Lauris) */
 	if (GTK_IS_WIDGET (SODIPODI)) fake_dialogs ();
 }
 
+unsigned int
+sp_ui_close_all (void)
+{
+	while (SP_ACTIVE_DESKTOP) {
+		GtkObject *w;
+		if (sp_view_shutdown (SP_VIEW (SP_ACTIVE_DESKTOP))) return FALSE;
+		w = gtk_object_get_data (GTK_OBJECT (SP_ACTIVE_DESKTOP), "window");
+		gtk_object_destroy (w);
+	}
+
+	return TRUE;
+}
+
 static gint
 sp_ui_delete (GtkWidget *widget, GdkEvent *event, SPView *view)
 {
@@ -199,7 +213,7 @@ static void
 sp_ui_file_menu (GtkMenu *fm, SPDocument *doc)
 {
 	sp_ui_menu_append_item (GTK_MENU (fm), GNOME_STOCK_MENU_NEW, _("New"), sp_file_new, NULL);
-	sp_ui_menu_append_item (GTK_MENU (fm), GNOME_STOCK_MENU_OPEN, _("Open"), sp_file_open, NULL);
+	sp_ui_menu_append_item (GTK_MENU (fm), GNOME_STOCK_MENU_OPEN, _("Open"), sp_file_open_dialog, NULL);
 	sp_ui_menu_append_item (GTK_MENU (fm), GNOME_STOCK_MENU_SAVE, _("Save"), sp_file_save, NULL);
 	sp_ui_menu_append_item (GTK_MENU (fm), GNOME_STOCK_MENU_SAVE_AS, _("Save as"), sp_file_save_as, NULL);
 	sp_ui_menu_append_item (GTK_MENU (fm), NULL, NULL, NULL, NULL);
@@ -209,10 +223,11 @@ sp_ui_file_menu (GtkMenu *fm, SPDocument *doc)
 	sp_ui_menu_append_item (GTK_MENU (fm), GNOME_STOCK_MENU_PRINT, _("Print"), sp_file_print, NULL);
 	sp_ui_menu_append_item (GTK_MENU (fm), GNOME_STOCK_MENU_PRINT, _("Print Preview"), sp_file_print_preview, NULL);
 	sp_ui_menu_append_item (GTK_MENU (fm), NULL, NULL, NULL, NULL);
-	sp_ui_menu_append_item (GTK_MENU (fm), GNOME_STOCK_MENU_ABOUT, _("About Sodipodi"), sp_help_about, NULL);
-	sp_ui_menu_append_item (GTK_MENU (fm), NULL, NULL, NULL, NULL);
-	sp_ui_menu_append_item (GTK_MENU (fm), GNOME_STOCK_MENU_CLOSE, _("Close View"), sp_ui_close_view, NULL);
-	sp_ui_menu_append_item (GTK_MENU (fm), GNOME_STOCK_MENU_EXIT, _("Exit Program"), sp_file_exit, NULL);
+	sp_menu_append_recent_documents (GTK_WIDGET (fm));
+	sp_ui_menu_append_item (fm, GNOME_STOCK_MENU_CLOSE, _("Close View"), sp_ui_close_view, NULL);
+	sp_ui_menu_append_item (fm, GNOME_STOCK_MENU_EXIT, _("Exit Program"), sp_file_exit, NULL);
+	sp_ui_menu_append_item (fm, NULL, NULL, NULL, NULL);
+	sp_ui_menu_append_item (fm, GNOME_STOCK_MENU_ABOUT, _("About Sodipodi"), sp_help_about, NULL);
 }
 
 static void
@@ -336,7 +351,7 @@ sp_ui_dialog_menu (GtkMenu *menu, SPDocument *doc)
 	sp_ui_menu_append_item (menu, GNOME_STOCK_MENU_BLANK, _("Size and Position"), sp_object_properties_layout, NULL);
 	sp_ui_menu_append_item (menu, GNOME_STOCK_MENU_BLANK, _("Align Objects"), sp_quick_align_dialog, NULL);
 	sp_ui_menu_append_item (menu, GNOME_STOCK_MENU_BLANK, _("Text Editing"), sp_text_edit_dialog, NULL);
-	sp_ui_menu_append_item (menu, GNOME_STOCK_MENU_BLANK, _("Transformations"), sp_transformation_dialog, NULL);
+	sp_ui_menu_append_item (menu, GNOME_STOCK_MENU_BLANK, _("Transformations"), sp_transformation_dialog_move, NULL);
 	sp_ui_menu_append_item (menu, GNOME_STOCK_MENU_BLANK, _("Tool Options"), sp_tool_options_dialog, NULL);
 	sp_ui_menu_append_item (menu, GNOME_STOCK_MENU_BLANK, _("Tool Attributes"), sp_tool_attributes_dialog, NULL);
 	sp_ui_menu_append_item (menu, GNOME_STOCK_MENU_BLANK, _("Document"), sp_document_dialog, NULL);
@@ -355,8 +370,9 @@ sp_ui_main_menu (void)
 	m = gtk_menu_new ();
 
 	sp_ui_menu_append_item (GTK_MENU (m), GNOME_STOCK_MENU_NEW, _("New"), sp_file_new, NULL);
-	sp_ui_menu_append_item (GTK_MENU (m), GNOME_STOCK_MENU_OPEN, _("Open"), sp_file_open, NULL);
+	sp_ui_menu_append_item (GTK_MENU (m), GNOME_STOCK_MENU_OPEN, _("Open"), sp_file_open_dialog, NULL);
 	sp_ui_menu_append_item (GTK_MENU (m), NULL, NULL, NULL, NULL);
+	sp_menu_append_recent_documents (m);
 	sp_ui_menu_append_item (GTK_MENU (m), GNOME_STOCK_MENU_ABOUT, _("About Sodipodi"), sp_help_about, NULL);
 	sp_ui_menu_append_item (GTK_MENU (m), NULL, NULL, NULL, NULL);
 	sp_ui_menu_append_item (GTK_MENU (m), GNOME_STOCK_MENU_EXIT, _("Exit Program"), sp_file_exit, NULL);
@@ -439,6 +455,30 @@ sp_ui_generic_menu (SPView *v, SPItem *item)
 	return m;
 }
 
+static void
+sp_recent_open (GtkWidget *widget, const guchar *uri)
+{
+	sp_file_open (uri);
+}
+
+void
+sp_menu_append_recent_documents (GtkWidget *menu)
+{
+	SPRepr *recent;
+	recent = sodipodi_get_repr (SODIPODI, "documents.recent");
+	if (recent) {
+		SPRepr *child;
+		for (child = recent->children; child != NULL; child = child->next) {
+			const guchar *uri, *name;
+			uri = sp_repr_attr (child, "uri");
+			name = sp_repr_attr (child, "name");
+			/* fixme: I am pretty sure this is safe, but double check (Lauris) */
+			sp_ui_menu_append_item (GTK_MENU (menu), GNOME_STOCK_MENU_BLANK, name, sp_recent_open, (gpointer) uri);
+		}
+		sp_ui_menu_append_item (GTK_MENU (menu), NULL, NULL, NULL, NULL);
+	}
+}
+
 void
 fake_dialogs (void)
 {
@@ -453,6 +493,16 @@ fake_dialogs (void)
 	sp_desktop_dialog ();
 	sp_document_dialog ();
 	sp_display_dialog ();
+
+	sp_node_path_edit_add ();
+	sp_node_path_edit_delete ();
+	sp_node_path_edit_break ();
+	sp_node_path_edit_join ();
+	sp_node_path_edit_toline ();
+	sp_node_path_edit_tocurve ();
+	sp_node_path_edit_cusp ();
+	sp_node_path_edit_smooth ();
+	sp_node_path_edit_symmetric ();
 }
 
 /* Drag and Drop */
