@@ -14,6 +14,7 @@
 
 #include <math.h>
 #include <gtk/gtksignal.h>
+#include "helper/sp-canvas-util.h"
 #include "helper/sodipodi-ctrl.h"
 #include "desktop.h"
 #include "desktop-handles.h"
@@ -23,6 +24,8 @@
 
 #include "sp-guide.h"
 
+#define KNOT_EVENT_MASK (GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK | GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK)
+
 #define hypot(a,b) sqrt ((a) * (a) + (b) * (b))
 
 #define noKNOT_NOGRAB
@@ -59,7 +62,7 @@ static void sp_knot_init (SPKnot * knot);
 static void sp_knot_destroy (GtkObject * object);
 static void sp_knot_set_arg (GtkObject * object, GtkArg * arg, guint id);
 
-static void sp_knot_handler (GnomeCanvasItem * item, GdkEvent * event, gpointer data);
+static void sp_knot_handler (SPCanvasItem * item, GdkEvent * event, gpointer data);
 static void sp_knot_set_flag (SPKnot * knot, guint flag, gboolean set);
 static void sp_knot_update_ctrl (SPKnot * knot);
 static void sp_knot_set_ctrl_state (SPKnot *knot);
@@ -206,7 +209,7 @@ sp_knot_destroy (GtkObject * object)
 	knot = (SPKnot *) object;
 
 	/* ungrab pointer if still grabbed by mouseover, find a different way */
-	if (gdk_pointer_is_grabbed) gdk_pointer_ungrab (0);
+	if (gdk_pointer_is_grabbed) gdk_pointer_ungrab (gdk_time_get ());
 
 	if (knot->item) gtk_object_destroy (GTK_OBJECT (knot->item));
 	/*
@@ -310,10 +313,9 @@ sp_knot_set_arg (GtkObject * object, GtkArg * arg, guint id)
 }
 
 static void
-sp_knot_handler (GnomeCanvasItem * item, GdkEvent * event, gpointer data)
+sp_knot_handler (SPCanvasItem * item, GdkEvent * event, gpointer data)
 {
 	SPKnot * knot;
-	ArtPoint p;
 	gboolean consumed;
 	static gboolean grabbed = FALSE;
 	static gboolean moved = FALSE;
@@ -334,6 +336,7 @@ sp_knot_handler (GnomeCanvasItem * item, GdkEvent * event, gpointer data)
 	switch (event->type) {
 	case GDK_BUTTON_PRESS:
 		if (event->button.button == 1) {
+			NRPointF p;
 			sp_desktop_w2d_xy_point (knot->desktop,
 				&p,
 				event->button.x,
@@ -341,12 +344,20 @@ sp_knot_handler (GnomeCanvasItem * item, GdkEvent * event, gpointer data)
 			knot->hx = p.x - knot->x;
 			knot->hy = p.y - knot->y;
 #ifndef KNOT_NOGRAB
+#if 0
 			gdk_pointer_ungrab (event->button.time);
-			gnome_canvas_item_grab (knot->item,
-						GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
-						knot->cursor[SP_KNOT_STATE_DRAGGING],
-						event->button.time);
+#endif
+			sp_canvas_item_grab (knot->item,
+					     KNOT_EVENT_MASK,
+					     knot->cursor[SP_KNOT_STATE_DRAGGING],
+					     event->button.time);
 			
+#if 0
+			/* fixme: Top hack (Lauris) */
+			/* fixme: If we add key masks to event mask, Gdk will abort (Lauris) */
+			/* fixme: But Canvas actualle does get key events, so all we need is routing these here */
+			knot->item->canvas->grabbed_event_mask |= (GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK);
+#endif
 #endif
 			sp_knot_set_flag (knot, SP_KNOT_GRABBED, TRUE);
 			grabbed = TRUE;
@@ -357,7 +368,8 @@ sp_knot_handler (GnomeCanvasItem * item, GdkEvent * event, gpointer data)
 		if (event->button.button == 1) {
 			sp_knot_set_flag (knot, SP_KNOT_GRABBED, FALSE);
 #ifndef KNOT_NOGRAB
-			gnome_canvas_item_ungrab (knot->item, event->button.time);
+			sp_canvas_item_ungrab (knot->item, event->button.time);
+#if 0
 			gdk_pointer_grab (knot->item->canvas->layout.bin_window,
 					  FALSE,
 					  GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK | 
@@ -365,7 +377,7 @@ sp_knot_handler (GnomeCanvasItem * item, GdkEvent * event, gpointer data)
 					  NULL,
 					  knot->cursor[SP_KNOT_STATE_MOUSEOVER],
 					  event->button.time);
-
+#endif
 #endif
 			if (moved) {
 				sp_knot_set_flag (knot,
@@ -386,6 +398,8 @@ sp_knot_handler (GnomeCanvasItem * item, GdkEvent * event, gpointer data)
 		break;
 	case GDK_MOTION_NOTIFY:
 		if (grabbed) {
+			ArtPoint p;
+			NRPointF fp;
 			if (!moved) {
 				gtk_signal_emit (GTK_OBJECT (knot),
 					knot_signals[GRABBED],
@@ -394,18 +408,16 @@ sp_knot_handler (GnomeCanvasItem * item, GdkEvent * event, gpointer data)
 					SP_KNOT_DRAGGING,
 					TRUE);
 			}
-			sp_desktop_w2d_xy_point (knot->desktop,
-				&p,
-				event->motion.x,
-				event->motion.y);
-			p.x -= knot->hx;
-			p.y -= knot->hy;
+			sp_desktop_w2d_xy_point (knot->desktop, &fp, event->motion.x, event->motion.y);
+			p.x = fp.x - knot->hx;
+			p.y = fp.y - knot->hy;
 			sp_knot_request_position (knot, &p, event->motion.state);
 			moved = TRUE;
 			consumed = TRUE;
 		}
 		break;
 	case GDK_ENTER_NOTIFY:
+#if 0
 		gdk_pointer_grab (knot->item->canvas->layout.bin_window,
 				  FALSE,
 				  GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK | 
@@ -413,12 +425,15 @@ sp_knot_handler (GnomeCanvasItem * item, GdkEvent * event, gpointer data)
 				  NULL,
 				  knot->cursor[SP_KNOT_STATE_MOUSEOVER],
 				  event->button.time);
+#endif
 
 		sp_knot_set_flag (knot, SP_KNOT_MOUSEOVER, TRUE);
 		consumed = TRUE;
 		break;
 	case GDK_LEAVE_NOTIFY:
+#if 0
 		gdk_pointer_ungrab (event->button.time);
+#endif
 		sp_knot_set_flag (knot, SP_KNOT_MOUSEOVER, FALSE);
 		consumed = TRUE;
 		break;
@@ -445,7 +460,7 @@ sp_knot_new (SPDesktop * desktop)
 	knot->desktop = desktop;
 	knot->flags = SP_KNOT_VISIBLE;
 
-	knot->item = gnome_canvas_item_new (SP_DT_CONTROLS (desktop),
+	knot->item = sp_canvas_item_new (SP_DT_CONTROLS (desktop),
 		SP_TYPE_CTRL,
 		"anchor", GTK_ANCHOR_CENTER,
 		"size", 8.0,
@@ -569,9 +584,9 @@ sp_knot_set_flag (SPKnot * knot, guint flag, gboolean set)
 	switch (flag) {
 	case SP_KNOT_VISIBLE:
 		if (set) {
-			gnome_canvas_item_show (knot->item);
+			sp_canvas_item_show (knot->item);
 		} else {
-			gnome_canvas_item_hide (knot->item);
+			sp_canvas_item_hide (knot->item);
 		}
 		break;
 	case SP_KNOT_MOUSEOVER:
