@@ -1,5 +1,5 @@
-#ifndef SP_KNOTHOLDER_H
-#define SP_KNOTHOLDER_H
+#ifndef __SP_KNOTHOLDER_H__
+#define __SP_KNOTHOLDER_H__
 
 /*
  * SPKnotHolder - Hold SPKnot list and manage signals
@@ -21,6 +21,8 @@
 
 typedef void (* SPKnotHolderSetFunc) (SPItem *item, const ArtPoint *p, guint state);
 typedef void (* SPKnotHolderGetFunc) (SPItem *item, ArtPoint *p);
+/* fixme: Think how to make callbacks most sensitive (Lauris) */
+typedef void (* SPKnotHolderReleasedFunc) (SPItem *item);
 
 typedef struct _SPKnotHolderEntity SPKnotHolderEntity;
 typedef struct _SPKnotHolder       SPKnotHolder;
@@ -37,22 +39,22 @@ struct _SPKnotHolder {
 	SPDesktop *desktop;
 	SPItem *item;
 	GSList *entity;
+
+	SPKnotHolderReleasedFunc released;
 };
 
 
-SPKnotHolder   *sp_knot_holder_new	(SPDesktop     *desktop,
-					 SPItem	       *item);
-void		sp_knot_holder_destroy	(SPKnotHolder       *knots);
+/* fixme: As a temporary solution, if released is NULL knotholder flushes undo itself (Lauris) */
+SPKnotHolder *sp_knot_holder_new (SPDesktop *desktop, SPItem *item, SPKnotHolderReleasedFunc relhandler);
 
+void sp_knot_holder_destroy (SPKnotHolder *knots);
 
-void		sp_knot_holder_add	(SPKnotHolder       *knot_holder,
-					 SPKnotHolderSetFunc knot_set,
-					 SPKnotHolderGetFunc knot_get);
-void		sp_knot_holder_add_full	(SPKnotHolder       *knot_holder,
-					 SPKnotHolderSetFunc knot_set,
-					 SPKnotHolderGetFunc knot_get,
-					 SPKnotShapeType     shape,
-					 SPKnotModeType      mode);
+void sp_knot_holder_add (SPKnotHolder *knot_holder, SPKnotHolderSetFunc knot_set, SPKnotHolderGetFunc knot_get);
+void sp_knot_holder_add_full (SPKnotHolder *knot_holder,
+			      SPKnotHolderSetFunc knot_set,
+			      SPKnotHolderGetFunc knot_get,
+			      SPKnotShapeType shape,
+			      SPKnotModeType mode);
 
 
 #endif
