@@ -13,7 +13,18 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#include <config.h>
 #include <math.h>
+#include <string.h>
+#include <glib.h>
+#include <libgnome/gnome-defs.h>
+#include <libgnome/gnome-i18n.h>
+#include <gtk/gtksignal.h>
+#include <gtk/gtktable.h>
+#include <gtk/gtkbutton.h>
+#include <gtk/gtkspinbutton.h>
+#include <gtk/gtklabel.h>
+#include "helper/sp-canvas.h"
 #include "sp-rect.h"
 #include "sodipodi.h"
 #include "document.h"
@@ -29,11 +40,13 @@ static void sp_rect_context_class_init (SPRectContextClass * klass);
 static void sp_rect_context_init (SPRectContext * rect_context);
 static void sp_rect_context_destroy (GtkObject * object);
 
-#if 0
+#if 1
 static void sp_rect_context_setup (SPEventContext *ec);
+static void sp_rect_context_set (SPEventContext *ec, const guchar *key, const guchar *val);
 #endif
 static gint sp_rect_context_root_handler (SPEventContext * event_context, GdkEvent * event);
 static gint sp_rect_context_item_handler (SPEventContext * event_context, SPItem * item, GdkEvent * event);
+static GtkWidget *sp_rect_context_config_widget (SPEventContext *ec);
 
 static void sp_rect_drag (SPRectContext * rc, double x, double y, guint state);
 static void sp_rect_finish (SPRectContext * rc);
@@ -77,11 +90,13 @@ sp_rect_context_class_init (SPRectContextClass * klass)
 
 	object_class->destroy = sp_rect_context_destroy;
 
-#if 0
+#if 1
 	event_context_class->setup = sp_rect_context_setup;
+	event_context_class->set = sp_rect_context_set;
 #endif
-	event_context_class->root_handler = sp_rect_context_root_handler;
-	event_context_class->item_handler = sp_rect_context_item_handler;
+	event_context_class->root_handler  = sp_rect_context_root_handler;
+	event_context_class->item_handler  = sp_rect_context_item_handler;
+	event_context_class->config_widget = sp_rect_context_config_widget;
 }
 
 static void
@@ -96,6 +111,9 @@ sp_rect_context_init (SPRectContext * rect_context)
 	event_context->hot_y = 4;
 
 	rect_context->item = NULL;
+
+	rect_context->rx_ratio = 0.0;
+	rect_context->ry_ratio = 0.0;
 }
 
 static void
@@ -112,16 +130,35 @@ sp_rect_context_destroy (GtkObject * object)
 		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
 
-#if 0
+#if 1
 static void
 sp_rect_context_setup (SPEventContext *ec)
 {
 	SPRectContext * rc;
 
-	rc = SP_RECT_CONTEXT (event_context);
+	rc = SP_RECT_CONTEXT (ec);
 
 	if (SP_EVENT_CONTEXT_CLASS (parent_class)->setup)
-		SP_EVENT_CONTEXT_CLASS (parent_class)->setup (event_context, desktop);
+		SP_EVENT_CONTEXT_CLASS (parent_class)->setup (ec);
+
+	sp_event_context_read (ec, "rx_ratio");
+	sp_event_context_read (ec, "ry_ratio");
+}
+
+static void
+sp_rect_context_set (SPEventContext *ec, const guchar *key, const guchar *val)
+{
+	SPRectContext *rc;
+
+	rc = SP_RECT_CONTEXT (ec);
+
+	if (!strcmp (key, "rx_ratio")) {
+		rc->rx_ratio = (val) ? atof (val) : 0.0;
+		rc->rx_ratio = CLAMP (rc->rx_ratio, 0.0, 1.0);
+	} else if (!strcmp (key, "ry_ratio")) {
+		rc->ry_ratio = (val) ? atof (val) : 0.0;
+		rc->ry_ratio = CLAMP (rc->ry_ratio, 0.0, 1.0);
+	}
 }
 #endif
 
@@ -153,12 +190,15 @@ sp_rect_context_root_handler (SPEventContext * event_context, GdkEvent * event)
 	switch (event->type) {
 	case GDK_BUTTON_PRESS:
 		if (event->button.button == 1) {
+			NRPointF fp;
 			dragging = TRUE;
 			/* Position center */
-			sp_desktop_w2d_xy_point (event_context->desktop, &rc->center, event->button.x, event->button.y);
+			sp_desktop_w2d_xy_point (event_context->desktop, &fp, event->button.x, event->button.y);
+			rc->center.x = fp.x;
+			rc->center.y = fp.y;
 			/* Snap center to nearest magnetic point */
 			sp_desktop_free_snap (event_context->desktop, &rc->center);
-			gnome_canvas_item_grab (GNOME_CANVAS_ITEM (desktop->acetate),
+			sp_canvas_item_grab (SP_CANVAS_ITEM (desktop->acetate),
 						GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK,
 						NULL, event->button.time);
 			ret = TRUE;
@@ -166,7 +206,7 @@ sp_rect_context_root_handler (SPEventContext * event_context, GdkEvent * event)
 		break;
 	case GDK_MOTION_NOTIFY:
 		if (dragging && event->motion.state && GDK_BUTTON1_MASK) {
-			ArtPoint p;
+			NRPointF p;
 			sp_desktop_w2d_xy_point (event_context->desktop, &p, event->motion.x, event->motion.y);
 			sp_rect_drag (rc, p.x, p.y, event->motion.state);
 			ret = TRUE;
@@ -177,7 +217,7 @@ sp_rect_context_root_handler (SPEventContext * event_context, GdkEvent * event)
 			dragging = FALSE;
 			sp_rect_finish (rc);
 			ret = TRUE;
-			gnome_canvas_item_ungrab (GNOME_CANVAS_ITEM (desktop->acetate), event->button.time);
+			sp_canvas_item_ungrab (SP_CANVAS_ITEM (desktop->acetate), event->button.time);
 		}
 		break;
 	default:
@@ -197,9 +237,10 @@ sp_rect_drag (SPRectContext * rc, double x, double y, guint state)
 {
 	SPDesktop * desktop;
 	ArtPoint p0, p1;
-	gdouble x0, y0, x1, y1;
+	gdouble x0, y0, x1, y1, w, h;
 	GString * xs, * ys;
 	gchar status[80];
+	NRPointF fp;
 
 	desktop = SP_EVENT_CONTEXT (rc)->desktop;
 
@@ -283,15 +324,25 @@ sp_rect_drag (SPRectContext * rc, double x, double y, guint state)
 		sp_desktop_free_snap (desktop, &p1);
 	}
 
-	sp_desktop_d2doc_xy_point (desktop, &p0, p0.x, p0.y);
-	sp_desktop_d2doc_xy_point (desktop, &p1, p1.x, p1.y);
+	sp_desktop_dt2root_xy_point (desktop, &fp, p0.x, p0.y);
+	p0.x = fp.x;
+	p0.y = fp.y;
+	sp_desktop_dt2root_xy_point (desktop, &fp, p1.x, p1.y);
+	p1.x = fp.x;
+	p1.y = fp.y;
 
 	x0 = MIN (p0.x, p1.x);
 	y0 = MIN (p0.y, p1.y);
 	x1 = MAX (p0.x, p1.x);
 	y1 = MAX (p0.y, p1.y);
+	w  = x1 - x0;
+	h  = y1 - y0;
 
-	sp_rect_set (SP_RECT (rc->item), x0, y0, x1 - x0, y1 - y0);
+	sp_rect_set (SP_RECT (rc->item), x0, y0, w, h);
+	if (rc->rx_ratio != 0.0)
+		sp_rect_set_rx(SP_RECT (rc->item), TRUE, 0.5 * rc->rx_ratio * w); 
+	if (rc->ry_ratio != 0.0)
+		sp_rect_set_ry(SP_RECT (rc->item), TRUE, 0.5 * rc->ry_ratio * h); 
 
 	// status text
 	xs = SP_PT_TO_METRIC_STRING (fabs(x1-x0), SP_DEFAULT_METRIC);
@@ -327,3 +378,71 @@ sp_rect_finish (SPRectContext * rc)
 	}
 }
 
+static void
+sp_rc_rx_ratio_value_changed (GtkAdjustment *adj, SPRectContext *rc)
+{
+  	sp_repr_set_double (SP_EVENT_CONTEXT_REPR (rc), "rx_ratio", adj->value);
+}
+
+static void
+sp_rc_ry_ratio_value_changed (GtkAdjustment *adj, SPRectContext *rc)
+{
+	sp_repr_set_double (SP_EVENT_CONTEXT_REPR (rc), "ry_ratio", adj->value);
+}
+
+static void
+sp_rc_defaults (GtkWidget *widget, GtkObject *obj)
+{
+	GtkAdjustment *adj;
+
+	adj = gtk_object_get_data (obj, "rx_ratio");
+	gtk_adjustment_set_value (adj, 0.0);
+	adj = gtk_object_get_data (obj, "ry_ratio");
+	gtk_adjustment_set_value (adj, 0.0);
+}
+
+static GtkWidget *
+sp_rect_context_config_widget (SPEventContext *ec)
+{
+	SPRectContext *rc;
+	GtkWidget *tbl, *l, *sb, *b;
+	GtkObject *a;
+
+	rc = SP_RECT_CONTEXT (ec);
+
+	tbl = gtk_table_new (3, 2, FALSE);
+	gtk_container_set_border_width (GTK_CONTAINER (tbl), 4);
+	gtk_table_set_row_spacings (GTK_TABLE (tbl), 4);
+
+	/* rx_ratio */
+	l = gtk_label_new (_("Roundness ratio for x:"));
+	gtk_widget_show (l);
+	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
+	gtk_table_attach (GTK_TABLE (tbl), l, 0, 1, 0, 1, 0, 0, 0, 0);
+	a = gtk_adjustment_new (rc->rx_ratio, 0.0, 1.0, 0.01, 0.1, 0.1);
+	gtk_object_set_data (GTK_OBJECT (tbl), "rx_ratio", a);
+	sb = gtk_spin_button_new (GTK_ADJUSTMENT (a), 0.1, 2);
+	gtk_widget_show (sb);
+	gtk_table_attach (GTK_TABLE (tbl), sb, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+	gtk_signal_connect (GTK_OBJECT (a), "value_changed", GTK_SIGNAL_FUNC (sp_rc_rx_ratio_value_changed), rc);
+
+	/* ry_ratio */
+	l = gtk_label_new (_("Roundness ratio for y:"));
+	gtk_widget_show (l);
+	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
+	gtk_table_attach (GTK_TABLE (tbl), l, 0, 1, 1, 2, 0, 0, 0, 0);
+	a = gtk_adjustment_new (rc->ry_ratio, 0.0, 1.0, 0.01, 0.1, 0.1);
+	gtk_object_set_data (GTK_OBJECT (tbl), "ry_ratio", a);
+	sb = gtk_spin_button_new (GTK_ADJUSTMENT (a), 0.1, 2);
+	gtk_widget_show (sb);
+	gtk_table_attach (GTK_TABLE (tbl), sb, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+	gtk_signal_connect (GTK_OBJECT (a), "value_changed", GTK_SIGNAL_FUNC (sp_rc_ry_ratio_value_changed), rc);
+
+	/* Reset */
+	b = gtk_button_new_with_label (_("Defaults"));
+	gtk_widget_show (b);
+	gtk_table_attach (GTK_TABLE (tbl), b, 0, 2, 2, 3, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+	gtk_signal_connect (GTK_OBJECT (b), "clicked", GTK_SIGNAL_FUNC (sp_rc_defaults), tbl);
+
+	return tbl;
+}
