@@ -36,7 +36,7 @@ sp_rubberband_move (double x, double y)
 	g_return_if_fail (sp_rb_dragging);
 #endif
 	if (sp_rb == NULL) {
-		sp_rb = (SPCtrlRect *) gnome_canvas_item_new (SP_DT_CONTROLS (sp_rb_desktop), SP_TYPE_CTRLRECT, NULL);
+		sp_rb = (SPCtrlRect *) sp_canvas_item_new (SP_DT_CONTROLS (sp_rb_desktop), SP_TYPE_CTRLRECT, NULL);
 	}
 
 	sp_rb_rect.x1 = x;
