@@ -15,9 +15,16 @@
 #include <math.h>
 #include <string.h>
 #include <glib.h>
+#include <libart_lgpl/art_affine.h>
 #include <gdk/gdkkeysyms.h>
+#include <gtk/gtksignal.h>
+#include <gtk/gtkframe.h>
+#include <gtk/gtkradiobutton.h>
+#include <gtk/gtkvbox.h>
+#include <gtk/gtktogglebutton.h>
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
+#include "helper/sp-canvas-util.h"
 #include "rubberband.h"
 #include "sodipodi-private.h"
 #include "document.h"
@@ -28,10 +35,6 @@
 #include "pixmaps/cursor-select-m.xpm"
 #include "pixmaps/cursor-select-d.xpm"
 #include "pixmaps/handles.xpm"
-#if 0
-/* This is not needed, so kill warning (Lauris) */
-#include "pixmaps/cursor-select.xpm"
-#endif
 #include "select-context.h"
 #include "selection-chemistry.h"
 #include "path-chemistry.h"
@@ -137,7 +140,7 @@ sp_select_context_destroy (GtkObject * object)
 	sc = SP_SELECT_CONTEXT (object);
 
 	if (sc->grabbed) {
-		gnome_canvas_item_ungrab (sc->grabbed, gdk_time_get ());
+		sp_canvas_item_ungrab (sc->grabbed, gdk_time_get ());
 		sc->grabbed = NULL;
 	}
 
@@ -193,7 +196,7 @@ sp_select_context_item_handler (SPEventContext *event_context, SPItem *item, Gdk
 	SPSelTrans *seltrans;
 	SPSelection *selection;
 	GdkCursor *cursor;
-	ArtPoint p;
+	NRPointF p;
 	gint ret = FALSE;
 
 	desktop = event_context->desktop;
@@ -211,10 +214,11 @@ sp_select_context_item_handler (SPEventContext *event_context, SPItem *item, Gdk
 			sc->dragging = TRUE;
 			sc->moved = FALSE;
 			sc->item = item;
-			gnome_canvas_item_grab (GNOME_CANVAS_ITEM (desktop->drawing),
-						GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK,
-						NULL, event->button.time);
-			sc->grabbed = GNOME_CANVAS_ITEM (desktop->drawing);
+			sp_canvas_item_grab (SP_CANVAS_ITEM (desktop->drawing),
+					     GDK_KEY_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
+					     GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK,
+					     NULL, event->button.time);
+			sc->grabbed = SP_CANVAS_ITEM (desktop->drawing);
 			ret = TRUE;
 		}
 		break;
@@ -270,11 +274,11 @@ sp_select_context_item_handler (SPEventContext *event_context, SPItem *item, Gdk
 			sc->dragging = FALSE;
 			sc->item = NULL;
 			if (sc->grabbed) {
-				gnome_canvas_item_ungrab (sc->grabbed, event->button.time);
+				sp_canvas_item_ungrab (sc->grabbed, event->button.time);
 				sc->grabbed = NULL;
 			}
 			ret = TRUE;
-		}
+		} 
 		break;
 	case GDK_ENTER_NOTIFY:
 		cursor = gdk_cursor_new (GDK_FLEUR);
@@ -284,6 +288,13 @@ sp_select_context_item_handler (SPEventContext *event_context, SPItem *item, Gdk
 	case GDK_LEAVE_NOTIFY:
 		gdk_window_set_cursor (GTK_WIDGET (SP_DT_CANVAS (desktop))->window, event_context->cursor);
 		break;
+	case GDK_KEY_PRESS:
+		if (event->key.keyval == GDK_space) {
+			/* stamping mode: show content mode moving */
+			sp_sel_trans_stamp(seltrans);
+			ret = TRUE;
+		}
+		break;
 	default:
 		break;
 	}
@@ -305,7 +316,7 @@ sp_select_context_root_handler (SPEventContext *event_context, GdkEvent * event)
 	SPSelection *selection;
 	SPItem *item;
 	gint ret = FALSE;
-	ArtPoint p;
+	NRPointF p;
 	ArtDRect b;
 	GSList *l;
 
@@ -319,11 +330,14 @@ sp_select_context_root_handler (SPEventContext *event_context, GdkEvent * event)
 		if (event->button.button == 1) {
 			sp_desktop_w2d_xy_point (desktop, &p, event->button.x, event->button.y);
 			sp_rubberband_start (desktop, p.x, p.y);
-			gnome_canvas_item_grab (GNOME_CANVAS_ITEM (desktop->acetate),
-						GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK,
-						NULL, event->button.time);
-			sc->grabbed = GNOME_CANVAS_ITEM (desktop->acetate);
+			sp_canvas_item_grab (SP_CANVAS_ITEM (desktop->acetate),
+					     GDK_KEY_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK,
+					     NULL, event->button.time);
+			sc->grabbed = SP_CANVAS_ITEM (desktop->acetate);
+#if 0
+			/* We cannot assign shift for partial selects, as it is used for add mode */
 			sc->button_press_shift = (event->button.state & GDK_SHIFT_MASK) ? TRUE : FALSE;
+#endif
 			ret = TRUE;
 		}
 		break;
@@ -349,7 +363,7 @@ sp_select_context_root_handler (SPEventContext *event_context, GdkEvent * event)
 		}
 		break;
 	case GDK_BUTTON_RELEASE:
-		if (event->button.button == 1) {
+		if ((event->button.button == 1) && (sc->grabbed)) {
 			if (sc->dragging) {
 				if (sc->moved) {
 					// item has been moved
@@ -385,10 +399,11 @@ sp_select_context_root_handler (SPEventContext *event_context, GdkEvent * event)
 				if (sp_rubberband_rect (&b)) {
 					sp_rubberband_stop ();
 					sp_sel_trans_reset_state (seltrans);
-					if (sc->button_press_shift)
+					if (sc->button_press_shift) {
 						l = sp_document_partial_items_in_box (SP_DT_DOCUMENT (desktop), &b);
-					else
+					} else {
 						l = sp_document_items_in_box (SP_DT_DOCUMENT (desktop), &b);
+					}
 					if (event->button.state & GDK_SHIFT_MASK) {
 						while (l) {
 							item = SP_ITEM (l->data);
@@ -411,10 +426,11 @@ sp_select_context_root_handler (SPEventContext *event_context, GdkEvent * event)
 				ret = TRUE;
 			}
 			if (sc->grabbed) {
-				gnome_canvas_item_ungrab (sc->grabbed, event->button.time);
+				sp_canvas_item_ungrab (sc->grabbed, event->button.time);
+				sc->grabbed = NULL;
 			}
-		}
-		sc->button_press_shift = FALSE;
+			sc->button_press_shift = FALSE;
+		} 
 		break;
 	case GDK_KEY_PRESS: // keybindings for select context
           switch (event->key.keyval) {  
@@ -522,6 +538,13 @@ sp_select_context_root_handler (SPEventContext *event_context, GdkEvent * event)
 	    sp_selection_item_prev ();
 	    ret = TRUE;
 	    break;
+	  case GDK_space:
+	    /* stamping mode: show outline mode moving */
+	    /* FIXME: Is next condition ok? */
+	    if (sc->dragging && sc->grabbed) 
+		    sp_sel_trans_stamp(seltrans);
+	    ret = TRUE;
+	    break;
           }
 	  break;
 	default:
