@@ -12,6 +12,10 @@
  */
 
 #include <math.h>
+#include <gdk/gdkkeysyms.h>
+#include <libart_lgpl/art_affine.h>
+#include <gtk/gtksignal.h>
+
 #include "svg/svg.h"
 #include "sodipodi-private.h"
 #include "sodipodi.h"
@@ -45,6 +49,33 @@ static void sp_sel_trans_sel_modified (SPSelection * selection, guint flags, gpo
 extern GdkCursor * CursorSelectMouseover, * CursorSelectDragging;
 extern GdkPixbuf * handles[];
 
+static gboolean
+sp_seltrans_handle_event (SPKnot *knot, GdkEvent *event, gpointer data)
+{
+	SPDesktop * desktop;
+	SPSelTrans * seltrans;
+	
+	switch (event->type) {
+	case GDK_MOTION_NOTIFY:
+		break;
+	case GDK_KEY_PRESS:
+		if (event->key.keyval == GDK_space) {
+			/* stamping mode: both mode(show content and outline) operation with knot */
+			if (!SP_KNOT_IS_GRABBED (knot)) return FALSE;
+			desktop = knot->desktop;
+			seltrans = &SP_SELECT_CONTEXT (desktop->event_context)->seltrans;
+			sp_sel_trans_stamp (seltrans);
+			return TRUE;
+		}
+		break;
+	default:
+		g_print ("Event type %d occured\n", event->type);
+		break;
+	}
+
+	return FALSE;
+}
+
 void
 sp_sel_trans_init (SPSelTrans * seltrans, SPDesktop * desktop)
 {
@@ -77,52 +108,50 @@ sp_sel_trans_init (SPSelTrans * seltrans, SPDesktop * desktop)
 	gtk_signal_connect (GTK_OBJECT (seltrans->selection), "changed", GTK_SIGNAL_FUNC (sp_sel_trans_sel_changed), seltrans);
 	gtk_signal_connect (GTK_OBJECT (seltrans->selection), "modified", GTK_SIGNAL_FUNC (sp_sel_trans_sel_modified), seltrans);
 
-	seltrans->norm = gnome_canvas_item_new (SP_DT_CONTROLS (desktop),
+	seltrans->norm = sp_canvas_item_new (SP_DT_CONTROLS (desktop),
 		SP_TYPE_CTRL,
 		"anchor", GTK_ANCHOR_CENTER,
 		"mode", SP_CTRL_MODE_COLOR,
 		"shape", SP_CTRL_SHAPE_BITMAP,
 		"size", 13.0,
 		"filled", TRUE,
-		"fill_color", 0x40ff40a0,
+		"fill_color", 0x00000000,
 		"stroked", TRUE,
 		"stroke_color", 0x000000a0,
-		"pixbuf", handles[12],				
+		"pixbuf", handles[12],
 		NULL);
-	seltrans->grip = gnome_canvas_item_new (SP_DT_CONTROLS (desktop),
+	seltrans->grip = sp_canvas_item_new (SP_DT_CONTROLS (desktop),
 		SP_TYPE_CTRL,
 		"anchor", GTK_ANCHOR_CENTER,
 		"mode", SP_CTRL_MODE_XOR,
 		"shape", SP_CTRL_SHAPE_CROSS,
 		"size", 7.0,
 		"filled", TRUE,
-		"fill_color", 0xff40ffa0,
+		"fill_color", 0xffffff7f,
 		"stroked", TRUE,
-		"stroke_color", 0xffffffFF,
-		"pixbuf", handles[12],				
+		"stroke_color", 0xffffffff,
+		"pixbuf", handles[12],
 		NULL);
-	gnome_canvas_item_hide (seltrans->grip);
-	gnome_canvas_item_hide (seltrans->norm);
+	sp_canvas_item_hide (seltrans->grip);
+	sp_canvas_item_hide (seltrans->norm);
 
-	seltrans->l1 = gnome_canvas_item_new (SP_DT_CONTROLS (desktop), SP_TYPE_CTRLLINE, NULL);
-	gnome_canvas_item_hide (seltrans->l1);
-	seltrans->l2 = gnome_canvas_item_new (SP_DT_CONTROLS (desktop), SP_TYPE_CTRLLINE, NULL);
-	gnome_canvas_item_hide (seltrans->l2);
-	seltrans->l3 = gnome_canvas_item_new (SP_DT_CONTROLS (desktop), SP_TYPE_CTRLLINE, NULL);
-	gnome_canvas_item_hide (seltrans->l3);
-	seltrans->l4 = gnome_canvas_item_new (SP_DT_CONTROLS (desktop), SP_TYPE_CTRLLINE, NULL);
-	gnome_canvas_item_hide (seltrans->l4);
+	seltrans->l1 = sp_canvas_item_new (SP_DT_CONTROLS (desktop), SP_TYPE_CTRLLINE, NULL);
+	sp_canvas_item_hide (seltrans->l1);
+	seltrans->l2 = sp_canvas_item_new (SP_DT_CONTROLS (desktop), SP_TYPE_CTRLLINE, NULL);
+	sp_canvas_item_hide (seltrans->l2);
+	seltrans->l3 = sp_canvas_item_new (SP_DT_CONTROLS (desktop), SP_TYPE_CTRLLINE, NULL);
+	sp_canvas_item_hide (seltrans->l3);
+	seltrans->l4 = sp_canvas_item_new (SP_DT_CONTROLS (desktop), SP_TYPE_CTRLLINE, NULL);
+	sp_canvas_item_hide (seltrans->l4);
+
+	seltrans->stamp_cache = NULL;
 }
 
 void
 sp_sel_trans_shutdown (SPSelTrans *seltrans)
 {
 	gint i;
-#if 0
-	seltrans->show_handles = FALSE;
 
-	sp_sel_trans_update_handles (seltrans);
-#else
 	for (i = 0; i < 8; i++) {
 		if (seltrans->shandle[i]) {
 			gtk_object_unref (GTK_OBJECT (seltrans->shandle[i]));
@@ -137,7 +166,6 @@ sp_sel_trans_shutdown (SPSelTrans *seltrans)
 		gtk_object_unref (GTK_OBJECT (seltrans->chandle));
 		seltrans->chandle = NULL;
 	}
-#endif
 
 	if (seltrans->norm) {
 		gtk_object_destroy (GTK_OBJECT (seltrans->norm));
@@ -197,7 +225,7 @@ sp_sel_trans_set_center (SPSelTrans * seltrans, gdouble x, gdouble y)
 }
 
 void
-sp_sel_trans_grab (SPSelTrans * seltrans, ArtPoint * p, gdouble x, gdouble y, gboolean show_handles)
+sp_sel_trans_grab (SPSelTrans * seltrans, NRPointF *p, gdouble x, gdouble y, gboolean show_handles)
 {
 	SPSelection * selection;
 
@@ -216,7 +244,8 @@ sp_sel_trans_grab (SPSelTrans * seltrans, ArtPoint * p, gdouble x, gdouble y, gb
 
 	art_affine_identity (seltrans->current);
 
-	seltrans->point = *p;
+	seltrans->point.x = p->x;
+	seltrans->point.y = p->y;
 
 	seltrans->snappoints = sp_selection_snappoints (selection);
 
@@ -224,19 +253,20 @@ sp_sel_trans_grab (SPSelTrans * seltrans, ArtPoint * p, gdouble x, gdouble y, gb
 	seltrans->opposit.y = seltrans->box.y0 + (1 - y) * fabs (seltrans->box.y1 - seltrans->box.y0);
 
 	if ((x != -1) && (y != -1)) {
-		gnome_canvas_item_show (seltrans->norm);
-		gnome_canvas_item_show (seltrans->grip);
+		sp_canvas_item_show (seltrans->norm);
+		sp_canvas_item_show (seltrans->grip);
 	}
 
 	if (seltrans->show == SP_SELTRANS_SHOW_OUTLINE) {
-		gnome_canvas_item_show (seltrans->l1);
-		gnome_canvas_item_show (seltrans->l2);
-		gnome_canvas_item_show (seltrans->l3);
-		gnome_canvas_item_show (seltrans->l4);
+		sp_canvas_item_show (seltrans->l1);
+		sp_canvas_item_show (seltrans->l2);
+		sp_canvas_item_show (seltrans->l3);
+		sp_canvas_item_show (seltrans->l4);
 	}
 
 
 	sp_sel_trans_update_handles (seltrans);
+	g_return_if_fail(seltrans->stamp_cache == NULL);
 }
 
 void
@@ -362,18 +392,85 @@ sp_sel_trans_ungrab (SPSelTrans * seltrans)
 	}
 #endif
 
-	gnome_canvas_item_hide (seltrans->norm);
-	gnome_canvas_item_hide (seltrans->grip);
+	sp_canvas_item_hide (seltrans->norm);
+	sp_canvas_item_hide (seltrans->grip);
 
         if (seltrans->show == SP_SELTRANS_SHOW_OUTLINE) {
-		gnome_canvas_item_hide (seltrans->l1);
-		gnome_canvas_item_hide (seltrans->l2);
-		gnome_canvas_item_hide (seltrans->l3);
-		gnome_canvas_item_hide (seltrans->l4);
+		sp_canvas_item_hide (seltrans->l1);
+		sp_canvas_item_hide (seltrans->l2);
+		sp_canvas_item_hide (seltrans->l3);
+		sp_canvas_item_hide (seltrans->l4);
 	}
 
 	sp_sel_trans_update_volatile_state (seltrans);
 	sp_sel_trans_update_handles (seltrans);
+	if (seltrans->stamp_cache) {
+		g_slist_free(seltrans->stamp_cache);
+		seltrans->stamp_cache = NULL;
+	}
+
+}
+
+/* fixme: This is really bad, as we compare positoons for each stamp (Lauris) */
+/* fixme: IMHO the best way to keep sort cache would be to implement timestamping at last */
+
+static int
+sp_object_compare_position (SPObject *a, SPObject *b)
+{
+	return sp_repr_compare_position (a->repr, b->repr);
+}
+
+void
+sp_sel_trans_stamp (SPSelTrans * seltrans)
+{
+	/* stamping mode */
+	SPItem * original_item, * copy_item;
+	SPRepr * original_repr, * copy_repr;
+	GSList *l;
+
+	gchar tstr[80];
+	gdouble i2d[6], i2dnew[6];
+	
+	gdouble * new_affine;
+
+	tstr[79] = '\0';
+	
+	if (!seltrans->empty) {
+		
+		if (seltrans->stamp_cache) {
+			l = seltrans->stamp_cache;
+		} else {
+			/* Build cache */
+			l  = (GSList *) sp_selection_item_list (SP_DT_SELECTION (seltrans->desktop));
+			l  = g_slist_copy (l);
+			l  = g_slist_sort (l, (GCompareFunc) sp_object_compare_position);
+			seltrans->stamp_cache = l;
+		}
+
+		while (l) {
+			original_item = SP_ITEM(l->data);
+			original_repr = (SPRepr *)(SP_OBJECT (original_item)->repr);
+			copy_repr = sp_repr_duplicate (original_repr);
+			copy_item = (SPItem *) sp_document_add_repr (SP_DT_DOCUMENT (seltrans->desktop), 
+								     copy_repr);
+			
+			if (seltrans->show == SP_SELTRANS_SHOW_OUTLINE) {
+				sp_item_i2d_affine (original_item, i2d);
+				art_affine_multiply (i2dnew, i2d, seltrans->current);
+				sp_item_set_i2d_affine (copy_item, i2dnew);
+				new_affine = copy_item->affine;
+			} else 
+				new_affine = original_item->affine;
+
+			if (sp_svg_write_affine (tstr, 79, new_affine)) 
+				sp_repr_set_attr (copy_repr, "transform", tstr);
+			else
+				sp_repr_set_attr (copy_repr, "transform", NULL);
+			sp_repr_unref (copy_repr);
+			l = l->next;
+		}
+		sp_document_done (SP_DT_DOCUMENT (seltrans->desktop));
+	}
 }
 
 ArtPoint *
@@ -428,10 +525,10 @@ sp_sel_trans_update_handles (SPSelTrans * seltrans)
 			  "shape", SP_CTRL_SHAPE_BITMAP,
 			  "size", 13,
 			  "mode", SP_CTRL_MODE_COLOR,
-					"fill", 0x40ff40a0,
-					"fill_mouseover", 0xff4040f0,
-					"stroke", 0x000000a0,
-					"stroke_mouseover", 0x000000FF,
+					"fill", 0x00000000,
+					"fill_mouseover", 0x0000007f,
+					"stroke", 0x000000ff,
+					"stroke_mouseover", 0x000000ff,
 			  //"fill", 0xff40ffa0,
 			  //"fill_mouseover", 0x40ffffa0,
 			  //"stroke", 0xFFb0b0ff,
@@ -467,6 +564,12 @@ sp_sel_trans_update_volatile_state (SPSelTrans * seltrans)
 	if (seltrans->empty) return;
 
 	sp_selection_bbox (SP_DT_SELECTION (seltrans->desktop), &seltrans->box);
+
+	if ((seltrans->box.x0 > seltrans->box.x1) || (seltrans->box.y0 > seltrans->box.y1)) {
+		seltrans->empty = TRUE;
+		return;
+	}
+
 	art_affine_identity (seltrans->current);
 }
 
@@ -517,6 +620,7 @@ sp_show_handles (SPSelTrans * seltrans, SPKnot * knot[], const SPSelTransHandle
 				GTK_SIGNAL_FUNC (sp_sel_trans_handle_grab), (gpointer) &handle[i]);
 			gtk_signal_connect (GTK_OBJECT (knot[i]), "ungrabbed",
 				GTK_SIGNAL_FUNC (sp_sel_trans_handle_ungrab), (gpointer) &handle[i]);
+			gtk_signal_connect (GTK_OBJECT (knot[i]), "event", GTK_SIGNAL_FUNC (sp_seltrans_handle_event), (gpointer) &handle[i]);
 		}
 		sp_knot_show (knot[i]);
 
@@ -539,7 +643,8 @@ sp_sel_trans_handle_grab (SPKnot * knot, guint state, gpointer data)
 	SPSelTrans * seltrans;
 	SPSelTransHandle * handle;
 	ArtPoint p;
-						
+	NRPointF pf;
+
 	desktop = knot->desktop;
 	seltrans = &SP_SELECT_CONTEXT (desktop->event_context)->seltrans;
 	handle = (SPSelTransHandle *) data;
@@ -550,22 +655,24 @@ sp_sel_trans_handle_grab (SPKnot * knot, guint state, gpointer data)
 			  "shape", SP_CTRL_SHAPE_BITMAP,
 			  "size", 13.0,
 			  NULL);
-	  gnome_canvas_item_show (seltrans->grip);
+	  sp_canvas_item_show (seltrans->grip);
 	  break;
 	default:
 	  gtk_object_set (GTK_OBJECT (seltrans->grip),
 			  "shape", SP_CTRL_SHAPE_CROSS,
 			  "size", 7.0,
 			  NULL);
-	  gnome_canvas_item_show (seltrans->norm);
-	  gnome_canvas_item_show (seltrans->grip);
+	  sp_canvas_item_show (seltrans->norm);
+	  sp_canvas_item_show (seltrans->grip);
 
 	  break;
 	}
 
 	sp_knot_position (knot, &p);
 
-	sp_sel_trans_grab (seltrans, &p, handle->x, handle->y, FALSE);
+	pf.x = p.x;
+	pf.y = p.y;
+	sp_sel_trans_grab (seltrans, &pf, handle->x, handle->y, FALSE);
 }
 
 static void
@@ -594,7 +701,6 @@ sp_sel_trans_handle_new_event (SPKnot * knot, ArtPoint * position, guint state,
 	desktop = knot->desktop;
 	seltrans = &SP_SELECT_CONTEXT (desktop->event_context)->seltrans;
 	handle = (SPSelTransHandle *) data;
-
 	handle->action (seltrans, handle, position, state);
 
 	//sp_desktop_coordinate_status (desktop, position->x, position->y, 4);
@@ -617,6 +723,7 @@ sp_sel_trans_handle_request (SPKnot * knot, ArtPoint * position, guint state, gb
 	handle = (SPSelTransHandle *) data;
 
 	sp_desktop_set_coordinate_status (desktop, position->x, position->y, 0);
+	sp_view_set_position (SP_VIEW (desktop), position->x, position->y);
 
 	if (state & GDK_MOD1_MASK) {
 	  sp_sel_trans_point_desktop (seltrans, &point);
@@ -679,40 +786,51 @@ sp_sel_trans_sel_modified (SPSelection *selection, guint flags, gpointer data)
  */
 
 gboolean
-sp_sel_trans_scale_request (SPSelTrans * seltrans, SPSelTransHandle * handle, ArtPoint * p, guint state)
+sp_sel_trans_scale_request (SPSelTrans *seltrans, SPSelTransHandle *handle, ArtPoint *p, guint state)
 {
 	ArtPoint norm, point;
-	gdouble sx, sy, ratio;
+	gdouble sx, sy;
 	gchar status[80];
-	SPDesktop * desktop;
+	SPDesktop *desktop;
+	int xd, yd;
 
 	desktop = seltrans->desktop;
 
 	sp_sel_trans_point_desktop (seltrans, &point);
 	sp_sel_trans_origin_desktop (seltrans, &norm);
 
-	if ((fabs(point.x - norm.x)<1e-15) || (fabs(point.x - norm.x)<1e-15)) return FALSE;
-
-	sx = (p->x - norm.x) / (point.x - norm.x);
-	sy = (p->y - norm.y) / (point.y - norm.y);
-
-	if (state & GDK_CONTROL_MASK) {
-        	if (fabs (sx)<1e-15) sx = 1e-15;
-	        if (fabs (sy)<1e-15) sy = 1e-15;
-	        if (fabs (sy) > fabs (sx)) sy = fabs (sx) * sy / fabs (sy);
-		if (fabs (sx) > fabs (sy)) sx = fabs (sy) * sx / fabs (sx);
-		ratio = sp_desktop_vector_snap_list (desktop, seltrans->snappoints, &norm, sx, sy);
-		sx = fabs (ratio) * sx / fabs (sx);
-		sy = fabs (ratio) * sy / fabs (sy);
+	if (fabs (point.x - norm.x) > 0.0625) {
+		sx = (p->x - norm.x) / (point.x - norm.x);
+		if (fabs (sx) < 1e-9) sx = 1e-9;
+		xd = TRUE;
 	} else {
-	  sx = sp_desktop_horizontal_snap_list_scale (desktop, seltrans->snappoints, &norm, sx);
-	  sy = sp_desktop_vertical_snap_list_scale (desktop, seltrans->snappoints, &norm, sy);
+		sx = 0.0;
+		xd = FALSE;
+	}
+	if (fabs (point.y - norm.y) > 0.0625) {
+		sy = (p->y - norm.y) / (point.y - norm.y);
+		if (fabs (sy) < 1e-9) sy = 1e-9;
+		yd = TRUE;
+	} else {
+		sy = 0.0;
+		yd = FALSE;
 	}
 
-	if (fabs (sx)<1e-15) sx = 1e-15;
-	if (fabs (sy)<1e-15) sy = 1e-15;
-	p->x = (point.x - norm.x)* sx + norm.x;
-	p->y = (point.y - norm.y)* sy + norm.y;
+	if (state & GDK_CONTROL_MASK) {
+		double r;
+		if (!xd || !yd) return FALSE;
+	        if (fabs (sy) > fabs (sx)) sy = fabs (sx) * sy / fabs (sy);
+		if (fabs (sx) > fabs (sy)) sx = fabs (sy) * sx / fabs (sx);
+		r = sp_desktop_vector_snap_list (desktop, seltrans->snappoints, &norm, sx, sy);
+		sx = fabs (r) * sx / fabs (sx);
+		sy = fabs (r) * sy / fabs (sy);
+	} else {
+		if (xd) sx = sp_desktop_horizontal_snap_list_scale (desktop, seltrans->snappoints, &norm, sx);
+		if (yd) sy = sp_desktop_vertical_snap_list_scale (desktop, seltrans->snappoints, &norm, sy);
+	}
+
+	p->x = (point.x - norm.x) * sx + norm.x;
+	p->y = (point.y - norm.y) * sy + norm.y;
 
 	// status text
 	sprintf (status, "Scale  %0.2f%c, %0.2f%c", 100 * sx, '%', 100 * sy, '%');
@@ -737,19 +855,19 @@ sp_sel_trans_stretch_request (SPSelTrans * seltrans, SPSelTransHandle * handle,
 	switch (handle->cursor) {
 	case GDK_TOP_SIDE:
 	case GDK_BOTTOM_SIDE:
-	  if (fabs (point.y - norm.y) < 1e-15) return FALSE;
-	  sy = (p->y - norm.y) / (point.y - norm.y);	    
-	  if (fabs (sy) < 1e-15) sy = 1e-15;
-	  if (state & GDK_CONTROL_MASK) {
-	    if (fabs (sy) > 1) sy = sy / fabs (sy);
-	    sx = fabs (sy);
-	    ratio = sp_desktop_vector_snap_list (desktop, seltrans->snappoints, &norm, sx, sy);
-	    sy = (fabs (ratio) < 1) ? fabs (ratio) * sy / fabs (sy) : sy / fabs (sy);
-	    sx = fabs (sy);
-	  } else {
-	    sy = sp_desktop_vertical_snap_list_scale (desktop, seltrans->snappoints, &norm, sy); 
-	  }
-	  break;
+		if (fabs (point.y - norm.y) < 1e-15) return FALSE;
+		sy = (p->y - norm.y) / (point.y - norm.y);	    
+		if (fabs (sy) < 1e-15) sy = 1e-15;
+		if (state & GDK_CONTROL_MASK) {
+			if (fabs (sy) > 1) sy = sy / fabs (sy);
+			sx = fabs (sy);
+			ratio = sp_desktop_vector_snap_list (desktop, seltrans->snappoints, &norm, sx, sy);
+			sy = (fabs (ratio) < 1) ? fabs (ratio) * sy / fabs (sy) : sy / fabs (sy);
+			sx = fabs (sy);
+		} else {
+			sy = sp_desktop_vertical_snap_list_scale (desktop, seltrans->snappoints, &norm, sy); 
+		}
+		break;
 	case GDK_LEFT_SIDE:
 	case GDK_RIGHT_SIDE:
 		if (fabs (point.x - norm.x) < 1e-5) return FALSE;
@@ -951,24 +1069,24 @@ sp_sel_trans_stretch (SPSelTrans * seltrans, SPSelTransHandle * handle, ArtPoint
 	switch (handle->cursor) {
 	case GDK_TOP_SIDE:
 	case GDK_BOTTOM_SIDE:
-	  if (fabs (point.y - norm.y) < 1e-15) return;
-	  sy = (p->y - norm.y) / (point.y - norm.y);
-	  if (fabs (sy) < 1e-15) sy = 1e-15;
-	  if (state & GDK_CONTROL_MASK) {
-	    if (fabs (sy) > fabs (sx)) sy = sy / fabs (sy);
-	    if (fabs (sy) < fabs (sx)) sx = fabs (sy) * sx / fabs (sx);
-	}
-	  break;
+		if (fabs (point.y - norm.y) < 1e-15) return;
+		sy = (p->y - norm.y) / (point.y - norm.y);
+		if (fabs (sy) < 1e-15) sy = 1e-15;
+		if (state & GDK_CONTROL_MASK) {
+			if (fabs (sy) > fabs (sx)) sy = sy / fabs (sy);
+			if (fabs (sy) < fabs (sx)) sx = fabs (sy) * sx / fabs (sx);
+		}
+		break;
 	case GDK_LEFT_SIDE:
 	case GDK_RIGHT_SIDE:
-	  if (fabs (point.x - norm.x) < 1e-15) return;
-	  sx = (p->x - norm.x) / (point.x - norm.x);
-	  if (fabs (sx) < 1e-15) sx = 1e-15;
-	  if (state & GDK_CONTROL_MASK) {
-	    if (fabs (sx) > fabs (sy)) sx = sx / fabs (sx);
-	    if (fabs (sx) < fabs (sy)) sy = fabs (sx) * sy / fabs (sy);
-	  }
-	  break;
+		if (fabs (point.x - norm.x) < 1e-15) return;
+		sx = (p->x - norm.x) / (point.x - norm.x);
+		if (fabs (sx) < 1e-15) sx = 1e-15;
+		if (state & GDK_CONTROL_MASK) {
+			if (fabs (sx) > fabs (sy)) sx = sx / fabs (sx);
+			if (fabs (sx) < fabs (sy)) sy = fabs (sx) * sy / fabs (sy);
+		}
+		break;
 	default:
 		break;
 	}
@@ -988,13 +1106,21 @@ sp_sel_trans_scale (SPSelTrans * seltrans, SPSelTransHandle * handle, ArtPoint *
 	sp_sel_trans_point_desktop (seltrans, &point);
 	sp_sel_trans_origin_desktop (seltrans, &norm);
 
-	if ((fabs(point.x - norm.x)<1e-15) || (fabs(point.x - norm.x)<1e-15)) return;
+	if (fabs (point.x - norm.x) > 1e-9) {
+		sx = (p->x - norm.x) / (point.x - norm.x);
+	} else {
+		sx = 1.0;
+	}
 
-	sx = (p->x - norm.x) / (point.x - norm.x);
-	sy = (p->y - norm.y) / (point.y - norm.y);
+	if (fabs (point.y - norm.y) > 1e-9) {
+		sy = (p->y - norm.y) / (point.y - norm.y);
+	} else {
+		sy = 1.0;
+	}
+
+	if (fabs (sx) < 1e-9) sx = 1e-9; 
+	if (fabs (sy) < 1e-9) sy = 1e-9;
 
-	if (fabs (sx) < 1e-15) sx = 1e-15; 
-	if (fabs (sy) < 1e-15) sy = 1e-15;
 	art_affine_scale (scale, sx, sy);
 	sp_sel_trans_transform (seltrans, scale, &norm);
 }
