@@ -14,7 +14,7 @@
  */
 
 #include <math.h>
-#include <gnome.h>
+#include <string.h>
 #include "helper/art-utils.h"
 #include "svg/svg.h"
 #include "style.h"
@@ -54,9 +54,8 @@ static double sp_round (double x, double y)
 
 static void sp_genericellipse_class_init (SPGenericEllipseClass *klass);
 static void sp_genericellipse_init (SPGenericEllipse *ellipse);
-static void sp_genericellipse_destroy (GtkObject *object);
+
 static void sp_genericellipse_build (SPObject *object, SPDocument *document, SPRepr *repr);
-static void sp_genericellipse_write_repr (SPObject *object, SPRepr *repr);
 static void sp_genericellipse_read_attr (SPObject * object, const gchar * attr);
 
 static GSList *sp_genericellipse_snappoints (SPItem *item, GSList *points);
@@ -100,10 +99,7 @@ sp_genericellipse_class_init (SPGenericEllipseClass *klass)
 
 	ge_parent_class = gtk_type_class (sp_shape_get_type ());
 
-	gtk_object_class->destroy = sp_genericellipse_destroy;
-
 	sp_object_class->build = sp_genericellipse_build;
-	sp_object_class->write_repr = sp_genericellipse_write_repr;
 	sp_object_class->read_attr = sp_genericellipse_read_attr;
 
 	item_class->snappoints = sp_genericellipse_snappoints;
@@ -126,29 +122,11 @@ sp_genericellipse_init (SPGenericEllipse *ellipse)
 	ellipse->closed = TRUE;
 }
 
-static void
-sp_genericellipse_destroy (GtkObject *object)
-{
-	if (GTK_OBJECT_CLASS (ge_parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (ge_parent_class)->destroy) (object);
-}
-
 static void
 sp_genericellipse_build (SPObject *object, SPDocument *document, SPRepr *repr)
 {
-	if (SP_OBJECT_CLASS (ge_parent_class)->build)
-		(* SP_OBJECT_CLASS (ge_parent_class)->build) (object, document, repr);
-}
-
-static void
-sp_genericellipse_write_repr (SPObject *object, SPRepr *repr)
-{
-	SPGenericEllipse *ellipse;
-
-	ellipse = SP_GENERICELLIPSE (object);
-
-	if (SP_OBJECT_CLASS (ge_parent_class)->write_repr)
-		(* SP_OBJECT_CLASS (ge_parent_class)->write_repr) (object, repr);
+	if (((SPObjectClass *) ge_parent_class)->build)
+		(* ((SPObjectClass *) ge_parent_class)->build) (object, document, repr);
 }
 
 static void
@@ -165,8 +143,8 @@ sp_genericellipse_read_attr (SPObject * object, const gchar * attr)
 
 	str = sp_repr_attr (object->repr, attr);
 
-	if (SP_OBJECT_CLASS(ge_parent_class)->read_attr)
-		(* SP_OBJECT_CLASS (ge_parent_class)->read_attr) (object, attr);
+	if (((SPObjectClass *) ge_parent_class)->read_attr)
+		(* ((SPObjectClass *) ge_parent_class)->read_attr) (object, attr);
 }
 
 static void sp_genericellipse_glue_set_shape (SPShape *shape)
@@ -264,10 +242,11 @@ static void sp_genericellipse_set_shape (SPGenericEllipse *ellipse)
 	affine[5] = ellipse->cy.computed;
 
 	i = 0;
-	if (ellipse->closed)
+	if (ellipse->closed) {
 		bpath[i].code = ART_MOVETO;
-	else
+	} else {
 		bpath[i].code = ART_MOVETO_OPEN;
+	}
 	bpath[i].x3 = cos (ellipse->start);
 	bpath[i].y3 = sin (ellipse->start);
 	i++;
@@ -320,6 +299,8 @@ g_print ("step %d s %f e %f coords %f %f %f %f %f %f\n",
 	c = sp_curve_new_from_bpath (abp);
 	sp_path_add_bpath (SP_PATH (ellipse), c, TRUE, NULL);
 	sp_curve_unref (c);
+
+	sp_object_request_modified (SP_OBJECT (ellipse), SP_OBJECT_MODIFIED_FLAG);
 }
 
 static GSList * 
@@ -386,10 +367,9 @@ sp_genericellipse_side (SPGenericEllipse *ellipse, const ArtPoint *p)
 
 static void sp_ellipse_class_init (SPEllipseClass *class);
 static void sp_ellipse_init (SPEllipse *ellipse);
-static void sp_ellipse_destroy (GtkObject *object);
 
 static void sp_ellipse_build (SPObject * object, SPDocument * document, SPRepr * repr);
-static void sp_ellipse_write_repr (SPObject *object, SPRepr *repr);
+static SPRepr *sp_ellipse_write (SPObject *object, SPRepr *repr, guint flags);
 static void sp_ellipse_read_attr (SPObject * object, const gchar * attr);
 static gchar * sp_ellipse_description (SPItem * item);
 
@@ -426,10 +406,8 @@ sp_ellipse_class_init (SPEllipseClass *class)
 
 	ellipse_parent_class = gtk_type_class (SP_TYPE_GENERICELLIPSE);
 
-	gtk_object_class->destroy = sp_ellipse_destroy;
-
 	sp_object_class->build = sp_ellipse_build;
-	sp_object_class->write_repr = sp_ellipse_write_repr;
+	sp_object_class->write = sp_ellipse_write;
 	sp_object_class->read_attr = sp_ellipse_read_attr;
 
 	item_class->description = sp_ellipse_description;
@@ -441,22 +419,11 @@ sp_ellipse_init (SPEllipse *ellipse)
 	/* Nothing special */
 }
 
-static void
-sp_ellipse_destroy (GtkObject *object)
-{
-	SPEllipse *ellipse;
-
-	ellipse = SP_ELLIPSE (object);
-
-	if (GTK_OBJECT_CLASS (ellipse_parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (ellipse_parent_class)->destroy) (object);
-}
-
 static void
 sp_ellipse_build (SPObject *object, SPDocument *document, SPRepr *repr)
 {
-	if (SP_OBJECT_CLASS (ellipse_parent_class)->build)
-		(* SP_OBJECT_CLASS (ellipse_parent_class)->build) (object, document, repr);
+	if (((SPObjectClass *) ellipse_parent_class)->build)
+		(* ((SPObjectClass *) ellipse_parent_class)->build) (object, document, repr);
 
 	sp_ellipse_read_attr (object, "cx");
 	sp_ellipse_read_attr (object, "cy");
@@ -464,20 +431,26 @@ sp_ellipse_build (SPObject *object, SPDocument *document, SPRepr *repr)
 	sp_ellipse_read_attr (object, "ry");
 }
 
-static void
-sp_ellipse_write_repr (SPObject *object, SPRepr *repr)
+static SPRepr *
+sp_ellipse_write (SPObject *object, SPRepr *repr, guint flags)
 {
 	SPGenericEllipse *ellipse;
 
 	ellipse = SP_GENERICELLIPSE (object);
 
+	if ((flags & SP_OBJECT_WRITE_BUILD) && !repr) {
+		repr = sp_repr_new ("ellipse");
+	}
+
 	sp_repr_set_double_attribute (repr, "cx", ellipse->cx.computed);
 	sp_repr_set_double_attribute (repr, "cy", ellipse->cy.computed);
 	sp_repr_set_double_attribute (repr, "rx", ellipse->rx.computed);
 	sp_repr_set_double_attribute (repr, "ry", ellipse->ry.computed);
 	
-	if (SP_OBJECT_CLASS (ellipse_parent_class)->write_repr)
-		(* SP_OBJECT_CLASS (ellipse_parent_class)->write_repr) (object, repr);
+	if (((SPObjectClass *) ellipse_parent_class)->write)
+		(* ((SPObjectClass *) ellipse_parent_class)->write) (object, repr, flags);
+
+	return repr;
 }
 
 static void
@@ -526,8 +499,8 @@ sp_ellipse_read_attr (SPObject * object, const gchar * attr)
 		}
 		sp_genericellipse_set_shape (ellipse);
 	} else {
-		if (SP_OBJECT_CLASS(ellipse_parent_class)->read_attr)
-			(* SP_OBJECT_CLASS (ellipse_parent_class)->read_attr) (object, attr);
+		if (((SPObjectClass *) ellipse_parent_class)->read_attr)
+			(* ((SPObjectClass *) ellipse_parent_class)->read_attr) (object, attr);
 	}
 }
 
@@ -560,10 +533,9 @@ sp_ellipse_set (SPEllipse *ellipse, gdouble x, gdouble y, gdouble rx, gdouble ry
 
 static void sp_circle_class_init (SPCircleClass *class);
 static void sp_circle_init (SPCircle *circle);
-static void sp_circle_destroy (GtkObject *object);
 
 static void sp_circle_build (SPObject * object, SPDocument * document, SPRepr * repr);
-static void sp_circle_write_repr (SPObject *object, SPRepr *repr);
+static SPRepr *sp_circle_write (SPObject *object, SPRepr *repr, guint flags);
 static void sp_circle_read_attr (SPObject * object, const gchar * attr);
 static gchar * sp_circle_description (SPItem * item);
 
@@ -600,10 +572,8 @@ sp_circle_class_init (SPCircleClass *class)
 
 	circle_parent_class = gtk_type_class (SP_TYPE_GENERICELLIPSE);
 
-	gtk_object_class->destroy = sp_circle_destroy;
-
 	sp_object_class->build = sp_circle_build;
-	sp_object_class->write_repr = sp_circle_write_repr;
+	sp_object_class->write = sp_circle_write;
 	sp_object_class->read_attr = sp_circle_read_attr;
 
 	item_class->description = sp_circle_description;
@@ -615,41 +585,36 @@ sp_circle_init (SPCircle *circle)
 	/* Nothing special */
 }
 
-static void
-sp_circle_destroy (GtkObject *object)
-{
-	SPCircle *circle;
-
-	circle = SP_CIRCLE (object);
-
-	if (GTK_OBJECT_CLASS (circle_parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (circle_parent_class)->destroy) (object);
-}
-
 static void
 sp_circle_build (SPObject *object, SPDocument *document, SPRepr *repr)
 {
-	if (SP_OBJECT_CLASS (circle_parent_class)->build)
-		(* SP_OBJECT_CLASS (circle_parent_class)->build) (object, document, repr);
+	if (((SPObjectClass *) circle_parent_class)->build)
+		(* ((SPObjectClass *) circle_parent_class)->build) (object, document, repr);
 
 	sp_circle_read_attr (object, "cx");
 	sp_circle_read_attr (object, "cy");
 	sp_circle_read_attr (object, "r");
 }
 
-static void
-sp_circle_write_repr (SPObject *object, SPRepr *repr)
+static SPRepr *
+sp_circle_write (SPObject *object, SPRepr *repr, guint flags)
 {
 	SPGenericEllipse *ellipse;
 
 	ellipse = SP_GENERICELLIPSE (object);
 
+	if ((flags & SP_OBJECT_WRITE_BUILD) && !repr) {
+		repr = sp_repr_new ("circle");
+	}
+
 	sp_repr_set_double_attribute (repr, "cx", ellipse->cx.computed);
 	sp_repr_set_double_attribute (repr, "cy", ellipse->cy.computed);
 	sp_repr_set_double_attribute (repr, "r", ellipse->rx.computed);
 	
-	if (SP_OBJECT_CLASS (circle_parent_class)->write_repr)
-		(* SP_OBJECT_CLASS (circle_parent_class)->write_repr) (object, repr);
+	if (((SPObjectClass *) circle_parent_class)->write)
+		((SPObjectClass *) circle_parent_class)->write (object, repr, flags);
+
+	return repr;
 }
 
 static void
@@ -690,8 +655,8 @@ sp_circle_read_attr (SPObject *object, const gchar * attr)
 		circle->ry.computed = circle->rx.computed;
 		sp_genericellipse_set_shape (circle);
 	} else {
-		if (SP_OBJECT_CLASS(circle_parent_class)->read_attr)
-			(* SP_OBJECT_CLASS (circle_parent_class)->read_attr) (object, attr);
+		if (((SPObjectClass *) circle_parent_class)->read_attr)
+			(* ((SPObjectClass *) circle_parent_class)->read_attr) (object, attr);
 	}
 }
 
@@ -705,11 +670,12 @@ sp_circle_description (SPItem * item)
 
 static void sp_arc_class_init (SPArcClass *class);
 static void sp_arc_init (SPArc *arc);
-static void sp_arc_destroy (GtkObject *object);
 
 static void sp_arc_build (SPObject * object, SPDocument * document, SPRepr * repr);
-static void sp_arc_write_repr (SPObject *object, SPRepr *repr);
-static void sp_arc_read_attr (SPObject * object, const gchar * attr);
+static SPRepr *sp_arc_write (SPObject *object, SPRepr *repr, guint flags);
+static void sp_arc_read_attr (SPObject *object, const gchar *attr);
+static void sp_arc_modified (SPObject *object, guint flags);
+
 static gchar * sp_arc_description (SPItem * item);
 static SPKnotHolder *sp_arc_knot_holder (SPItem * item, SPDesktop *desktop);
 
@@ -746,11 +712,10 @@ sp_arc_class_init (SPArcClass *class)
 
 	arc_parent_class = gtk_type_class (SP_TYPE_GENERICELLIPSE);
 
-	gtk_object_class->destroy = sp_arc_destroy;
-
 	sp_object_class->build = sp_arc_build;
-	sp_object_class->write_repr = sp_arc_write_repr;
+	sp_object_class->write = sp_arc_write;
 	sp_object_class->read_attr = sp_arc_read_attr;
+	sp_object_class->modified = sp_arc_modified;
 
 	item_class->description = sp_arc_description;
 	item_class->knot_holder = sp_arc_knot_holder;
@@ -759,18 +724,7 @@ sp_arc_class_init (SPArcClass *class)
 static void
 sp_arc_init (SPArc *arc)
 {
-	arc->is_closed = TRUE;
-}
-
-static void
-sp_arc_destroy (GtkObject *object)
-{
-	SPArc *arc;
-
-	arc = SP_ARC (object);
-
-	if (GTK_OBJECT_CLASS (arc_parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (arc_parent_class)->destroy) (object);
+	/* Nothing special */
 }
 
 /* fixme: Better place (Lauris) */
@@ -794,8 +748,8 @@ sp_arc_build (SPObject *object, SPDocument *document, SPRepr *repr)
 {
 	guint version;
 
-	if (SP_OBJECT_CLASS (arc_parent_class)->build)
-		(* SP_OBJECT_CLASS (arc_parent_class)->build) (object, document, repr);
+	if (((SPObjectClass *) arc_parent_class)->build)
+		(* ((SPObjectClass *) arc_parent_class)->build) (object, document, repr);
 
 	version = sp_arc_find_version (object);
 
@@ -819,7 +773,7 @@ sp_arc_build (SPObject *object, SPDocument *document, SPRepr *repr)
 
 	if (version < 25) {
 		/* fixme: I am 99.9% sure we can do this here safely, but check nevertheless (Lauris) */
-		sp_arc_write_repr (object, repr);
+		sp_arc_write (object, repr, SP_OBJECT_WRITE_SODIPODI);
 		sp_repr_set_attr (repr, "cx", NULL);
 		sp_repr_set_attr (repr, "cy", NULL);
 		sp_repr_set_attr (repr, "rx", NULL);
@@ -838,11 +792,11 @@ sp_arc_build (SPObject *object, SPDocument *document, SPRepr *repr)
 static gboolean
 sp_arc_set_elliptical_path_attribute (SPArc *arc, SPRepr *repr)
 {
-#define ARC_BUFSIZE 128
+#define ARC_BUFSIZE 256
 	SPGenericEllipse *ge;
 	ArtPoint p1, p2;
-	gdouble  dt;
 	gint fa, fs;
+	gdouble  dt;
 	gchar c[ARC_BUFSIZE];
 
 	ge = SP_GENERICELLIPSE (arc);
@@ -850,45 +804,91 @@ sp_arc_set_elliptical_path_attribute (SPArc *arc, SPRepr *repr)
 	sp_arc_get_xy (arc, ge->start, &p1);
 	sp_arc_get_xy (arc, ge->end, &p2);
 
-	dt = fmod(ge->end - ge->start, SP_2PI);
-	fa = (fabs(dt) > M_PI) ? 1 : 0;
-	fs = (dt > 0) ? 1 : 0;
+	dt = fmod (ge->end - ge->start, SP_2PI);
+	if (fabs (dt) < 1e-6) {
+		ArtPoint ph;
+		sp_arc_get_xy (arc, (ge->start + ge->end) / 2.0, &ph);
+		g_snprintf (c, ARC_BUFSIZE, "M %f %f A %f %f 0 %d %d %f,%f A %g %g 0 %d %d %g %g L %f %f z",
+			    p1.x, p1.y,
+			    ge->rx.computed, ge->ry.computed,
+			    1, (dt > 0),
+			    ph.x, ph.y,
+			    ge->rx.computed, ge->ry.computed,
+			    1, (dt > 0),
+			    p2.x, p2.y,
+			    ge->cx.computed, ge->cy.computed);
+	} else {
+		fa = (fabs (dt) > M_PI) ? 1 : 0;
+		fs = (dt > 0) ? 1 : 0;
 #ifdef ARC_VERBOSE
-	g_print ("start:%g end:%g fa=%d fs=%d\n", ge->start, ge->end, fa, fs);
+		g_print ("start:%g end:%g fa=%d fs=%d\n", ge->start, ge->end, fa, fs);
 #endif
-	if (arc->is_closed)
-		g_snprintf (c, ARC_BUFSIZE, "M %f,%f A %f,%f 0 %d %d %f,%f L %f,%f z",
-			    p1.x, p1.y, ge->rx.computed, ge->ry.computed, fa, fs, p2.x, p2.y, ge->cx.computed, ge->cy.computed);
-	else
-		g_snprintf (c, ARC_BUFSIZE, "M %f,%f A %f,%f 0 %d %d %f,%f",
-			    p1.x, p1.y, ge->rx.computed, ge->ry.computed, fa, fs, p2.x, p2.y);
+		if (ge->closed) {
+			g_snprintf (c, ARC_BUFSIZE, "M %f,%f A %f,%f 0 %d %d %f,%f L %f,%f z",
+				    p1.x, p1.y,
+				    ge->rx.computed, ge->ry.computed,
+				    fa, fs,
+				    p2.x, p2.y,
+				    ge->cx.computed, ge->cy.computed);
+		} else {
+			g_snprintf (c, ARC_BUFSIZE, "M %f,%f A %f,%f 0 %d %d %f,%f",
+				    p1.x, p1.y,
+				    ge->rx.computed, ge->ry.computed,
+				    fa, fs, p2.x, p2.y);
+		}
+	}
 
 
 	return sp_repr_set_attr (repr, "d", c);
 }
 
-static void
-sp_arc_write_repr (SPObject *object, SPRepr *repr)
+static SPRepr *
+sp_arc_write (SPObject *object, SPRepr *repr, guint flags)
 {
-	SPGenericEllipse *ellipse;
+	SPGenericEllipse *ge;
 	SPArc *arc;
 
-	ellipse = SP_GENERICELLIPSE (object);
+	ge = SP_GENERICELLIPSE (object);
 	arc = SP_ARC (object);
 
-	sp_repr_set_double_attribute (repr, "sodipodi:cx", ellipse->cx.computed);
-	sp_repr_set_double_attribute (repr, "sodipodi:cy", ellipse->cy.computed);
-	sp_repr_set_double_attribute (repr, "sodipodi:rx", ellipse->rx.computed);
-	sp_repr_set_double_attribute (repr, "sodipodi:ry", ellipse->ry.computed);
-	sp_repr_set_double_attribute (repr, "sodipodi:start", ellipse->start);
-	sp_repr_set_double_attribute (repr, "sodipodi:end", ellipse->end);
-	if (! ellipse->closed)
-		sp_repr_set_attr (repr, "sodipodi:open", "true");
+	if (flags & SP_OBJECT_WRITE_SODIPODI) {
+		if ((flags & SP_OBJECT_WRITE_BUILD) && !repr) {
+			repr = sp_repr_new ("path");
+		}
 
-	sp_arc_set_elliptical_path_attribute (arc, repr);
-	
-/*  	if (SP_OBJECT_CLASS (arc_parent_class)->write_repr) */
-/*  		(* SP_OBJECT_CLASS (arc_parent_class)->write_repr) (object, repr); */
+		sp_repr_set_attr (repr, "sodipodi:type", "arc");
+		sp_repr_set_double_attribute (repr, "sodipodi:cx", ge->cx.computed);
+		sp_repr_set_double_attribute (repr, "sodipodi:cy", ge->cy.computed);
+		sp_repr_set_double_attribute (repr, "sodipodi:rx", ge->rx.computed);
+		sp_repr_set_double_attribute (repr, "sodipodi:ry", ge->ry.computed);
+		sp_repr_set_double_attribute (repr, "sodipodi:start", ge->start);
+		sp_repr_set_double_attribute (repr, "sodipodi:end", ge->end);
+		sp_repr_set_attr (repr, "sodipodi:open", (!ge->closed) ? "true" : NULL);
+
+		sp_arc_set_elliptical_path_attribute (arc, repr);
+	} else {
+		gdouble dt;
+		dt = fmod (ge->end - ge->start, SP_2PI);
+		if (fabs (dt) < 1e-6) {
+			if ((flags & SP_OBJECT_WRITE_BUILD) && !repr) {
+				repr = sp_repr_new ("ellipse");
+			}
+			sp_repr_set_double_attribute (repr, "cx", ge->cx.computed);
+			sp_repr_set_double_attribute (repr, "cy", ge->cy.computed);
+			sp_repr_set_double_attribute (repr, "rx", ge->rx.computed);
+			sp_repr_set_double_attribute (repr, "ry", ge->ry.computed);
+		} else {
+			if ((flags & SP_OBJECT_WRITE_BUILD) && !repr) {
+				repr = sp_repr_new ("path");
+			}
+			sp_arc_set_elliptical_path_attribute (arc, repr);
+		}
+	}
+
+	if (((SPObjectClass *) arc_parent_class)->write)
+		((SPObjectClass *) arc_parent_class)->write (object, repr, flags);
+
+	return repr;
 }
 
 static void
@@ -939,6 +939,7 @@ sp_arc_read_attr (SPObject *object, const gchar *attr)
 			ellipse->ry.set = FALSE;
 			ellipse->ry.computed = 0.0;
 		}
+		sp_genericellipse_set_shape (ellipse);
 	} else if (!strcmp (attr, "sodipodi:start")) {
 		n = sp_repr_get_double_attribute (object->repr, attr, ellipse->start);
 		ellipse->start = n;
@@ -954,11 +955,22 @@ sp_arc_read_attr (SPObject *object, const gchar *attr)
 			ellipse->closed = TRUE;
 		sp_genericellipse_set_shape (ellipse);
 	} else {
-		if (SP_OBJECT_CLASS(arc_parent_class)->read_attr)
-			(* SP_OBJECT_CLASS (arc_parent_class)->read_attr) (object, attr);
+		if (((SPObjectClass *) arc_parent_class)->read_attr)
+			((SPObjectClass *) arc_parent_class)->read_attr (object, attr);
 	}
 }
 
+static void
+sp_arc_modified (SPObject *object, guint flags)
+{
+	if (flags & SP_OBJECT_MODIFIED_FLAG) {
+		sp_arc_set_elliptical_path_attribute (SP_ARC (object), SP_OBJECT_REPR (object));
+	}
+
+	if (((SPObjectClass *) arc_parent_class)->modified)
+		((SPObjectClass *) arc_parent_class)->modified (object, flags);
+}
+
 static gchar *
 sp_arc_description (SPItem * item)
 {
@@ -993,7 +1005,7 @@ sp_arc_start_set (SPItem *item, const ArtPoint *p, guint state)
 	ge = SP_GENERICELLIPSE (item);
 	arc = SP_ARC(item);
 
-	arc->is_closed = (sp_genericellipse_side (ge, p) == -1) ? TRUE : FALSE;
+	ge->closed = (sp_genericellipse_side (ge, p) == -1) ? TRUE : FALSE;
 
 	dx = p->x - ge->cx.computed;
 	dy = p->y - ge->cy.computed;
@@ -1027,7 +1039,7 @@ sp_arc_end_set (SPItem *item, const ArtPoint *p, guint state)
 	ge = SP_GENERICELLIPSE (item);
 	arc = SP_ARC(item);
 
-	arc->is_closed = (sp_genericellipse_side (ge, p) == -1) ? TRUE : FALSE;
+	ge->closed = (sp_genericellipse_side (ge, p) == -1) ? TRUE : FALSE;
 
 	dx = p->x - ge->cx.computed;
 	dy = p->y - ge->cy.computed;
@@ -1057,7 +1069,7 @@ sp_arc_knot_holder (SPItem *item, SPDesktop *desktop)
 	SPKnotHolder *knot_holder;
 
 	arc = SP_ARC (item);
-	knot_holder = sp_knot_holder_new (desktop, item);
+	knot_holder = sp_knot_holder_new (desktop, item, NULL);
 	
 	sp_knot_holder_add (knot_holder,
 			    sp_arc_start_set,
