@@ -15,6 +15,9 @@
 #include <math.h>
 #include <string.h>
 #include <stdlib.h>
+
+#include <libnr/nr-gradient.h>
+
 #include <libart_lgpl/art_affine.h>
 #include <gtk/gtksignal.h>
 
@@ -29,13 +32,14 @@
 #include "macros.h"
 
 /* Has to be power of 2 */
-#define NCOLORS 1024
+#define NCOLORS NR_GRADIENT_VECTOR_LENGTH
 
 static void sp_stop_class_init (SPStopClass * klass);
 static void sp_stop_init (SPStop * stop);
 
 static void sp_stop_build (SPObject * object, SPDocument * document, SPRepr * repr);
 static void sp_stop_read_attr (SPObject * object, const gchar * key);
+static SPRepr *sp_stop_write (SPObject *object, SPRepr *repr, guint flags);
 
 static SPObjectClass * stop_parent_class;
 
@@ -68,6 +72,7 @@ sp_stop_class_init (SPStopClass * klass)
 
 	sp_object_class->build = sp_stop_build;
 	sp_object_class->read_attr = sp_stop_read_attr;
+	sp_object_class->write = sp_stop_write;
 }
 
 static void
@@ -80,8 +85,8 @@ sp_stop_init (SPStop *stop)
 static void
 sp_stop_build (SPObject * object, SPDocument * document, SPRepr * repr)
 {
-	if (SP_OBJECT_CLASS (stop_parent_class)->build)
-		(* SP_OBJECT_CLASS (stop_parent_class)->build) (object, document, repr);
+	if (((SPObjectClass *) stop_parent_class)->build)
+		(* ((SPObjectClass *) stop_parent_class)->build) (object, document, repr);
 
 	sp_stop_read_attr (object, "offset");
 	sp_stop_read_attr (object, "stop-color");
@@ -129,28 +134,54 @@ sp_stop_read_attr (SPObject * object, const gchar * key)
 		stop->offset = sp_svg_read_percentage (str, 0.0);
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
 	} else {
-		if (SP_OBJECT_CLASS (stop_parent_class)->read_attr)
-			(* SP_OBJECT_CLASS (stop_parent_class)->read_attr) (object, key);
+		if (((SPObjectClass *) stop_parent_class)->read_attr)
+			(* ((SPObjectClass *) stop_parent_class)->read_attr) (object, key);
 	}
 }
 
+static SPRepr *
+sp_stop_write (SPObject *object, SPRepr *repr, guint flags)
+{
+	SPStop *stop;
+	unsigned char c[64], s[1024];
+
+	stop = SP_STOP (object);
+
+	if ((flags & SP_OBJECT_WRITE_BUILD) & !repr) {
+		repr = sp_repr_new ("stop");
+	}
+
+	sp_svg_write_color (c, 64, sp_color_get_rgba32_ualpha (&stop->color, 255));
+	g_snprintf (s, 1024, "stop-color:%s;stop-opacity:%g;", c, stop->opacity);
+	sp_repr_set_attr (repr, "style", s);
+	sp_repr_set_attr (repr, "stop-color", NULL);
+	sp_repr_set_attr (repr, "stop-opacity", NULL);
+	sp_repr_set_double (repr, "offset", stop->offset);
+
+	if (((SPObjectClass *) stop_parent_class)->write)
+		(* ((SPObjectClass *) stop_parent_class)->write) (object, repr, flags);
+
+	return repr;
+}
+
 /*
  * Gradient
  */
 
 static void sp_gradient_class_init (SPGradientClass *klass);
 static void sp_gradient_init (SPGradient *gr);
-static void sp_gradient_destroy (GtkObject *object);
 
 static void sp_gradient_build (SPObject *object, SPDocument *document, SPRepr *repr);
+static void sp_gradient_release (SPObject *object);
 static void sp_gradient_read_attr (SPObject *object, const gchar *key);
 static void sp_gradient_child_added (SPObject *object, SPRepr *child, SPRepr *ref);
 static void sp_gradient_remove_child (SPObject *object, SPRepr *child);
 static void sp_gradient_modified (SPObject *object, guint flags);
+static SPRepr *sp_gradient_write (SPObject *object, SPRepr *repr, guint flags);
 
 static void sp_gradient_flatten_attributes (SPGradient *gradient, SPRepr *repr, gboolean set_missing);
 
-static void sp_gradient_href_destroy (SPObject *href, SPGradient *gradient);
+static void sp_gradient_href_release (SPObject *href, SPGradient *gradient);
 static void sp_gradient_href_modified (SPObject *href, guint flags, SPGradient *gradient);
 
 static void sp_gradient_invalidate_vector (SPGradient *gr);
@@ -187,13 +218,13 @@ sp_gradient_class_init (SPGradientClass *klass)
 
 	gradient_parent_class = gtk_type_class (SP_TYPE_PAINT_SERVER);
 
-	gtk_object_class->destroy = sp_gradient_destroy;
-
 	sp_object_class->build = sp_gradient_build;
+	sp_object_class->release = sp_gradient_release;
 	sp_object_class->read_attr = sp_gradient_read_attr;
 	sp_object_class->child_added = sp_gradient_child_added;
 	sp_object_class->remove_child = sp_gradient_remove_child;
 	sp_object_class->modified = sp_gradient_modified;
+	sp_object_class->write = sp_gradient_write;
 
 	klass->flatten_attributes = sp_gradient_flatten_attributes;
 }
@@ -221,46 +252,6 @@ sp_gradient_init (SPGradient *gr)
 	gr->len = 0.0;
 }
 
-static void
-sp_gradient_destroy (GtkObject *object)
-{
-	SPGradient * gradient;
-
-	gradient = (SPGradient *) object;
-
-	if (SP_OBJECT_DOCUMENT (object)) {
-		/* Unregister ourselves */
-		sp_document_remove_resource (SP_OBJECT_DOCUMENT (object), "gradient", SP_OBJECT (object));
-	}
-
-	if (gradient->href) {
-		gtk_signal_disconnect_by_data (GTK_OBJECT (gradient->href), gradient);
-		gradient->href = (SPGradient *) sp_object_hunref (SP_OBJECT (gradient->href), object);
-	}
-
-	if (gradient->color) {
-		g_free (gradient->color);
-		gradient->color = NULL;
-	}
-
-	if (gradient->vector) {
-		g_free (gradient->vector);
-		gradient->vector = NULL;
-	}
-
-	while (gradient->stops) {
-		SPObject *o;
-		o = gradient->stops;
-		gradient->stops = o->next;
-		o->parent = NULL;
-		o->next = NULL;
-		gtk_object_unref (GTK_OBJECT (o));
-	}
-
-	if (GTK_OBJECT_CLASS (gradient_parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (gradient_parent_class)->destroy) (object);
-}
-
 static void
 sp_gradient_build (SPObject *object, SPDocument *document, SPRepr *repr)
 {
@@ -270,8 +261,8 @@ sp_gradient_build (SPObject *object, SPDocument *document, SPRepr *repr)
 
 	gradient = SP_GRADIENT (object);
 
-	if (SP_OBJECT_CLASS (gradient_parent_class)->build)
-		(* SP_OBJECT_CLASS (gradient_parent_class)->build) (object, document, repr);
+	if (((SPObjectClass *) gradient_parent_class)->build)
+		(* ((SPObjectClass *) gradient_parent_class)->build) (object, document, repr);
 
 	/* fixme: Add all children, not only stops? */
 	last = NULL;
@@ -298,6 +289,41 @@ sp_gradient_build (SPObject *object, SPDocument *document, SPRepr *repr)
 	sp_document_add_resource (document, "gradient", object);
 }
 
+static void
+sp_gradient_release (SPObject *object)
+{
+	SPGradient *gradient;
+
+	gradient = (SPGradient *) object;
+
+	if (SP_OBJECT_DOCUMENT (object)) {
+		/* Unregister ourselves */
+		sp_document_remove_resource (SP_OBJECT_DOCUMENT (object), "gradient", SP_OBJECT (object));
+	}
+
+	if (gradient->href) {
+		gtk_signal_disconnect_by_data (GTK_OBJECT (gradient->href), gradient);
+		gradient->href = (SPGradient *) sp_object_hunref (SP_OBJECT (gradient->href), object);
+	}
+
+	if (gradient->color) {
+		g_free (gradient->color);
+		gradient->color = NULL;
+	}
+
+	if (gradient->vector) {
+		g_free (gradient->vector);
+		gradient->vector = NULL;
+	}
+
+	while (gradient->stops) {
+		gradient->stops = sp_object_detach_unref (object, gradient->stops);
+	}
+
+	if (((SPObjectClass *) gradient_parent_class)->release)
+		((SPObjectClass *) gradient_parent_class)->release (object);
+}
+
 static void
 sp_gradient_read_attr (SPObject *object, const gchar *key)
 {
@@ -357,10 +383,8 @@ sp_gradient_read_attr (SPObject *object, const gchar *key)
 			href = sp_document_lookup_id (object->document, val + 1);
 			if (SP_IS_GRADIENT (href)) {
 				gr->href = (SPGradient *) sp_object_href (href, object);
-				gtk_signal_connect (GTK_OBJECT (href), "destroy",
-						    GTK_SIGNAL_FUNC (sp_gradient_href_destroy), gr);
-				gtk_signal_connect (GTK_OBJECT (href), "modified",
-						    GTK_SIGNAL_FUNC (sp_gradient_href_modified), gr);
+				gtk_signal_connect (GTK_OBJECT (href), "release", GTK_SIGNAL_FUNC (sp_gradient_href_release), gr);
+				gtk_signal_connect (GTK_OBJECT (href), "modified", GTK_SIGNAL_FUNC (sp_gradient_href_modified), gr);
 			}
 		}
 		sp_gradient_invalidate_vector (gr);
@@ -368,8 +392,8 @@ sp_gradient_read_attr (SPObject *object, const gchar *key)
 		return;
 	}
 
-	if (SP_OBJECT_CLASS (gradient_parent_class)->read_attr)
-		(* SP_OBJECT_CLASS (gradient_parent_class)->read_attr) (object, key);
+	if (((SPObjectClass *) gradient_parent_class)->read_attr)
+		(* ((SPObjectClass *) gradient_parent_class)->read_attr) (object, key);
 }
 
 static void
@@ -486,6 +510,42 @@ sp_gradient_modified (SPObject *object, guint flags)
 	}
 }
 
+static SPRepr *
+sp_gradient_write (SPObject *object, SPRepr *repr, guint flags)
+{
+	SPGradient *gr;
+
+	gr = SP_GRADIENT (object);
+
+	if (((SPObjectClass *) gradient_parent_class)->write)
+		(* ((SPObjectClass *) gradient_parent_class)->write) (object, repr, flags);
+
+	if (flags & SP_OBJECT_WRITE_BUILD) {
+		SPObject *child;
+		GSList *l;
+		l = NULL;
+		for (child = gr->stops; child != NULL; child = child->next) {
+			SPRepr *crepr;
+			crepr = sp_object_invoke_write (child, NULL, flags);
+			if (crepr) l = g_slist_prepend (l, crepr);
+		}
+		while (l) {
+			sp_repr_add_child (repr, (SPRepr *) l->data, NULL);
+			sp_repr_unref ((SPRepr *) l->data);
+			l = g_slist_remove (l, l->data);
+		}
+	}
+
+	if (gr->href) {
+		unsigned char *str;
+		str = g_strdup_printf ("#%s", SP_OBJECT_ID (gr->href));
+		sp_repr_set_attr (repr, "xlink:href", str);
+		g_free (str);
+	}
+	/* fixme: (Lauris) */
+	sp_gradient_flatten_attributes (SP_GRADIENT (object), repr, TRUE);
+}
+
 static void
 sp_gradient_flatten_attributes (SPGradient *gr, SPRepr *repr, gboolean set_missing)
 {
@@ -561,6 +621,24 @@ sp_gradient_set_vector (SPGradient *gradient, SPGradientVector *vector)
 	sp_object_request_modified (SP_OBJECT (gradient), SP_OBJECT_MODIFIED_FLAG);
 }
 
+void
+sp_gradient_set_units (SPGradient *gr, unsigned int units)
+{
+	if (units != gr->units) {
+		gr->units = units;
+		sp_object_request_modified (SP_OBJECT (gr), SP_OBJECT_MODIFIED_FLAG);
+	}
+}
+
+void
+sp_gradient_set_spread (SPGradient *gr, unsigned int spread)
+{
+	if (spread != gr->spread) {
+		gr->spread = spread;
+		sp_object_request_modified (SP_OBJECT (gr), SP_OBJECT_MODIFIED_FLAG);
+	}
+}
+
 /* Gradient repr methods */
 void
 sp_gradient_repr_flatten_attributes (SPGradient *gr, SPRepr *repr, gboolean set_missing)
@@ -622,7 +700,7 @@ sp_gradient_repr_set_vector (SPGradient *gr, SPRepr *repr, SPGradientVector *vec
 }
 
 static void
-sp_gradient_href_destroy (SPObject *href, SPGradient *gradient)
+sp_gradient_href_release (SPObject *href, SPGradient *gradient)
 {
 	gradient->href = (SPGradient *) sp_object_hunref (href, gradient);
 	sp_gradient_invalidate_vector (gradient);
@@ -770,8 +848,8 @@ sp_gradient_ensure_colors (SPGradient *gr)
 		g1 = (color >> 16) & 0xff;
 		b1 = (color >> 8) & 0xff;
 		a1 = color & 0xff;
-		o0 = (gint) floor (gr->vector->stops[i].offset * (NCOLORS + 0.9999));
-		o1 = (gint) floor (gr->vector->stops[i + 1].offset * (NCOLORS + 0.9999));
+		o0 = (gint) floor (gr->vector->stops[i].offset * (NCOLORS - 0.001));
+		o1 = (gint) floor (gr->vector->stops[i + 1].offset * (NCOLORS - 0.001));
 		if (o1 > o0) {
 			dr = ((r1 - r0) << 16) / (o1 - o0);
 			dg = ((g1 - g0) << 16) / (o1 - o0);
@@ -942,12 +1020,12 @@ sp_gradient_render_vector_block_rgb (SPGradient *gradient, guchar *buf, gint wid
 			for (x = 0; x < width; x++) {
 				gint a, fc;
 				a = t[3];
-				fc = (t[0] - *buf) * a;
-				buf[0] = *buf + ((fc + (fc >> 8) + 0x80) >> 8);
-				fc = (t[1] - *buf) * a;
-				buf[1] = *buf + ((fc + (fc >> 8) + 0x80) >> 8);
-				fc = (t[2] - *buf) * a;
-				buf[2] = *buf + ((fc + (fc >> 8) + 0x80) >> 8);
+				fc = (t[0] - buf[0]) * a;
+				buf[0] = buf[0] + ((fc + (fc >> 8) + 0x80) >> 8);
+				fc = (t[1] - buf[1]) * a;
+				buf[1] = buf[1] + ((fc + (fc >> 8) + 0x80) >> 8);
+				fc = (t[2] - buf[2]) * a;
+				buf[2] = buf[2] + ((fc + (fc >> 8) + 0x80) >> 8);
 				buf += 3;
 				t += 4;
 			}
@@ -964,17 +1042,167 @@ sp_gradient_render_vector_block_rgb (SPGradient *gradient, guchar *buf, gint wid
 			for (x = 0; x < width; x++) {
 				gint a, fc;
 				a = t[3];
-				fc = (t[0] - *buf) * a;
-				buf[0] = *buf + ((fc + (fc >> 8) + 0x80) >> 8);
-				fc = (t[1] - *buf) * a;
-				buf[1] = *buf + ((fc + (fc >> 8) + 0x80) >> 8);
-				fc = (t[2] - *buf) * a;
-				buf[2] = *buf + ((fc + (fc >> 8) + 0x80) >> 8);
+				fc = (t[0] - buf[0]) * a;
+				buf[0] = buf[0] + ((fc + (fc >> 8) + 0x80) >> 8);
+				fc = (t[1] - buf[1]) * a;
+				buf[1] = buf[1] + ((fc + (fc >> 8) + 0x80) >> 8);
+				fc = (t[2] - buf[2]) * a;
+				buf[2] = buf[2] + ((fc + (fc >> 8) + 0x80) >> 8);
 			}
 		}
 	}
 }
 
+NRMatrixF *
+sp_gradient_get_g2d_matrix_f (SPGradient *gr, NRMatrixF *ctm, NRRectF *bbox, NRMatrixF *g2d)
+{
+	if (gr->units == SP_GRADIENT_UNITS_OBJECTBOUNDINGBOX) {
+		NRMatrixF bb2u;
+
+		bb2u.c[0] = bbox->x1 - bbox->x0;
+		bb2u.c[1] = 0.0;
+		bb2u.c[2] = 0.0;
+		bb2u.c[3] = bbox->y1 - bbox->y0;
+		bb2u.c[4] = bbox->x0;
+		bb2u.c[5] = bbox->y0;
+
+		nr_matrix_multiply_fff (g2d, &bb2u, ctm);
+	} else {
+		*g2d = *ctm;
+	}
+
+	return g2d;
+}
+
+NRMatrixF *
+sp_gradient_get_gs2d_matrix_f (SPGradient *gr, NRMatrixF *ctm, NRRectF *bbox, NRMatrixF *gs2d)
+{
+	if (gr->units == SP_GRADIENT_UNITS_OBJECTBOUNDINGBOX) {
+		NRMatrixF bb2u, gs2u;
+
+		bb2u.c[0] = bbox->x1 - bbox->x0;
+		bb2u.c[1] = 0.0;
+		bb2u.c[2] = 0.0;
+		bb2u.c[3] = bbox->y1 - bbox->y0;
+		bb2u.c[4] = bbox->x0;
+		bb2u.c[5] = bbox->y0;
+
+		nr_matrix_multiply_fdf (&gs2u, (NRMatrixD *) gr->transform, &bb2u);
+		nr_matrix_multiply_fff (gs2d, &gs2u, ctm);
+	} else {
+		nr_matrix_multiply_fdf (gs2d, (NRMatrixD *) gr->transform, ctm);
+	}
+
+	return gs2d;
+}
+
+void
+sp_gradient_set_gs2d_matrix_f (SPGradient *gr, NRMatrixF *ctm, NRRectF *bbox, NRMatrixF *gs2d)
+{
+	NRMatrixF g2d, d2g, gs2g;
+
+	SP_PRINT_MATRIX ("* GS2D:", gs2d);
+
+	if (gr->units == SP_GRADIENT_UNITS_OBJECTBOUNDINGBOX) {
+		NRMatrixF bb2u;
+
+		bb2u.c[0] = bbox->x1 - bbox->x0;
+		bb2u.c[1] = 0.0;
+		bb2u.c[2] = 0.0;
+		bb2u.c[3] = bbox->y1 - bbox->y0;
+		bb2u.c[4] = bbox->x0;
+		bb2u.c[5] = bbox->y0;
+
+		SP_PRINT_MATRIX ("* BB2U:", &bb2u);
+
+		nr_matrix_multiply_fff (&g2d, &bb2u, ctm);
+	} else {
+		g2d = *ctm;
+	}
+
+	SP_PRINT_MATRIX ("* G2D:", &g2d);
+
+	nr_matrix_f_invert (&d2g, &g2d);
+	SP_PRINT_MATRIX ("* D2G:", &d2g);
+	SP_PRINT_MATRIX ("* G2D:", &g2d);
+	nr_matrix_f_invert (&g2d, &d2g);
+	SP_PRINT_MATRIX ("* D2G:", &d2g);
+	SP_PRINT_MATRIX ("* G2D:", &g2d);
+
+
+	nr_matrix_multiply_fff (&gs2g, gs2d, &d2g);
+	SP_PRINT_MATRIX ("* GS2G:", &gs2g);
+
+	gr->transform[0] = gs2g.c[0];
+	gr->transform[1] = gs2g.c[1];
+	gr->transform[2] = gs2g.c[2];
+	gr->transform[3] = gs2g.c[3];
+	gr->transform[4] = gs2g.c[4];
+	gr->transform[5] = gs2g.c[5];
+
+	sp_object_request_modified (SP_OBJECT (gr), SP_OBJECT_MODIFIED_FLAG);
+}
+
+void
+sp_gradient_from_position_xy (SPGradient *gr, gdouble *ctm, ArtDRect *bbox, NRPointF *p, float x, float y)
+{
+	NRMatrixF gs2d;
+
+	g_return_if_fail (gr != NULL);
+	g_return_if_fail (SP_IS_GRADIENT (gr));
+	g_return_if_fail (p != NULL);
+
+	if (gr->units == SP_GRADIENT_UNITS_OBJECTBOUNDINGBOX) {
+		NRMatrixF bb2u, gs2u;
+
+		bb2u.c[0] = bbox->x1 - bbox->x0;
+		bb2u.c[1] = 0.0;
+		bb2u.c[2] = 0.0;
+		bb2u.c[3] = bbox->y1 - bbox->y0;
+		bb2u.c[4] = bbox->x0;
+		bb2u.c[5] = bbox->y0;
+
+		nr_matrix_multiply_fdf (&gs2u, (NRMatrixD *) gr->transform, &bb2u);
+		nr_matrix_multiply_ffd (&gs2d, &gs2u, (NRMatrixD *) ctm);
+	} else {
+		nr_matrix_multiply_fdd (&gs2d, (NRMatrixD *) gr->transform, (NRMatrixD *) ctm);
+	}
+
+	p->x = gs2d.c[0] * x + gs2d.c[2] * y + gs2d.c[4];
+	p->y = gs2d.c[1] * x + gs2d.c[3] * y + gs2d.c[5];
+}
+
+void
+sp_gradient_to_position_xy (SPGradient *gr, gdouble *ctm, ArtDRect *bbox, NRPointF *p, float x, float y)
+{
+	NRMatrixF gs2d, d2gs;
+
+	g_return_if_fail (gr != NULL);
+	g_return_if_fail (SP_IS_GRADIENT (gr));
+	g_return_if_fail (p != NULL);
+
+	if (gr->units == SP_GRADIENT_UNITS_OBJECTBOUNDINGBOX) {
+		NRMatrixF bb2u, gs2u;
+
+		bb2u.c[0] = bbox->x1 - bbox->x0;
+		bb2u.c[1] = 0.0;
+		bb2u.c[2] = 0.0;
+		bb2u.c[3] = bbox->y1 - bbox->y0;
+		bb2u.c[4] = bbox->x0;
+		bb2u.c[5] = bbox->y0;
+
+		nr_matrix_multiply_fdf (&gs2u, (NRMatrixD *) gr->transform, &bb2u);
+		nr_matrix_multiply_ffd (&gs2d, &gs2u, (NRMatrixD *) ctm);
+	} else {
+		nr_matrix_multiply_fdd (&gs2d, (NRMatrixD *) gr->transform, (NRMatrixD *) ctm);
+	}
+
+	nr_matrix_f_invert (&d2gs, &gs2d);
+
+	p->x = d2gs.c[0] * x + d2gs.c[2] * y + d2gs.c[4];
+	p->y = d2gs.c[1] * x + d2gs.c[3] * y + d2gs.c[5];
+}
+
 /*
  * Linear Gradient
  */
@@ -984,17 +1212,16 @@ typedef struct _SPLGPainter SPLGPainter;
 struct _SPLGPainter {
 	SPPainter painter;
 	SPLinearGradient *lg;
-	gdouble len;
-	gdouble x0, y0;
-	gdouble dx, dy;
+
+	NRLGradientRenderer lgr;
 };
 
 static void sp_lineargradient_class_init (SPLinearGradientClass * klass);
 static void sp_lineargradient_init (SPLinearGradient * lg);
-static void sp_lineargradient_destroy (GtkObject * object);
 
-static void sp_lineargradient_build (SPObject * object, SPDocument * document, SPRepr * repr);
+static void sp_lineargradient_build (SPObject *object, SPDocument * document, SPRepr * repr);
 static void sp_lineargradient_read_attr (SPObject * object, const gchar * key);
+static SPRepr *sp_lineargradient_write (SPObject *object, SPRepr *repr, guint flags);
 
 static SPPainter *sp_lineargradient_painter_new (SPPaintServer *ps, const gdouble *affine, const ArtDRect *bbox);
 static void sp_lineargradient_painter_free (SPPaintServer *ps, SPPainter *painter);
@@ -1038,10 +1265,9 @@ sp_lineargradient_class_init (SPLinearGradientClass * klass)
 
 	lg_parent_class = gtk_type_class (SP_TYPE_GRADIENT);
 
-	gtk_object_class->destroy = sp_lineargradient_destroy;
-
 	sp_object_class->build = sp_lineargradient_build;
 	sp_object_class->read_attr = sp_lineargradient_read_attr;
+	sp_object_class->write = sp_lineargradient_write;
 
 	ps_class->painter_new = sp_lineargradient_painter_new;
 	ps_class->painter_free = sp_lineargradient_painter_free;
@@ -1058,17 +1284,6 @@ sp_lineargradient_init (SPLinearGradient * lg)
 	sp_svg_length_unset (&lg->y2, SP_SVG_UNIT_PERCENT, 0.0, 0.0);
 }
 
-static void
-sp_lineargradient_destroy (GtkObject * object)
-{
-	SPLinearGradient * lg;
-
-	lg = (SPLinearGradient *) object;
-
-	if (GTK_OBJECT_CLASS (lg_parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (lg_parent_class)->destroy) (object);
-}
-
 static void
 sp_lineargradient_build (SPObject * object, SPDocument * document, SPRepr * repr)
 {
@@ -1076,8 +1291,8 @@ sp_lineargradient_build (SPObject * object, SPDocument * document, SPRepr * repr
 
 	lg = SP_LINEARGRADIENT (object);
 
-	if (SP_OBJECT_CLASS (lg_parent_class)->build)
-		(* SP_OBJECT_CLASS (lg_parent_class)->build) (object, document, repr);
+	if (((SPObjectClass *) lg_parent_class)->build)
+		(* ((SPObjectClass *) lg_parent_class)->build) (object, document, repr);
 
 	sp_lineargradient_read_attr (object, "x1");
 	sp_lineargradient_read_attr (object, "y1");
@@ -1116,11 +1331,33 @@ sp_lineargradient_read_attr (SPObject * object, const gchar * key)
 		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
 	} else {
-		if (SP_OBJECT_CLASS (lg_parent_class)->read_attr)
-			(* SP_OBJECT_CLASS (lg_parent_class)->read_attr) (object, key);
+		if (((SPObjectClass *) lg_parent_class)->read_attr)
+			(* ((SPObjectClass *) lg_parent_class)->read_attr) (object, key);
 	}
 }
 
+static SPRepr *
+sp_lineargradient_write (SPObject *object, SPRepr *repr, guint flags)
+{
+	SPLinearGradient * lg;
+
+	lg = SP_LINEARGRADIENT (object);
+
+	if ((flags & SP_OBJECT_WRITE_BUILD) && !repr) {
+		repr = sp_repr_new ("linearGradient");
+	}
+
+	sp_repr_set_double (repr, "x1", lg->x1.computed);
+	sp_repr_set_double (repr, "y1", lg->y1.computed);
+	sp_repr_set_double (repr, "x2", lg->x2.computed);
+	sp_repr_set_double (repr, "y2", lg->y2.computed);
+
+	if (((SPObjectClass *) lg_parent_class)->write)
+		(* ((SPObjectClass *) lg_parent_class)->write) (object, repr, flags);
+
+	return repr;
+}
+
 /*
  * Basically we have to deal with transformations
  *
@@ -1138,7 +1375,8 @@ sp_lineargradient_painter_new (SPPaintServer *ps, const gdouble *ctm, const ArtD
 	SPLinearGradient *lg;
 	SPGradient *gr;
 	SPLGPainter *lgp;
-	gdouble color2norm[6];
+	gdouble color2norm[6], color2px[6];
+	NRMatrixF v2px;
 
 	lg = SP_LINEARGRADIENT (ps);
 	gr = SP_GRADIENT (ps);
@@ -1156,14 +1394,19 @@ sp_lineargradient_painter_new (SPPaintServer *ps, const gdouble *ctm, const ArtD
 	/* fixme: I almost think, we should fill color array start and end in that case (Lauris) */
 	/* fixme: The alternative would be to leave these just empty garbage or something similar (Lauris) */
 	/* fixme: Originally I had 1023.9999 here - not sure, whether we have really to cut out ceil int (Lauris) */
+#if 0
 	art_affine_scale (color2norm, gr->len / (gdouble) NCOLORS, gr->len / (gdouble) NCOLORS);
 	SP_PRINT_TRANSFORM ("color2norm", color2norm);
 	/* Now we have normalized vector */
+#else
+	art_affine_identity (color2norm);
+#endif
 
 	if (gr->units == SP_GRADIENT_UNITS_OBJECTBOUNDINGBOX) {
 		gdouble norm2pos[6], bbox2user[6];
-		gdouble color2pos[6], color2tpos[6], color2user[6], color2px[6], px2color[6];
+		gdouble color2pos[6], color2tpos[6], color2user[6];
 
+#if 0
 		/* This is easy case, as we can just ignore percenting here */
 		/* fixme: Still somewhat tricky, but I think I got it correct (lauris) */
 		norm2pos[0] = lg->x2.computed - lg->x1.computed;
@@ -1173,6 +1416,9 @@ sp_lineargradient_painter_new (SPPaintServer *ps, const gdouble *ctm, const ArtD
 		norm2pos[4] = lg->x1.computed;
 		norm2pos[5] = lg->y1.computed;
 		SP_PRINT_TRANSFORM ("norm2pos", norm2pos);
+#else
+		art_affine_identity (norm2pos);
+#endif
 
 		/* gradientTransform goes here (Lauris) */
 		SP_PRINT_TRANSFORM ("gradientTransform", gr->transform);
@@ -1197,20 +1443,13 @@ sp_lineargradient_painter_new (SPPaintServer *ps, const gdouble *ctm, const ArtD
 		SP_PRINT_TRANSFORM ("color2user", color2user);
 		art_affine_multiply (color2px, color2user, ctm);
 		SP_PRINT_TRANSFORM ("color2px", color2px);
-
-		art_affine_invert (px2color, color2px);
-		SP_PRINT_TRANSFORM ("px2color", px2color);
-
-		lgp->x0 = color2px[4];
-		lgp->y0 = color2px[5];
-		lgp->dx = px2color[0];
-		lgp->dy = px2color[2];
 	} else {
 		gdouble norm2pos[6];
-		gdouble color2pos[6], color2tpos[6], color2px[6], px2color[6];
+		gdouble color2pos[6], color2tpos[6];
 		/* Problem: What to do, if we have mixed lengths and percentages? */
 		/* Currently we do ignore percentages at all, but that is not good (lauris) */
 
+#if 0
 		/* fixme: Do percentages (Lauris) */
 		norm2pos[0] = lg->x2.computed - lg->x1.computed;
 		norm2pos[1] = lg->y2.computed - lg->y1.computed;
@@ -1219,6 +1458,9 @@ sp_lineargradient_painter_new (SPPaintServer *ps, const gdouble *ctm, const ArtD
 		norm2pos[4] = lg->x1.computed;
 		norm2pos[5] = lg->y1.computed;
 		SP_PRINT_TRANSFORM ("norm2pos", norm2pos);
+#else
+		art_affine_identity (norm2pos);
+#endif
 
 		/* gradientTransform goes here (Lauris) */
 		SP_PRINT_TRANSFORM ("gradientTransform", gr->transform);
@@ -1232,16 +1474,18 @@ sp_lineargradient_painter_new (SPPaintServer *ps, const gdouble *ctm, const ArtD
 		SP_PRINT_TRANSFORM ("color2tpos", color2tpos);
 		art_affine_multiply (color2px, color2tpos, ctm);
 		SP_PRINT_TRANSFORM ("color2px", color2px);
-
-		art_affine_invert (px2color, color2px);
-		SP_PRINT_TRANSFORM ("px2color", px2color);
-
-		lgp->x0 = color2px[4];
-		lgp->y0 = color2px[5];
-		lgp->dx = px2color[0];
-		lgp->dy = px2color[2];
 	}
 
+	v2px.c[0] = color2px[0];
+	v2px.c[1] = color2px[1];
+	v2px.c[2] = color2px[2];
+	v2px.c[3] = color2px[3];
+	v2px.c[4] = color2px[4];
+	v2px.c[5] = color2px[5];
+
+	nr_lgradient_renderer_setup (&lgp->lgr, gr->color, gr->spread, &v2px,
+				     lg->x1.computed, lg->y1.computed, lg->x2.computed, lg->y2.computed);
+
 	return (SPPainter *) lgp;
 }
 
@@ -1269,84 +1513,6 @@ sp_lineargradient_flatten_attributes (SPGradient *gr, SPRepr *repr, gboolean set
 	if (set_missing || lg->y2.set) sp_repr_set_double_attribute (repr, "y2", lg->y2.computed);
 }
 
-void
-sp_lineargradient_from_position (SPLinearGradient *lg, gdouble *ctm, ArtDRect *bbox, ArtPoint *p)
-{
-	SPGradient *gr;
-	gdouble p2b[6];
-	gdouble x, y;
-
-	g_return_if_fail (lg != NULL);
-	g_return_if_fail (SP_IS_LINEARGRADIENT (lg));
-	g_return_if_fail (p != NULL);
-
-	gr = SP_GRADIENT (lg);
-
-	if (gr->units == SP_GRADIENT_UNITS_OBJECTBOUNDINGBOX) {
-		gdouble bbox2user[6];
-
-		/* BBox to user coordinate system */
-		bbox2user[0] = bbox->x1 - bbox->x0;
-		bbox2user[1] = 0.0;
-		bbox2user[2] = 0.0;
-		bbox2user[3] = bbox->y1 - bbox->y0;
-		bbox2user[4] = bbox->x0;
-		bbox2user[5] = bbox->y0;
-
-		/* CTM goes here */
-
-		art_affine_multiply (p2b, bbox2user, ctm);
-	} else {
-		memcpy (p2b, ctm, 6 * sizeof (gdouble));
-	}
-
-	x = p2b[0] * p->x + p2b[2] * p->y + p2b[4];
-	y = p2b[1] * p->x + p2b[3] * p->y + p2b[5];
-
-	p->x = x;
-	p->y = y;
-}
-
-void
-sp_lineargradient_to_position (SPLinearGradient *lg, gdouble *ctm, ArtDRect *bbox, ArtPoint *p)
-{
-	SPGradient *gr;
-	gdouble p2b[6], b2p[6];
-	gdouble x, y;
-
-	g_return_if_fail (lg != NULL);
-	g_return_if_fail (SP_IS_LINEARGRADIENT (lg));
-	g_return_if_fail (p != NULL);
-
-	gr = SP_GRADIENT (lg);
-
-	if (gr->units == SP_GRADIENT_UNITS_OBJECTBOUNDINGBOX) {
-		gdouble bbox2user[6];
-
-		/* BBox to user coordinate system */
-		bbox2user[0] = bbox->x1 - bbox->x0;
-		bbox2user[1] = 0.0;
-		bbox2user[2] = 0.0;
-		bbox2user[3] = bbox->y1 - bbox->y0;
-		bbox2user[4] = bbox->x0;
-		bbox2user[5] = bbox->y0;
-
-		/* CTM goes here */
-
-		art_affine_multiply (p2b, bbox2user, ctm);
-
-		art_affine_invert (b2p, p2b);
-	} else {
-		art_affine_invert (b2p, ctm);
-	}
-
-	x = b2p[0] * p->x + b2p[2] * p->y + b2p[4];
-	y = b2p[1] * p->x + b2p[3] * p->y + b2p[5];
-
-	p->x = x;
-	p->y = y;
-}
-
 void
 sp_lineargradient_set_position (SPLinearGradient *lg, gdouble x1, gdouble y1, gdouble x2, gdouble y2)
 {
@@ -1388,47 +1554,15 @@ static void
 sp_lg_fill (SPPainter *painter, guchar *px, gint x0, gint y0, gint width, gint height, gint rowstride)
 {
 	SPLGPainter *lgp;
-	SPLinearGradient *lg;
-	SPGradient *g;
-	gdouble pos;
-	gint x, y;
-	guchar *p;
+	NRPixBlock pb;
 
 	lgp = (SPLGPainter *) painter;
-	lg = lgp->lg;
-	g = (SPGradient *) lg;
 
-	if (!g->color) {
-#if 0
-		/* fixme: This is forbidden, so we should paint some mishmesh instead */
-		sp_gradient_ensure_colors (g);
-#else
-		nr_render_r8g8b8a8_gray_garbage (px, width, height, rowstride);
-		return;
-#endif
-	}
+	nr_pixblock_setup_extern (&pb, NR_PIXBLOCK_MODE_R8G8B8A8N, x0, y0, x0 + width, y0 + height, px, rowstride, FALSE, FALSE);
 
-	for (y = 0; y < height; y++) {
-		p = px + y * rowstride;
-		pos = (y + y0 - lgp->y0) * lgp->dy + (0 + x0 - lgp->x0) * lgp->dx;
-		for (x = 0; x < width; x++) {
-			gint idx;
-			if (g->spread == SP_GRADIENT_SPREAD_REFLECT) {
-				idx = ((gint) pos) & (2 * NCOLORS - 1);
-				if (idx & NCOLORS) idx = (2 * NCOLORS) - idx;
-			} else if (g->spread == SP_GRADIENT_SPREAD_REPEAT) {
-				idx = ((gint) pos) & (NCOLORS - 1);
-			} else {
-				idx = CLAMP (((gint) pos), 0, (NCOLORS - 1));
-			}
-			idx = idx * 4;
-			*p++ = g->color[idx];
-			*p++ = g->color[idx + 1];
-			*p++ = g->color[idx + 2];
-			*p++ = g->color[idx + 3];
-			pos += lgp->dx;
-		}
-	}
+	nr_render ((NRRenderer *) &lgp->lgr, &pb, NULL);
+
+	nr_pixblock_release (&pb);
 }
 
 /*
@@ -1440,18 +1574,15 @@ typedef struct _SPRGPainter SPRGPainter;
 struct _SPRGPainter {
 	SPPainter painter;
 	SPRadialGradient *rg;
-	gdouble len;
-	gdouble x0, y0;
-	gdouble dx, dy;
-	guint32 opacity;
+	NRRGradientRenderer rgr;
 };
 
 static void sp_radialgradient_class_init (SPRadialGradientClass *klass);
 static void sp_radialgradient_init (SPRadialGradient *rg);
-static void sp_radialgradient_destroy (GtkObject *object);
 
 static void sp_radialgradient_build (SPObject *object, SPDocument *document, SPRepr *repr);
 static void sp_radialgradient_read_attr (SPObject *object, const gchar *key);
+static SPRepr *sp_radialgradient_write (SPObject *object, SPRepr *repr, guint flags);
 
 static SPPainter *sp_radialgradient_painter_new (SPPaintServer *ps, const gdouble *affine, const ArtDRect *bbox);
 static void sp_radialgradient_painter_free (SPPaintServer *ps, SPPainter *painter);
@@ -1495,10 +1626,9 @@ sp_radialgradient_class_init (SPRadialGradientClass * klass)
 
 	rg_parent_class = gtk_type_class (SP_TYPE_GRADIENT);
 
-	gtk_object_class->destroy = sp_radialgradient_destroy;
-
 	sp_object_class->build = sp_radialgradient_build;
 	sp_object_class->read_attr = sp_radialgradient_read_attr;
+	sp_object_class->write = sp_radialgradient_write;
 
 	ps_class->painter_new = sp_radialgradient_painter_new;
 	ps_class->painter_free = sp_radialgradient_painter_free;
@@ -1509,22 +1639,11 @@ sp_radialgradient_class_init (SPRadialGradientClass * klass)
 static void
 sp_radialgradient_init (SPRadialGradient *rg)
 {
-	sp_svg_length_unset (&rg->cx, SP_SVG_UNIT_PERCENT, 0.5, 50.0);
-	sp_svg_length_unset (&rg->cy, SP_SVG_UNIT_PERCENT, 0.5, 50.0);
-	sp_svg_length_unset (&rg->r, SP_SVG_UNIT_PERCENT, 0.5, 50.0);
-	sp_svg_length_unset (&rg->fx, SP_SVG_UNIT_PERCENT, 0.5, 50.0);
-	sp_svg_length_unset (&rg->fy, SP_SVG_UNIT_PERCENT, 0.5, 50.0);
-}
-
-static void
-sp_radialgradient_destroy (GtkObject *object)
-{
-	SPRadialGradient * rg;
-
-	rg = (SPRadialGradient *) object;
-
-	if (GTK_OBJECT_CLASS (rg_parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (rg_parent_class)->destroy) (object);
+	sp_svg_length_unset (&rg->cx, SP_SVG_UNIT_PERCENT, 0.5, 0.5);
+	sp_svg_length_unset (&rg->cy, SP_SVG_UNIT_PERCENT, 0.5, 0.5);
+	sp_svg_length_unset (&rg->r, SP_SVG_UNIT_PERCENT, 0.5, 0.5);
+	sp_svg_length_unset (&rg->fx, SP_SVG_UNIT_PERCENT, 0.5, 0.5);
+	sp_svg_length_unset (&rg->fy, SP_SVG_UNIT_PERCENT, 0.5, 0.5);
 }
 
 static void
@@ -1534,8 +1653,8 @@ sp_radialgradient_build (SPObject *object, SPDocument *document, SPRepr *repr)
 
 	rg = SP_RADIALGRADIENT (object);
 
-	if (SP_OBJECT_CLASS (rg_parent_class)->build)
-		(* SP_OBJECT_CLASS (rg_parent_class)->build) (object, document, repr);
+	if (((SPObjectClass *) rg_parent_class)->build)
+		(* ((SPObjectClass *) rg_parent_class)->build) (object, document, repr);
 
 	sp_radialgradient_read_attr (object, "cx");
 	sp_radialgradient_read_attr (object, "cy");
@@ -1556,48 +1675,70 @@ sp_radialgradient_read_attr (SPObject *object, const gchar *key)
 
 	if (!strcmp (key, "cx")) {
 		if (!sp_svg_length_read (str, &rg->cx)) {
-			sp_svg_length_unset (&rg->cx, SP_SVG_UNIT_PERCENT, 0.5, 50.0);
+			sp_svg_length_unset (&rg->cx, SP_SVG_UNIT_PERCENT, 0.5, 0.5);
 		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
 	} else if (!strcmp (key, "cy")) {
 		if (!sp_svg_length_read (str, &rg->cy)) {
-			sp_svg_length_unset (&rg->cy, SP_SVG_UNIT_PERCENT, 0.5, 50.0);
+			sp_svg_length_unset (&rg->cy, SP_SVG_UNIT_PERCENT, 0.5, 0.5);
 		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
 	} else if (!strcmp (key, "r")) {
 		if (!sp_svg_length_read (str, &rg->r)) {
-			sp_svg_length_unset (&rg->r, SP_SVG_UNIT_PERCENT, 0.5, 50.0);
+			sp_svg_length_unset (&rg->r, SP_SVG_UNIT_PERCENT, 0.5, 0.5);
 		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
 	} else if (!strcmp (key, "fx")) {
 		if (!sp_svg_length_read (str, &rg->fx)) {
-			sp_svg_length_unset (&rg->fx, SP_SVG_UNIT_PERCENT, 0.5, 50.0);
+			sp_svg_length_unset (&rg->fx, SP_SVG_UNIT_PERCENT, 0.5, 0.5);
 		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
 	} else if (!strcmp (key, "fy")) {
 		if (!sp_svg_length_read (str, &rg->fy)) {
-			sp_svg_length_unset (&rg->fy, SP_SVG_UNIT_PERCENT, 0.5, 50.0);
+			sp_svg_length_unset (&rg->fy, SP_SVG_UNIT_PERCENT, 0.5, 0.5);
 		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
 	} else {
-		if (SP_OBJECT_CLASS (rg_parent_class)->read_attr)
-			(* SP_OBJECT_CLASS (rg_parent_class)->read_attr) (object, key);
+		if (((SPObjectClass *) rg_parent_class)->read_attr)
+			(* ((SPObjectClass *) rg_parent_class)->read_attr) (object, key);
 	}
 }
 
+static SPRepr *
+sp_radialgradient_write (SPObject *object, SPRepr *repr, guint flags)
+{
+	SPRadialGradient *rg;
+
+	rg = SP_RADIALGRADIENT (object);
+
+	if ((flags & SP_OBJECT_WRITE_BUILD) && !repr) {
+		repr = sp_repr_new ("radialGradient");
+	}
+
+	sp_repr_set_double (repr, "cx", rg->cx.computed);
+	sp_repr_set_double (repr, "cy", rg->cy.computed);
+	sp_repr_set_double (repr, "fx", rg->fx.computed);
+	sp_repr_set_double (repr, "fy", rg->fy.computed);
+	sp_repr_set_double (repr, "r", rg->r.computed);
+
+	if (((SPObjectClass *) rg_parent_class)->write)
+		(* ((SPObjectClass *) rg_parent_class)->write) (object, repr, flags);
+
+	return repr;
+}
+
 static SPPainter *
-sp_radialgradient_painter_new (SPPaintServer *ps, const gdouble *affine, const ArtDRect *bbox)
+sp_radialgradient_painter_new (SPPaintServer *ps, const gdouble *ctm, const ArtDRect *bbox)
 {
 	SPRadialGradient *rg;
 	SPGradient *gr;
 	SPRGPainter *rgp;
+	NRMatrixF gs2px;
 
 	rg = SP_RADIALGRADIENT (ps);
 	gr = SP_GRADIENT (ps);
 
-	if (!gr->color) {
-		sp_gradient_ensure_colors (gr);
-	}
+	if (!gr->color) sp_gradient_ensure_colors (gr);
 
 	rgp = g_new (SPRGPainter, 1);
 
@@ -1606,58 +1747,42 @@ sp_radialgradient_painter_new (SPPaintServer *ps, const gdouble *affine, const A
 
 	rgp->rg = rg;
 
-#if 0
-	if (rg->units == SP_GRADIENT_UNITS_OBJECTBOUNDINGBOX) {
-		gdouble b2c[6], vec2b[6], norm2vec[6], norm2c[6], c2norm[6];
-		/* fixme: we should use start & end here */
-		norm2vec[0] = gr->len / 1023.9999;
-		norm2vec[1] = 0.0;
-		norm2vec[2] = 0.0;
-		norm2vec[3] = gr->len / 1023.9999;
-		norm2vec[4] = 0.0;
-		norm2vec[5] = 0.0;
-#if 0
-		g_print ("\nnorm2vec: ");for (i = 0; i < 6; i++) g_print ("%g ", norm2vec[i]);g_print ("\n");
-#endif
-		/* fixme: gradient transform somewhere here */
-		vec2b[0] = lg->x2.distance - lg->x1.distance;
-		vec2b[1] = lg->y2.distance - lg->y1.distance;
-		vec2b[2] = lg->y2.distance - lg->y1.distance;
-		vec2b[3] = lg->x1.distance - lg->x2.distance;
-		vec2b[4] = lg->x1.distance;
-		vec2b[5] = lg->y1.distance;
-#if 0
-		g_print ("vec2b: ");for (i = 0; i < 6; i++) g_print ("%g ", vec2b[i]);g_print ("\n");
-#endif
-		b2c[0] = bbox->x1 - bbox->x0;
-		b2c[1] = 0.0;
-		b2c[2] = 0.0;
-		b2c[3] = bbox->y1 - bbox->y0;
-		b2c[4] = bbox->x0;
-		b2c[5] = bbox->y0;
-#if 0
-		g_print ("b2c: ");for (i = 0; i < 6; i++) g_print ("%g ", b2c[i]);g_print ("\n");
-#endif
-		art_affine_multiply (norm2c, norm2vec, vec2b);
-		art_affine_multiply (norm2c, norm2c, b2c);
-#if 0
-		g_print ("norm2c: ");for (i = 0; i < 6; i++) g_print ("%g ", norm2c[i]);g_print ("\n");
-#endif
-		art_affine_invert (c2norm, norm2c);
-#if 0
-		g_print ("c2norm: ");for (i = 0; i < 6; i++) g_print ("%g ", c2norm[i]);g_print ("\n");
-#endif
-		lgp->x0 = norm2c[4];
-		lgp->y0 = norm2c[5];
-		lgp->dx = c2norm[0];
-		lgp->dy = c2norm[2];
+	/* fixme: We may try to normalize here too, look at linearGradient (Lauris) */
+
+	if (gr->units == SP_GRADIENT_UNITS_OBJECTBOUNDINGBOX) {
+		NRMatrixF bbox2user;
+		NRMatrixF gs2user;
+
+		/* fixme: We may try to normalize here too, look at linearGradient (Lauris) */
+
+		/* gradientTransform goes here (Lauris) */
+
+		/* BBox to user coordinate system */
+		bbox2user.c[0] = bbox->x1 - bbox->x0;
+		bbox2user.c[1] = 0.0;
+		bbox2user.c[2] = 0.0;
+		bbox2user.c[3] = bbox->y1 - bbox->y0;
+		bbox2user.c[4] = bbox->x0;
+		bbox2user.c[5] = bbox->y0;
+
+		/* fixme: (Lauris) */
+		nr_matrix_multiply_fdf (&gs2user, (NRMatrixD *) gr->transform, &bbox2user);
+		nr_matrix_multiply_ffd (&gs2px, &gs2user, (NRMatrixD *) ctm);
 	} else {
-		lgp->x0 = affine[4];
-		lgp->y0 = affine[5];
-		lgp->dx = 127.0 / (bbox->x1 - bbox->x0);
-		lgp->dy = 127.0 / (bbox->y1 - bbox->y0);
+		/* Problem: What to do, if we have mixed lengths and percentages? */
+		/* Currently we do ignore percentages at all, but that is not good (lauris) */
+
+		/* fixme: We may try to normalize here too, look at linearGradient (Lauris) */
+
+		/* fixme: (Lauris) */
+		nr_matrix_multiply_fdd (&gs2px, (NRMatrixD *) gr->transform, (NRMatrixD *) ctm);
 	}
-#endif
+
+	nr_rgradient_renderer_setup (&rgp->rgr, gr->color, gr->spread,
+				     &gs2px,
+				     rg->cx.computed, rg->cy.computed,
+				     rg->fx.computed, rg->fy.computed,
+				     rg->r.computed);
 
 	return (SPPainter *) rgp;
 }
@@ -1686,6 +1811,23 @@ sp_radialgradient_flatten_attributes (SPGradient *gr, SPRepr *repr, gboolean set
 	if (set_missing || rg->fy.set) sp_repr_set_double_attribute (repr, "fy", rg->fy.computed);
 }
 
+void
+sp_radialgradient_set_position (SPRadialGradient *rg, gdouble cx, gdouble cy, gdouble fx, gdouble fy, gdouble r)
+{
+	g_return_if_fail (rg != NULL);
+	g_return_if_fail (SP_IS_RADIALGRADIENT (rg));
+
+	/* fixme: units? (Lauris)  */
+	sp_svg_length_unset (&rg->cx, SP_SVG_UNIT_NONE, cx, cx);
+	sp_svg_length_unset (&rg->cy, SP_SVG_UNIT_NONE, cy, cy);
+	sp_svg_length_unset (&rg->fx, SP_SVG_UNIT_NONE, fx, fx);
+	sp_svg_length_unset (&rg->fy, SP_SVG_UNIT_NONE, fy, fy);
+	sp_svg_length_unset (&rg->r, SP_SVG_UNIT_NONE, r, r);
+
+	sp_object_request_modified (SP_OBJECT (rg), SP_OBJECT_MODIFIED_FLAG);
+}
+
+
 /* Builds flattened repr tree of gradient - i.e. no href */
 
 SPRepr *
@@ -1711,53 +1853,15 @@ sp_radialgradient_build_repr (SPRadialGradient *rg, gboolean vector)
 static void
 sp_rg_fill (SPPainter *painter, guchar *px, gint x0, gint y0, gint width, gint height, gint rowstride)
 {
-#if 0
-	SPLGPainter *lgp;
-	SPRadialGradient *lg;
-	SPGradient *g;
-	gdouble pos;
-	gint x, y;
-	guint32 *p;
-	guint32 tmp;
+	SPRGPainter *rgp;
+	NRPixBlock pb;
 
-	lgp = (SPLGPainter *) painter;
-	lg = lgp->lg;
-	g = (SPGradient *) lg;
+	rgp = (SPRGPainter *) painter;
 
-	if (!g->color) {
-		/* fixme: This is forbidden, so we should paint some mishmesh instead */
-		sp_gradient_ensure_colors (g);
-	}
+	nr_pixblock_setup_extern (&pb, NR_PIXBLOCK_MODE_R8G8B8A8N, x0, y0, x0 + width, y0 + height, px, rowstride, FALSE, FALSE);
 
-	for (y = 0; y < height; y++) {
-		p = buf + y * rowstride;
-		pos = (y + y0 - lgp->y0) * lgp->dy + (0 + x0 - lgp->x0) * lgp->dx;
-		for (x = 0; x < width; x++) {
-			gint ip, idx;
-			ip = (gint) pos;
-			switch (g->spread) {
-			case SP_GRADIENT_SPREAD_PAD:
-				idx = CLAMP (ip, 0, 1023);
-				break;
-			case SP_GRADIENT_SPREAD_REFLECT:
-				idx = ip & 0x3ff;
-				if (ip & 0x400) idx = 1023 - idx;
-				break;
-			case SP_GRADIENT_SPREAD_REPEAT:
-				idx = ip & 0x3ff;
-				break;
-			default:
-				g_assert_not_reached ();
-				idx = 0;
-				break;
-			}
-			/* a * b / 255 = a * b * (256 * 256 / 255) / 256 / 256 */
-			tmp = (g->color[idx] & 0xff) * lgp->opacity;
-			tmp = (tmp << 16) + tmp + (tmp >> 8);
-			*p++ = (g->color[idx] & 0xffffff00) | (tmp >> 24);
-			pos += lgp->dx;
-		}
-	}
-#endif
+	nr_render ((NRRenderer *) &rgp->rgr, &pb, NULL);
+
+	nr_pixblock_release (&pb);
 }
 
