@@ -13,12 +13,6 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include "forward.h"
-
-/*
- * Gradient Stop
- */
-
 #define SP_TYPE_STOP (sp_stop_get_type ())
 #define SP_STOP(o) (GTK_CHECK_CAST ((o), SP_TYPE_STOP, SPStop))
 #define SP_STOP_CLASS(k) (GTK_CHECK_CLASS_CAST ((k), SP_TYPE_STOP, SPStopClass))
@@ -43,11 +37,16 @@
 #define SP_IS_RADIALGRADIENT(o) (GTK_CHECK_TYPE ((o), SP_TYPE_RADIALGRADIENT))
 #define SP_IS_RADIALGRADIENT_CLASS(k) (GTK_CHECK_CLASS_TYPE ((k), SP_TYPE_RADIALGRADIENT))
 
-#include <libart_lgpl/art_point.h>
+#include <libnr/nr-types.h>
 #include "svg/svg-types.h"
+#include "forward.h"
 #include "color.h"
 #include "sp-paint-server.h"
 
+/*
+ * Gradient Stop
+ */
+
 struct _SPStop {
 	SPObject object;
 
@@ -94,8 +93,8 @@ typedef enum {
 } SPGradientState;
 
 typedef enum {
-	SP_GRADIENT_UNITS_USERSPACEONUSE,
-	SP_GRADIENT_UNITS_OBJECTBOUNDINGBOX
+	SP_GRADIENT_UNITS_OBJECTBOUNDINGBOX,
+	SP_GRADIENT_UNITS_USERSPACEONUSE
 } SPGradientUnits;
 
 typedef enum {
@@ -108,6 +107,8 @@ typedef enum {
 #define SP_GRADIENT_IS_VECTOR(g) (SP_GRADIENT(g)->state == SP_GRADIENT_STATE_VECTOR)
 #define SP_GRADIENT_IS_PRIVATE(g) (SP_GRADIENT(g)->state == SP_GRADIENT_STATE_PRIVATE)
 #define SP_GRADIENT_HAS_STOPS(g) (SP_GRADIENT(g)->has_stops)
+#define SP_GRADIENT_SPREAD(g) (SP_GRADIENT (g)->spread)
+#define SP_GRADIENT_UNITS(g) (SP_GRADIENT (g)->units)
 
 struct _SPGradient {
 	SPPaintServer paint_server;
@@ -151,6 +152,9 @@ void sp_gradient_ensure_colors (SPGradient *gradient);
 /* Sets gradient vector to given value, does not update reprs */
 void sp_gradient_set_vector (SPGradient *gradient, SPGradientVector *vector);
 
+void sp_gradient_set_units (SPGradient *gr, unsigned int units);
+void sp_gradient_set_spread (SPGradient *gr, unsigned int spread);
+
 /* Gradient repr methods */
 void sp_gradient_repr_flatten_attributes (SPGradient *gradient, SPRepr *repr, gboolean set_missing);
 void sp_gradient_repr_set_vector (SPGradient *gradient, SPRepr *repr, SPGradientVector *vector);
@@ -164,12 +168,17 @@ void sp_gradient_repr_set_vector (SPGradient *gradient, SPRepr *repr, SPGradient
  *
  * RGB buffer background should be set up before
  */
-void sp_gradient_render_vector_line_rgba (SPGradient *gradient, guchar *buf, gint len, gint pos, gint span);
-void sp_gradient_render_vector_line_rgb (SPGradient *gradient, guchar *buf, gint len, gint pos, gint span);
-void sp_gradient_render_vector_block_rgba (SPGradient *gradient, guchar *buf, gint width, gint height, gint rowstride,
-					   gint pos, gint span, gboolean horizontal);
-void sp_gradient_render_vector_block_rgb (SPGradient *gradient, guchar *buf, gint width, gint height, gint rowstride,
-					  gint pos, gint span, gboolean horizontal);
+void sp_gradient_render_vector_line_rgba (SPGradient *gr, guchar *px, gint len, gint pos, gint span);
+void sp_gradient_render_vector_line_rgb (SPGradient *gr, guchar *px, gint len, gint pos, gint span);
+void sp_gradient_render_vector_block_rgba (SPGradient *gr, guchar *px, gint w, gint h, gint rs, gint pos, gint span, gboolean horizontal);
+void sp_gradient_render_vector_block_rgb (SPGradient *gr, guchar *px, gint w, gint h, gint rs, gint pos, gint span, gboolean horizontal);
+
+/* Transforms to/from gradient position space in given environment */
+NRMatrixF *sp_gradient_get_g2d_matrix_f (SPGradient *gr, NRMatrixF *ctm, NRRectF *bbox, NRMatrixF *g2d);
+NRMatrixF *sp_gradient_get_gs2d_matrix_f (SPGradient *gr, NRMatrixF *ctm, NRRectF *bbox, NRMatrixF *gs2d);
+void sp_gradient_set_gs2d_matrix_f (SPGradient *gr, NRMatrixF *ctm, NRRectF *bbox, NRMatrixF *gs2d);
+void sp_gradient_from_position_xy (SPGradient *gr, gdouble *ctm, ArtDRect *bbox, NRPointF *p, float x, float y);
+void sp_gradient_to_position_xy (SPGradient *gr, gdouble *ctm, ArtDRect *bbox, NRPointF *p, float x, float y);
 
 /*
  * Linear Gradient
@@ -190,10 +199,6 @@ struct _SPLinearGradientClass {
 
 GtkType sp_lineargradient_get_type (void);
 
-/* Transforms to/from gradient position space in given environment */
-void sp_lineargradient_from_position (SPLinearGradient *lg, gdouble *ctm, ArtDRect *bbox, ArtPoint *p);
-void sp_lineargradient_to_position (SPLinearGradient *lg, gdouble *ctm, ArtDRect *bbox, ArtPoint *p);
-
 void sp_lineargradient_set_position (SPLinearGradient *lg, gdouble x1, gdouble y1, gdouble x2, gdouble y2);
 
 /* Builds flattened repr tree of gradient - i.e. no href */
@@ -220,6 +225,8 @@ struct _SPRadialGradientClass {
 
 GtkType sp_radialgradient_get_type (void);
 
+void sp_radialgradient_set_position (SPRadialGradient *rg, gdouble cx, gdouble cy, gdouble fx, gdouble fy, gdouble r);
+
 /* Builds flattened repr tree of gradient - i.e. no href */
 
 SPRepr *sp_radialgradient_build_repr (SPRadialGradient *lg, gboolean vector);
