@@ -1,4 +1,15 @@
-#define SP_GUIDE_C
+#define __SP_GUIDE_C__
+
+/*
+ * Sodipodi guideline implementation
+ *
+ * Authors:
+ *   Lauris Kaplinski <lauris@kaplinski.com>
+ *
+ * Copyright (C) 2000-2002 authors
+ *
+ * Released under GNU GPL, read the file 'COPYING' for more information
+ */
 
 #include <string.h>
 #include <gtk/gtksignal.h>
@@ -9,10 +20,10 @@ enum {ARG_0, ARG_COLOR, ARG_HICOLOR};
 
 static void sp_guide_class_init (SPGuideClass * klass);
 static void sp_guide_init (SPGuide * guide);
-static void sp_guide_destroy (GtkObject * object);
 static void sp_guide_set_arg (GtkObject * object, GtkArg * arg, guint arg_id);
 
 static void sp_guide_build (SPObject * object, SPDocument * document, SPRepr * repr);
+static void sp_guide_release (SPObject *object);
 static void sp_guide_read_attr (SPObject * object, const gchar * key);
 
 static SPObjectClass * parent_class;
@@ -51,10 +62,10 @@ sp_guide_class_init (SPGuideClass * klass)
 	gtk_object_add_arg_type ("SPGuide::color", GTK_TYPE_UINT, GTK_ARG_WRITABLE, ARG_COLOR);
 	gtk_object_add_arg_type ("SPGuide::hicolor", GTK_TYPE_UINT, GTK_ARG_WRITABLE, ARG_HICOLOR);
 
-	gtk_object_class->destroy = sp_guide_destroy;
 	gtk_object_class->set_arg = sp_guide_set_arg;
 
 	sp_object_class->build = sp_guide_build;
+	sp_object_class->release = sp_guide_release;
 	sp_object_class->read_attr = sp_guide_read_attr;
 }
 
@@ -67,22 +78,6 @@ sp_guide_init (SPGuide * guide)
 	guide->hicolor = 0xff00007f;
 }
 
-static void
-sp_guide_destroy (GtkObject * object)
-{
-	SPGuide * guide;
-
-	guide = (SPGuide *) object;
-
-	while (guide->views) {
-		gtk_object_destroy (GTK_OBJECT (guide->views->data));
-		guide->views = g_slist_remove (guide->views, guide->views->data);
-	}
-
-	if (((GtkObjectClass *) (parent_class))->destroy)
-		(* ((GtkObjectClass *) (parent_class))->destroy) (object);
-}
-
 static void
 sp_guide_set_arg (GtkObject * object, GtkArg * arg, guint arg_id)
 {
@@ -115,6 +110,22 @@ sp_guide_build (SPObject * object, SPDocument * document, SPRepr * repr)
 
 }
 
+static void
+sp_guide_release (SPObject *object)
+{
+	SPGuide *guide;
+
+	guide = (SPGuide *) object;
+
+	while (guide->views) {
+		gtk_object_destroy (GTK_OBJECT (guide->views->data));
+		guide->views = g_slist_remove (guide->views, guide->views->data);
+	}
+
+	if (((SPObjectClass *) parent_class)->release)
+		((SPObjectClass *) parent_class)->release (object);
+}
+
 static void
 sp_guide_read_attr (SPObject * object, const gchar * key)
 {
@@ -142,11 +153,11 @@ sp_guide_read_attr (SPObject * object, const gchar * key)
 }
 
 void
-sp_guide_show (SPGuide * guide, GnomeCanvasGroup * group, gpointer handler)
+sp_guide_show (SPGuide * guide, SPCanvasGroup * group, gpointer handler)
 {
-	GnomeCanvasItem * item;
+	SPCanvasItem * item;
 
-	item = gnome_canvas_item_new (group, SP_TYPE_GUIDELINE,
+	item = sp_canvas_item_new (group, SP_TYPE_GUIDELINE,
 				      "orientation", guide->orientation,
 				      "color", guide->color,
 				      NULL);
@@ -160,17 +171,17 @@ sp_guide_show (SPGuide * guide, GnomeCanvasGroup * group, gpointer handler)
 }
 
 void
-sp_guide_hide (SPGuide * guide, GnomeCanvas * canvas)
+sp_guide_hide (SPGuide * guide, SPCanvas * canvas)
 {
 	GSList * l;
 
 	g_assert (guide != NULL);
 	g_assert (SP_IS_GUIDE (guide));
 	g_assert (canvas != NULL);
-	g_assert (GNOME_IS_CANVAS (canvas));
+	g_assert (SP_IS_CANVAS (canvas));
 
 	for (l = guide->views; l != NULL; l = l->next) {
-		if (canvas == GNOME_CANVAS_ITEM (l->data)->canvas) {
+		if (canvas == SP_CANVAS_ITEM (l->data)->canvas) {
 			gtk_object_destroy (GTK_OBJECT (l->data));
 			guide->views = g_slist_remove (guide->views, l->data);
 			return;
@@ -180,17 +191,17 @@ sp_guide_hide (SPGuide * guide, GnomeCanvas * canvas)
 }
 
 void
-sp_guide_sensitize (SPGuide * guide, GnomeCanvas * canvas, gboolean sensitive)
+sp_guide_sensitize (SPGuide * guide, SPCanvas * canvas, gboolean sensitive)
 {
 	GSList * l;
 
 	g_assert (guide != NULL);
 	g_assert (SP_IS_GUIDE (guide));
 	g_assert (canvas != NULL);
-	g_assert (GNOME_IS_CANVAS (canvas));
+	g_assert (SP_IS_CANVAS (canvas));
 
 	for (l = guide->views; l != NULL; l = l->next) {
-		if (canvas == GNOME_CANVAS_ITEM (l->data)->canvas) {
+		if (canvas == SP_CANVAS_ITEM (l->data)->canvas) {
 			sp_guideline_sensitize (SP_GUIDELINE (l->data), sensitive);
 			return;
 		}
