@@ -10,7 +10,7 @@
  *
  */
 
-#include <libgnomeui/gnome-canvas.h>
+#include "helper/helper-forward.h"
 #include "sp-object.h"
 
 typedef enum {
@@ -18,9 +18,6 @@ typedef enum {
 	SP_GUIDE_VERTICAL
 } SPGuideOrientation;
 
-typedef struct _SPGuide SPGuide;
-typedef struct _SPGuideClass SPGuideClass;
-
 #define SP_TYPE_GUIDE            (sp_guide_get_type ())
 #define SP_GUIDE(obj)            (GTK_CHECK_CAST ((obj), SP_TYPE_GUIDE, SPGuide))
 #define SP_GUIDE_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), SP_TYPE_GUIDE, SPGuideClass))
@@ -42,11 +39,13 @@ struct _SPGuideClass {
 
 GtkType sp_guide_get_type (void);
 
-void sp_guide_show (SPGuide * guide, GnomeCanvasGroup * group, gpointer handler);
-void sp_guide_hide (SPGuide * guide, GnomeCanvas * canvas);
-void sp_guide_sensitize (SPGuide * guide, GnomeCanvas * canvas, gboolean sensitive);
+void sp_guide_show (SPGuide * guide, SPCanvasGroup * group, gpointer handler);
+void sp_guide_hide (SPGuide * guide, SPCanvas * canvas);
+void sp_guide_sensitize (SPGuide * guide, SPCanvas * canvas, gboolean sensitive);
 
 void sp_guide_moveto (SPGuide * guide, gdouble x, gdouble y);
+void sp_guide_set (SPGuide * guide, gdouble x, gdouble y);
+void sp_guide_remove (SPGuide * guide);
 
 gint sp_guide_compare (gconstpointer a, gconstpointer b);
 
