@@ -16,12 +16,17 @@
 #include <math.h>
 #include <string.h>
 #include <ctype.h>
+#include <libnr/nr-matrix.h>
 #include <glib.h>
+#include <libart_lgpl/art_affine.h>
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
+#include <gtk/gtksignal.h>
+#include <gtk/gtkmenuitem.h>
 #include <gdk-pixbuf/gdk-pixbuf-loader.h>
 #include "display/nr-arena-image.h"
 #include "svg/svg.h"
+#include "print.h"
 #include "style.h"
 #include "brokenimage.xpm"
 #include "document.h"
@@ -34,13 +39,14 @@
 
 static void sp_image_class_init (SPImageClass * class);
 static void sp_image_init (SPImage * image);
-static void sp_image_destroy (GtkObject * object);
 
 static void sp_image_build (SPObject * object, SPDocument * document, SPRepr * repr);
+static void sp_image_release (SPObject * object);
 static void sp_image_read_attr (SPObject * object, const gchar * key);
+static SPRepr *sp_image_write (SPObject *object, SPRepr *repr, guint flags);
 
 static void sp_image_bbox (SPItem *item, ArtDRect *bbox, const gdouble *transform);
-static void sp_image_print (SPItem * item, GnomePrintContext * gpc);
+static void sp_image_print (SPItem * item, SPPrintContext *ctx);
 static gchar * sp_image_description (SPItem * item);
 static GSList * sp_image_snappoints (SPItem * item, GSList * points);
 static NRArenaItem *sp_image_show (SPItem *item, NRArena *arena);
@@ -92,10 +98,10 @@ sp_image_class_init (SPImageClass * klass)
 
 	parent_class = gtk_type_class (sp_item_get_type ());
 
-	gtk_object_class->destroy = sp_image_destroy;
-
 	sp_object_class->build = sp_image_build;
+	sp_object_class->release = sp_image_release;
 	sp_object_class->read_attr = sp_image_read_attr;
+	sp_object_class->write = sp_image_write;
 
 	item_class->bbox = sp_image_bbox;
 	item_class->print = sp_image_print;
@@ -120,7 +126,23 @@ sp_image_init (SPImage *image)
 }
 
 static void
-sp_image_destroy (GtkObject *object)
+sp_image_build (SPObject *object, SPDocument *document, SPRepr *repr)
+{
+	if (((SPObjectClass *) parent_class)->build)
+		((SPObjectClass *) parent_class)->build (object, document, repr);
+
+	sp_image_read_attr (object, "xlink:href");
+	sp_image_read_attr (object, "x");
+	sp_image_read_attr (object, "y");
+	sp_image_read_attr (object, "width");
+	sp_image_read_attr (object, "height");
+
+	/* Register */
+	sp_document_add_resource (document, "image", object);
+}
+
+static void
+sp_image_release (SPObject *object)
 {
 	SPImage *image;
 
@@ -141,24 +163,8 @@ sp_image_destroy (GtkObject *object)
 		image->pixbuf = NULL;
 	}
 
-	if (GTK_OBJECT_CLASS (parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
-}
-
-static void
-sp_image_build (SPObject *object, SPDocument *document, SPRepr *repr)
-{
-	if (SP_OBJECT_CLASS (parent_class)->build)
-		SP_OBJECT_CLASS (parent_class)->build (object, document, repr);
-
-	sp_image_read_attr (object, "xlink:href");
-	sp_image_read_attr (object, "x");
-	sp_image_read_attr (object, "y");
-	sp_image_read_attr (object, "width");
-	sp_image_read_attr (object, "height");
-
-	/* Register */
-	sp_document_add_resource (document, "image", object);
+	if (((SPObjectClass *) parent_class)->release)
+		((SPObjectClass *) parent_class)->release (object);
 }
 
 static void
@@ -260,6 +266,30 @@ sp_image_read_attr (SPObject * object, const gchar *key)
 	}
 }
 
+static SPRepr *
+sp_image_write (SPObject *object, SPRepr *repr, guint flags)
+{
+	SPImage *image;
+
+	image = SP_IMAGE (object);
+
+	if ((flags & SP_OBJECT_WRITE_BUILD) && !repr) {
+		repr = sp_repr_new ("image");
+	}
+
+	sp_repr_set_attr (repr, "xlink:href", image->href);
+	/* fixme: Reset attribute if needed (Lauris) */
+	if (image->x.set) sp_repr_set_double (repr, "x", image->x.computed);
+	if (image->y.set) sp_repr_set_double (repr, "y", image->y.computed);
+	if (image->width.set) sp_repr_set_double (repr, "width", image->width.computed);
+	if (image->height.set) sp_repr_set_double (repr, "height", image->height.computed);
+
+	if (((SPObjectClass *) (parent_class))->write)
+		((SPObjectClass *) (parent_class))->write (object, repr, flags);
+
+	return repr;
+}
+
 static void
 sp_image_bbox (SPItem *item, ArtDRect *bbox, const gdouble *transform)
 {
@@ -280,8 +310,34 @@ sp_image_bbox (SPItem *item, ArtDRect *bbox, const gdouble *transform)
 }
 
 static void
-sp_image_print (SPItem *item, GnomePrintContext *gpc)
+sp_image_print (SPItem *item, SPPrintContext *ctx)
 {
+#if 1
+	SPImage *image;
+	NRMatrixF tp, ti, s, t;
+	unsigned char *px;
+	int w, h, rs;
+
+	image = SP_IMAGE (item);
+
+	if (!image->pixbuf) return;
+	if ((image->width.computed <= 0.0) || (image->height.computed <= 0.0)) return;
+
+	px = gdk_pixbuf_get_pixels (image->pixbuf);
+	w = gdk_pixbuf_get_width (image->pixbuf);
+	h = gdk_pixbuf_get_height (image->pixbuf);
+	rs = gdk_pixbuf_get_rowstride (image->pixbuf);
+
+	/* fixme: (Lauris) */
+	nr_matrix_f_set_translate (&tp, image->x.computed, image->y.computed);
+	nr_matrix_f_set_scale (&s, image->width.computed, -image->height.computed);
+	nr_matrix_f_set_translate (&ti, 0.0, -1.0);
+
+	nr_matrix_multiply_fff (&t, &s, &tp);
+	nr_matrix_multiply_fff (&t, &ti, &t);
+
+	sp_print_image_R8G8B8A8_N (ctx, px, w, h, rs, &t, SP_OBJECT_STYLE (item));
+#else
 	SPObject *object;
 	SPImage *image;
 	guchar *pixels;
@@ -327,6 +383,7 @@ sp_image_print (SPItem *item, GnomePrintContext *gpc)
 	}
 
 	gnome_print_grestore (gpc);
+#endif
 }
 
 static gchar *
@@ -769,7 +826,7 @@ static void handle_msg(at_string msg, at_msg_type msg_type, at_address client_da
 static GtkWidget *  build_header_area(SPRepr *repr);
 
 static void
-autotrace_dialog(SPImage * img)
+autotrace_dialog (SPImage * img)
 {
 	gchar *title = _("Trace");
 
@@ -778,12 +835,16 @@ autotrace_dialog(SPImage * img)
 	GtkWidget * header_sep;
 	at_bitmap_type * bitmap;
 
+#ifdef FRONTLINE_INIT
+	frontline_init();
+#endif /* Def: FRONTLINE_INIT */
+
 	trace_dialog = frontline_dialog_new();
 	gtk_window_set_title (GTK_WINDOW (trace_dialog), title);
 
 	gtk_signal_connect_object_while_alive (GTK_OBJECT (img), 
-					       "destroy",
-					       GTK_SIGNAL_FUNC(gtk_widget_destroy),
+					       "release",
+					       GTK_SIGNAL_FUNC (gtk_widget_destroy),
 					       GTK_OBJECT (trace_dialog));
 	gtk_signal_connect_object (GTK_OBJECT(FRONTLINE_DIALOG(trace_dialog)->close_button),
 				   "clicked",
@@ -880,10 +941,8 @@ load_splines(at_splines_type * splines)
 	umask(old_mask);
   
 	writer = at_output_get_handler_by_suffix ("svg");
-	at_splines_write(splines, tmp_fp, filename, AT_DEFAULT_DPI,
-			 writer, 
-			 handle_msg, 
-			 filename);
+	at_splines_write (writer, tmp_fp, filename, NULL, splines,
+			  handle_msg, filename);
 	fclose(tmp_fp);
   
 	load_file (filename);
@@ -894,12 +953,14 @@ load_splines(at_splines_type * splines)
 
 
 static void
-load_file(const guchar * filename)
+load_file (const guchar *filename)
 {
 	SPDocument * doc;
 	SPViewWidget *dtw;
   
-	doc = sp_document_new (filename);
+	/* fixme: Either use file:: method, or amke this private (Lauris) */
+	/* fixme: In latter case we may want to publish it on save (Lauris) */
+	doc = sp_document_new (filename, TRUE);
 	dtw = sp_desktop_widget_new (sp_document_namedview (doc, NULL));
 	sp_document_unref (doc);
 	sp_create_window (dtw, TRUE);
