@@ -18,9 +18,12 @@
 #include <glib.h>
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
+#include <gtk/gtksignal.h>
+#include <gtk/gtkmenuitem.h>
 #include "helper/art-utils.h"
 #include "helper/nr-plain-stuff.h"
 #include "svg/svg.h"
+#include "print.h"
 #include "display/nr-arena.h"
 #include "display/nr-arena-item.h"
 #include "document.h"
@@ -40,14 +43,15 @@
 
 #define noSP_ITEM_DEBUG_IDLE
 
-static void sp_item_class_init (SPItemClass * klass);
-static void sp_item_init (SPItem * item);
-static void sp_item_destroy (GtkObject * object);
+static void sp_item_class_init (SPItemClass *klass);
+static void sp_item_init (SPItem *item);
 
 static void sp_item_build (SPObject * object, SPDocument * document, SPRepr * repr);
+static void sp_item_release (SPObject *object);
 static void sp_item_read_attr (SPObject *object, const gchar *key);
 static void sp_item_modified (SPObject *object, guint flags);
 static void sp_item_style_modified (SPObject *object, guint flags);
+static SPRepr *sp_item_write (SPObject *object, SPRepr *repr, guint flags);
 
 static gchar * sp_item_private_description (SPItem * item);
 static GSList * sp_item_private_snappoints (SPItem * item, GSList * points);
@@ -62,45 +66,43 @@ static void sp_item_reset_transformation (GtkMenuItem *menuitem, SPItem *item);
 static void sp_item_toggle_sensitivity (GtkMenuItem *menuitem, SPItem *item);
 static void sp_item_create_link (GtkMenuItem *menuitem, SPItem *item);
 
-static SPObjectClass * parent_class;
+static SPObjectClass *parent_class;
 
 GtkType
 sp_item_get_type (void)
 {
-	static GtkType item_type = 0;
-	if (!item_type) {
-		GtkTypeInfo item_info = {
+	static GtkType type = 0;
+	if (!type) {
+		GtkTypeInfo info = {
 			"SPItem",
 			sizeof (SPItem),
 			sizeof (SPItemClass),
 			(GtkClassInitFunc) sp_item_class_init,
 			(GtkObjectInitFunc) sp_item_init,
-			NULL, /* reserved_1 */
-			NULL, /* reserved_2 */
-			(GtkClassInitFunc) NULL
+			NULL, NULL, NULL
 		};
-		item_type = gtk_type_unique (sp_object_get_type (), &item_info);
+		type = gtk_type_unique (SP_TYPE_OBJECT, &info);
 	}
-	return item_type;
+	return type;
 }
 
 static void
-sp_item_class_init (SPItemClass * klass)
+sp_item_class_init (SPItemClass *klass)
 {
-	GtkObjectClass * gtk_object_class;
-	SPObjectClass * sp_object_class;
+	GtkObjectClass *gtk_object_class;
+	SPObjectClass *sp_object_class;
 
 	gtk_object_class = (GtkObjectClass *) klass;
 	sp_object_class = (SPObjectClass *) klass;
 
-	parent_class = gtk_type_class (sp_object_get_type ());
-
-	gtk_object_class->destroy = sp_item_destroy;
+	parent_class = gtk_type_class (SP_TYPE_OBJECT);
 
 	sp_object_class->build = sp_item_build;
+	sp_object_class->release = sp_item_release;
 	sp_object_class->read_attr = sp_item_read_attr;
 	sp_object_class->modified = sp_item_modified;
 	sp_object_class->style_modified = sp_item_style_modified;
+	sp_object_class->write = sp_item_write;
 
 	klass->description = sp_item_private_description;
 	klass->show = sp_item_private_show;
@@ -127,27 +129,6 @@ sp_item_init (SPItem *item)
 	if (!object->style) object->style = sp_style_new_from_object (SP_OBJECT (item));
 }
 
-static void
-sp_item_destroy (GtkObject * object)
-{
-	SPItem * item;
-
-	item = (SPItem *) object;
-
-	while (item->display) {
-		nr_arena_item_destroy (item->display->arenaitem);
-		item->display = sp_item_view_list_remove (item->display, item->display);
-	}
-
-	if (item->clip) {
-		gtk_signal_disconnect_by_data (GTK_OBJECT (item->clip), item);
-		item->clip = (SPClipPath *) sp_object_hunref (SP_OBJECT (item->clip), SP_OBJECT (item));
-	}
-
-	if (((GtkObjectClass *) (parent_class))->destroy)
-		(* ((GtkObjectClass *) (parent_class))->destroy) (object);
-}
-
 static void
 sp_item_build (SPObject * object, SPDocument * document, SPRepr * repr)
 {
@@ -161,9 +142,30 @@ sp_item_build (SPObject * object, SPDocument * document, SPRepr * repr)
 }
 
 static void
-sp_item_clip_destroyed (SPClipPath *cp, SPItem *item)
+sp_item_release (SPObject * object)
 {
-	g_warning ("Item %s clip path %s destroyed", SP_OBJECT_ID (item), SP_OBJECT_ID (cp));
+	SPItem *item;
+
+	item = (SPItem *) object;
+
+	while (item->display) {
+		nr_arena_item_destroy (item->display->arenaitem);
+		item->display = sp_item_view_list_remove (item->display, item->display);
+	}
+
+	if (item->clip) {
+		gtk_signal_disconnect_by_data (GTK_OBJECT (item->clip), item);
+		item->clip = (SPClipPath *) sp_object_hunref (SP_OBJECT (item->clip), object);
+	}
+
+	if (((SPObjectClass *) (parent_class))->release)
+		((SPObjectClass *) parent_class)->release (object);
+}
+
+static void
+sp_item_clip_released (SPClipPath *cp, SPItem *item)
+{
+	g_warning ("Item %s clip path %s released", SP_OBJECT_ID (item), SP_OBJECT_ID (cp));
 }
 
 static void
@@ -199,10 +201,8 @@ sp_item_read_attr (SPObject * object, const gchar * key)
 		if (SP_IS_CLIPPATH (cp)) {
 			SPItemView *v;
 			item->clip = (SPClipPath *) sp_object_href (cp, object);
-			gtk_signal_connect (GTK_OBJECT (item->clip), "destroy",
-					    GTK_SIGNAL_FUNC (sp_item_clip_destroyed), item);
-			gtk_signal_connect (GTK_OBJECT (item->clip), "modified",
-					    GTK_SIGNAL_FUNC (sp_item_clip_modified), item);
+			gtk_signal_connect (GTK_OBJECT (item->clip), "release", GTK_SIGNAL_FUNC (sp_item_clip_released), item);
+			gtk_signal_connect (GTK_OBJECT (item->clip), "modified", GTK_SIGNAL_FUNC (sp_item_clip_modified), item);
 			for (v = item->display; v != NULL; v = v->next) {
 				NRArenaItem *ai;
 				ai = sp_clippath_show (item->clip, v->arena);
@@ -278,24 +278,41 @@ sp_item_style_modified (SPObject *object, guint flags)
 	g_print ("S");
 #endif
 
-#if 0
-	if (!style->real_opacity_set) {
-		SPObject *parent;
-		style->real_opacity = style->opacity;
-		parent = object->parent;
-		while (parent && !parent->style) parent = parent->parent;
-		/* fixme: Currently it does not work, if parent has not set real_opacity (but it shouldn't happen) */
-		if (parent) style->real_opacity = style->real_opacity * parent->style->real_opacity;
-		style->real_opacity_set = TRUE;
-	}
-#endif
-
-
 	for (v = item->display; v != NULL; v = v->next) {
 		nr_arena_item_set_opacity (v->arenaitem, SP_SCALE24_TO_FLOAT (object->style->opacity.value));
 	}
 }
 
+static SPRepr *
+sp_item_write (SPObject *object, SPRepr *repr, guint flags)
+{
+	SPItem *item;
+	guchar c[256];
+	guchar *s;
+	gint len;
+
+	item = SP_ITEM (object);
+
+	len = sp_svg_write_affine (c, 256, item->affine);
+	sp_repr_set_attr (repr, "transform", (len > 0) ? c : NULL);
+	if (SP_OBJECT_PARENT (object)) {
+		s = sp_style_write_difference (SP_OBJECT_STYLE (object), SP_OBJECT_STYLE (SP_OBJECT_PARENT (object)));
+		sp_repr_set_attr (repr, "style", (s && *s) ? s : NULL);
+		g_free (s);
+	} else {
+		sp_repr_set_attr (repr, "style", NULL);
+	}
+
+	if (flags & SP_OBJECT_WRITE_SODIPODI) {
+		sp_repr_set_attr (repr, "sodipodi:insensitive", item->sensitive ? NULL : "true");
+	}
+
+	if (((SPObjectClass *) (parent_class))->write)
+		((SPObjectClass *) (parent_class))->write (object, repr, flags);
+
+	return repr;
+}
+
 void
 sp_item_invoke_bbox (SPItem *item, ArtDRect *bbox, const gdouble *transform)
 {
@@ -304,14 +321,15 @@ sp_item_invoke_bbox (SPItem *item, ArtDRect *bbox, const gdouble *transform)
 	g_assert (bbox != NULL);
 	g_assert (transform != NULL);
 
-	bbox->x0 = bbox->y0 = 0.0;
-	bbox->x1 = bbox->y1 = 0.0;
+	bbox->x0 = bbox->y0 = 1e18;
+	bbox->x1 = bbox->y1 = -1e18;
 
 	if (SP_ITEM_CLASS (((GtkObject *)(item))->klass)->bbox)
 		SP_ITEM_CLASS (((GtkObject *)(item))->klass)->bbox (item, bbox, transform);
 }
 
-void sp_item_bbox_desktop (SPItem *item, ArtDRect *bbox)
+void
+sp_item_bbox_desktop (SPItem *item, ArtDRect *bbox)
 {
 	gdouble i2d[6];
 
@@ -371,7 +389,8 @@ sp_item_private_snappoints (SPItem * item, GSList * points)
 	return points;
 }
 
-GSList * sp_item_snappoints (SPItem * item)
+GSList *
+sp_item_snappoints (SPItem * item)
 {
         GSList * points = NULL;
 
@@ -384,14 +403,29 @@ GSList * sp_item_snappoints (SPItem * item)
 }
 
 void
-sp_item_print (SPItem * item, GnomePrintContext * gpc)
+sp_item_invoke_print (SPItem *item, SPPrintContext *ctx)
 {
 	g_assert (item != NULL);
 	g_assert (SP_IS_ITEM (item));
-	g_assert (gpc != NULL);
+	g_assert (ctx != NULL);
 
-	if (SP_ITEM_CLASS (((GtkObject *)(item))->klass)->print)
-		(* SP_ITEM_CLASS (((GtkObject *)(item))->klass)->print) (item, gpc);
+	if (SP_ITEM_CLASS (((GtkObject *)(item))->klass)->print) {
+		if (!nr_matrix_d_test_identity ((NRMatrixD *) item->affine, NR_EPSILON_F) ||
+		    SP_OBJECT_STYLE (item)->opacity.value != SP_SCALE24_MAX) {
+			NRMatrixF t;
+			t.c[0] = item->affine[0];
+			t.c[1] = item->affine[1];
+			t.c[2] = item->affine[2];
+			t.c[3] = item->affine[3];
+			t.c[4] = item->affine[4];
+			t.c[5] = item->affine[5];
+			sp_print_bind (ctx, &t, SP_SCALE24_TO_FLOAT (SP_OBJECT_STYLE (item)->opacity.value));
+			SP_ITEM_CLASS (((GtkObject *)(item))->klass)->print (item, ctx);
+			sp_print_release (ctx);
+		} else {
+			SP_ITEM_CLASS (((GtkObject *)(item))->klass)->print (item, ctx);
+		}
+	}
 }
 
 static gchar *
@@ -479,12 +513,13 @@ sp_item_hide (SPItem *item, NRArena *arena)
 		(* SP_ITEM_CLASS (((GtkObject *)(item))->klass)->hide) (item, arena);
 }
 
+#if 0
 gboolean
 sp_item_paint (SPItem *item, ArtPixBuf *buf, gdouble affine[])
 {
 	NRArena *arena;
 	NRArenaItem *root;
-	NRIRect bbox;
+	NRRectL bbox;
 	NRGC gc;
 	NRBuffer *b;
 
@@ -507,7 +542,7 @@ sp_item_paint (SPItem *item, ArtPixBuf *buf, gdouble affine[])
 	bbox.y1 = buf->height;
 	/* Update to renderable state */
 	art_affine_identity (gc.affine);
-	nr_arena_item_invoke_update (root, &bbox, &gc, NR_ARENA_ITEM_STATE_ALL, NR_ARENA_ITEM_STATE_NONE);
+	nr_arena_item_invoke_update (root, &bbox, &gc, NR_ARENA_ITEM_STATE_ALL, NR_ARENA_ITEM_STATE_ALL);
 	/* Get RGBA buffer */
 	b = nr_buffer_get (NR_IMAGE_R8G8B8A8, buf->width, buf->height, TRUE, FALSE);
 	/* Render */
@@ -522,6 +557,7 @@ sp_item_paint (SPItem *item, ArtPixBuf *buf, gdouble affine[])
 
 	return FALSE;
 }
+#endif
 
 void
 sp_item_write_transform (SPItem *item, SPRepr *repr, gdouble *transform)
@@ -590,13 +626,40 @@ sp_item_set_item_transform (SPItem *item, const gdouble *transform)
 gdouble *
 sp_item_i2doc_affine (SPItem * item, gdouble affine[])
 {
+	SPRoot *root;
+
 	g_return_val_if_fail (item != NULL, NULL);
 	g_return_val_if_fail (SP_IS_ITEM (item), NULL);
 	g_return_val_if_fail (affine != NULL, NULL);
 
 	art_affine_identity (affine);
 
-	while (item) {
+	while (SP_OBJECT_PARENT (item)) {
+		art_affine_multiply (affine, affine, item->affine);
+		item = (SPItem *) SP_OBJECT_PARENT (item);
+	}
+
+	g_return_val_if_fail (SP_IS_ROOT (item), NULL);
+
+	root = SP_ROOT (item);
+
+	/* fixme: (Lauris) */
+	art_affine_multiply (affine, affine, root->viewbox.c);
+	art_affine_multiply (affine, affine, item->affine);
+
+	return affine;
+}
+
+gdouble *
+sp_item_i2root_affine (SPItem *item, gdouble affine[])
+{
+	g_return_val_if_fail (item != NULL, NULL);
+	g_return_val_if_fail (SP_IS_ITEM (item), NULL);
+	g_return_val_if_fail (affine != NULL, NULL);
+
+	art_affine_identity (affine);
+
+	while (SP_OBJECT_PARENT (item)) {
 		art_affine_multiply (affine, affine, item->affine);
 		item = (SPItem *) SP_OBJECT_PARENT (item);
 	}
@@ -624,11 +687,11 @@ sp_item_i2vp_affine (SPItem *item, gdouble affine[])
 
 	g_return_val_if_fail (SP_IS_ROOT (item), NULL);
 
-	/* fixme: Viewbox is specified using root transformation (Lauris) */
-	art_affine_multiply (affine, affine, item->affine);
-
 	root = SP_ROOT (item);
 
+	/* fixme: (Lauris) */
+	art_affine_multiply (affine, affine, root->viewbox.c);
+
 	affine[0] /= root->width.computed;
 	affine[1] /= root->height.computed;
 	affine[2] /= root->width.computed;
