@@ -1,22 +1,32 @@
-#define SP_LINE_C
+#define __SP_LINE_C__
+
+/*
+ * SVG <line> implementation
+ *
+ * Authors:
+ *   Lauris Kaplinski <lauris@kaplinski.com>
+ *
+ * Copyright (C) 1999-2002 Lauris Kaplinski
+ *
+ * Released under GNU GPL, read the file 'COPYING' for more information
+ */
 
 #include <math.h>
 #include <string.h>
+#include "style.h"
 #include "sp-line.h"
 
 #define hypot(a,b) sqrt ((a) * (a) + (b) * (b))
 
-enum {ARG_0, ARG_X1, ARG_Y1, ARG_X2, ARG_Y2};
-
 static void sp_line_class_init (SPLineClass *class);
 static void sp_line_init (SPLine *line);
-static void sp_line_destroy (GtkObject *object);
-static void sp_line_set_arg (GtkObject * object, GtkArg * arg, guint arg_id);
 
 static void sp_line_build (SPObject * object, SPDocument * document, SPRepr * repr);
 static void sp_line_read_attr (SPObject * object, const gchar * attr);
+static SPRepr *sp_line_write (SPObject *object, SPRepr *repr, guint flags);
 
-static gchar * sp_line_description (SPItem * item);
+static gchar *sp_line_description (SPItem * item);
+static void sp_line_write_transform (SPItem *item, SPRepr *repr, gdouble *t);
 
 static void sp_line_set_shape (SPLine * line);
 
@@ -55,18 +65,12 @@ sp_line_class_init (SPLineClass *class)
 
 	parent_class = gtk_type_class (sp_shape_get_type ());
 
-	gtk_object_add_arg_type ("SPLine::x1", GTK_TYPE_DOUBLE, GTK_ARG_WRITABLE, ARG_X1);
-	gtk_object_add_arg_type ("SPLine::y1", GTK_TYPE_DOUBLE, GTK_ARG_WRITABLE, ARG_Y1);
-	gtk_object_add_arg_type ("SPLine::x2", GTK_TYPE_DOUBLE, GTK_ARG_WRITABLE, ARG_X2);
-	gtk_object_add_arg_type ("SPLine::y2", GTK_TYPE_DOUBLE, GTK_ARG_WRITABLE, ARG_Y2);
-
-	gtk_object_class->destroy = sp_line_destroy;
-	gtk_object_class->set_arg = sp_line_set_arg;
-
 	sp_object_class->build = sp_line_build;
 	sp_object_class->read_attr = sp_line_read_attr;
+	sp_object_class->write = sp_line_write;
 
 	item_class->description = sp_line_description;
+	item_class->write_transform = sp_line_write_transform;
 }
 
 static void
@@ -76,50 +80,12 @@ sp_line_init (SPLine * line)
 	line->x1 = line->y1 = line->x2 = line->y2 = 0.0;
 }
 
-static void
-sp_line_destroy (GtkObject *object)
-{
-	SPLine *line;
-
-	line = SP_LINE (object);
-
-	if (GTK_OBJECT_CLASS (parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
-}
-
-static void
-sp_line_set_arg (GtkObject * object, GtkArg * arg, guint arg_id)
-{
-	SPLine * line;
-
-	line = SP_LINE (object);
-
-	switch (arg_id) {
-	case ARG_X1:
-		line->x1 = GTK_VALUE_DOUBLE (*arg);
-		sp_line_set_shape (line);
-		break;
-	case ARG_Y1:
-		line->y1 = GTK_VALUE_DOUBLE (*arg);
-		sp_line_set_shape (line);
-		break;
-	case ARG_X2:
-		line->x2 = GTK_VALUE_DOUBLE (*arg);
-		sp_line_set_shape (line);
-		break;
-	case ARG_Y2:
-		line->y2 = GTK_VALUE_DOUBLE (*arg);
-		sp_line_set_shape (line);
-		break;
-	}
-}
-
 static void
 sp_line_build (SPObject * object, SPDocument * document, SPRepr * repr)
 {
 
-	if (SP_OBJECT_CLASS(parent_class)->build)
-		(* SP_OBJECT_CLASS(parent_class)->build) (object, document, repr);
+	if (((SPObjectClass *) parent_class)->build)
+		((SPObjectClass *) parent_class)->build (object, document, repr);
 
 	sp_line_read_attr (object, "x1");
 	sp_line_read_attr (object, "y1");
@@ -162,17 +128,69 @@ sp_line_read_attr (SPObject * object, const gchar * attr)
 		return;
 	}
 
-	if (SP_OBJECT_CLASS (parent_class)->read_attr)
-		SP_OBJECT_CLASS (parent_class)->read_attr (object, attr);
+	if (((SPObjectClass *) parent_class)->read_attr)
+		((SPObjectClass *) parent_class)->read_attr (object, attr);
 
 }
 
+static SPRepr *
+sp_line_write (SPObject *object, SPRepr *repr, guint flags)
+{
+	SPLine *line;
+
+	line = SP_LINE (object);
+
+	if ((flags & SP_OBJECT_WRITE_BUILD) && !repr) {
+		repr = sp_repr_new ("line");
+	}
+
+	if (repr != SP_OBJECT_REPR (object)) {
+		sp_repr_merge (repr, SP_OBJECT_REPR (object), "id");
+	}
+
+	if (((SPObjectClass *) (parent_class))->write)
+		((SPObjectClass *) (parent_class))->write (object, repr, flags);
+
+	return repr;
+}
+
 static gchar *
 sp_line_description (SPItem * item)
 {
 	return g_strdup ("Line");
 }
 
+static void
+sp_line_write_transform (SPItem *item, SPRepr *repr, gdouble *t)
+{
+	double sw, sh;
+	SPLine *line;
+
+	line = SP_LINE (item);
+
+	/* fixme: Would be nice to preserve units here */
+	sp_repr_set_double (repr, "x1", t[0] * line->x1 + t[2] * line->y1 + t[4]);
+	sp_repr_set_double (repr, "y1", t[1] * line->x1 + t[3] * line->y1 + t[5]);
+	sp_repr_set_double (repr, "x2", t[0] * line->x2 + t[2] * line->y2 + t[4]);
+	sp_repr_set_double (repr, "y2", t[1] * line->x2 + t[3] * line->y2 + t[5]);
+
+	/* Scalers */
+	sw = sqrt (t[0] * t[0] + t[1] * t[1]);
+	sh = sqrt (t[2] * t[2] + t[3] * t[3]);
+
+	/* And last but not least */
+	if ((fabs (sw - 1.0) > 1e-9) || (fabs (sh - 1.0) > 1e-9)) {
+		SPStyle *style;
+		guchar *str;
+		/* Scale changed, so we have to adjust stroke width */
+		style = SP_OBJECT_STYLE (item);
+		style->stroke_width.computed *= sqrt (fabs (sw * sh));
+		str = sp_style_write_difference (style, SP_OBJECT_STYLE (SP_OBJECT_PARENT (item)));
+		sp_repr_set_attr (SP_OBJECT_REPR (item), "style", str);
+		g_free (str);
+	}
+}
+
 static void
 sp_line_set_shape (SPLine * line)
 {
