@@ -1,5 +1,17 @@
 #define __SP_NAMEDVIEW_C__
 
+/*
+ * <sodipodi:namedview> implementation
+ *
+ * Authors:
+ *   Lauris Kaplinski <lauris@kaplinski.com>
+ *
+ * Copyright (C) 1999-2002 Authors
+ * Copyright (C) 2000-2001 Ximian, Inc.
+ *
+ * Released under GNU GPL, read the file 'COPYING' for more information
+ */
+
 #include <math.h>
 #include <string.h>
 #include <ctype.h>
@@ -21,15 +33,16 @@
 
 static void sp_namedview_class_init (SPNamedViewClass * klass);
 static void sp_namedview_init (SPNamedView * namedview);
-static void sp_namedview_destroy (GtkObject * object);
 
 static void sp_namedview_build (SPObject * object, SPDocument * document, SPRepr * repr);
+static void sp_namedview_release (SPObject *object);
 static void sp_namedview_read_attr (SPObject * object, const gchar * key);
 static void sp_namedview_child_added (SPObject * object, SPRepr * child, SPRepr * ref);
-static void sp_namedview_remove_child (SPObject * object, SPRepr * child);
+static void sp_namedview_remove_child (SPObject *object, SPRepr *child);
+static SPRepr *sp_namedview_write (SPObject *object, SPRepr *repr, guint flags);
 
 static void sp_namedview_setup_grid (SPNamedView * nv);
-static void sp_namedview_setup_grid_item (SPNamedView * nv, GnomeCanvasItem * item);
+static void sp_namedview_setup_grid_item (SPNamedView * nv, SPCanvasItem * item);
 
 static gboolean sp_str_to_bool (const guchar *str);
 static gboolean sp_nv_read_length (const guchar *str, guint base, gdouble *val, const SPUnit **unit);
@@ -48,9 +61,7 @@ sp_namedview_get_type (void)
 			sizeof (SPNamedViewClass),
 			(GtkClassInitFunc) sp_namedview_class_init,
 			(GtkObjectInitFunc) sp_namedview_init,
-			NULL, /* reserved_1 */
-			NULL, /* reserved_2 */
-			(GtkClassInitFunc) NULL
+			NULL, NULL, NULL
 		};
 		namedview_type = gtk_type_unique (sp_objectgroup_get_type (), &namedview_info);
 	}
@@ -68,12 +79,12 @@ sp_namedview_class_init (SPNamedViewClass * klass)
 
 	parent_class = gtk_type_class (sp_objectgroup_get_type ());
 
-	gtk_object_class->destroy = sp_namedview_destroy;
-
 	sp_object_class->build = sp_namedview_build;
+	sp_object_class->release = sp_namedview_release;
 	sp_object_class->read_attr = sp_namedview_read_attr;
 	sp_object_class->child_added = sp_namedview_child_added;
 	sp_object_class->remove_child = sp_namedview_remove_child;
+	sp_object_class->write = sp_namedview_write;
 }
 
 static void
@@ -84,48 +95,13 @@ sp_namedview_init (SPNamedView * nv)
 	nv->snaptogrid = FALSE;
 	nv->showguides = FALSE;
 	nv->snaptoguides = FALSE;
-#if 0
-	/* Defualt will be assigned in build anyways */
-	nv->gridunit = sp_unit_get_by_abbreviation ("mm");
-	nv->gridorigin.x = nv->gridorigin.y = 0.0;
-	nv->gridspacing.x = nv->gridspacing.y = PTPERMM;
-	nv->gridtoleranceunit = sp_unit_get_identity (SP_UNIT_DEVICE);
-	nv->gridtolerance = DEFAULTTOLERANCE;
-	nv->gridtoleranceunit = sp_unit_get_identity (SP_UNIT_DEVICE);
-	nv->guidetolerance = DEFAULTTOLERANCE;
-#endif
-#if 0
-	nv->gridcolor = 0x3f3fff3f;
-	nv->guidecolor = 0x0000ff7f;
-	nv->guidehicolor = 0xff00007f;
-#endif
+	nv->showborder = TRUE;
 
 	nv->hguides = NULL;
 	nv->vguides = NULL;
 	nv->viewcount = 0;
 }
 
-static void
-sp_namedview_destroy (GtkObject * object)
-{
-	SPNamedView * namedview;
-
-	namedview = (SPNamedView *) object;
-
-	g_slist_free (namedview->hguides);
-	g_slist_free (namedview->vguides);
-
-	g_assert (!namedview->views);
-
-	while (namedview->gridviews) {
-		gtk_object_destroy (GTK_OBJECT (namedview->gridviews->data));
-		namedview->gridviews = g_slist_remove (namedview->gridviews, namedview->gridviews->data);
-	}
-
-	if (((GtkObjectClass *) (parent_class))->destroy)
-		(* ((GtkObjectClass *) (parent_class))->destroy) (object);
-}
-
 static void
 sp_namedview_build (SPObject * object, SPDocument * document, SPRepr * repr)
 {
@@ -156,6 +132,7 @@ sp_namedview_build (SPObject * object, SPDocument * document, SPRepr * repr)
 	sp_namedview_read_attr (object, "guideopacity");
 	sp_namedview_read_attr (object, "guidehicolor");
 	sp_namedview_read_attr (object, "guidehiopacity");
+	sp_namedview_read_attr (object, "showborder");
 
 	/* Construct guideline list */
 
@@ -173,6 +150,34 @@ sp_namedview_build (SPObject * object, SPDocument * document, SPRepr * repr)
 	}
 }
 
+static void
+sp_namedview_release (SPObject * object)
+{
+	SPNamedView *namedview;
+
+	namedview = (SPNamedView *) object;
+
+	if (namedview->hguides) {
+		g_slist_free (namedview->hguides);
+		namedview->hguides = NULL;
+	}
+
+	if (namedview->vguides) {
+		g_slist_free (namedview->vguides);
+		namedview->vguides = NULL;
+	}
+
+	g_assert (!namedview->views);
+
+	while (namedview->gridviews) {
+		gtk_object_destroy (GTK_OBJECT (namedview->gridviews->data));
+		namedview->gridviews = g_slist_remove (namedview->gridviews, namedview->gridviews->data);
+	}
+
+	if (((SPObjectClass *) parent_class)->release)
+		((SPObjectClass *) parent_class)->release (object);
+}
+
 static void
 sp_namedview_read_attr (SPObject * object, const gchar * key)
 {
@@ -194,24 +199,19 @@ sp_namedview_read_attr (SPObject * object, const gchar * key)
 	if (!strcmp (key, "viewonly")) {
 		nv->editable = (astr == NULL);
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		return;
 	} else if (!strcmp (key, "showgrid")) {
 		nv->showgrid = sp_str_to_bool (astr);
 		sp_namedview_setup_grid (nv);
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		return;
 	} else if (!strcmp (key, "snaptogrid")) {
 		nv->snaptogrid = sp_str_to_bool (astr);
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		return;
 	} else if (!strcmp (key, "showguides")) {
 		nv->showguides = sp_str_to_bool (astr);
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		return;
 	} else if (!strcmp (key, "snaptoguides")) {
 		nv->snaptoguides = sp_str_to_bool (astr);
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		return;
 	} else if (!strcmp (key, "gridtolerance")) {
 		nv->gridtoleranceunit = px;
 		nv->gridtolerance = DEFAULTTOLERANCE;
@@ -219,7 +219,6 @@ sp_namedview_read_attr (SPObject * object, const gchar * key)
 			sp_nv_read_length (astr, SP_UNIT_ABSOLUTE | SP_UNIT_DEVICE, &nv->gridtolerance, &nv->gridtoleranceunit);
 		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		return;
 	} else if (!strcmp (key, "guidetolerance")) {
 		nv->guidetoleranceunit = px;
 		nv->guidetolerance = DEFAULTTOLERANCE;
@@ -227,7 +226,6 @@ sp_namedview_read_attr (SPObject * object, const gchar * key)
 			sp_nv_read_length (astr, SP_UNIT_ABSOLUTE | SP_UNIT_DEVICE, &nv->guidetolerance, &nv->guidetoleranceunit);
 		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		return;
 	} else if (strcmp (key, "gridoriginx") == 0) {
 		nv->gridunit = mm;
 		nv->gridoriginx = 0.0;
@@ -237,7 +235,6 @@ sp_namedview_read_attr (SPObject * object, const gchar * key)
 		sp_convert_distance (&nv->gridoriginx, nv->gridunit, pt);
 		sp_namedview_setup_grid (nv);
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		return;
 	} else if (!strcmp (key, "gridoriginy")) {
 		nv->gridunit = mm;
 		nv->gridoriginy = 0.0;
@@ -247,7 +244,6 @@ sp_namedview_read_attr (SPObject * object, const gchar * key)
 		sp_convert_distance (&nv->gridoriginy, nv->gridunit, pt);
 		sp_namedview_setup_grid (nv);
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		return;
 	} else if (!strcmp (key, "gridspacingx")) {
 		nv->gridunit = mm;
 		nv->gridspacingx = 5.0;
@@ -257,7 +253,6 @@ sp_namedview_read_attr (SPObject * object, const gchar * key)
 		sp_convert_distance (&nv->gridspacingx, nv->gridunit, pt);
 		sp_namedview_setup_grid (nv);
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		return;
 	} else if (!strcmp (key, "gridspacingy")) {
 		nv->gridunit = mm;
 		nv->gridspacingy = 5.0;
@@ -267,7 +262,6 @@ sp_namedview_read_attr (SPObject * object, const gchar * key)
 		sp_convert_distance (&nv->gridspacingy, nv->gridunit, pt);
 		sp_namedview_setup_grid (nv);
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		return;
 	} else if (!strcmp (key, "gridcolor")) {
 		nv->gridcolor = (nv->gridcolor & 0xff) | (DEFAULTGRIDCOLOR & 0xffffff00);
 		if (astr) {
@@ -275,13 +269,11 @@ sp_namedview_read_attr (SPObject * object, const gchar * key)
 		}
 		sp_namedview_setup_grid (nv);
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		return;
 	} else if (!strcmp (key, "gridopacity")) {
 		nv->gridcolor = (nv->gridcolor & 0xffffff00) | (DEFAULTGRIDCOLOR & 0xff);
 		sp_nv_read_opacity (astr, &nv->gridcolor);
 		sp_namedview_setup_grid (nv);
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		return;
 	} else if (!strcmp (key, "guidecolor")) {
 		nv->guidecolor = (nv->guidecolor & 0xff) | (DEFAULTGUIDECOLOR & 0xffffff00);
 		if (astr) {
@@ -294,7 +286,6 @@ sp_namedview_read_attr (SPObject * object, const gchar * key)
 			gtk_object_set (GTK_OBJECT (l->data), "color", nv->guidecolor, NULL);
 		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		return;
 	} else if (!strcmp (key, "guideopacity")) {
 		nv->guidecolor = (nv->guidecolor & 0xffffff00) | (DEFAULTGUIDECOLOR & 0xff);
 		sp_nv_read_opacity (astr, &nv->guidecolor);
@@ -305,7 +296,6 @@ sp_namedview_read_attr (SPObject * object, const gchar * key)
 			gtk_object_set (GTK_OBJECT (l->data), "color", nv->guidecolor, NULL);
 		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		return;
 	} else if (!strcmp (key, "guidehicolor")) {
 		nv->guidehicolor = (nv->guidehicolor & 0xff) | (DEFAULTGUIDEHICOLOR & 0xffffff00);
 		if (astr) {
@@ -318,7 +308,6 @@ sp_namedview_read_attr (SPObject * object, const gchar * key)
 			gtk_object_set (GTK_OBJECT (l->data), "hicolor", nv->guidehicolor, NULL);
 		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		return;
 	} else if (!strcmp (key, "guidehiopacity")) {
 		nv->guidehicolor = (nv->guidehicolor & 0xffffff00) | (DEFAULTGUIDEHICOLOR & 0xff);
 		sp_nv_read_opacity (astr, &nv->guidehicolor);
@@ -329,11 +318,13 @@ sp_namedview_read_attr (SPObject * object, const gchar * key)
 			gtk_object_set (GTK_OBJECT (l->data), "hicolor", nv->guidehicolor, NULL);
 		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		return;
+	} else if (!strcmp (key, "showborder")) {
+		nv->showborder = (astr) ? sp_str_to_bool (astr) : TRUE;
+		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+	} else {
+		if (((SPObjectClass *) (parent_class))->read_attr)
+			(* ((SPObjectClass *) (parent_class))->read_attr) (object, key);
 	}
-
-	if (((SPObjectClass *) (parent_class))->read_attr)
-		(* ((SPObjectClass *) (parent_class))->read_attr) (object, key);
 }
 
 static void
@@ -398,12 +389,30 @@ sp_namedview_remove_child (SPObject * object, SPRepr * child)
 		(* ((SPObjectClass *) (parent_class))->remove_child) (object, child);
 }
 
+static SPRepr *
+sp_namedview_write (SPObject *object, SPRepr *repr, guint flags)
+{
+	SPNamedView *nv;
+
+	nv = SP_NAMEDVIEW (object);
+
+	if (flags & SP_OBJECT_WRITE_SODIPODI) {
+		if (repr) {
+			sp_repr_merge (repr, SP_OBJECT_REPR (object), "id");
+		} else {
+			repr = sp_repr_duplicate (SP_OBJECT_REPR (object));
+		}
+	}
+
+	return repr;
+}
+
 void
 sp_namedview_show (SPNamedView * nv, gpointer desktop)
 {
 	SPDesktop * dt;
 	GSList * l;
-	GnomeCanvasItem * item;
+	SPCanvasItem * item;
 
 	dt = SP_DESKTOP (desktop);
 
@@ -419,7 +428,7 @@ sp_namedview_show (SPNamedView * nv, gpointer desktop)
 
 	nv->views = g_slist_prepend (nv->views, desktop);
 
-	item = gnome_canvas_item_new (SP_DT_GRID (dt), SP_TYPE_CGRID, NULL);
+	item = sp_canvas_item_new (SP_DT_GRID (dt), SP_TYPE_CGRID, NULL);
 	nv->gridviews = g_slist_prepend (nv->gridviews, item);
 	sp_namedview_setup_grid_item (nv, item);
 }
@@ -449,7 +458,7 @@ sp_namedview_hide (SPNamedView * nv, gpointer desktop)
 	nv->views = g_slist_remove (nv->views, desktop);
 
 	for (l = nv->gridviews; l != NULL; l = l->next) {
-		if (GNOME_CANVAS_ITEM (l->data)->canvas == SP_DT_CANVAS (dt)) break;
+		if (SP_CANVAS_ITEM (l->data)->canvas == SP_DT_CANVAS (dt)) break;
 	}
 
 	g_assert (l);
@@ -487,52 +496,30 @@ sp_namedview_setup_grid (SPNamedView * nv)
 	GSList * l;
 
 	for (l = nv->gridviews; l != NULL; l = l->next) {
-		sp_namedview_setup_grid_item (nv, GNOME_CANVAS_ITEM (l->data));
+		sp_namedview_setup_grid_item (nv, SP_CANVAS_ITEM (l->data));
 	}
 }
 
 static void
-sp_namedview_setup_grid_item (SPNamedView * nv, GnomeCanvasItem * item)
+sp_namedview_setup_grid_item (SPNamedView * nv, SPCanvasItem * item)
 {
 	const SPUnit *pt = NULL;
-#if 0
-	gdouble x0, y0, xs, ys;
-#endif
 
 	if (nv->showgrid) {
-		gnome_canvas_item_show (item);
+		sp_canvas_item_show (item);
 	} else {
-		gnome_canvas_item_hide (item);
+		sp_canvas_item_hide (item);
 	}
 
 	if (!pt) pt = sp_unit_get_identity (SP_UNIT_ABSOLUTE);
 
-#if 0
-	x0 = nv->gridoriginx;
-	sp_convert_distance (&x0, nv->gridunit, pt);
-	y0 = nv->gridoriginy;
-	sp_convert_distance (&y0, nv->gridunit, pt);
-	xs = nv->gridspacingx;
-	sp_convert_distance (&xs, nv->gridunit, pt);
-	ys = nv->gridspacingy;
-	sp_convert_distance (&ys, nv->gridunit, pt);
-
-	gnome_canvas_item_set (item,
-			       "color", nv->gridcolor,
-			       "originx", x0,
-			       "originy", y0,
-			       "spacingx", xs,
-			       "spacingy", ys,
-			       NULL);
-#else
-	gnome_canvas_item_set (item,
+	sp_canvas_item_set ((GtkObject *) item,
 			       "color", nv->gridcolor,
 			       "originx", nv->gridoriginx,
 			       "originy", nv->gridoriginy,
 			       "spacingx", nv->gridspacingx,
 			       "spacingy", nv->gridspacingy,
 			       NULL);
-#endif
 }
 
 const gchar *
