@@ -1,20 +1,21 @@
-#ifndef SP_NAMEDVIEW_H
-#define SP_NAMEDVIEW_H
+#ifndef __SP_NAMEDVIEW_H__
+#define __SP_NAMEDVIEW_H__
 
 /*
- * SPNamedView
+ * <sodipodi:namedview> implementation
  *
- * A view into document, from which desktop can be constructed.
- * It holds per-desktop data, such as guides, grids etc.
+ * Authors:
+ *   Lauris Kaplinski <lauris@kaplinski.com>
  *
- * Copyright (C) Lauris Kaplinski 2000
+ * Copyright (C) Lauris Kaplinski 2000-2002
  *
+ * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#define SP_TYPE_NAMEDVIEW            (sp_namedview_get_type ())
-#define SP_NAMEDVIEW(obj)            (GTK_CHECK_CAST ((obj), SP_TYPE_NAMEDVIEW, SPNamedView))
-#define SP_NAMEDVIEW_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), SP_TYPE_NAMEDVIEW, SPNamedViewClass))
-#define SP_IS_NAMEDVIEW(obj)         (GTK_CHECK_TYPE ((obj), SP_TYPE_NAMEDVIEW))
+#define SP_TYPE_NAMEDVIEW (sp_namedview_get_type ())
+#define SP_NAMEDVIEW(obj) (GTK_CHECK_CAST ((obj), SP_TYPE_NAMEDVIEW, SPNamedView))
+#define SP_NAMEDVIEW_CLASS(klass) (GTK_CHECK_CLASS_CAST ((klass), SP_TYPE_NAMEDVIEW, SPNamedViewClass))
+#define SP_IS_NAMEDVIEW(obj) (GTK_CHECK_TYPE ((obj), SP_TYPE_NAMEDVIEW))
 #define SP_IS_NAMEDVIEW_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), SP_TYPE_NAMEDVIEW))
 
 #include "helper/units.h"
@@ -27,6 +28,7 @@ struct _SPNamedView {
 	guint snaptogrid : 1;
 	guint showguides : 1;
 	guint snaptoguides : 1;
+	guint showborder : 1;
 
 	const SPUnit *gridunit;
 	/* Grid data is in points regardless of unit */
