@@ -1,16 +1,15 @@
 #define __SP_OBJECTGROUP_C__
 
 /*
- * Abstract base class for SPObjects with multiple children, which are not
- * items themselves (i.e. not directly renderable)
+ * Abstract base class for all nodes
  *
- * Author:
- *   Lauris Kaplinski <lauris@ximian.com>
+ * Authors:
+ *   Lauris Kaplinski <lauris@kaplinski.com>
  *
- * Copyright (C) 2000-2001 Lauris Kaplinski and Ximian, Inc.
- *
- * Released under GNU GPL
+ * Copyright (C) 1999-2002 authors
+ * Copyright (C) 2001-2002 Ximian, Inc.
  *
+ * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
 #include "xml/repr-private.h"
@@ -19,12 +18,13 @@
 
 static void sp_objectgroup_class_init (SPObjectGroupClass *klass);
 static void sp_objectgroup_init (SPObjectGroup *objectgroup);
-static void sp_objectgroup_destroy (GtkObject *object);
 
 static void sp_objectgroup_build (SPObject * object, SPDocument * document, SPRepr * repr);
+static void sp_objectgroup_release (SPObject *object);
 static void sp_objectgroup_child_added (SPObject * object, SPRepr * child, SPRepr * ref);
 static void sp_objectgroup_remove_child (SPObject * object, SPRepr * child);
 static void sp_objectgroup_order_changed (SPObject * object, SPRepr * child, SPRepr * old, SPRepr * new);
+static SPRepr *sp_objectgroup_write (SPObject *object, SPRepr *repr, guint flags);
 
 static SPObject *sp_objectgroup_get_le_child_by_repr (SPObjectGroup *og, SPRepr *ref);
 
@@ -59,12 +59,12 @@ sp_objectgroup_class_init (SPObjectGroupClass *klass)
 
 	parent_class = gtk_type_class (sp_object_get_type ());
 
-	gtk_object_class->destroy = sp_objectgroup_destroy;
-
 	sp_object_class->build = sp_objectgroup_build;
+	sp_object_class->release = sp_objectgroup_release;
 	sp_object_class->child_added = sp_objectgroup_child_added;
 	sp_object_class->remove_child = sp_objectgroup_remove_child;
 	sp_object_class->order_changed = sp_objectgroup_order_changed;
+	sp_object_class->write = sp_objectgroup_write;
 }
 
 static void
@@ -73,21 +73,6 @@ sp_objectgroup_init (SPObjectGroup *objectgroup)
 	objectgroup->children = NULL;
 }
 
-static void
-sp_objectgroup_destroy (GtkObject *object)
-{
-	SPObjectGroup * og;
-
-	og = SP_OBJECTGROUP (object);
-
-	while (og->children) {
-		og->children = sp_object_detach_unref (SP_OBJECT (og), og->children);
-	}
-
-	if (((GtkObjectClass *) (parent_class))->destroy)
-		(* ((GtkObjectClass *) (parent_class))->destroy) (object);
-}
-
 static void sp_objectgroup_build (SPObject *object, SPDocument *document, SPRepr *repr)
 {
 	SPObjectGroup *og;
@@ -113,6 +98,21 @@ static void sp_objectgroup_build (SPObject *object, SPDocument *document, SPRepr
 	}
 }
 
+static void
+sp_objectgroup_release (SPObject *object)
+{
+	SPObjectGroup *og;
+
+	og = SP_OBJECTGROUP (object);
+
+	while (og->children) {
+		og->children = sp_object_detach_unref (object, og->children);
+	}
+
+	if (((SPObjectClass *) parent_class)->release)
+		((SPObjectClass *) parent_class)->release (object);
+}
+
 static void
 sp_objectgroup_child_added (SPObject *object, SPRepr *child, SPRepr *ref)
 {
@@ -185,6 +185,40 @@ sp_objectgroup_order_changed (SPObject *object, SPRepr *child, SPRepr *old, SPRe
 	}
 }
 
+static SPRepr *
+sp_objectgroup_write (SPObject *object, SPRepr *repr, guint flags)
+{
+	SPObjectGroup *group;
+	SPObject *child;
+	SPRepr *crepr;
+
+	group = SP_OBJECTGROUP (object);
+
+	if (flags & SP_OBJECT_WRITE_BUILD) {
+		GSList *l;
+		if (!repr) repr = sp_repr_new ("g");
+		l = NULL;
+		for (child = group->children; child != NULL; child = child->next) {
+			crepr = sp_object_invoke_write (child, NULL, flags);
+			if (crepr) l = g_slist_prepend (l, crepr);
+		}
+		while (l) {
+			sp_repr_add_child (repr, (SPRepr *) l->data, NULL);
+			sp_repr_unref ((SPRepr *) l->data);
+			l = g_slist_remove (l, l->data);
+		}
+	} else {
+		for (child = group->children; child != NULL; child = child->next) {
+			sp_object_invoke_write (child, SP_OBJECT_REPR (child), flags);
+		}
+	}
+
+	if (((SPObjectClass *) (parent_class))->write)
+		((SPObjectClass *) (parent_class))->write (object, repr, flags);
+
+	return repr;
+}
+
 static SPObject *
 sp_objectgroup_get_le_child_by_repr (SPObjectGroup *og, SPRepr *ref)
 {
