@@ -1,12 +1,15 @@
-#define SP_OBJECT_C
+#define __SP_OBJECT_C__
 
 /*
- * SPObject
+ * Abstract base class for all nodes
  *
- * This is most abstract of all typed objects
+ * Authors:
+ *   Lauris Kaplinski <lauris@kaplinski.com>
  *
- * Copyright (C) Lauris Kaplinski <lauris@ximian.com> 1999-2000
+ * Copyright (C) 1999-2002 authors
+ * Copyright (C) 2001-2002 Ximian, Inc.
  *
+ * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
 #include <stdlib.h>
@@ -23,7 +26,10 @@ static void sp_object_init (SPObject * object);
 static void sp_object_destroy (GtkObject * object);
 
 static void sp_object_build (SPObject * object, SPDocument * document, SPRepr * repr);
+static void sp_object_release (SPObject *object);
+
 static void sp_object_read_attr (SPObject * object, const gchar * key);
+static SPRepr *sp_object_private_write (SPObject *object, SPRepr *repr, guint flags);
 
 /* Real handlers of repr signals */
 
@@ -39,7 +45,7 @@ static void sp_object_repr_order_changed (SPRepr *repr, SPRepr *child, SPRepr *o
 
 static gchar * sp_object_get_unique_id (SPObject * object, const gchar * defid);
 
-enum {MODIFIED, LAST_SIGNAL};
+enum {RELEASE, MODIFIED, LAST_SIGNAL};
 
 SPReprEventVector object_event_vector = {
 	NULL, /* Destroy */
@@ -87,6 +93,12 @@ sp_object_class_init (SPObjectClass * klass)
 
 	parent_class = gtk_type_class (gtk_object_get_type ());
 
+	object_signals[RELEASE] =  gtk_signal_new ("release",
+						   GTK_RUN_FIRST,
+						   gtk_object_class->type,
+						   GTK_SIGNAL_OFFSET (SPObjectClass, release),
+						   gtk_marshal_NONE__NONE,
+						   GTK_TYPE_NONE, 0);
 	object_signals[MODIFIED] = gtk_signal_new ("modified",
 						   GTK_RUN_FIRST,
 						   gtk_object_class->type,
@@ -98,8 +110,9 @@ sp_object_class_init (SPObjectClass * klass)
 	gtk_object_class->destroy = sp_object_destroy;
 
 	klass->build = sp_object_build;
-	klass->write_repr = NULL;
+	klass->release = sp_object_release;
 	klass->read_attr = sp_object_read_attr;
+	klass->write = sp_object_private_write;
 }
 
 static void
@@ -118,34 +131,16 @@ sp_object_init (SPObject * object)
 static void
 sp_object_destroy (GtkObject * object)
 {
-	SPObject * spobject;
+	SPObject *spobject;
 
 	spobject = (SPObject *) object;
 
-	/* Parent refcount us, so there shouldn't be any */
-	g_assert (!spobject->parent);
-	g_assert (!spobject->next);
-	g_assert (spobject->document);
-	g_assert (spobject->repr);
-	/* href holders HAVE to release it in "destroy" signal handler */
-	g_assert (spobject->hrefcount == 0);
-
-	if (spobject->style) {
-		sp_style_unref (spobject->style);
-		spobject->style = NULL;
+	/* fixme: This is here temporarily */
+	/* fixme: I f everything is ported to ::release, we can remove destroy at all */
+	if (spobject->document) {
+		sp_object_invoke_release (spobject);
 	}
 
-	if (!SP_OBJECT_IS_CLONED (object)) {
-		g_assert (spobject->id);
-		sp_document_undef_id (spobject->document, spobject->id);
-		g_free (spobject->id);
-	} else {
-		g_assert (!spobject->id);
-	}
-
-	sp_repr_remove_listener_by_data (spobject->repr, spobject);
-	sp_repr_unref (spobject->repr);
-
 	if (((GtkObjectClass *) (parent_class))->destroy)
 		(* ((GtkObjectClass *) (parent_class))->destroy) (object);
 }
@@ -242,6 +237,8 @@ sp_object_detach (SPObject *parent, SPObject *object)
 	object->parent = NULL;
 	object->next = NULL;
 
+	sp_object_invoke_release (object);
+
 	return next;
 }
 
@@ -259,6 +256,9 @@ sp_object_detach_unref (SPObject *parent, SPObject *object)
 	next = object->next;
 	object->parent = NULL;
 	object->next = NULL;
+
+	sp_object_invoke_release (object);
+
 	sp_object_unref (object, parent);
 
 	return next;
@@ -274,6 +274,12 @@ sp_object_build (SPObject * object, SPDocument * document, SPRepr * repr)
 	/* Nothing specific here */
 }
 
+static void
+sp_object_release (SPObject * object)
+{
+	/* Nothing specific here */
+}
+
 void
 sp_object_invoke_build (SPObject * object, SPDocument * document, SPRepr * repr, gboolean cloned)
 {
@@ -326,6 +332,43 @@ sp_object_invoke_build (SPObject * object, SPDocument * document, SPRepr * repr,
 	sp_repr_add_listener (repr, &object_event_vector, object);
 }
 
+void
+sp_object_invoke_release (SPObject *object)
+{
+	g_assert (object != NULL);
+	g_assert (SP_IS_OBJECT (object));
+
+	/* Parent refcount us, so there shouldn't be any */
+	g_assert (!object->parent);
+	g_assert (!object->next);
+	g_assert (object->document);
+	g_assert (object->repr);
+
+	gtk_signal_emit (GTK_OBJECT (object), object_signals[RELEASE]);
+
+	/* href holders HAVE to release it in signal handler */
+	g_assert (object->hrefcount == 0);
+
+	if (object->style) {
+		object->style = sp_style_unref (object->style);
+	}
+
+	if (!SP_OBJECT_IS_CLONED (object)) {
+		g_assert (object->id);
+		sp_document_undef_id (object->document, object->id);
+		g_free (object->id);
+		object->id = NULL;
+	} else {
+		g_assert (!object->id);
+	}
+
+	sp_repr_remove_listener_by_data (object->repr, object);
+	sp_repr_unref (object->repr);
+
+	object->document = NULL;
+	object->repr = NULL;
+}
+
 static void
 sp_object_repr_child_added (SPRepr *repr, SPRepr *child, SPRepr *ref, gpointer data)
 {
@@ -369,23 +412,6 @@ sp_object_repr_order_changed (SPRepr * repr, SPRepr * child, SPRepr * old, SPRep
 	sp_document_order_changed (object->document, object, child, old, new);
 }
 
-void
-sp_object_invoke_write_repr (SPObject * object,
-			     SPRepr * repr)
-{
-	g_assert (object != NULL);
-	g_assert (SP_IS_OBJECT (object));
-	g_assert (repr != NULL);
-
-	g_assert (SP_IS_DOCUMENT (object->document));
-	g_assert (object->repr != NULL);
-	/* fixme: rething that cloning issue */
-	g_assert (SP_OBJECT_IS_CLONED (object) || object->id != NULL);
-
-	if (((SPObjectClass *)(((GtkObject *) object)->klass))->write_repr)
-		(*((SPObjectClass *)(((GtkObject *) object)->klass))->write_repr) (object, repr);
-}
-
 static void
 sp_object_read_attr (SPObject * object, const gchar * key)
 {
@@ -470,6 +496,70 @@ sp_object_repr_content_changed (SPRepr *repr, const guchar *oldcontent, const gu
 	sp_document_content_changed (object->document, object, oldcontent, newcontent);
 }
 
+void
+sp_object_invoke_forall (SPObject *object, SPObjectMethod func, gpointer data)
+{
+	SPRepr *repr, *child;
+
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (SP_IS_OBJECT (object));
+	g_return_if_fail (func != NULL);
+
+	func (object, data);
+
+#if 0
+
+	if (((SPObjectClass *) (((GtkObject *) object)->klass))->forall)
+		((SPObjectClass *) (((GtkObject *) object)->klass))->forall (object, func, data);
+#else
+	repr = SP_OBJECT_REPR (object);
+	for (child = repr->children; child != NULL; child = child->next) {
+		const unsigned char *id;
+		SPObject *cho;
+		id = sp_repr_attr (child, "id");
+		cho = sp_document_lookup_id (SP_OBJECT_DOCUMENT (object), id);
+		if (cho) sp_object_invoke_forall (cho, func, data);
+	}
+#endif
+}
+
+static SPRepr *
+sp_object_private_write (SPObject *object, SPRepr *repr, guint flags)
+{
+	if (!repr && (flags & SP_OBJECT_WRITE_BUILD)) {
+		repr = sp_repr_duplicate (SP_OBJECT_REPR (object));
+	} else {
+		sp_repr_set_attr (repr, "id", object->id);
+	}
+
+	return repr;
+}
+
+SPRepr *
+sp_object_invoke_write (SPObject *object, SPRepr *repr, guint flags)
+{
+	g_return_val_if_fail (object != NULL, NULL);
+	g_return_val_if_fail (SP_IS_OBJECT (object), NULL);
+
+	if (((SPObjectClass *) (((GtkObject *) object)->klass))->write) {
+		if (!(flags & SP_OBJECT_WRITE_BUILD) && !repr) {
+			repr = SP_OBJECT_REPR (object);
+		}
+		return ((SPObjectClass *) (((GtkObject *) object)->klass))->write (object, repr, flags);
+	} else {
+		g_warning ("Class %s does not implement ::write", gtk_type_name (GTK_OBJECT_TYPE (object)));
+		if (!repr) {
+			if (flags & SP_OBJECT_WRITE_BUILD) {
+				repr = sp_repr_duplicate (SP_OBJECT_REPR (object));
+			}
+			/* fixme: else probably error (Lauris) */
+		} else {
+			sp_repr_merge (repr, SP_OBJECT_REPR (object), "id");
+		}
+		return repr;
+	}
+}
+
 /* Modification */
 
 void
