@@ -1,21 +1,18 @@
-#ifndef SP_OBJECT_H
-#define SP_OBJECT_H
+#ifndef __SP_OBJECT_H__
+#define __SP_OBJECT_H__
 
 /*
- * SPObject
+ * Abstract base class for all nodes
  *
- * This is most abstract of all typed objects
+ * Authors:
+ *   Lauris Kaplinski <lauris@kaplinski.com>
  *
- * Copyright (C) Lauris Kaplinski <lauris@ariman.ee> 1999-2000
+ * Copyright (C) 1999-2002 authors
+ * Copyright (C) 2001-2002 Ximian, Inc.
  *
+ * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#define SP_TYPE_OBJECT (sp_object_get_type ())
-#define SP_OBJECT(obj) (GTK_CHECK_CAST ((obj), SP_TYPE_OBJECT, SPObject))
-#define SP_OBJECT_CLASS(klass) (GTK_CHECK_CLASS_CAST ((klass), SP_TYPE_OBJECT, SPObjectClass))
-#define SP_IS_OBJECT(obj) (GTK_CHECK_TYPE ((obj), SP_TYPE_OBJECT))
-#define SP_IS_OBJECT_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), SP_TYPE_OBJECT))
-
 /* Generic */
 #define SP_OBJECT_FLAGS GTK_OBJECT_FLAGS
 #define SP_OBJECT_SET_FLAGS GTK_OBJECT_SET_FLAGS
@@ -44,6 +41,10 @@
 #define SP_OBJECT_IS_MODIFIED(o) (GTK_OBJECT_FLAGS (o) & SP_OBJECT_MODIFIED_FLAG)
 #define SP_OBJECT_CHILD_IS_MODIFIED(o) (GTK_OBJECT_FLAGS (o) & SP_OBJECT_CHILD_MODIFIED_FLAG)
 
+/* Write flags */
+#define SP_OBJECT_WRITE_BUILD (1 << 0)
+#define SP_OBJECT_WRITE_SODIPODI (1 << 1)
+
 /* Convenience stuff */
 #define SP_OBJECT_ID(o) (SP_OBJECT (o)->id)
 #define SP_OBJECT_REPR(o) (SP_OBJECT (o)->repr)
@@ -52,12 +53,16 @@
 #define SP_OBJECT_NEXT(o) (SP_OBJECT (o)->next)
 #define SP_OBJECT_HREFCOUNT(o) (SP_OBJECT (o)->hrefcount)
 #define SP_OBJECT_STYLE(o) (SP_OBJECT (o)->style)
+#define SP_OBJECT_TITLE(o) (SP_OBJECT (o)->title)
+#define SP_OBJECT_DESCRIPTION(o) (SP_OBJECT (o)->description)
 
 #include <gtk/gtktypeutils.h>
 #include <gtk/gtkobject.h>
 #include "xml/repr.h"
 #include "forward.h"
 
+typedef void (* SPObjectMethod) (SPObject *object, gpointer data);
+
 typedef enum {
 	SP_NO_EXCEPTION,
 	SP_INDEX_SIZE_ERR,
@@ -95,15 +100,15 @@ struct _SPObject {
 	SPRepr *repr; /* Our xml representation */
 	gchar *id; /* Our very own unique id */
 	SPStyle *style;
-	const gchar *title; /* Our title, if any */
-	const gchar *description; /* Our description, if any */
+	const guchar *title; /* Our title, if any */
+	const guchar *description; /* Our description, if any */
 };
 
 struct _SPObjectClass {
 	GtkObjectClass parent_class;
 
-	void (* build) (SPObject * object, SPDocument * document, SPRepr * repr);
-	void (* write_repr) (SPObject * object, SPRepr * repr);
+	void (* build) (SPObject *object, SPDocument *doc, SPRepr *repr);
+	void (* release) (SPObject *object);
 
 	/* Virtual handlers of repr signals */
 	void (* child_added) (SPObject * object, SPRepr * child, SPRepr * ref);
@@ -121,9 +126,10 @@ struct _SPObjectClass {
 
 	/* Compute next sequence number */
 	gint (* sequence) (SPObject *object, gint seq);
-};
+	void (* forall) (SPObject *object, SPObjectMethod func, gpointer data);
 
-GtkType sp_object_get_type (void);
+	SPRepr * (* write) (SPObject *object, SPRepr *repr, guint flags);
+};
 
 /*
  * Refcounting
@@ -150,17 +156,12 @@ SPObject *sp_object_detach (SPObject *parent, SPObject *object);
 SPObject *sp_object_detach_unref (SPObject *parent, SPObject *object);
 
 void sp_object_invoke_build (SPObject * object, SPDocument * document, SPRepr * repr, gboolean cloned);
-void sp_object_invoke_write_repr (SPObject * object, SPRepr * repr);
+void sp_object_invoke_release (SPObject *object);
+
 void sp_object_invoke_read_attr (SPObject * object, const gchar * key);
 
 /* Styling */
 
-#if 0
-/* fixme: this is potentially dangerous - use load/set style instead (Lauris) */
-/* flags are the same as for modification */
-void sp_object_style_changed (SPObject *object, guint flags);
-#endif
-
 /* Modification */
 void sp_object_request_modified (SPObject *object, guint flags);
 void sp_object_modified (SPObject *object, guint flags);
@@ -168,6 +169,10 @@ void sp_object_modified (SPObject *object, guint flags);
 /* Sequence */
 gint sp_object_sequence (SPObject *object, gint seq);
 
+void sp_object_invoke_forall (SPObject *object, SPObjectMethod func, gpointer data);
+/* Write object to repr */
+SPRepr *sp_object_invoke_write (SPObject *object, SPRepr *repr, guint flags);
+
 /* Public */
 
 const guchar *sp_object_getAttribute (SPObject *object, const guchar *key, SPException *ex);
