@@ -1,24 +1,24 @@
 #define __SP_PAINT_SERVER_C__
 
 /*
- * Abstract base class for different paint types
+ * Base class for gradients and patterns
  *
  * Author:
- *   Lauris Kaplinski <lauris@ximian.com>
+ *   Lauris Kaplinski <lauris@kaplinski.com>
  *
- * Copyright (C) 2002 Lauris Kaplinski
- * Copyright (C) 2001 Ximian, Inc.
+ * Copyright (C) 1999-2002 Lauris Kaplinski
+ * Copyright (C) 2000-2001 Ximian, Inc.
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include "helper/nr-plain-stuff.h"
+#include <libnr/nr-pixblock-pattern.h>
 #include "sp-paint-server.h"
 
 static void sp_paint_server_class_init (SPPaintServerClass *klass);
 static void sp_paint_server_init (SPPaintServer *ps);
 
-static void sp_paint_server_destroy (GtkObject *object);
+static void sp_paint_server_release (SPObject *object);
 
 static void sp_painter_stale_fill (SPPainter *painter, guchar *px, gint x0, gint y0, gint width, gint height, gint rowstride);
 
@@ -46,13 +46,13 @@ sp_paint_server_get_type (void)
 static void
 sp_paint_server_class_init (SPPaintServerClass *klass)
 {
-	GtkObjectClass *object_class;
+	SPObjectClass *sp_object_class;
 
-	object_class = (GtkObjectClass *) klass;
+	sp_object_class = (SPObjectClass *) klass;
 
 	parent_class = gtk_type_class (SP_TYPE_OBJECT);
 
-	object_class->destroy = sp_paint_server_destroy;
+	sp_object_class->release = sp_paint_server_release;
 }
 
 static void
@@ -62,7 +62,7 @@ sp_paint_server_init (SPPaintServer *ps)
 }
 
 static void
-sp_paint_server_destroy (GtkObject *object)
+sp_paint_server_release (SPObject *object)
 {
 	SPPaintServer *ps;
 
@@ -78,8 +78,8 @@ sp_paint_server_destroy (GtkObject *object)
 		painter->fill = sp_painter_stale_fill;
 	}
 
-	if (((GtkObjectClass *) parent_class)->destroy)
-		(* ((GtkObjectClass *) parent_class)->destroy) (object);
+	if (((SPObjectClass *) parent_class)->release)
+		((SPObjectClass *) parent_class)->release (object);
 }
 
 SPPainter *
@@ -152,7 +152,10 @@ sp_painter_free (SPPainter *painter)
 static void
 sp_painter_stale_fill (SPPainter *painter, guchar *px, gint x0, gint y0, gint width, gint height, gint rowstride)
 {
-	nr_render_r8g8b8a8_gray_garbage (px, width, height, rowstride);
+	NRPixBlock pb;
+	nr_pixblock_setup_extern (&pb, NR_PIXBLOCK_MODE_R8G8B8A8N, x0, y0, x0 + width, y0 + height, px, rowstride, FALSE, FALSE);
+	nr_pixblock_render_gray_noise (&pb, NULL);
+	nr_pixblock_release (&pb);
 }
 
 
