@@ -15,9 +15,13 @@
 #include <config.h>
 #include <math.h>
 #include <string.h>
+#include <libnr/nr-values.h>
+#include <libnr/nr-macros.h>
 #include <glib.h>
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
+#include <gtk/gtksignal.h>
+#include <gtk/gtkmenuitem.h>
 #include "helper/art-utils.h"
 #include "svg/svg.h"
 #include "style.h"
@@ -29,16 +33,15 @@
 
 static void sp_rect_class_init (SPRectClass *class);
 static void sp_rect_init (SPRect *rect);
-static void sp_rect_destroy (GtkObject *object);
 
 static void sp_rect_build (SPObject *object, SPDocument *document, SPRepr *repr);
 static void sp_rect_read_attr (SPObject *object, const gchar *attr);
 static void sp_rect_modified (SPObject *object, guint flags);
+static SPRepr *sp_rect_write (SPObject *object, SPRepr *repr, guint flags);
 
 static gchar * sp_rect_description (SPItem * item);
 static GSList * sp_rect_snappoints (SPItem * item, GSList * points);
 static void sp_rect_write_transform (SPItem *item, SPRepr *repr, gdouble *transform);
-static void sp_rect_write_repr (SPObject * object, SPRepr *repr);
 static void sp_rect_menu (SPItem *item, SPDesktop *desktop, GtkMenu *menu);
 
 static void sp_rect_rect_properties (GtkMenuItem *menuitem, SPAnchor *anchor);
@@ -82,10 +85,8 @@ sp_rect_class_init (SPRectClass *class)
 
 	parent_class = gtk_type_class (sp_shape_get_type ());
 
-	gtk_object_class->destroy = sp_rect_destroy;
-
 	sp_object_class->build = sp_rect_build;
-	sp_object_class->write_repr = sp_rect_write_repr;
+	sp_object_class->write = sp_rect_write;
 	sp_object_class->read_attr = sp_rect_read_attr;
 	sp_object_class->modified = sp_rect_modified;
 
@@ -116,22 +117,12 @@ sp_rect_init (SPRect * rect)
 	rect->ry.computed = 0.0;
 }
 
-static void
-sp_rect_destroy (GtkObject *object)
-{
-	SPRect *rect;
-
-	rect = SP_RECT (object);
-
-	GTK_OBJECT_CLASS (parent_class)->destroy (object);
-}
-
 static void
 sp_rect_build (SPObject * object, SPDocument * document, SPRepr * repr)
 {
 
-	if (SP_OBJECT_CLASS(parent_class)->build)
-		(* SP_OBJECT_CLASS(parent_class)->build) (object, document, repr);
+	if (((SPObjectClass *) parent_class)->build)
+		((SPObjectClass *) parent_class)->build (object, document, repr);
 
 	sp_rect_read_attr (object, "x");
 	sp_rect_read_attr (object, "y");
@@ -215,20 +206,44 @@ sp_rect_read_attr (SPObject *object, const gchar *attr)
 		sp_rect_set_shape (rect);
 	}
 
-	if (SP_OBJECT_CLASS (parent_class)->read_attr)
-		SP_OBJECT_CLASS (parent_class)->read_attr (object, attr);
+	if (((SPObjectClass *) parent_class)->read_attr)
+		((SPObjectClass *) parent_class)->read_attr (object, attr);
 
 }
 
 static void
 sp_rect_modified (SPObject *object, guint flags)
 {
-	if (SP_OBJECT_CLASS (parent_class)->modified)
-		SP_OBJECT_CLASS (parent_class)->modified (object, flags);
-
 	if ((flags & SP_OBJECT_STYLE_MODIFIED_FLAG) || (flags & SP_OBJECT_VIEWPORT_MODIFIED_FLAG)) {
 		sp_rect_set_shape (SP_RECT (object));
 	}
+
+	if (((SPObjectClass *) parent_class)->modified)
+		((SPObjectClass *) parent_class)->modified (object, flags);
+}
+
+static SPRepr *
+sp_rect_write (SPObject *object, SPRepr *repr, guint flags)
+{
+	SPRect *rect;
+
+	rect = SP_RECT (object);
+
+	if ((flags & SP_OBJECT_WRITE_BUILD) && !repr) {
+		repr = sp_repr_new ("rect");
+	}
+
+	sp_repr_set_double (repr, "width", rect->width.computed);
+	sp_repr_set_double (repr, "height", rect->height.computed);
+	sp_repr_set_double_default (repr, "rx", rect->rx.computed, 0.0, 1e-6);
+	sp_repr_set_double_default (repr, "ry", rect->ry.computed, 0.0, 1e-6);
+	sp_repr_set_double (repr, "x", rect->x.computed);
+	sp_repr_set_double (repr, "y", rect->y.computed);
+
+	if (((SPObjectClass *) parent_class)->write)
+		((SPObjectClass *) parent_class)->write (object, repr, flags);
+
+	return repr;
 }
 
 static gchar *
@@ -369,6 +384,30 @@ sp_rect_set (SPRect * rect, gdouble x, gdouble y, gdouble width, gdouble height)
 	sp_rect_set_shape (rect);
 }
 
+void
+sp_rect_set_rx(SPRect * rect, gboolean set, gdouble value)
+{
+	g_return_if_fail (rect != NULL);
+	g_return_if_fail (SP_IS_RECT (rect));
+
+	rect->rx.set = set;
+	if (set)
+		rect->rx.computed = value;
+	sp_rect_set_shape (rect);
+}
+
+void
+sp_rect_set_ry(SPRect * rect, gboolean set, gdouble value)
+{
+	g_return_if_fail (rect != NULL);
+	g_return_if_fail (SP_IS_RECT (rect));
+
+	rect->ry.set = set;
+	if (set)
+		rect->ry.computed = value;
+	sp_rect_set_shape (rect);
+}
+
 static GSList * 
 sp_rect_snappoints (SPItem *item, GSList *points)
 {
@@ -421,6 +460,7 @@ sp_rect_write_transform (SPItem *item, SPRepr *repr, gdouble *transform)
 	gdouble rev[6];
 	gdouble px, py, sw, sh;
 	guchar t[80];
+	SPStyle *style;
 
 	rect = SP_RECT (item);
 
@@ -467,33 +507,26 @@ sp_rect_write_transform (SPItem *item, SPRepr *repr, gdouble *transform)
 	}
 
 	/* And last but not least */
-	if ((fabs (sw - 1.0) > 1e-9) || (fabs (sh - 1.0) > 1e-9)) {
-		SPStyle *style;
-		guchar *str;
-		/* Scale changed, so we have to adjust stroke width */
-		style = SP_OBJECT_STYLE (item);
-		style->stroke_width.computed *= sqrt (fabs (sw * sh));
-		str = sp_style_write_difference (style, SP_OBJECT_STYLE (SP_OBJECT_PARENT (item)));
-		sp_repr_set_attr (SP_OBJECT_REPR (item), "style", str);
-		g_free (str);
+	style = SP_OBJECT_STYLE (item);
+	if (style->stroke.type != SP_PAINT_TYPE_NONE) {
+		if (!NR_DF_TEST_CLOSE (sw, 1.0, NR_EPSILON_D) || !NR_DF_TEST_CLOSE (sh, 1.0, NR_EPSILON_D)) {
+			double scale;
+			guchar *str;
+			/* Scale changed, so we have to adjust stroke width */
+			scale = sqrt (fabs (sw * sh));
+			style->stroke_width.computed *= scale;
+			if (style->stroke_dash.n_dash != 0) {
+				int i;
+				for (i = 0; i < style->stroke_dash.n_dash; i++) style->stroke_dash.dash[i] *= scale;
+				style->stroke_dash.offset *= scale;
+			}
+			str = sp_style_write_difference (style, SP_OBJECT_STYLE (SP_OBJECT_PARENT (item)));
+			sp_repr_set_attr (SP_OBJECT_REPR (item), "style", str);
+			g_free (str);
+		}
 	}
 }
 
-static void
-sp_rect_write_repr (SPObject * object, SPRepr *repr)
-{
-	SPRect *rect;
-
-	rect = SP_RECT (object);
-
-	sp_repr_set_double_attribute (repr, "width", rect->width.computed);
-	sp_repr_set_double_attribute (repr, "height", rect->height.computed);
-	sp_repr_set_double_attribute (repr, "rx", rect->rx.computed);
-	sp_repr_set_double_attribute (repr, "ry", rect->ry.computed);
-	sp_repr_set_double_attribute (repr, "x", rect->x.computed);
-	sp_repr_set_double_attribute (repr, "y", rect->y.computed);
-}
-
 /* Generate context menu item section */
 
 static void
@@ -589,7 +622,7 @@ sp_rect_knot_holder (SPItem *item, SPDesktop *desktop)
 	SPKnotHolder *knot_holder;
 
 	rect = SP_RECT (item);
-	knot_holder = sp_knot_holder_new (desktop, item);
+	knot_holder = sp_knot_holder_new (desktop, item, NULL);
 	
 	sp_knot_holder_add (knot_holder, sp_rect_rx_set, sp_rect_rx_get);
 	sp_knot_holder_add (knot_holder, sp_rect_ry_set, sp_rect_ry_get);
