@@ -19,14 +19,18 @@
 #include <glib.h>
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
+#include <libnr/nr-pixblock.h>
 #include <libart_lgpl/art_svp.h>
 #include <libart_lgpl/art_svp_wind.h>
+#include <gtk/gtksignal.h>
+#include <gtk/gtkmenuitem.h>
 
 #include "macros.h"
 #include "helper/art-utils.h"
 #include "svg/svg.h"
 #include "dialogs/fill-style.h"
 #include "display/nr-arena-shape.h"
+#include "print.h"
 #include "document.h"
 #include "desktop.h"
 #include "selection.h"
@@ -40,14 +44,13 @@
 
 static void sp_shape_class_init (SPShapeClass *class);
 static void sp_shape_init (SPShape *shape);
-static void sp_shape_destroy (GtkObject *object);
 
 static void sp_shape_build (SPObject * object, SPDocument * document, SPRepr * repr);
-static void sp_shape_write_repr (SPObject * object, SPRepr * repr);
 static void sp_shape_modified (SPObject *object, guint flags);
 static void sp_shape_style_modified (SPObject *object, guint flags);
+static SPRepr *sp_shape_write (SPObject *object, SPRepr *repr, guint flags);
 
-void sp_shape_print (SPItem * item, GnomePrintContext * gpc);
+void sp_shape_print (SPItem * item, SPPrintContext * ctx);
 static gchar * sp_shape_description (SPItem * item);
 static NRArenaItem *sp_shape_show (SPItem *item, NRArena *arena);
 static void sp_shape_menu (SPItem *item, SPDesktop *desktop, GtkMenu *menu);
@@ -70,9 +73,7 @@ sp_shape_get_type (void)
 			sizeof (SPShapeClass),
 			(GtkClassInitFunc) sp_shape_class_init,
 			(GtkObjectInitFunc) sp_shape_init,
-			NULL, /* reserved_1 */
-			NULL, /* reserved_2 */
-			(GtkClassInitFunc) NULL
+			NULL, NULL, NULL
 		};
 		shape_type = gtk_type_unique (sp_path_get_type (), &shape_info);
 	}
@@ -94,10 +95,8 @@ sp_shape_class_init (SPShapeClass * klass)
 
 	parent_class = gtk_type_class (sp_path_get_type ());
 
-	gtk_object_class->destroy = sp_shape_destroy;
-
 	sp_object_class->build = sp_shape_build;
-	sp_object_class->write_repr = sp_shape_write_repr;
+	sp_object_class->write = sp_shape_write;
 	sp_object_class->modified = sp_shape_modified;
 	sp_object_class->style_modified = sp_shape_style_modified;
 
@@ -119,17 +118,6 @@ sp_shape_init (SPShape *shape)
 	/* Nothing here */
 }
 
-static void
-sp_shape_destroy (GtkObject *object)
-{
-	SPShape *shape;
-
-	shape = SP_SHAPE (object);
-
-	if (GTK_OBJECT_CLASS (parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
-}
-
 /* fixme: Better place (Lauris) */
 
 static guint
@@ -210,28 +198,35 @@ sp_shape_build (SPObject *object, SPDocument *document, SPRepr *repr)
 #endif
 }
 
-static void
-sp_shape_write_repr (SPObject * object, SPRepr * repr)
+static SPRepr *
+sp_shape_write (SPObject *object, SPRepr *repr, guint flags)
 {
-	SPPath     *path;
-	SPPathComp *pathcomp;
-	ArtBpath   *abp;
-	gchar      *str;
+	SPPath *path;
 
-	path = SP_PATH(object);
-	g_assert (path->comp);
-	pathcomp = path->comp->data;
-	g_assert (pathcomp);
-	abp = sp_curve_first_bpath (pathcomp->curve);
-	str = sp_svg_write_path (abp);
-	g_assert (str != NULL);
-	sp_repr_set_attr (repr, "d", str);
-	g_free (str);
+	path = SP_PATH (object);
 
-#if 0
-	if (((SPObjectClass *) (parent_class))->write_repr)
-		(*((SPObjectClass *) (parent_class))->write_repr) (object, repr);
-#endif
+	if ((flags & SP_OBJECT_WRITE_BUILD) && !repr) {
+		repr = sp_repr_new ("path");
+	}
+
+	if (path->independent || (flags & SP_SHAPE_WRITE_PATH)) {
+		SPPathComp *pathcomp;
+		ArtBpath *abp;
+		gchar *str;
+		g_assert (path->comp);
+		pathcomp = path->comp->data;
+		g_assert (pathcomp);
+		abp = sp_curve_first_bpath (pathcomp->curve);
+		str = sp_svg_write_path (abp);
+		g_assert (str != NULL);
+		sp_repr_set_attr (repr, "d", str);
+		g_free (str);
+	}
+
+	if (((SPObjectClass *) (parent_class))->write)
+		((SPObjectClass *) (parent_class))->write (object, repr, flags);
+
+	return repr;
 }
 
 static void
@@ -250,7 +245,7 @@ sp_shape_modified (SPObject *object, guint flags)
 		ArtDRect paintbox;
 		/* This is suboptimal, because changing parent style schedules recalculation */
 		/* But on the other hand - how can we know that parent does not tie style and transform */
-		sp_item_invoke_bbox (SP_ITEM (object), &paintbox, SP_MATRIX_D_IDENTITY);
+		sp_item_invoke_bbox (SP_ITEM (object), &paintbox, NR_MATRIX_D_IDENTITY.c);
 		for (v = SP_ITEM (shape)->display; v != NULL; v = v->next) {
 			nr_arena_shape_set_paintbox (NR_ARENA_SHAPE (v->arenaitem), &paintbox);
 		}
@@ -270,139 +265,57 @@ sp_shape_style_modified (SPObject *object, guint flags)
 		(* ((SPObjectClass *) (parent_class))->style_modified) (object, flags);
 
 	for (v = SP_ITEM (shape)->display; v != NULL; v = v->next) {
-#if 0
-		/* fixme: */
-		nr_arena_shape_group_set_style (NR_ARENA_SHAPE_GROUP (v->arenaitem), object->style);
-#else
 		nr_arena_shape_set_style (NR_ARENA_SHAPE (v->arenaitem), object->style);
-#endif
 	}
 }
 
 void
-sp_shape_print (SPItem *item, GnomePrintContext *gpc)
+sp_shape_print (SPItem *item, SPPrintContext *ctx)
 {
-
-	gfloat rgb[3], opacity;
-	SPObject *object;
 	SPPath *path;
-	SPShape * shape;
-	SPPathComp * comp;
-	GSList * l;
-	ArtBpath * bpath;
+	SPPathComp *comp;
+	NRRectF pbox, dbox, bbox;
+	ArtDRect box;
+	double i2d[6];
 
-	object = SP_OBJECT (item);
 	path = SP_PATH (item);
-	shape = SP_SHAPE (item);
 
-	gnome_print_gsave (gpc);
+	if (!path->comp) return;
+	comp = path->comp->data;
+	if (!comp->curve) return;
 
-	for (l = path->comp; l != NULL; l = l->next) {
-		comp = (SPPathComp *) l->data;
-		if (comp->curve != NULL) {
-			const ArtBpath *bp;
-			gboolean closed;
+	/* fixme: Think (Lauris) */
+	sp_item_invoke_bbox (item, &box, NR_MATRIX_D_IDENTITY.c);
+	pbox.x0 = box.x0;
+	pbox.y0 = box.y0;
+	pbox.x1 = box.x1;
+	pbox.y1 = box.y1;
+	dbox.x0 = 0.0;
+	dbox.y0 = 0.0;
+	dbox.x1 = sp_document_width (SP_OBJECT_DOCUMENT (item));
+	dbox.y1 = sp_document_height (SP_OBJECT_DOCUMENT (item));
+	sp_item_bbox_desktop (item, &box);
+	bbox.x0 = box.x0;
+	bbox.y0 = box.y0;
+	bbox.x1 = box.x1;
+	bbox.y1 = box.y1;
+	sp_item_i2d_affine (item, i2d);
 
-			gnome_print_gsave (gpc);
-			gnome_print_concat (gpc, comp->affine);
-			bpath = comp->curve->bpath;
-
-			closed = TRUE;
-			for (bp = bpath; bp->code != ART_END; bp++) {
-				if (bp->code == ART_MOVETO_OPEN) {
-					closed = FALSE;
-					break;
-				}
-			}
-
-			gnome_print_bpath (gpc, bpath, FALSE);
-
-			if (closed) {
-				if (object->style->fill.type == SP_PAINT_TYPE_COLOR) {
-					sp_color_get_rgb_floatv (&object->style->fill.value.color, rgb);
-					/* fixme: */
-					opacity = SP_SCALE24_TO_FLOAT (object->style->fill_opacity.value)
-						* SP_SCALE24_TO_FLOAT (object->style->opacity.value);
-					gnome_print_gsave (gpc);
-					gnome_print_setrgbcolor (gpc, rgb[0], rgb[1], rgb[2]);
-					gnome_print_setopacity (gpc, opacity);
-					if (object->style->fill_rule.value == ART_WIND_RULE_ODDEVEN) {
-						gnome_print_eofill (gpc);
-					} else {
-						gnome_print_fill (gpc);
-					}
-					gnome_print_grestore (gpc);
-				} else if (object->style->fill.type == SP_PAINT_TYPE_PAINTSERVER) {
-					SPPainter *painter;
-					gdouble ctm[6];
-					ArtDRect pbox;
-					sp_item_i2d_affine (item, ctm);
-					sp_item_invoke_bbox (item, &pbox, SP_MATRIX_D_IDENTITY);
-					/* fixme: */
-					painter = sp_paint_server_painter_new (SP_OBJECT_STYLE_FILL_SERVER (object),
-									       ctm,
-									       &pbox);
-					if (painter) {
-						ArtDRect dbox, bbox, cbox;
-						ArtIRect ibox;
-						gdouble i2d[6], d2i[6];
-						gint x, y;
-						guchar *rgba;
-
-						dbox.x0 = 0.0;
-						dbox.y0 = 0.0;
-						dbox.x1 = sp_document_width (SP_OBJECT_DOCUMENT (item));
-						dbox.y1 = sp_document_height (SP_OBJECT_DOCUMENT (item));
-						sp_item_bbox_desktop (item, &bbox);
-						art_drect_intersect (&cbox, &dbox, &bbox);
-						art_drect_to_irect (&ibox, &cbox);
-						sp_item_i2d_affine (item, i2d);
-						art_affine_invert (d2i, i2d);
-
-						gnome_print_gsave (gpc);
-						if (object->style->fill_rule.value == ART_WIND_RULE_ODDEVEN) {
-							gnome_print_eoclip (gpc);
-						} else {
-							gnome_print_clip (gpc);
-						}
-						gnome_print_bpath (gpc, bpath, FALSE);
-						gnome_print_concat (gpc, d2i);
-						/* Now we are in desktop coordinates */
-						rgba = nr_buffer_4_4096_get (FALSE, 0x00000000);
-						for (y = ibox.y0; y < ibox.y1; y+= 64) {
-							for (x = ibox.x0; x < ibox.x1; x+= 64) {
-								painter->fill (painter, rgba, x, y, 64, 64, 4 * 64);
-								gnome_print_gsave (gpc);
-								gnome_print_translate (gpc, x, y + 64);
-								gnome_print_scale (gpc, 64, -64);
-								gnome_print_rgbaimage (gpc, rgba, 64, 64, 4 * 64);
-								gnome_print_grestore (gpc);
-							}
-						}
-						nr_buffer_4_4096_free (rgba);
-						gnome_print_grestore (gpc);
-						sp_painter_free (painter);
-					}
-				}
-			}
-			if (object->style->stroke.type == SP_PAINT_TYPE_COLOR) {
-				sp_color_get_rgb_floatv (&object->style->stroke.value.color, rgb);
-				/* fixme: */
-				opacity = SP_SCALE24_TO_FLOAT (object->style->stroke_opacity.value)
-					* SP_SCALE24_TO_FLOAT (object->style->opacity.value);
-				gnome_print_gsave (gpc);
-				gnome_print_setrgbcolor (gpc, rgb[0], rgb[1], rgb[2]);
-				gnome_print_setopacity (gpc, opacity);
-				gnome_print_setlinewidth (gpc, object->style->stroke_width.computed);
-				gnome_print_setlinejoin (gpc, object->style->stroke_linejoin.value);
-				gnome_print_setlinecap (gpc, object->style->stroke_linecap.value);
-				gnome_print_stroke (gpc);
-				gnome_print_grestore (gpc);
-			}
-		}
-		gnome_print_grestore (gpc);
+	if (SP_OBJECT_STYLE (item)->fill.type != SP_PAINT_TYPE_NONE) {
+		NRMatrixF ctm;
+		NRBPath bp;
+		nr_matrix_multiply_fdd (&ctm, (NRMatrixD *) comp->affine, (NRMatrixD *) i2d);
+		bp.path = comp->curve->bpath;
+		sp_print_fill (ctx, &bp, &ctm, SP_OBJECT_STYLE (item), &pbox, &dbox, &bbox);
+	}
+
+	if (SP_OBJECT_STYLE (item)->stroke.type != SP_PAINT_TYPE_NONE) {
+		NRMatrixF ctm;
+		NRBPath bp;
+		nr_matrix_multiply_fdd (&ctm, (NRMatrixD *) comp->affine, (NRMatrixD *) i2d);
+		bp.path = comp->curve->bpath;
+		sp_print_stroke (ctx, &bp, &ctm, SP_OBJECT_STYLE (item), &pbox, &dbox, &bbox);
 	}
-	gnome_print_grestore (gpc);
 }
 
 static gchar *
@@ -430,7 +343,7 @@ sp_shape_show (SPItem *item, NRArena *arena)
 		ArtDRect paintbox;
 		comp = (SPPathComp *) path->comp->data;
 		nr_arena_shape_set_path (NR_ARENA_SHAPE (arenaitem), comp->curve, comp->private, comp->affine);
-		sp_item_invoke_bbox (SP_ITEM (object), &paintbox, SP_MATRIX_D_IDENTITY);
+		sp_item_invoke_bbox (SP_ITEM (object), &paintbox, NR_MATRIX_D_IDENTITY.c);
 		nr_arena_shape_set_paintbox (NR_ARENA_SHAPE (arenaitem), &paintbox);
 	}
 
@@ -493,11 +406,7 @@ sp_shape_remove_comp (SPPath *path, SPPathComp *comp)
 
 	/* fixme: */
 	for (v = item->display; v != NULL; v = v->next) {
-#if 0
-		nr_arena_shape_group_clear (NR_ARENA_SHAPE_GROUP (v->arenaitem));
-#else
 		nr_arena_shape_set_path (NR_ARENA_SHAPE (v->arenaitem), NULL, FALSE, NULL);
-#endif
 	}
 
 	if (SP_PATH_CLASS (parent_class)->remove_comp)
@@ -515,11 +424,7 @@ sp_shape_add_comp (SPPath *path, SPPathComp *comp)
 	shape = SP_SHAPE (path);
 
 	for (v = item->display; v != NULL; v = v->next) {
-#if 0
-		nr_arena_shape_group_add_component (NR_ARENA_SHAPE_GROUP (v->arenaitem), comp->curve, comp->private, comp->affine);
-#else
 		nr_arena_shape_set_path (NR_ARENA_SHAPE (v->arenaitem), comp->curve, comp->private, comp->affine);
-#endif
 	}
 
 	if (SP_PATH_CLASS (parent_class)->add_comp)
