@@ -14,11 +14,16 @@
  */
 
 #include <config.h>
+
 #include <math.h>
 #include <string.h>
+#include <stdlib.h>
 #include <glib.h>
+#include <libart_lgpl/art_affine.h>
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
+#include <gtk/gtksignal.h>
+#include <gtk/gtkmenuitem.h>
 #include "svg/svg.h"
 #include "knotholder.h"
 #include "helper/bezier-utils.h"
@@ -39,10 +44,9 @@
 
 static void sp_spiral_class_init (SPSpiralClass *class);
 static void sp_spiral_init (SPSpiral *spiral);
-static void sp_spiral_destroy (GtkObject *object);
 
 static void sp_spiral_build (SPObject * object, SPDocument * document, SPRepr * repr);
-static void sp_spiral_write_repr (SPObject * object, SPRepr * repr);
+static SPRepr *sp_spiral_write (SPObject *object, SPRepr *repr, guint flags);
 static void sp_spiral_read_attr (SPObject * object, const gchar * attr);
 
 static SPKnotHolder *sp_spiral_knot_holder (SPItem * item, SPDesktop *desktop);
@@ -91,10 +95,8 @@ sp_spiral_class_init (SPSpiralClass *class)
 
 	parent_class = gtk_type_class (sp_shape_get_type ());
 
-	gtk_object_class->destroy = sp_spiral_destroy;
-
 	sp_object_class->build = sp_spiral_build;
-	sp_object_class->write_repr = sp_spiral_write_repr;
+	sp_object_class->write = sp_spiral_write;
 	sp_object_class->read_attr = sp_spiral_read_attr;
 
 	item_class->knot_holder = sp_spiral_knot_holder;
@@ -119,26 +121,13 @@ sp_spiral_init (SPSpiral * spiral)
 	spiral->t0         = 0.0;
 }
 
-static void
-sp_spiral_destroy (GtkObject *object)
-{
-	SPSpiral *spiral;
-
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (SP_IS_SPIRAL (object));
-
-	spiral = SP_SPIRAL (object);
-
-	if (GTK_OBJECT_CLASS (parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
-}
-
 static void
 sp_spiral_build (SPObject * object, SPDocument * document, SPRepr * repr)
 {
 
-	if (SP_OBJECT_CLASS(parent_class)->build)
-		(* SP_OBJECT_CLASS(parent_class)->build) (object, document, repr);
+	if (((SPObjectClass *) parent_class)->build)
+		((SPObjectClass *) parent_class)->build (object, document, repr);
+
 	sp_spiral_read_attr (object, "sodipodi:cx");
 	sp_spiral_read_attr (object, "sodipodi:cy");
 	sp_spiral_read_attr (object, "sodipodi:expansion");
@@ -148,28 +137,35 @@ sp_spiral_build (SPObject * object, SPDocument * document, SPRepr * repr)
 	sp_spiral_read_attr (object, "sodipodi:t0");
 }
 
-static void
-sp_spiral_write_repr (SPObject * object, SPRepr * repr)
+static SPRepr *
+sp_spiral_write (SPObject *object, SPRepr *repr, guint flags)
 {
 	SPSpiral *spiral;
 
 	spiral = SP_SPIRAL (object);
 
-	/* Fixme: we may replace these attributes by
-	 * sodipodi:spiral="cx cy exp revo rad arg t0"
-	 */
-	if ((spiral->cx > SP_EPSILON) || (spiral->cx < -SP_EPSILON))
-		sp_repr_set_double_attribute (repr, "sodipodi:cx", spiral->cx);
-	if ((spiral->cy > SP_EPSILON) || (spiral->cy < -SP_EPSILON))
-		sp_repr_set_double_attribute (repr, "sodipodi:cy", spiral->cy);
-	sp_repr_set_double_attribute (repr, "sodipodi:expansion", spiral->exp);
-	sp_repr_set_double_attribute (repr, "sodipodi:revolution", spiral->revo);
-	sp_repr_set_double_attribute (repr, "sodipodi:radius", spiral->rad);
-	sp_repr_set_double_attribute (repr, "sodipodi:argument", spiral->arg);
-	sp_repr_set_double_attribute (repr, "sodipodi:t0", spiral->t0);
+	if ((flags & SP_OBJECT_WRITE_BUILD) && !repr) {
+		repr = sp_repr_new ("path");
+	}
 
-	if (((SPObjectClass *) (parent_class))->write_repr)
-		(*((SPObjectClass *) (parent_class))->write_repr) (object, repr);
+	if (flags & SP_OBJECT_WRITE_SODIPODI) {
+		/* Fixme: we may replace these attributes by
+		 * sodipodi:spiral="cx cy exp revo rad arg t0"
+		 */
+		sp_repr_set_attr (repr, "sodipodi:type", "spiral");
+		sp_repr_set_double_attribute (repr, "sodipodi:cx", spiral->cx);
+		sp_repr_set_double_attribute (repr, "sodipodi:cy", spiral->cy);
+		sp_repr_set_double_attribute (repr, "sodipodi:expansion", spiral->exp);
+		sp_repr_set_double_attribute (repr, "sodipodi:revolution", spiral->revo);
+		sp_repr_set_double_attribute (repr, "sodipodi:radius", spiral->rad);
+		sp_repr_set_double_attribute (repr, "sodipodi:argument", spiral->arg);
+		sp_repr_set_double_attribute (repr, "sodipodi:t0", spiral->t0);
+	}
+
+	if (((SPObjectClass *) (parent_class))->write)
+		((SPObjectClass *) (parent_class))->write (object, repr, flags | SP_SHAPE_WRITE_PATH);
+
+	return repr;
 }
 
 #if 0
@@ -262,8 +258,8 @@ sp_spiral_read_attr (SPObject * object, const gchar * attr)
 		}
 		sp_shape_set_shape (shape);
 	} else {
-		if (SP_OBJECT_CLASS (parent_class)->read_attr) {
-			SP_OBJECT_CLASS (parent_class)->read_attr (object, attr);
+		if (((SPObjectClass *) parent_class)->read_attr) {
+			((SPObjectClass *) parent_class)->read_attr (object, attr);
 		}
 	}
 }
@@ -504,7 +500,7 @@ sp_spiral_knot_holder (SPItem * item, SPDesktop *desktop)
 	SPKnotHolder *knot_holder;
 
 	spiral = SP_SPIRAL (item);
-	knot_holder = sp_knot_holder_new (desktop, item);
+	knot_holder = sp_knot_holder_new (desktop, item, NULL);
 
 	sp_knot_holder_add (knot_holder,
 			    sp_spiral_inner_set,
