@@ -14,11 +14,16 @@
  */
 
 #include <config.h>
+
 #include <math.h>
 #include <string.h>
+#include <stdlib.h>
 #include <glib.h>
+#include <libart_lgpl/art_affine.h>
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
+#include <gtk/gtksignal.h>
+#include <gtk/gtkmenuitem.h>
 #include "svg/svg.h"
 #include "sp-star.h"
 #include "desktop.h"
@@ -31,10 +36,9 @@
 
 static void sp_star_class_init (SPStarClass *class);
 static void sp_star_init (SPStar *star);
-static void sp_star_destroy (GtkObject *object);
 
 static void sp_star_build (SPObject * object, SPDocument * document, SPRepr * repr);
-static void sp_star_write_repr (SPObject * object, SPRepr * repr);
+static SPRepr *sp_star_write (SPObject *object, SPRepr *repr, guint flags);
 static void sp_star_read_attr (SPObject * object, const gchar * attr);
 
 static SPKnotHolder *sp_star_knot_holder (SPItem * item, SPDesktop *desktop);
@@ -84,10 +88,8 @@ sp_star_class_init (SPStarClass *class)
 
 	parent_class = gtk_type_class (sp_polygon_get_type ());
 
-	gtk_object_class->destroy = sp_star_destroy;
-
 	sp_object_class->build = sp_star_build;
-	sp_object_class->write_repr = sp_star_write_repr;
+	sp_object_class->write = sp_star_write;
 	sp_object_class->read_attr = sp_star_read_attr;
 
 	item_class->knot_holder = sp_star_knot_holder;
@@ -111,23 +113,12 @@ sp_star_init (SPStar * star)
 	star->arg1 = star->arg2 = 0.0;
 }
 
-static void
-sp_star_destroy (GtkObject *object)
-{
-	SPStar *star;
-
-	star = SP_STAR (object);
-
-	if (GTK_OBJECT_CLASS (parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
-}
-
 static void
 sp_star_build (SPObject * object, SPDocument * document, SPRepr * repr)
 {
 
-	if (SP_OBJECT_CLASS(parent_class)->build)
-		(* SP_OBJECT_CLASS(parent_class)->build) (object, document, repr);
+	if (((SPObjectClass *) parent_class)->build)
+		((SPObjectClass *) parent_class)->build (object, document, repr);
 
 	sp_star_read_attr (object, "sodipodi:cx");
 	sp_star_read_attr (object, "sodipodi:cy");
@@ -138,27 +129,32 @@ sp_star_build (SPObject * object, SPDocument * document, SPRepr * repr)
 	sp_star_read_attr (object, "sodipodi:arg2");
 }
 
-static void
-sp_star_write_repr (SPObject * object, SPRepr * repr)
+static SPRepr *
+sp_star_write (SPObject *object, SPRepr *repr, guint flags)
 {
 	SPStar *star;
 
 	star = SP_STAR (object);
 
-	sp_repr_set_int_attribute (repr, "sodipodi:sides", star->sides);
-        if (fabs (star->cx) > SP_EPSILON) {
+	if ((flags & SP_OBJECT_WRITE_BUILD) && !repr) {
+		repr = sp_repr_new ("polygon");
+	}
+
+	if (flags & SP_OBJECT_WRITE_SODIPODI) {
+		sp_repr_set_attr (repr, "sodipodi:type", "star");
+		sp_repr_set_int_attribute (repr, "sodipodi:sides", star->sides);
 		sp_repr_set_double_attribute (repr, "sodipodi:cx", star->cx);
-	}
-        if (fabs (star->cy) > SP_EPSILON) {
 		sp_repr_set_double_attribute (repr, "sodipodi:cy", star->cy);
+		sp_repr_set_double_attribute (repr, "sodipodi:r1", star->r1);
+		sp_repr_set_double_attribute (repr, "sodipodi:r2", star->r2);
+		sp_repr_set_double_attribute (repr, "sodipodi:arg1", star->arg1);
+		sp_repr_set_double_attribute (repr, "sodipodi:arg2", star->arg2);
 	}
-	sp_repr_set_double_attribute (repr, "sodipodi:r1", star->r1);
-	sp_repr_set_double_attribute (repr, "sodipodi:r2", star->r2);
-	sp_repr_set_double_attribute (repr, "sodipodi:arg1", star->arg1);
-	sp_repr_set_double_attribute (repr, "sodipodi:arg2", star->arg2);
-	
-	if (((SPObjectClass *) (parent_class))->write_repr)
-		(*((SPObjectClass *) (parent_class))->write_repr) (object, repr);
+
+	if (((SPObjectClass *) (parent_class))->write)
+		((SPObjectClass *) (parent_class))->write (object, repr, flags | SP_POLYGON_WRITE_POINTS);
+
+	return repr;
 }
 
 static void
@@ -236,8 +232,8 @@ sp_star_read_attr (SPObject * object, const gchar * attr)
 		}
 		sp_shape_set_shape (shape);
 	} else {
-		if (SP_OBJECT_CLASS (parent_class)->read_attr)
-			SP_OBJECT_CLASS (parent_class)->read_attr (object, attr);
+		if (((SPObjectClass *) parent_class)->read_attr)
+			((SPObjectClass *) parent_class)->read_attr (object, attr);
 	}
 }
 
@@ -375,7 +371,7 @@ sp_star_knot_holder (SPItem *item, SPDesktop *desktop)
 		knot_holder = (* SP_ITEM_CLASS(parent_class)->knot_holder) (item);
 #else
 	/* we don't need to get parent knot_holder */
-	knot_holder = sp_knot_holder_new (desktop, item);
+	knot_holder = sp_knot_holder_new (desktop, item, NULL);
 #endif
 
 	sp_knot_holder_add (knot_holder,
