@@ -19,6 +19,12 @@
 #include <glib.h>
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
+#include <gtk/gtksignal.h>
+#include <gtk/gtktable.h>
+#include <gtk/gtkbutton.h>
+#include <gtk/gtkspinbutton.h>
+#include <gtk/gtklabel.h>
+#include "helper/sp-canvas.h"
 #include "sp-spiral.h"
 #include "sodipodi.h"
 #include "document.h"
@@ -169,12 +175,15 @@ sp_spiral_context_root_handler (SPEventContext * event_context, GdkEvent * event
 	switch (event->type) {
 	case GDK_BUTTON_PRESS:
 		if (event->button.button == 1) {
+			NRPointF fp;
 			dragging = TRUE;
 			/* Position center */
-			sp_desktop_w2d_xy_point (event_context->desktop, &sc->center, event->button.x, event->button.y);
+			sp_desktop_w2d_xy_point (event_context->desktop, &fp, event->button.x, event->button.y);
+			sc->center.x = fp.x;
+			sc->center.y = fp.y;
 			/* Snap center to nearest magnetic point */
 			sp_desktop_free_snap (event_context->desktop, &sc->center);
-			gnome_canvas_item_grab (GNOME_CANVAS_ITEM (desktop->acetate),
+			sp_canvas_item_grab (SP_CANVAS_ITEM (desktop->acetate),
 						GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK,
 						NULL, event->button.time);
 			ret = TRUE;
@@ -182,7 +191,7 @@ sp_spiral_context_root_handler (SPEventContext * event_context, GdkEvent * event
 		break;
 	case GDK_MOTION_NOTIFY:
 		if (dragging && event->motion.state && GDK_BUTTON1_MASK) {
-			ArtPoint p;
+			NRPointF p;
 			sp_desktop_w2d_xy_point (event_context->desktop, &p, event->motion.x, event->motion.y);
 			sp_spiral_drag (sc, p.x, p.y, event->motion.state);
 			ret = TRUE;
@@ -193,7 +202,7 @@ sp_spiral_context_root_handler (SPEventContext * event_context, GdkEvent * event
 			dragging = FALSE;
 			sp_spiral_finish (sc);
 			ret = TRUE;
-			gnome_canvas_item_ungrab (GNOME_CANVAS_ITEM (desktop->acetate), event->button.time);
+			sp_canvas_item_ungrab (SP_CANVAS_ITEM (desktop->acetate), event->button.time);
 		}
 		break;
 	default:
@@ -217,6 +226,7 @@ sp_spiral_drag (SPSpiralContext * sc, double x, double y, guint state)
 	gdouble dx, dy, rad, arg;
 	GString *xs, *ys;
 	gchar status[80];
+	NRPointF fp;
 
 	desktop = SP_EVENT_CONTEXT (sc)->desktop;
 
@@ -246,8 +256,12 @@ sp_spiral_drag (SPSpiralContext * sc, double x, double y, guint state)
 /*  	} else if (state & GDK_SHIFT_MASK) { */
 
 	/* Free movement for corner point */
-	sp_desktop_d2doc_xy_point (desktop, &p0, sc->center.x, sc->center.y);
-	sp_desktop_d2doc_xy_point (desktop, &p1, x, y);
+	sp_desktop_dt2root_xy_point (desktop, &fp, sc->center.x, sc->center.y);
+	p0.x = fp.x;
+	p0.y = fp.y;
+	sp_desktop_dt2root_xy_point (desktop, &fp, x, y);
+	p1.x = fp.x;
+	p1.y = fp.y;
 	sp_desktop_free_snap (desktop, &p1);
 	
 	spiral = SP_SPIRAL (sc->item);
@@ -279,14 +293,12 @@ sp_spiral_finish (SPSpiralContext * sc)
 	if (sc->item != NULL) {
 		SPDesktop *desktop;
 		SPSpiral  *spiral;
-		SPRepr    *repr;
 
 		desktop = SP_EVENT_CONTEXT (sc)->desktop;
 		spiral = SP_SPIRAL (sc->item);
-		repr = SP_OBJECT (sc->item)->repr;
 
 		sp_shape_set_shape(SP_SHAPE(spiral));
-		sp_object_invoke_write_repr (SP_OBJECT(spiral), repr);
+		sp_object_invoke_write (SP_OBJECT (spiral), NULL, SP_OBJECT_WRITE_SODIPODI);
 
 		sp_selection_set_item (SP_DT_SELECTION (desktop), sc->item);
 		sp_document_done (SP_DT_DOCUMENT (desktop));
