@@ -5,9 +5,9 @@
  *
  * Authors:
  *   Mitsuru Oka <oka326@parkcity.ne.jp>
- *   Lauris Kaplinski <lauris@ximian.com>
+ *   Lauris Kaplinski <lauris@kaplinski.com>
  *
- * Copyright (C) 1999-2001 Lauris Kaplinski
+ * Copyright (C) 1999-2002 Lauris Kaplinski
  * Copyright (C) 2001-2002 Mitsuru Oka
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
@@ -19,6 +19,12 @@
 #include <glib.h>
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
+#include <gtk/gtksignal.h>
+#include <gtk/gtktable.h>
+#include <gtk/gtkbutton.h>
+#include <gtk/gtkspinbutton.h>
+#include <gtk/gtklabel.h>
+#include "helper/sp-canvas.h"
 #include "sp-star.h"
 #include "sodipodi.h"
 #include "document.h"
@@ -164,12 +170,15 @@ sp_star_context_root_handler (SPEventContext * event_context, GdkEvent * event)
 	switch (event->type) {
 	case GDK_BUTTON_PRESS:
 		if (event->button.button == 1) {
+			NRPointF fp;
 			dragging = TRUE;
 			/* Position center */
-			sp_desktop_w2d_xy_point (event_context->desktop, &sc->center, event->button.x, event->button.y);
+			sp_desktop_w2d_xy_point (event_context->desktop, &fp, event->button.x, event->button.y);
+			sc->center.x = fp.x;
+			sc->center.y = fp.y;
 			/* Snap center to nearest magnetic point */
 			sp_desktop_free_snap (event_context->desktop, &sc->center);
-			gnome_canvas_item_grab (GNOME_CANVAS_ITEM (desktop->acetate),
+			sp_canvas_item_grab (SP_CANVAS_ITEM (desktop->acetate),
 						GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK,
 						NULL, event->button.time);
 			ret = TRUE;
@@ -177,7 +186,7 @@ sp_star_context_root_handler (SPEventContext * event_context, GdkEvent * event)
 		break;
 	case GDK_MOTION_NOTIFY:
 		if (dragging && event->motion.state && GDK_BUTTON1_MASK) {
-			ArtPoint p;
+			NRPointF p;
 			sp_desktop_w2d_xy_point (event_context->desktop, &p, event->motion.x, event->motion.y);
 			sp_star_drag (sc, p.x, p.y, event->motion.state);
 			ret = TRUE;
@@ -188,7 +197,7 @@ sp_star_context_root_handler (SPEventContext * event_context, GdkEvent * event)
 			dragging = FALSE;
 			sp_star_finish (sc);
 			ret = TRUE;
-			gnome_canvas_item_ungrab (GNOME_CANVAS_ITEM (desktop->acetate), event->button.time);
+			sp_canvas_item_ungrab (SP_CANVAS_ITEM (desktop->acetate), event->button.time);
 		}
 		break;
 	default:
@@ -212,6 +221,7 @@ sp_star_drag (SPStarContext * sc, double x, double y, guint state)
 	gdouble sides, dx, dy, r1, arg1;
 	GString * xs, * ys;
 	gchar status[80];
+	NRPointF fp;
 
 	desktop = SP_EVENT_CONTEXT (sc)->desktop;
 
@@ -238,8 +248,12 @@ sp_star_drag (SPStarContext * sc, double x, double y, guint state)
 /*  	} else if (state & GDK_SHIFT_MASK) { */
 
 	/* Free movement for corner point */
-	sp_desktop_d2doc_xy_point (desktop, &p0, sc->center.x, sc->center.y);
-	sp_desktop_d2doc_xy_point (desktop, &p1, x, y);
+	sp_desktop_dt2root_xy_point (desktop, &fp, sc->center.x, sc->center.y);
+	p0.x = fp.x;
+	p0.y = fp.y;
+	sp_desktop_dt2root_xy_point (desktop, &fp, x, y);
+	p1.x = fp.x;
+	p1.y = fp.y;
 	sp_desktop_free_snap (desktop, &p1);
 
 	star = SP_STAR(sc->item);
@@ -275,8 +289,9 @@ sp_star_finish (SPStarContext * sc)
 		desktop = SP_EVENT_CONTEXT (sc)->desktop;
 		object  = SP_OBJECT(sc->item);
 		
-                sp_shape_set_shape(SP_SHAPE(sc->item));
-		sp_object_invoke_write_repr (object, object->repr);
+                sp_shape_set_shape (SP_SHAPE (sc->item));
+
+		sp_object_invoke_write (object, NULL, SP_OBJECT_WRITE_SODIPODI);
 
 		sp_selection_set_item (SP_DT_SELECTION (desktop), sc->item);
 		sp_document_done (SP_DT_DOCUMENT (desktop));
@@ -299,6 +314,26 @@ sp_sc_proportion_value_changed (GtkAdjustment *adj, SPStarContext *sc)
 	sp_repr_set_double (SP_EVENT_CONTEXT_REPR (sc), "proportion", adj->value);
 }
 
+static void
+sp_sc_make_sides_flat_clicked(GtkWidget *widget, GtkObject *obj)
+{
+	GtkAdjustment *adj;
+	gint n;
+	gfloat proportion;
+	gdouble k;
+	
+	adj = gtk_object_get_data (obj, "magnitude");
+	n   = (gint)adj->value;
+	
+	
+	k = sin(M_PI/n);
+	k = (1 - k) * (1 + k);
+	proportion = (gfloat)sqrt((double)k);
+
+	adj = gtk_object_get_data (obj, "proportion");
+	gtk_adjustment_set_value (adj, proportion);
+}
+
 static void
 sp_sc_defaults (GtkWidget *widget, GtkObject *obj)
 {
@@ -319,7 +354,7 @@ sp_star_context_config_widget (SPEventContext *ec)
 
 	sc = SP_STAR_CONTEXT (ec);
 
-	tbl = gtk_table_new (3, 2, FALSE);
+	tbl = gtk_table_new (4, 2, FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (tbl), 4);
 	gtk_table_set_row_spacings (GTK_TABLE (tbl), 4);
 
@@ -346,11 +381,18 @@ sp_star_context_config_widget (SPEventContext *ec)
 	gtk_widget_show (sb);
 	gtk_table_attach (GTK_TABLE (tbl), sb, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, 0, 0, 0);
 	gtk_signal_connect (GTK_OBJECT (a), "value_changed", GTK_SIGNAL_FUNC (sp_sc_proportion_value_changed), sc);
+	
 
+	/* Making sides flat */
+	b = gtk_button_new_with_label (_("Make sides flat"));
+	gtk_widget_show (b);
+	gtk_table_attach (GTK_TABLE (tbl), b, 0, 2, 2, 3, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+	gtk_signal_connect (GTK_OBJECT (b), "clicked", GTK_SIGNAL_FUNC (sp_sc_make_sides_flat_clicked), tbl);
+	
 	/* Reset */
 	b = gtk_button_new_with_label (_("Defaults"));
 	gtk_widget_show (b);
-	gtk_table_attach (GTK_TABLE (tbl), b, 0, 2, 2, 3, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+	gtk_table_attach (GTK_TABLE (tbl), b, 0, 2, 3, 4, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
 	gtk_signal_connect (GTK_OBJECT (b), "clicked", GTK_SIGNAL_FUNC (sp_sc_defaults), tbl);
 
 	return tbl;
