@@ -41,9 +41,6 @@ static void sp_style_read_dash (ArtVpathDash *dash, const guchar *str);
 
 static SPTextStyle *sp_text_style_new (void);
 static void sp_text_style_clear (SPTextStyle *ts);
-#if 0
-static SPTextStyle *sp_text_style_ref (SPTextStyle *st);
-#endif
 static SPTextStyle *sp_text_style_unref (SPTextStyle *st);
 static SPTextStyle *sp_text_style_duplicate_unset (SPTextStyle *st);
 static guint sp_text_style_write (guchar *p, guint len, SPTextStyle *st);
@@ -191,7 +188,7 @@ static const SPStyleEnum enum_writing_mode[] = {
 };
 
 static void
-sp_style_object_destroyed (GtkObject *object, SPStyle *style)
+sp_style_object_released (GtkObject *object, SPStyle *style)
 {
 	style->object = NULL;
 }
@@ -224,7 +221,7 @@ sp_style_new_from_object (SPObject *object)
 	style = sp_style_new ();
 
 	style->object = object;
-	gtk_signal_connect (GTK_OBJECT (object), "destroy", GTK_SIGNAL_FUNC (sp_style_object_destroyed), style);
+	gtk_signal_connect (GTK_OBJECT (object), "release", GTK_SIGNAL_FUNC (sp_style_object_released), style);
 
 	return style;
 }
@@ -273,25 +270,29 @@ sp_style_read (SPStyle *style, SPObject *object, SPRepr *repr)
 
 	sp_style_clear (style);
 
-	/* 1. Presentation-only attributes */
-	/* CMYK has precedence and can only be presentation attribute */
-	val = sp_repr_attr (repr, "fill-cmyk");
-	if (val && sp_style_read_color_cmyk (&style->fill.value.color, val)) {
-		style->fill.set = TRUE;
-		style->fill.inherit = FALSE;
-	}
-	val = sp_repr_attr (repr, "stroke-cmyk");
-	if (val && sp_style_read_color_cmyk (&style->stroke.value.color, val)) {
-		style->stroke.set = TRUE;
-		style->stroke.inherit = FALSE;
-	}
-
-	/* 2. Style itself */
+	/* 1. Style itself */
 	val = sp_repr_attr (repr, "style");
 	if (val != NULL) {
 		sp_style_merge_from_style_string (style, val);
 	}
 
+	/* 2. Presentation-only attributes */
+	/* CMYK has precedence and can only be presentation attribute */
+	if (!style->fill.set || (style->fill.type == SP_PAINT_TYPE_COLOR)) {
+		val = sp_repr_attr (repr, "fill-cmyk");
+		if (val && sp_style_read_color_cmyk (&style->fill.value.color, val)) {
+			style->fill.set = TRUE;
+			style->fill.inherit = FALSE;
+		}
+	}
+	if (!style->stroke.set || (style->stroke.type == SP_PAINT_TYPE_COLOR)) {
+		val = sp_repr_attr (repr, "stroke-cmyk");
+		if (val && sp_style_read_color_cmyk (&style->stroke.value.color, val)) {
+			style->stroke.set = TRUE;
+			style->stroke.inherit = FALSE;
+		}
+	}
+
 	/* fixme: CSS etc. parsing goes here */
 	/* 3. Other styling methods */
 
@@ -343,6 +344,20 @@ sp_style_read (SPStyle *style, SPObject *object, SPRepr *repr)
 			sp_style_read_iscale24 (&style->stroke_opacity, val);
 		}
 	}
+	if (!style->stroke_dasharray_set) {
+		val = sp_repr_attr (repr, "stroke-dasharray");
+		sp_style_read_dash (&style->stroke_dash, val);
+		style->stroke_dasharray_set = TRUE;
+	}
+	if (!style->stroke_dashoffset_set) {
+		/* fixme */
+		val = sp_repr_attr (repr, "stroke-dashoffset");
+		if (sp_svg_read_number_d (val, &style->stroke_dash.offset)) {
+			style->stroke_dashoffset_set = TRUE;
+		} else {
+			style->stroke_dashoffset_set = FALSE;
+		}
+	}
 
 	/* font-family */
 	if (!style->text_private || !style->text->font_family.set) {
@@ -722,7 +737,7 @@ sp_style_merge_from_parent (SPStyle *style, SPStyle *parent)
 		style->fill_opacity.value = parent->fill_opacity.value;
 	}
 	if (!style->fill_rule.set || style->fill_rule.inherit) {
-		style->fill_rule.value = parent->fill_rule.value;
+		style->fill_rule.computed = parent->fill_rule.computed;
 	}
 	/* Stroke */
 	if (!style->stroke.set || style->stroke.inherit) {
@@ -741,10 +756,10 @@ sp_style_merge_from_parent (SPStyle *style, SPStyle *parent)
 		style->stroke_width.computed = style->font_size.computed * 0.5;
 	}
 	if (!style->stroke_linecap.set || style->stroke_linecap.inherit) {
-		style->stroke_linecap.value = parent->stroke_linecap.value;
+		style->stroke_linecap.computed = parent->stroke_linecap.computed;
 	}
 	if (!style->stroke_linejoin.set || style->stroke_linejoin.inherit) {
-		style->stroke_linejoin.value = parent->stroke_linejoin.value;
+		style->stroke_linejoin.computed = parent->stroke_linejoin.computed;
 	}
 	if (!style->stroke_miterlimit.set || style->stroke_miterlimit.inherit) {
 		style->stroke_miterlimit.value = parent->stroke_miterlimit.value;
@@ -781,7 +796,7 @@ sp_style_merge_from_parent (SPStyle *style, SPStyle *parent)
 }
 
 static void
-sp_style_paint_server_destroy (SPPaintServer *server, SPStyle *style)
+sp_style_paint_server_release (SPPaintServer *server, SPStyle *style)
 {
 	if ((style->fill.type == SP_PAINT_TYPE_PAINTSERVER) && (server == style->fill.value.server)) {
 		sp_style_paint_clear (style, &style->fill, TRUE, FALSE);
@@ -824,8 +839,8 @@ sp_style_merge_ipaint (SPStyle *style, SPIPaint *paint, SPIPaint *parent)
 		paint->value.server = parent->value.server;
 		if (paint->value.server) {
 			sp_object_href (SP_OBJECT (paint->value.server), style);
-			gtk_signal_connect (GTK_OBJECT (paint->value.server), "destroy",
-					    GTK_SIGNAL_FUNC (sp_style_paint_server_destroy), style);
+			gtk_signal_connect (GTK_OBJECT (paint->value.server), "release",
+					    GTK_SIGNAL_FUNC (sp_style_paint_server_release), style);
 			gtk_signal_connect (GTK_OBJECT (paint->value.server), "modified",
 					    GTK_SIGNAL_FUNC (sp_style_paint_server_modified), style);
 		}
@@ -946,7 +961,7 @@ sp_style_write_difference (SPStyle *from, SPStyle *to)
 	p += sp_style_write_iscale24 (p, c + BMAX - p, "stroke-opacity", &from->stroke_opacity, &to->stroke_opacity, SP_STYLE_FLAG_IFDIFF);
 
 	/* fixme: */
-	sp_text_style_write (p, c + BMAX - p, from->text);
+	p += sp_text_style_write (p, c + BMAX - p, from->text);
 
 	p += sp_style_write_ienum (p, c + BMAX - p, "text-anchor", enum_text_anchor, &from->text_anchor, &to->text_anchor, SP_STYLE_FLAG_IFDIFF);
 	p += sp_style_write_ienum (p, c + BMAX - p, "writing-mode", enum_writing_mode, &from->writing_mode, &to->writing_mode, SP_STYLE_FLAG_IFDIFF);
@@ -1043,6 +1058,7 @@ sp_style_read_dash (ArtVpathDash *dash, const guchar *str)
 			n_dash += 1;
 			str = e;
 		}
+		while (str && *str && !isalnum (*str)) str += 1;
 	}
 
 	if (n_dash > 0) {
@@ -1173,22 +1189,10 @@ sp_text_style_clear (SPTextStyle *ts)
 	ts->font_size_adjust_set = FALSE;
 
 	ts->direction_set = FALSE;
-	ts->letter_spacing_set = FALSE;
 	ts->text_decoration_set = FALSE;
 	ts->unicode_bidi_set = FALSE;
-	ts->word_spacing_set = FALSE;
 }
 
-#if 0
-static SPTextStyle *
-sp_text_style_ref (SPTextStyle *st)
-{
-	st->refcount += 1;
-
-	return st;
-}
-#endif
-
 static SPTextStyle *
 sp_text_style_unref (SPTextStyle *st)
 {
@@ -1197,7 +1201,6 @@ sp_text_style_unref (SPTextStyle *st)
 	if (st->refcount < 1) {
 		if (st->font.value) g_free (st->font.value);
 		if (st->font_family.value) g_free (st->font_family.value);
-		if (st->face) gnome_font_face_unref (st->face);
 		g_free (st);
 	}
 
@@ -1219,27 +1222,6 @@ sp_text_style_duplicate_unset (SPTextStyle *st)
 	return nt;
 }
 
-#if 0
-static guint
-sp_text_style_write_property (guchar *p, guint len, const guchar *key, const guchar *value)
-{
-	guint klen, vlen;
-
-	klen = strlen (key);
-	vlen = strlen (value);
-
-	if ((klen + vlen + 2) <= len) {
-		memcpy (p, key, klen);
-		*(p + klen) = ':';
-		memcpy (p + klen + 1, value, vlen);
-		*(p + klen + vlen + 1) = ';';
-		return klen + vlen + 2;
-	}
-
-	return 0;
-}
-#endif
-
 static guint
 sp_text_style_write (guchar *p, guint len, SPTextStyle *st)
 {
@@ -1379,6 +1361,7 @@ sp_style_read_iscale24 (SPIScale24 *val, const guchar *str)
 			val->set = TRUE;
 			val->inherit = FALSE;
 			val->value = SP_SCALE24_FROM_FLOAT (value);
+			val->value = CLAMP (val->value, 0, SP_SCALE24_MAX);
 		}
 	}
 }
@@ -1502,8 +1485,8 @@ sp_style_read_ipaint (SPIPaint *paint, const guchar *str, SPStyle *style, SPDocu
 				paint->type = SP_PAINT_TYPE_PAINTSERVER;
 				paint->value.server = SP_PAINT_SERVER (ps);
 				sp_object_href (SP_OBJECT (paint->value.server), style);
-				gtk_signal_connect (GTK_OBJECT (paint->value.server), "destroy",
-						    GTK_SIGNAL_FUNC (sp_style_paint_server_destroy), style);
+				gtk_signal_connect (GTK_OBJECT (paint->value.server), "release",
+						    GTK_SIGNAL_FUNC (sp_style_paint_server_release), style);
 				gtk_signal_connect (GTK_OBJECT (paint->value.server), "modified",
 						    GTK_SIGNAL_FUNC (sp_style_paint_server_modified), style);
 				paint->set = TRUE;
@@ -1668,7 +1651,7 @@ static gint
 sp_style_write_ienum (guchar *p, gint len, const guchar *key, const SPStyleEnum *dict, SPIEnum *val, SPIEnum *base, guint flags)
 {
 	if (((flags & SP_STYLE_FLAG_IFSET) && val->set) ||
-	    ((flags & SP_STYLE_FLAG_IFDIFF) && (val->value != base->value))) {
+	    ((flags & SP_STYLE_FLAG_IFDIFF) && (val->computed != base->computed))) {
 		gint i;
 		for (i = 0; dict[i].key; i++) {
 			if (dict[i].value == val->value) {
