@@ -14,10 +14,8 @@
 
 #include <glib.h>
 #include <libart_lgpl/art_bpath.h>
-#include <libgnomeprint/gnome-font.h>
 #include "svg-types.h"
 #include "../helper/units.h"
-#include "../display/stroke.h"
 
 /* Generic */
 
@@ -46,9 +44,6 @@ gboolean sp_svg_length_read (const unsigned char *str, SPSVGLength *length);
 gboolean sp_svg_length_read_lff (const unsigned char *str, unsigned long *unit, float *value, float *computed);
 void sp_svg_length_unset (SPSVGLength *length, unsigned long unit, float value, float computed);
 
-#if 0
-gdouble sp_svg_read_length (const SPUnit **unit, const gchar *str, gdouble def);
-#endif
 gint sp_svg_write_length (gchar *buf, gint buflen, gdouble val, const SPUnit *unit);
 
 gdouble sp_svg_read_percentage (const gchar * str, gdouble def);
@@ -65,11 +60,4 @@ gint sp_svg_write_affine (gchar * buf, gint buflen, gdouble affine[]);
 ArtBpath * sp_svg_read_path (const gchar * str);
 gchar * sp_svg_write_path (const ArtBpath * bpath);
 
-SPStrokeType sp_svg_read_stroke_type (const gchar * str);
-ArtPathStrokeJoinType sp_svg_read_stroke_join (const gchar * str);
-ArtPathStrokeCapType sp_svg_read_stroke_cap (const gchar * str);
-
-GnomeFontWeight sp_svg_read_font_weight (const gchar * str);
-gboolean sp_svg_read_font_italic (const gchar * str);
-
 #endif
