@@ -5,12 +5,12 @@
  * SPTextContext
  *
  * Authors:
- *   Lauris Kaplinski <lauris@ximian.com>
+ *   Lauris Kaplinski <lauris@kaplinski.com>
  *
- * Copyright (C) 1999-2000 Lauris Kaplinski
+ * Copyright (C) 1999-2002 Lauris Kaplinski
  * Copyright (C) 2001 Ximian, Inc.
  *
- * Licensed under GNU GPL
+ * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
 #include "event-context.h"
@@ -31,12 +31,16 @@ struct _SPTextContext {
 
 	SPItem *text;
 
-	/* Text item position in document coordinates */
-	ArtPoint pdoc;
+	/* Text item position in root coordinates */
+	NRPointF pdoc;
 	/* Insertion point position */
 	gint ipos;
 
-	GnomeCanvasItem *cursor;
+	guchar uni[5];
+	guchar unimode;
+	guchar unipos;
+
+	SPCanvasItem *cursor;
 	gint timeout;
 	guint show : 1;
 	guint phase : 1;
