@@ -9,7 +9,7 @@
  *
  */
 
-#define noSPPS_PREVIEW
+#define noSP_PS_VERBOSE
 
 #include <config.h>
 
@@ -17,6 +17,9 @@
 #include <string.h>
 #include <stdlib.h>
 
+#include <libnr/nr-values.h>
+#include <libnr/nr-matrix.h>
+
 #include <gtk/gtksignal.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkradiobutton.h>
@@ -33,7 +36,8 @@
 
 #include <libgnomeui/gnome-stock.h>
 
-#include "../sp-object.h"
+#include "../sp-item.h"
+#include "../sp-gradient.h"
 
 #include "sp-color-selector.h"
 /* fixme: Move it from dialogs to here */
@@ -157,12 +161,14 @@ sp_paint_selector_init (SPPaintSelector *psel)
 							  SP_PAINT_SELECTOR_MODE_COLOR_RGB, GTK_RADIO_BUTTON (psel->none));
 	psel->gradient = sp_paint_selector_style_button_add (psel, "fill_gradient.xpm",
 							     SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR, GTK_RADIO_BUTTON (psel->solid));
+	psel->radial = sp_paint_selector_style_button_add (psel, "fill_radial.xpm",
+							   SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL, GTK_RADIO_BUTTON (psel->gradient));
 	psel->pattern = sp_paint_selector_style_button_add (psel, "fill_pattern.xpm",
-							    SP_PAINT_SELECTOR_MODE_PATTERN, GTK_RADIO_BUTTON (psel->gradient));
-	gtk_widget_set_sensitive (psel->pattern, FALSE);
+							    SP_PAINT_SELECTOR_MODE_PATTERN, GTK_RADIO_BUTTON (psel->radial));
+	gtk_widget_hide (psel->pattern);
 	psel->fractal = sp_paint_selector_style_button_add (psel, "fill_fractal.xpm",
 							    SP_PAINT_SELECTOR_MODE_FRACTAL, GTK_RADIO_BUTTON (psel->pattern));
-	gtk_widget_set_sensitive (psel->fractal, FALSE);
+	gtk_widget_hide (psel->fractal);
 
 	/* Horizontal separator */
 	w = gtk_hseparator_new ();
@@ -174,6 +180,9 @@ sp_paint_selector_init (SPPaintSelector *psel)
 	gtk_widget_show (psel->frame);
 	gtk_container_set_border_width (GTK_CONTAINER (psel->frame), 4);
 	gtk_box_pack_start (GTK_BOX (psel), psel->frame, TRUE, TRUE, 0);
+
+	/* Last used color */
+	psel->rgba = 0x000000ff;
 }
 
 static void
@@ -233,7 +242,9 @@ sp_paint_selector_set_mode (SPPaintSelector *psel, SPPaintSelectorMode mode)
 {
 	if (psel->mode != mode) {
 		psel->update = TRUE;
+#ifdef SP_PS_VERBOSE
 		g_print ("Mode change %d -> %d\n", psel->mode, mode);
+#endif
 		switch (mode) {
 		case SP_PAINT_SELECTOR_MODE_EMPTY:
 			sp_paint_selector_set_mode_empty (psel);
@@ -272,9 +283,9 @@ void
 sp_paint_selector_set_color_rgba_floatv (SPPaintSelector *psel, gfloat *rgba)
 {
 	SPColorSelector *csel;
-
+#ifdef SP_PS_VERBOSE
 	g_print ("PaintSelector set RGBA\n");
-
+#endif
 	sp_paint_selector_set_mode (psel, SP_PAINT_SELECTOR_MODE_COLOR_RGB);
 
 	csel = gtk_object_get_data (GTK_OBJECT (psel->selector), "color-selector");
@@ -286,9 +297,9 @@ void
 sp_paint_selector_set_color_cmyka_floatv (SPPaintSelector *psel, gfloat *cmyka)
 {
 	SPColorSelector *csel;
-
+#ifdef SP_PS_VERBOSE
 	g_print ("PaintSelector set CMYKA\n");
-
+#endif
 	sp_paint_selector_set_mode (psel, SP_PAINT_SELECTOR_MODE_COLOR_CMYK);
 
 	csel = gtk_object_get_data (GTK_OBJECT (psel->selector), "color-selector");
@@ -300,16 +311,60 @@ void
 sp_paint_selector_set_gradient_linear (SPPaintSelector *psel, SPGradient *vector)
 {
 	SPGradientSelector *gsel;
-
+#ifdef SP_PS_VERBOSE
 	g_print ("PaintSelector set GRADIENT LINEAR\n");
-
+#endif
 	sp_paint_selector_set_mode (psel, SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR);
 
 	gsel = gtk_object_get_data (GTK_OBJECT (psel->selector), "gradient-selector");
 
+	sp_gradient_selector_set_mode (gsel, SP_GRADIENT_SELECTOR_MODE_LINEAR);
 	sp_gradient_selector_set_vector (gsel, (vector) ? SP_OBJECT_DOCUMENT (vector) : NULL, vector);
 }
 
+void
+sp_paint_selector_set_lgradient_position (SPPaintSelector *psel, gdouble x0, gdouble y0, gdouble x1, gdouble y1)
+{
+	SPGradientSelector *gsel;
+
+	g_return_if_fail (psel != NULL);
+	g_return_if_fail (SP_IS_PAINT_SELECTOR (psel));
+	g_return_if_fail (psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR);
+
+	gsel = gtk_object_get_data (GTK_OBJECT (psel->selector), "gradient-selector");
+
+	sp_gradient_selector_set_lgradient_position (gsel, x0, y0, x1, y1);
+}
+
+void
+sp_paint_selector_set_gradient_radial (SPPaintSelector *psel, SPGradient *vector)
+{
+	SPGradientSelector *gsel;
+#ifdef SP_PS_VERBOSE
+	g_print ("PaintSelector set GRADIENT RADIAL\n");
+#endif
+	sp_paint_selector_set_mode (psel, SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL);
+
+	gsel = gtk_object_get_data (GTK_OBJECT (psel->selector), "gradient-selector");
+
+	sp_gradient_selector_set_mode (gsel, SP_GRADIENT_SELECTOR_MODE_RADIAL);
+	sp_gradient_selector_set_vector (gsel, (vector) ? SP_OBJECT_DOCUMENT (vector) : NULL, vector);
+}
+
+void
+sp_paint_selector_set_rgradient_position (SPPaintSelector *psel, gdouble cx, gdouble cy, gdouble fx, gdouble fy, gdouble r)
+{
+	SPGradientSelector *gsel;
+
+	g_return_if_fail (psel != NULL);
+	g_return_if_fail (SP_IS_PAINT_SELECTOR (psel));
+	g_return_if_fail (psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL);
+
+	gsel = gtk_object_get_data (GTK_OBJECT (psel->selector), "gradient-selector");
+
+	sp_gradient_selector_set_rgradient_position (gsel, cx, cy, fx, fy, r);
+}
+
 void
 sp_paint_selector_set_gradient_bbox (SPPaintSelector *psel, gdouble x0, gdouble y0, gdouble x1, gdouble y1)
 {
@@ -317,25 +372,66 @@ sp_paint_selector_set_gradient_bbox (SPPaintSelector *psel, gdouble x0, gdouble
 
 	g_return_if_fail (psel != NULL);
 	g_return_if_fail (SP_IS_PAINT_SELECTOR (psel));
-	g_return_if_fail ((psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR) || (psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR));
+	g_return_if_fail ((psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR) ||
+			  (psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL));
 
 	gsel = gtk_object_get_data (GTK_OBJECT (psel->selector), "gradient-selector");
 
-	sp_gradient_selector_set_gradient_bbox (gsel, x0, y0, x1, y1);
+	sp_gradient_selector_set_bbox (gsel, x0, y0, x1, y1);
 }
 
 void
-sp_paint_selector_set_gradient_position (SPPaintSelector *psel, gdouble x0, gdouble y0, gdouble x1, gdouble y1)
+sp_paint_selector_set_gradient_gs2d_matrix_f (SPPaintSelector *psel, NRMatrixF *gs2d)
 {
 	SPGradientSelector *gsel;
 
 	g_return_if_fail (psel != NULL);
 	g_return_if_fail (SP_IS_PAINT_SELECTOR (psel));
-	g_return_if_fail ((psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR) || (psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR));
+	g_return_if_fail ((psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR) ||
+			  (psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL));
 
 	gsel = gtk_object_get_data (GTK_OBJECT (psel->selector), "gradient-selector");
 
-	sp_gradient_selector_set_gradient_position (gsel, x0, y0, x1, y1);
+	sp_gradient_selector_set_gs2d_matrix_f (gsel, gs2d);
+}
+
+void
+sp_paint_selector_get_gradient_gs2d_matrix_f (SPPaintSelector *psel, NRMatrixF *gs2d)
+{
+	SPGradientSelector *gsel;
+
+	g_return_if_fail (psel != NULL);
+	g_return_if_fail (SP_IS_PAINT_SELECTOR (psel));
+	g_return_if_fail ((psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR) ||
+			  (psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL));
+
+	gsel = gtk_object_get_data (GTK_OBJECT (psel->selector), "gradient-selector");
+
+	sp_gradient_selector_get_gs2d_matrix_f (gsel, gs2d);
+}
+
+void
+sp_paint_selector_set_gradient_properties (SPPaintSelector *psel, unsigned int units, unsigned int spread)
+{
+	SPGradientSelector *gsel;
+	g_return_if_fail (SP_IS_PAINT_SELECTOR (psel));
+	g_return_if_fail ((psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR) ||
+			  (psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL));
+	gsel = gtk_object_get_data (GTK_OBJECT (psel->selector), "gradient-selector");
+	sp_gradient_selector_set_units (gsel, units);
+	sp_gradient_selector_set_spread (gsel, spread);
+}
+
+void
+sp_paint_selector_get_gradient_properties (SPPaintSelector *psel, unsigned int *units, unsigned int *spread)
+{
+	SPGradientSelector *gsel;
+	g_return_if_fail (SP_IS_PAINT_SELECTOR (psel));
+	g_return_if_fail ((psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR) ||
+			  (psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL));
+	gsel = gtk_object_get_data (GTK_OBJECT (psel->selector), "gradient-selector");
+	if (units) *units = gsel->gradientUnits;
+	if (spread) *spread = gsel->gradientSpread;
 }
 
 void
@@ -368,7 +464,7 @@ sp_paint_selector_get_gradient_vector (SPPaintSelector *psel)
 	SPGradientSelector *gsel;
 
 	g_return_val_if_fail ((psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR) ||
-			      (psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR), NULL);
+			      (psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL), NULL);
 
 	gsel = gtk_object_get_data (GTK_OBJECT (psel->selector), "gradient-selector");
 
@@ -380,11 +476,140 @@ sp_paint_selector_get_gradient_position_floatv (SPPaintSelector *psel, gfloat *p
 {
 	SPGradientSelector *gsel;
 
-	g_return_if_fail (psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR);
+	g_return_if_fail ((psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR) ||
+			  (psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL));
 
 	gsel = gtk_object_get_data (GTK_OBJECT (psel->selector), "gradient-selector");
 
-	return sp_gradient_selector_get_position_floatv (gsel, pos);
+	if (psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR) {
+		return sp_gradient_selector_get_lgradient_position_floatv (gsel, pos);
+	} else {
+		return sp_gradient_selector_get_rgradient_position_floatv (gsel, pos);
+	}
+}
+
+void
+sp_paint_selector_write_lineargradient (SPPaintSelector *psel, SPLinearGradient *lg, SPItem *item)
+{
+	gfloat p[4], gp[4];
+	ArtDRect bbox;
+	gdouble ctm[6];
+	NRMatrixF fctm, gs2d, g2d, d2g, gs2g, g2gs;
+	NRRectF fbb;
+	double e;
+	unsigned int units, spread;
+
+	g_return_if_fail (psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR);
+
+	/* Units have to be set before position calculations */
+	sp_paint_selector_get_gradient_properties (psel, &units, &spread);
+	sp_gradient_set_units (SP_GRADIENT (lg), units);
+	sp_gradient_set_spread (SP_GRADIENT (lg), spread);
+	/* Now position */
+	sp_paint_selector_get_gradient_position_floatv (psel, p);
+	/* Calculate raw gradient transform */
+	sp_item_invoke_bbox (item, &bbox, (gdouble *) &NR_MATRIX_D_IDENTITY);
+	sp_item_i2doc_affine (item, ctm);
+	fctm.c[0] = ctm[0];
+	fctm.c[1] = ctm[1];
+	fctm.c[2] = ctm[2];
+	fctm.c[3] = ctm[3];
+	fctm.c[4] = ctm[4];
+	fctm.c[5] = ctm[5];
+	fbb.x0 = bbox.x0;
+	fbb.y0 = bbox.y0;
+	fbb.x1 = bbox.x1;
+	fbb.y1 = bbox.y1;
+	sp_paint_selector_get_gradient_gs2d_matrix_f (psel, &gs2d);
+	sp_gradient_get_g2d_matrix_f (SP_GRADIENT (lg), &fctm, &fbb, &g2d);
+	nr_matrix_f_invert (&d2g, &g2d);
+	nr_matrix_multiply_fff (&gs2g, &gs2d, &d2g);
+	/* Normalize transform */
+	gp[0] = NR_MATRIX_DF_TRANSFORM_X (&gs2g, p[0], p[1]);
+	gp[1] = NR_MATRIX_DF_TRANSFORM_Y (&gs2g, p[0], p[1]);
+	gp[2] = NR_MATRIX_DF_TRANSFORM_X (&gs2g, p[2], p[3]);
+	gp[3] = NR_MATRIX_DF_TRANSFORM_Y (&gs2g, p[2], p[3]);
+	e = NR_MATRIX_DF_EXPANSION (&gs2g);
+	if (e < 0.001) e = 0.001;
+	gs2g.c[0] /= e;
+	gs2g.c[1] /= e;
+	gs2g.c[2] /= e;
+	gs2g.c[3] /= e;
+	gs2g.c[4] = 0.0;
+	gs2g.c[5] = 0.0;
+	nr_matrix_multiply_fff (&gs2d, &gs2g, &g2d);
+	sp_gradient_set_gs2d_matrix_f (SP_GRADIENT (lg), &fctm, &fbb, &gs2d);
+	nr_matrix_f_invert (&g2gs, &gs2g);
+	p[0] = NR_MATRIX_DF_TRANSFORM_X (&g2gs, gp[0], gp[1]);
+	p[1] = NR_MATRIX_DF_TRANSFORM_Y (&g2gs, gp[0], gp[1]);
+	p[2] = NR_MATRIX_DF_TRANSFORM_X (&g2gs, gp[2], gp[3]);
+	p[3] = NR_MATRIX_DF_TRANSFORM_Y (&g2gs, gp[2], gp[3]);
+	sp_lineargradient_set_position (lg, p[0], p[1], p[2], p[3]);
+}
+
+void
+sp_paint_selector_write_radialgradient (SPPaintSelector *psel, SPRadialGradient *rg, SPItem *item)
+{
+	gfloat p[5], gp[4];
+	ArtDRect bbox;
+	gdouble ctm[6];
+	NRMatrixF fctm, gs2d, g2d, d2g, gs2g, g2gs;
+	NRRectF fbb;
+	double e;
+	unsigned int units, spread;
+
+	g_return_if_fail (psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL);
+
+	sp_paint_selector_get_gradient_properties (psel, &units, &spread);
+	sp_gradient_set_units (SP_GRADIENT (rg), units);
+	sp_gradient_set_spread (SP_GRADIENT (rg), spread);
+	/* Now position */
+	sp_paint_selector_get_gradient_position_floatv (psel, p);
+	/* Calculate raw gradient transform */
+	sp_item_invoke_bbox (item, &bbox, (gdouble *) &NR_MATRIX_D_IDENTITY);
+	sp_item_i2doc_affine (item, ctm);
+	fctm.c[0] = ctm[0];
+	fctm.c[1] = ctm[1];
+	fctm.c[2] = ctm[2];
+	fctm.c[3] = ctm[3];
+	fctm.c[4] = ctm[4];
+	fctm.c[5] = ctm[5];
+	fbb.x0 = bbox.x0;
+	fbb.y0 = bbox.y0;
+	fbb.x1 = bbox.x1;
+	fbb.y1 = bbox.y1;
+	sp_paint_selector_get_gradient_gs2d_matrix_f (psel, &gs2d);
+	sp_gradient_get_g2d_matrix_f (SP_GRADIENT (rg), &fctm, &fbb, &g2d);
+	nr_matrix_f_invert (&d2g, &g2d);
+	nr_matrix_multiply_fff (&gs2g, &gs2d, &d2g);
+	/* Normalize transform */
+	gp[0] = NR_MATRIX_DF_TRANSFORM_X (&gs2g, p[0], p[1]);
+	gp[1] = NR_MATRIX_DF_TRANSFORM_Y (&gs2g, p[0], p[1]);
+	gp[2] = NR_MATRIX_DF_TRANSFORM_X (&gs2g, p[2], p[3]);
+	gp[3] = NR_MATRIX_DF_TRANSFORM_Y (&gs2g, p[2], p[3]);
+#if 1
+	e = NR_MATRIX_DF_EXPANSION (&gs2g);
+	if (e < 0.001) e = 0.001;
+#else
+	e = 1.0;
+#endif
+	gs2g.c[0] /= e;
+	gs2g.c[1] /= e;
+	gs2g.c[2] /= e;
+	gs2g.c[3] /= e;
+#if 1
+	gs2g.c[4] = 0.0;
+	gs2g.c[5] = 0.0;
+#endif
+	nr_matrix_multiply_fff (&gs2d, &gs2g, &g2d);
+	sp_gradient_set_gs2d_matrix_f (SP_GRADIENT (rg), &fctm, &fbb, &gs2d);
+	nr_matrix_f_invert (&g2gs, &gs2g);
+	p[0] = NR_MATRIX_DF_TRANSFORM_X (&g2gs, gp[0], gp[1]);
+	p[1] = NR_MATRIX_DF_TRANSFORM_Y (&g2gs, gp[0], gp[1]);
+	p[2] = NR_MATRIX_DF_TRANSFORM_X (&g2gs, gp[2], gp[3]);
+	p[3] = NR_MATRIX_DF_TRANSFORM_Y (&g2gs, gp[2], gp[3]);
+	p[4] *= e;
+	sp_radialgradient_set_position (rg, p[0], p[1], p[2], p[3], p[4]);
 }
 
 static void
@@ -485,6 +710,8 @@ sp_paint_selector_color_released (SPColorSelector *csel, SPPaintSelector *psel)
 static void
 sp_paint_selector_color_changed (SPColorSelector *csel, SPPaintSelector *psel)
 {
+	psel->rgba = sp_color_selector_get_rgba32 (csel);
+
 	gtk_signal_emit (GTK_OBJECT (psel), psel_signals[CHANGED]);
 }
 
@@ -558,6 +785,9 @@ sp_paint_selector_set_mode_color (SPPaintSelector *psel, SPPaintSelectorMode mod
 		/* Pack everything to frame */
 		gtk_container_add (GTK_CONTAINER (psel->frame), vb);
 		psel->selector = vb;
+
+		/* Set color */
+		sp_color_selector_set_rgba32 (SP_COLOR_SELECTOR (csel), psel->rgba);
 	}
 
 	if (mode == SP_PAINT_SELECTOR_MODE_COLOR_RGB) {
@@ -569,8 +799,9 @@ sp_paint_selector_set_mode_color (SPPaintSelector *psel, SPPaintSelectorMode mod
 	}
 
 	gtk_frame_set_label (GTK_FRAME (psel->frame), _("Color paint"));
-
+#ifdef SP_PS_VERBOSE
 	g_print ("Color req\n");
+#endif
 }
 
 /* Gradient */
@@ -606,87 +837,61 @@ sp_paint_selector_set_mode_gradient (SPPaintSelector *psel, SPPaintSelectorMode
 
 	/* fixme: We do not need function-wide gsel at all */
 
-	sp_paint_selector_set_style_buttons (psel, psel->gradient);
+	if (mode == SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR) {
+		sp_paint_selector_set_style_buttons (psel, psel->gradient);
+	} else {
+		sp_paint_selector_set_style_buttons (psel, psel->radial);
+	}
 	gtk_widget_set_sensitive (psel->style, TRUE);
 
 	if ((psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR) || (psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL)) {
 		/* Already have gradient selector */
 		gsel = gtk_object_get_data (GTK_OBJECT (psel->selector), "gradient-selector");
 	} else {
-		GtkWidget *vb, *hb, *l, *om, *m, *i;
 		if (psel->selector) {
 			gtk_widget_destroy (psel->selector);
 			psel->selector = NULL;
 		}
 		/* Create new gradient selector */
-		/* Create vbox */
-		vb = gtk_vbox_new (FALSE, 4);
-		gtk_widget_show (vb);
-		/* Create hbox */
-		hb = gtk_hbox_new (FALSE, 4);
-		gtk_widget_show (hb);
-		gtk_box_pack_start (GTK_BOX (vb), hb, FALSE, FALSE, 4);
-		/* Label */
-		l = gtk_label_new (_("Mode:"));
-		gtk_widget_show (l);
-		gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
-		gtk_box_pack_start (GTK_BOX (hb), l, TRUE, TRUE, 4);
-		/* Create option menu */
-		om = gtk_option_menu_new ();
-		gtk_widget_show (om);
-		gtk_widget_set_sensitive (om, FALSE);
-		/* Create menu */
-		m = gtk_menu_new ();
-		gtk_widget_show (m);
-		i = gtk_menu_item_new_with_label (_("Linear"));
-		gtk_widget_show (i);
-		gtk_menu_append (GTK_MENU (m), i);
-		i = gtk_menu_item_new_with_label (_("Radial"));
-		gtk_widget_show (i);
-		gtk_menu_append (GTK_MENU (m), i);
-		gtk_option_menu_set_menu (GTK_OPTION_MENU (om), m);
-		gtk_object_set_data (GTK_OBJECT (vb), "mode-menu", om);
-		gtk_box_pack_start (GTK_BOX (hb), om, FALSE, FALSE, 0);
-		/* Gradient selector */
 		gsel = sp_gradient_selector_new ();
 		gtk_widget_show (gsel);
-		gtk_object_set_data (GTK_OBJECT (vb), "gradient-selector", gsel);
-		gtk_box_pack_start (GTK_BOX (vb), gsel, TRUE, TRUE, 0);
 		gtk_signal_connect (GTK_OBJECT (gsel), "grabbed", GTK_SIGNAL_FUNC (sp_paint_selector_gradient_grabbed), psel);
 		gtk_signal_connect (GTK_OBJECT (gsel), "dragged", GTK_SIGNAL_FUNC (sp_paint_selector_gradient_dragged), psel);
 		gtk_signal_connect (GTK_OBJECT (gsel), "released", GTK_SIGNAL_FUNC (sp_paint_selector_gradient_released), psel);
 		gtk_signal_connect (GTK_OBJECT (gsel), "changed", GTK_SIGNAL_FUNC (sp_paint_selector_gradient_changed), psel);
 		/* Pack everything to frame */
-		gtk_container_add (GTK_CONTAINER (psel->frame), vb);
-		psel->selector = vb;
+		gtk_container_add (GTK_CONTAINER (psel->frame), gsel);
+		psel->selector = gsel;
+		gtk_object_set_data (GTK_OBJECT (psel->selector), "gradient-selector", gsel);
 	}
 
 	/* Actually we have to set optiomenu history here */
 	if (mode == SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR) {
-		GtkWidget *om;
-		om = gtk_object_get_data (GTK_OBJECT (psel->selector), "mode-menu");
-		gtk_option_menu_set_history (GTK_OPTION_MENU (om), 0);
+		sp_gradient_selector_set_mode (SP_GRADIENT_SELECTOR (gsel), SP_GRADIENT_SELECTOR_MODE_LINEAR);
+		gtk_frame_set_label (GTK_FRAME (psel->frame), _("Linear gradient"));
 	} else {
-		GtkWidget *om;
-		om = gtk_object_get_data (GTK_OBJECT (psel->selector), "mode-menu");
-		gtk_option_menu_set_history (GTK_OPTION_MENU (om), 0);
+		sp_gradient_selector_set_mode (SP_GRADIENT_SELECTOR (gsel), SP_GRADIENT_SELECTOR_MODE_RADIAL);
+		gtk_frame_set_label (GTK_FRAME (psel->frame), _("Radial gradient"));
 	}
-
-	gtk_frame_set_label (GTK_FRAME (psel->frame), _("Gradient paint"));
-
+#ifdef SP_PS_VERBOSE
 	g_print ("Gradient req\n");
+#endif
 }
 
 static void
 sp_paint_selector_set_mode_pattern (SPPaintSelector *psel)
 {
+#ifdef SP_PS_VERBOSE
 	g_print ("Pattern req\n");
+#endif
 }
 
 static void
 sp_paint_selector_set_mode_fractal (SPPaintSelector *psel)
 {
+#ifdef SP_PS_VERBOSE
 	g_print ("Fractal req\n");
+#endif
 }
 
 static void
@@ -695,6 +900,7 @@ sp_paint_selector_set_style_buttons (SPPaintSelector *psel, GtkWidget *active)
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (psel->none), (active == psel->none));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (psel->solid), (active == psel->solid));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (psel->gradient), (active == psel->gradient));
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (psel->radial), (active == psel->radial));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (psel->pattern), (active == psel->pattern));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (psel->fractal), (active == psel->fractal));
 }
