@@ -59,6 +59,17 @@ sp_repr_new (const guchar *name)
 	return sp_repr_new_from_code (g_quark_from_string (name));
 }
 
+SPRepr *
+sp_repr_new_text (const guchar *content)
+{
+	SPRepr * repr;
+	g_return_val_if_fail (content != NULL, NULL);
+	repr = sp_repr_new_from_code (g_quark_from_string ("text"));
+	repr->content = g_strdup (content);
+	repr->type = SP_XML_TEXT_NODE;
+	return repr;
+}
+
 SPRepr *
 sp_repr_ref (SPRepr *repr)
 {
@@ -477,6 +488,29 @@ sp_repr_set_position_absolute (SPRepr * repr, gint pos)
 	sp_repr_change_order (parent, repr, ref);
 }
 
+void
+sp_repr_synthesize_events (SPRepr *repr, const SPReprEventVector *vector, gpointer data)
+{
+	
+	if (vector->attr_changed) {
+		SPReprAttr *attr;
+		attr = repr->attributes;
+		for ( ; attr ; attr = attr->next ) {
+			vector->attr_changed (repr, g_quark_to_string (attr->key), NULL, attr->value, data);
+		}
+	}
+	if (vector->child_added) {
+		SPRepr *child, *ref;
+		ref = NULL; child = repr->children;
+		for ( ; child ; ref = child, child = child->next ) {
+			vector->child_added (repr, child, ref, data);
+		}
+	}
+	if (vector->content_changed) {
+		vector->content_changed (repr, NULL, repr->content, data);
+	}
+}
+
 void
 sp_repr_add_listener (SPRepr *repr, const SPReprEventVector *vector, gpointer data)
 {
@@ -561,13 +595,19 @@ sp_repr_document_new (const gchar * rootname)
 {
 	SPRepr * repr, * root;
 
-	repr = sp_repr_new ("?xml");
+	repr = sp_repr_new ("xml");
 	if (!strcmp (rootname, "svg")) {
 		sp_repr_set_attr (repr, "version", "1.0");
 		sp_repr_set_attr (repr, "standalone", "no");
 		sp_repr_set_attr (repr, "doctype",
-				  "<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG August 1999//EN\"\n"
-				  "\"http://www.w3.org/Graphics/SVG/SVG-19990812.dtd\">");
+				  "<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 20010904//EN\"\n"
+				  "\"http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd\"\n"
+				  "[\n"
+				  " <!ATTLIST svg\n"
+				  "  xmlns:xlink CDATA #FIXED \"http://www.w3.org/1999/xlink\">\n"
+				  "]>\n");
+		sp_repr_set_attr (repr, "comment",
+				  "<!-- Created with Sodipodi (\"http://www.sodipodi.com/\") -->\n");
 	}
 
 	root = sp_repr_new (rootname);
@@ -672,9 +712,14 @@ sp_repr_merge (SPRepr *repr, const SPRepr *src, const guchar *key)
 		SPRepr *rch;
 		const guchar *id;
 		id = sp_repr_attr (child, key);
-		rch = sp_repr_lookup_child (repr, key, id);
-		if (rch) {
-			sp_repr_merge (rch, child, key);
+		if (id) {
+			rch = sp_repr_lookup_child (repr, key, id);
+			if (rch) {
+				sp_repr_merge (rch, child, key);
+			} else {
+				rch = sp_repr_duplicate (child);
+				sp_repr_append_child (repr, rch);
+			}
 		} else {
 			rch = sp_repr_duplicate (child);
 			sp_repr_append_child (repr, rch);
