@@ -49,6 +49,7 @@ typedef struct _SPRepr SPRepr;
 /* Create new repr & similar */
 
 SPRepr *sp_repr_new (const guchar *name);
+SPRepr *sp_repr_new_text (const guchar *content);
 SPRepr *sp_repr_ref (SPRepr *repr);
 SPRepr *sp_repr_unref (SPRepr *repr);
 SPRepr *sp_repr_duplicate (const SPRepr *repr);
@@ -145,6 +146,9 @@ gboolean sp_repr_get_double (SPRepr *repr, const guchar *key, gdouble *val);
 gboolean sp_repr_set_boolean (SPRepr *repr, const guchar *key, gboolean val);
 gboolean sp_repr_set_int (SPRepr *repr, const guchar *key, gint val);
 gboolean sp_repr_set_double (SPRepr *repr, const guchar *key, gdouble val);
+/* Defaults */
+gboolean sp_repr_set_double_default (SPRepr *repr, const guchar *key, gdouble val, gdouble def, gdouble e);
+
 /* Deprecated */
 gdouble sp_repr_get_double_attribute (SPRepr * repr, const gchar * key, gdouble def);
 gint sp_repr_get_int_attribute (SPRepr * repr, const gchar * key, gint def);
