@@ -1,6 +1,21 @@
-#define SP_ZOOM_CONTEXT_C
+#define __SP_ZOOM_CONTEXT_C__
+
+/*
+ * Base class for event processors
+ *
+ * Authors:
+ *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   Frank Felfe <innerspace@iname.com>
+ *
+ * Copyright (C) 1999-2002 Authors
+ *
+ * Released under GNU GPL, read the file 'COPYING' for more information
+ */
 
 #include <math.h>
+#include <stdlib.h>
+#include <gtk/gtkeditable.h>
+
 #include "rubberband.h"
 #include "sodipodi.h"
 #include "document.h"
@@ -15,9 +30,6 @@ static void sp_zoom_context_class_init (SPZoomContextClass * klass);
 static void sp_zoom_context_init (SPZoomContext * zoom_context);
 static void sp_zoom_context_destroy (GtkObject * object);
 
-#if 0
-static void sp_zoom_context_setup (SPEventContext * event_context, SPDesktop * desktop);
-#endif
 static gint sp_zoom_context_root_handler (SPEventContext * event_context, GdkEvent * event);
 static gint sp_zoom_context_item_handler (SPEventContext * event_context, SPItem * item, GdkEvent * event);
 
@@ -26,7 +38,7 @@ void sp_zoom_any (void);
 
 static SPEventContextClass * parent_class;
 
-extern GtkWidget * zoom_any;
+static GtkWidget *zoom_any = NULL;
 
 GtkType
 sp_zoom_context_get_type (void)
@@ -65,9 +77,6 @@ sp_zoom_context_class_init (SPZoomContextClass * klass)
 
 	object_class->destroy = sp_zoom_context_destroy;
 
-#if 0
-	event_context_class->setup = sp_zoom_context_setup;
-#endif
 	event_context_class->root_handler = sp_zoom_context_root_handler;
 	event_context_class->item_handler = sp_zoom_context_item_handler;
 }
@@ -91,15 +100,6 @@ sp_zoom_context_destroy (GtkObject * object)
 		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
 
-#if 0
-static void
-sp_zoom_context_setup (SPEventContext * event_context, SPDesktop * desktop)
-{
-	if (SP_EVENT_CONTEXT_CLASS (parent_class)->setup)
-		SP_EVENT_CONTEXT_CLASS (parent_class)->setup (event_context, desktop);
-}
-#endif
-
 static gint
 sp_zoom_context_item_handler (SPEventContext * event_context, SPItem * item, GdkEvent * event)
 {
@@ -117,7 +117,7 @@ static gint
 sp_zoom_context_root_handler (SPEventContext * event_context, GdkEvent * event)
 {
 	SPDesktop * desktop;
-	ArtPoint p;
+	NRPointF p;
 	ArtDRect b;
 	gint ret;
 
@@ -339,8 +339,8 @@ sp_zoom_string (gchar * zoom_str) {
 
 void
 sp_zoom_any (void) {
-  gchar * zoom_str;
-  zoom_str = gtk_editable_get_chars ((GtkEditable *) zoom_any, 0, 4);
-  sp_zoom_string (zoom_str);
+	gchar * zoom_str;
+	zoom_str = gtk_editable_get_chars ((GtkEditable *) zoom_any, 0, 4);
+	sp_zoom_string (zoom_str);
 }
 
